package ch.ige.adr.server.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import ch.ige.adr.vo.EmailadrVO;
import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.server.DBHelper;

/**
 * Datenbankzugriff auf die Emailadresse
 * <p>
 * Copyright (c) 2001, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Ueli Ryser
 * @version $Id: EmailadrDAO.java,v 1.15 2006/12/11 15:17:28 nydegger Exp $
 */
public class EmailadrDAO
{
  /**
   * Selektiert eine Emailadresse über die Adressnumer (Fremdschlüssel).
   * @param adrNr Adressnummer
   * @return Value Object der Emailadresse
   * @exception SQLException
   */
  public EmailadrVO selectByAdrNr(int adrNr) throws SQLException
  {
    String query = null;
    EmailadrVO emailadrVO = new EmailadrVO();
    Connection con = null;
    PreparedStatement ps = null;
    ResultSet rs = null;
    try
    {
      query = "SELECT emailadr_nr, adr_nr, email_name, email_domain, version " +
              "FROM emailadr WHERE adr_nr = ?";
      con = DBHelper.getConnection(DBHelper.BAGIS);
      ps = con.prepareStatement(query);
      ps.setInt(1, adrNr);
      rs = ps.executeQuery();
      if(!rs.next())
      {
        return null;
      }
      emailadrVO.setEmailadrNr(rs.getInt("emailadr_nr"));
      emailadrVO.setAdrNr(rs.getInt("adr_nr"));
      emailadrVO.setEmailName(rs.getString("email_name"));
      emailadrVO.setEmailDomain(rs.getString("email_domain"));
      emailadrVO.setVersion(rs.getInt("version"));
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, EmailadrDAO.class, query + " adr_nr = " + adrNr);
      throw ex;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
    return emailadrVO;
  }

  /**
   * Kreiert eine Emailadresse.
   * @param emailadrVO Value Object der Emailadresse
   * @exception SQLException
   */
  public void insert(EmailadrVO emailadrVO) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;
    try
    {
      query = "INSERT INTO emailadr(emailadr_nr, adr_nr, email_name, email_domain, " +
              "kz_hauptadres, anz_mail, dat_letzt_benu, dat_letzt_best, " +
              "dat_letzt_miss, sicher_kz, kz_bestaet, version)" +
              "VALUES (?, ?, ?, ?, 1, 0, null, null, null, 0, 0, ?)";
      con = DBHelper.getConnection(DBHelper.BAGIS);
      ps = con.prepareStatement(query);
      ps.setInt(1, emailadrVO.getEmailadrNr());
      ps.setInt(2, emailadrVO.getAdrNr());
      ps.setString(3, emailadrVO.getEmailName());
      ps.setString(4, emailadrVO.getEmailDomain());
      ps.setInt(5, emailadrVO.getVersion());
      ps.executeUpdate();
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, EmailadrDAO.class, query + emailadrVO.toString());
      throw ex;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

  /**
   * Setzt die Emailadresse auf den Wert des Value Objects.
   * @param emailadrVO Value Object der Emailadresse
   * @return Anzahl der aktualisierten Emailadresse
   * @exception SQLException
   */
  public int update(EmailadrVO emailadrVO) throws SQLException
  {
    String query = null;
    int rowCount;
    Connection con = null;
    PreparedStatement ps = null;
    try
    {
      query = "UPDATE emailadr SET " +
              "adr_nr = ?, " +
              "email_name = ?, " +
              "email_domain = ?, " +
              "version = ? " +
              "WHERE emailadr_nr = ?";
      con = DBHelper.getConnection(DBHelper.BAGIS);
      ps = con.prepareStatement(query);
      ps.setInt(1, emailadrVO.getAdrNr());
      ps.setString(2, emailadrVO.getEmailName());
      ps.setString(3, emailadrVO.getEmailDomain());
      ps.setInt(4, emailadrVO.getVersion());
      ps.setInt(5, emailadrVO.getEmailadrNr());
      rowCount = ps.executeUpdate();
      ps.close();
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, EmailadrDAO.class, query + emailadrVO.toString());
      throw ex;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
    return rowCount;
  }


  /**
   * Löscht eine Emailadresse.
   * @param emailadrNr Emailadressnummer der zu löschenden Adresse
   * @return Anzahl der gelöschten Emailadressen
   * @exception SQLException
   */
  public int delete(int emailadrNr) throws SQLException
  {
    String query = null;
    int rowCount;
    Connection con = null;
    PreparedStatement ps = null;
    try
    {
      query = "DELETE FROM emailadr WHERE emailadr_nr = ?";
      con = DBHelper.getConnection(DBHelper.BAGIS);
      ps = con.prepareStatement(query);
      ps.setInt(1, emailadrNr);
      rowCount = ps.executeUpdate();
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, EmailadrDAO.class, query + " emailadr_nr = " + emailadrNr);
      throw ex;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
    return rowCount;
  }
}
