package ch.ige.art.server.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;

import ch.ige.art.vo.BformartVO;
import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.server.DBHelper;

/**
 * DataAccessObject für bformart
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: BformartDAO.java,v 1.10 2008/06/25 13:44:29 ruegsegger Exp $
 */

public class BformartDAO
{
  /**
   * Selektiert einen Datensatz aus der Tabelle bformart
   *
   * @param  formartCd       Formart Code
   * @return BformartVO      Value Object
   * @throws SQLException    Unerwarteter SQL Fehler
   */
  public BformartVO select( String formartCd ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;
    ResultSet rs = null;
    BformartVO bformartVO = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );

      query = "SELECT trim(formart_cd) as formart_cd, " + "trim(bez) as bez, druck_system, version, u_version "
          + "FROM bformart WHERE formart_cd = RPAD(?, 3, ' ') ";
      ps = con.prepareStatement( query );
      ps.setString( 1, formartCd );

      rs = ps.executeQuery();
      if ( rs.next() )
      {
        bformartVO = new BformartVO();
        bformartVO.formartCd = rs.getString( "formart_cd" );
        bformartVO.bez = rs.getString( "bez" );
        bformartVO.druckSystem = rs.getString( "druck_system" );
        bformartVO.version = rs.getInt( "version" );
        bformartVO.uVersion = rs.getString( "u_version" );
      }
      return bformartVO;
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, BformartDAO.class, query + formartCd );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Selektiert alle Formulare aus der Tabelle bformart auf welche die Suchkriterien zutreffen.
   * @param formartCd         Primary Key
   * @param bez               Bezeichnung
   * @param druckSystem       Drucksystem
   * @return returnArrLi      ArrayList mit den ValueObjects
   * @exception SQLException  Unerwarteter SQL-Fehler
   */
  public ArrayList<BformartVO> select( String formartCd, String bez, String druckSystem ) throws SQLException
  {
    Connection con = null;
    Statement stmt = null;
    String query = null;
    ResultSet rs = null;
    ArrayList<BformartVO> al = new ArrayList<BformartVO>();

    StringBuffer select = new StringBuffer();
    StringBuffer from = new StringBuffer();
    StringBuffer where = new StringBuffer();
    StringBuffer order = new StringBuffer();

    select.append( "SELECT trim(formart_cd) as formart_cd, " );
    select.append( "trim(bez) as bez, druck_system, version, u_version " );
    from.append( "FROM bformart " );

    where.append( "WHERE " );
    order.append( " ORDER BY 1 " );
    String and = "";

    // kein Formular Code eingeben
    if ( formartCd == null || formartCd.equals( "" ) )
    {
      if ( bez != null && !bez.equals( "" ) )
      {
        // Suchbegriff Bezeichnung wurde eingegeben
        where.append( "trim(lower(bez)) " );
        if ( bez.indexOf( '%' ) == -1 )
        {
          where.append( "= " );
        }
        else
        {
          where.append( " LIKE " );
        }
        where.append( "'" );
        where.append( bez.toLowerCase() );
        where.append( "'" );
        and = " AND ";
      }
      if ( druckSystem != null && !druckSystem.equals( "" ) )
      {
        // Drucksystem wurde eingeben
        where.append( and );
        if ( druckSystem.equals( "CRESC" ) )
        {
          where.append( "(" );
        }
        where.append( "druck_system = " );
        where.append( "'" );
        where.append( druckSystem );
        where.append( "'" );

        if ( druckSystem.equals( "CRESC" ) )
        {
          where.append( " OR " );
          where.append( "druck_system IS NULL) " );
        }
        and = " AND ";
      }
    }
    else
    {
      // Formularcode wurde eingeben, nur nach Formularcode suchen
      where.append( "trim(formart_cd) = " );
      where.append( "'" );
      where.append( formartCd );
      where.append( "'" );
      and = " AND ";
    }
    query = select.toString() + from.toString() + where.toString() + order.toString();

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      stmt = con.createStatement();
      rs = stmt.executeQuery( query );
      while ( rs.next() )
      {
        BformartVO bformartVO = new BformartVO();

        bformartVO.formartCd = rs.getString( "formart_cd" );
        bformartVO.bez = rs.getString( "bez" );
        bformartVO.druckSystem = rs.getString( "druck_system" );
        bformartVO.version = rs.getInt( "version" );
        bformartVO.uVersion = rs.getString( "u_version" );

        al.add( bformartVO );
      }
      return al;
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, BformartDAO.class, query + " formartCd: " + formartCd + " bez: " + bez + " druckSystem: " + druckSystem );
      throw ex;
    }
    finally
    {
      DBHelper.close( stmt, con );
    }
  }

  /**
   * Fügt einen Datensatz in die Tabelle bformart ein
   *
   * @param     bformartVO       ValueObject BformartVO
   * @exception SQLException     Unerwarteter SQL Fehler
   */
  public void insert( BformartVO bformartVO ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "INSERT INTO bformart(" + "formart_cd, " + "bez, " + "druck_system, " + "version, " + "u_version )" + "VALUES( ?,?,?,?,?) ";

      ps = con.prepareStatement( query );
      ps.setString( 1, bformartVO.formartCd );
      ps.setString( 2, bformartVO.bez );
      ps.setString( 3, bformartVO.druckSystem );
      ps.setInt( 4, bformartVO.version );
      ps.setString( 5, bformartVO.uVersion );

      ps.executeUpdate();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, BformartDAO.class, query + bformartVO.toString() );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Aktualisiert einen Datensatz in der Tabelle bformart
   *
   * @param     bformartVO    BformartVO ValueObject
   * @return    int           Anzahl aktualisierter Datensätze
   * @exception SQLException  Unerwarteter SQL Fehler
   */
  public int update( BformartVO bformartVO ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "UPDATE bformart SET " + "bez = ?, " + "druck_system = ?, " + "version = ?, " + "u_version = ? " + "WHERE formart_cd = RPAD(?, 3, ' ') ";

      ps = con.prepareStatement( query );
      ps.setString( 1, bformartVO.bez );
      ps.setString( 2, bformartVO.druckSystem );
      ps.setInt( 3, bformartVO.version );
      ps.setString( 4, bformartVO.uVersion );
      ps.setString( 5, bformartVO.formartCd );

      return ps.executeUpdate();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, BformartDAO.class, query + bformartVO.toString() );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Löscht Datensätze aus der Tabelle bformart
   *
   * @param     formartCd      formart Code    Primary Key
   * @return    int            Anzahl gelöschter Datensätze
   * @exception SQLException   Unerwarteter SQL Fehler
   */
  public int delete( String formartCd ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "DELETE FROM bformart WHERE formart_cd = RPAD(?, 3, ' ') ";

      ps = con.prepareStatement( query );
      ps.setString( 1, formartCd );

      return ps.executeUpdate();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, BformartDAO.class, query + formartCd );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Prüft die Existenz eines Formart Codes
   *
   * @param     formartCd     zu prüfender FormartCode
   * @return    true          formartCode existiert.
   * @exception SQLException  Unerwarteter SQL-Fehler
   */
  public boolean exists( String formartCd ) throws SQLException
  {
    if ( formartCd == null )
    {
      throw new IllegalArgumentException( "formartCode darf nicht null sein" );
    }
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;
    ResultSet rs = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "SELECT formart_cd FROM bformart WHERE formart_cd = RPAD(?, 3, ' ') ";

      ps = con.prepareStatement( query );
      ps.setString( 1, formartCd );
      rs = ps.executeQuery();
      return rs.next();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, BformartDAO.class, query + formartCd );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }
}
