package ch.ige.art.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import ch.ige.art.server.ArtikelDelegate;
import ch.ige.art.vo.XxDomainVO;

/**
 * <p>Überschrift: XxDomain</p>
 * <p>Beschreibung: Aus der Tabelle xx_domain werden alle domains mit sprachcd=1 selektiert und in
 * eine HashMap abgefüllt.<br>
 * Ein HashMap EntrySet besteht aus dem DomainNamen als Schlüssel und einer ArrayList von VOs als Value</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author Anita Rueegsegger
 * @version $Id: XxDomain.java,v 1.10 2008/07/02 11:00:20 heinzer Exp $
 */

public class XxDomain
{
  private static XxDomain xxDomain = null; //Singleton. requires .init call prior to first use

  private static Map<String, List<XxDomainVO>> domainMap = null;

  /**
   * Private constructor because of singleton
   */
  private XxDomain() throws Exception
  {
    domainMap = new HashMap<String, List<XxDomainVO>>();
    readDomains();
  }

  /**
   * Initialise singleton
   * @throws Exception If the domain values could not be read
   */
  public static synchronized void init() throws Exception
  {
    if ( xxDomain == null )
    {
      xxDomain = new XxDomain();
    }
  }

  /**
   * gets XxDomainVO's of specific domains
   * 
   * @param domains List of domain identification names  
   * @return List of XxDomainVOs, empty list if nothing found
   */
  public static synchronized List<XxDomainVO> getDomains( String... domains )
  {
    List<XxDomainVO> list = new ArrayList<XxDomainVO>();
    for ( String domain: domains )
    {
      if ( domainMap.containsKey( domain ) )
      {
        list.addAll( domainMap.get( domain ) );
      }
    }

    return list;
  }

  /**
   * Liefert alle Kurzbezeichnungen einer domain aus xx_domain in einer ArrayList
   *
   * @param domain Name der Domain deren Kurzbezeichnungen angefordert werden
   * @return  gibt eine ArrayList von Kurzbezeichnungen einer Domain zurück.
   *          falls Domain nicht gefunden wird leere Liste zurückgegeben
   */
  public static synchronized ArrayList<String> getDomainKbez( String domain )
  {
    ArrayList<String> alKbez = new ArrayList<String>();

    ArrayList<XxDomainVO> al = (ArrayList<XxDomainVO>) domainMap.get( domain );
    if ( al != null )
    {
      alKbez = new ArrayList<String>();
      XxDomainVO xxDomainVO = new XxDomainVO();

      for ( int i = 0; i < al.size(); i++ )
      {
        xxDomainVO = al.get( i );
        alKbez.add( xxDomainVO.getKbez() );
      }
    }
    return alKbez;
  }

  /**
   * Liefert alle Bezeichnungen einer domain aus xx_domain in einer ArrayList
   *
   * @param domain Name der Domain deren Bezeichnungen angefordert werden
   * @return  gibt eine ArrayList von Bezeichnungen einer Doamin zurück
   *          falls Domain nicht gefunden wird leere Liste zurückgegeben
   */
  public static synchronized ArrayList<String> getDomainBez( String domain )
  {
    ArrayList<String> alBez = new ArrayList<String>();

    ArrayList<XxDomainVO> al = (ArrayList<XxDomainVO>) domainMap.get( domain );
    if ( al != null )
    {
      alBez = new ArrayList<String>();
      XxDomainVO xxDomainVO = new XxDomainVO();

      for ( int i = 0; i < al.size(); i++ )
      {
        xxDomainVO = al.get( i );
        alBez.add( xxDomainVO.getBez() );
      }
    }
    return alBez;
  }

  /**
   * Liefert ein ValueObject das die Daten einer Zeile aus der Tabelle xx_domain enthält
   * @param domain    Name der domain, Bestandteil des Primärschlüssels
   * @param domainLnr DomainNummer, Bestandteil des Primärschlüssels
   * @return          lifert ein XxDomainVO, Abbildung einer Zeile aus xx_domain,
   *                  falls nichts gefunden wird null zurückgegeben
   */
  public static synchronized XxDomainVO getDomainVO( String domain, int domainLnr )
  {
    ArrayList<XxDomainVO> al = (ArrayList<XxDomainVO>) domainMap.get( domain );
    if ( al != null )
    {
      for ( int i = 0; i < al.size(); i++ )
      {
        XxDomainVO xxDomainVO = al.get( i );
        if ( xxDomainVO.domainLnr == domainLnr )
        {
          return xxDomainVO;
        }
      }
    }
    return null;
  }

  /**
   * Liefert die Bezeichnung eines Domain Wertes
   *
   * @param domain Name der Domain
   * @param domainLnr Domain Wert
   * @return  Bezeichnung eines Wertes aus xx_domain,
   *          wenn keine Bezeichnung gefunden wird null zurückgegeben
   */
  public static synchronized String getBez( int domainLnr, String... domains )
  {
    List<XxDomainVO> listDomain = getDomains( domains );

    for ( XxDomainVO xxDomainVO: listDomain )
    {
      if ( xxDomainVO.domainLnr == domainLnr )
      {
        return xxDomainVO.getBez();
      }
    }
    return null;
  }

  /**
   * Liefert die Bezeichnung eines Domain Wertes, wenn der Domain Wert
   * im Feld kbez gespeichert wird (falls es sich um einen alphanumerischen Wert handelt)
   *
   * @param domain Name der Domain
   * @param kbez Domain Wert
   * @return  Bezeichnung eines Wertes aus xx_domain
   *          wenn keine Bezeichnung gefunden wird null zurückgegeben
   */
  public static synchronized String getBez( String kbez, String... domains )
  {
    List<XxDomainVO> listDomain = getDomains( domains );

    for ( XxDomainVO xxDomainVO: listDomain )
    {
      if ( xxDomainVO.getKbez().equals( kbez ) )
      {
        return xxDomainVO.getBez();
      }
    }
    return null;
  }

  /**
   * Liefert die Kurzbezeichnung eines Domain Wertes
   *
   * @param domain Name der Domain
   * @param domainLnr Domain Wert
   * @return  Kurzbezeichnung eines Wertes aus xx_domain
   *          wenn keine Bezeichnung gefunden wird null zurückgegeben
   */
  public static synchronized String getKbez( String domain, int domainLnr )
  {
    ArrayList<XxDomainVO> al = (ArrayList<XxDomainVO>) domainMap.get( domain );
    if ( al != null )
    {
      for ( int i = 0; i < al.size(); i++ )
      {
        XxDomainVO xxDomainVO = al.get( i );
        if ( xxDomainVO.domainLnr == domainLnr )
        {
          return xxDomainVO.getKbez();
        }
      }
    }
    return null;
  }

  /**
   * Liefert die Bezeichnung, welche in den Textfeldern angezeigt wird
   *
   * @param domain Name der Domain
   * @param domainLnr Domain Wert
   * @return  Kurzbezeichnung eines Wertes aus xx_domain
   *          wenn keine Bezeichnung gefunden wird "" zurückgegeben
   */
  public static synchronized String getText( String domain, String value )
  {
    ArrayList<XxDomainVO> al = (ArrayList<XxDomainVO>) domainMap.get( domain );
    if ( al != null )
    {
      for ( int i = 0; i < al.size(); i++ )
      {
        XxDomainVO xxDomainVO = al.get( i );
        if ( xxDomainVO.isEqual( value ) )
        {
          return xxDomainVO.getText();
        }
      }
    }
    return "";
  }

  /**
   * @param value key value
   * @param domain domains looking for value
   * @return legende to correspondig value, key and value separated by comma
   */
  public static synchronized String getLegend( String value, String... domains )
  {
    List<XxDomainVO> listDomain = getDomains( domains );

    for ( XxDomainVO xxDomainVO: listDomain )
    {
      if ( xxDomainVO.isEqual( value ) )
      {
        return xxDomainVO.getLegend();
      }
    }
    return value;
  }

  /**
   * Selektiert alle Domains aus der Tabelle xx_domain.
   * Die XxDomainVOs werden in einer HashMap sortiert nach domains abgelegt.
   * Es wird davon ausgegangen, dass die Werte sortiert selektiert werden,
   * damit die Ordnungszahl nicht berücksichtigt werden muss
   */
  private static synchronized void readDomains() throws Exception
  {
    ArrayList<XxDomainVO> alXxDomainVO = new ArrayList<XxDomainVO>();

    ArtikelDelegate delegate = ArtikelDelegate.getInstance();
    alXxDomainVO = delegate.findXxDomain();

    for ( int i = 0; i < alXxDomainVO.size(); i++ )
    {
      XxDomainVO xxDomainVO = alXxDomainVO.get( i );

      List<XxDomainVO> al = domainMap.get( xxDomainVO.domain );

      if ( al == null )
      {
        ArrayList<XxDomainVO> alDomain = new ArrayList<XxDomainVO>();
        alDomain.add( xxDomainVO );
        domainMap.put( xxDomainVO.domain, alDomain );
      }
      else
      {
        al.add( xxDomainVO );
      }
    }
  }
}
