package ch.ige.epband.clientneu;

import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.StringTokenizer;

import org.apache.log4j.Logger;

import ch.ige.epband.vo.StatistikVO;

/**
 * <p>
 * Überschrift: Europäische Patente (EP) einlesen
 * </p>
 * <p>
 * Beschreibung: Mir dieser Klasse wird ein Kontroll File geführt.<br>
 * In diesm File wird laufen abgespeichert, welches EP zuletzt vollständig verarbeitet werden konnte. Das File enthaelt
 * dazu eine File-Id und die Gesuch_nr. Die Fileid ist nötig, da die Gesuch-Nr mehrmals, sowohl als Gesuch als auch als
 * Patent vorkommen kann. Statistik wird gespeichert, damit sie bei einem Restart nicht neu erstellt werden muss.<br>
 * Es wurde ein randomAccessFile gewaehlt, damit das File immer offen ist und jeweils überschrieben werden kann.
 * </p>
 * <p>
 * Copyright: Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * <p>
 * Organisation: IGE
 * </p>
 * 
 * @author M. Trachsel
 * @version $Id: KontrollFileReader.java,v 1.7 2007/01/25 17:26:38 ruegsegger Exp $
 */

public class KontrollFileReader
{
  private static final Logger LOG = Logger.getLogger( KontrollFileReader.class );

  private static final String FILENAME = "\\Kontrollfile.txt";

  private static final String DELIM = "\r\n";

  private RandomAccessFile f;

  private String filenr = "";

  private String gesuchnr = "";

  private StatistikVO statistikVO = new StatistikVO();

/**
 * KontrollFile öffnen, bzw erstellen
 * @param zipFileName
 * @throws IOException
 */
  public KontrollFileReader( String zipFileName ) throws IOException
  {
    String name = zipFileName.substring( 0, zipFileName.lastIndexOf( "\\" ) ) + FILENAME;
    f = new RandomAccessFile( name, "rw" );
  }

  /**
   * Schliessen des Kontrollfiles
   */
  public void close()
  {
    if ( f != null )
    {
      try
      {
        f.close();
      }
      catch ( IOException e )
      {
        // nichts
      }
    }
  }

  /**
   * In welchem File die zuletzt verarbeitete Gesuch-Nr vorkommt diese Gesuchnr konnte nicht mehr erfolgreich
   * verarbeitet werden
   * 
   * @return filenr
    */
  public String getWhichFi()
  {
    return filenr;
  }

  /**
   * gibt zuletzt verarbeitete Gesuchnr zurück diese Gesuchnr konnte nicht mehr erfolgreich verarbeitet werden
   * 
   * @return Gesuch-nr
   */
  public String getGesuchNr()
  {
    return gesuchnr;
  }

  /**
   * gibt Statistik-Angaben zurück
   * 
   * @return StatistikVO mit Statistikdaten
   */
  public StatistikVO getStatistik()
  {
    return statistikVO;
  }

  /**
   * File-Eintrag lesen
   * 
   * @throws IOException
   */
  public void readFileEntry() throws IOException
  {
    statistikVO = new StatistikVO();

    byte[] data = new byte[(int) f.length()];

    f.seek( 0 );
    f.read( data );

    String s = new String( data );
    StringTokenizer st = new StringTokenizer( s, DELIM );

    try
    {
      filenr = ( st.hasMoreTokens() ? st.nextToken() : "" );
      statistikVO.anzWipo = ( st.hasMoreTokens() ? Integer.parseInt( st.nextToken() ) : 0 );
      statistikVO.anzWipofalse = ( st.hasMoreTokens() ? Integer.parseInt( st.nextToken() ) : 0 );
      statistikVO.anzChzurueck = ( st.hasMoreTokens() ? Integer.parseInt( st.nextToken() ) : 0 );
      statistikVO.anzChzurueckfalse = ( st.hasMoreTokens() ? Integer.parseInt( st.nextToken() ) : 0 );
      statistikVO.anzkeinEinsp = ( st.hasMoreTokens() ? Integer.parseInt( st.nextToken() ) : 0 );
      statistikVO.anzkeinEinspfalse = ( st.hasMoreTokens() ? Integer.parseInt( st.nextToken() ) : 0 );
      statistikVO.anzcreateEp = ( st.hasMoreTokens() ? Integer.parseInt( st.nextToken() ) : 0 );
      statistikVO.anzcreateEpfalse = ( st.hasMoreTokens() ? Integer.parseInt( st.nextToken() ) : 0 );
      gesuchnr = ( st.hasMoreTokens() ? st.nextToken() : "" );
    }
    catch ( NumberFormatException ex ) // NumberFormatException, NoSuchElementExcpetion
    {
      LOG.error( ex );
    }
  }

  /**
   * Eintrag ins File schreiben Einträge werden aus einer Batchdatei gelesen deshalb Bytes schreiben
   * 
   * @param fileNr FileId
   * @param gesuchNr Gesuch-Nr
   * @param statistikVo Statistikdaten
   * @throws IOException
   */
  public void writeFileEntry( String fileNr, String gesuchNr, StatistikVO statistikVo ) throws IOException
  {
    if ( fileNr.length() == 1 )
    {
      f.seek( 0 );
      f.writeBytes( fileNr + DELIM );
      f.writeBytes( String.valueOf( statistikVo.anzWipo ) + DELIM );
      f.writeBytes( String.valueOf( statistikVo.anzWipofalse ) + DELIM );
      f.writeBytes( String.valueOf( statistikVo.anzChzurueck ) + DELIM );
      f.writeBytes( String.valueOf( statistikVo.anzChzurueckfalse ) + DELIM );
      f.writeBytes( String.valueOf( statistikVo.anzkeinEinsp ) + DELIM );
      f.writeBytes( String.valueOf( statistikVo.anzkeinEinspfalse ) + DELIM );
      f.writeBytes( String.valueOf( statistikVo.anzcreateEp ) + DELIM );
      f.writeBytes( String.valueOf( statistikVo.anzcreateEpfalse ) + DELIM );
      f.writeBytes( gesuchNr );

      // Filelänge setzen da gesuchnr beliebige Länge haben kann
      // ansonsten werden bei grösseren Zahlen die hinteren Stellen nicht gelöscht
      f.setLength( f.getFilePointer() );
    }
    else
    {
      throw new IOException( "Länge von Fileid nicht 1" );
    }
  }
}
