package ch.ige.epband.clientneu;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.SortedMap;
import java.util.TreeMap;

import org.apache.commons.lang.StringEscapeUtils;
import org.apache.log4j.Logger;

import ch.ige.epband.util.Pair;
import ch.ige.epband.util.XMLDoc;
import ch.ige.util.Pruefziffer;

/**
 * <p>
 * Überschrift: Europäische Patente (EP) einlesen
 * </p>
 * <p>
 * Beschreibung: Aufbereiten der XML für die benötigten Listen
 * 
 * - EP_001: Liste der neuen Gesuche
 * - EP_002: Liste Spezial (Freie Texte)
 * - EP_003: Liste Mutation eingetragener EP
 * - EP_004: Liste der neuen Patente
 * 
 * </p>
 * <p>
 * Copyright: Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * <p>
 * Organisation: IGE
 * </p>
 * 
 * @author M. Trachsel
 * @version $Id: Listen.java,v 1.25 2008/07/07 08:29:46 ruegsegger Exp $
 */

public class Listen
{
  private static final Logger LOG = Logger.getLogger( Listen.class );

  private XMLDoc xml;

  private String epstatus = "";

  private String epkind = "";

  private String schutitNr = "";

  private String gesuchNr = "";

  private SimpleDateFormat sdfold = new SimpleDateFormat( "yyyyMMdd" );

  private SimpleDateFormat sdfnew = new SimpleDateFormat( "dd.MM.yyyy" );

  private static String publDat = null;

  private static String dirDat = null;

  private static SortedMap<String, String> listGesuchMP = new TreeMap<String, String>();

  private static ArrayList<String> listPatAL = new ArrayList<String>();

  private static StringBuffer sbXMLSpezial = new StringBuffer();

  private static StringBuffer sbXMLMutataion = new StringBuffer();

  private static String tagesDatum;

  // TODO encoding auf jipps testen
  private static final String XMLHEADER = "<?xml version = \"1.0\" encoding=\"ISO-8859-15\"?> ";

  // Speziell für Liste 3 Mutation EP
  private static final String IH = "<inhalt>";

  private static final String IHE = "</inhalt>";

  private ListeMutationLimitation listeMutationLimitation = new ListeMutationLimitation();

  private ListeMutationRevocation listeMutationRevocation = new ListeMutationRevocation();

  /**
   * Konstruktor
   *
   */
  public Listen()
  {
    sbXMLMutataion.append( XMLHEADER + "<liste>" );

    tagesDatum = sdfnew.format( new Date() );
  }

  /**
   * Diverse Listen im XML-Format aufbereiten und an JIPPS übergeben
   * 
   * @param xml SGML mit den Daten eines EP 
   */
  public void doListen( @SuppressWarnings( "hiding" )
  XMLDoc xml )
  {
    this.xml = xml;

    // Patentart und Änderungsstatus ermitteln
    epkind = xml.getAttribute( "ep-bulletin", "kind" );
    epstatus = xml.getAttribute( "ep-bulletin", "status" );

    // gelöschte EP's werden nich bearbeitet
    if ( epstatus.equalsIgnoreCase( "D" ) )
    {
      return;
    }

    // Publikationsdatum und BulletinNummer ist zwar pro SQML immer gleich
    // aber es könnte ja sein, dass .... (sonst müssen diese Daten nicht pro
    // Patent und Gesuch angegeben werden
    dirDat = xml.getValueOf( "B405", "date" );
    publDat = printDate( xml.getValueOf( "B405", "date" ) );
    String kw = xml.getValueOf( "B405", "bnum" );
    if ( !kw.equals( "" ) )
    {
      publDat += " " + kw.substring( 0, 4 ) + "/" + kw.substring( 4 );
    }

    // gesuchNR und schutitNr aufbereiten (nicht Druckfertig)
    schutitNr = Pruefziffer.modulo10_ep( xml.getValueOf( "B110" ) );
    while ( schutitNr.length() < 9 )
    {
      schutitNr = "0" + schutitNr;
    }
    gesuchNr = xml.getValueOf( "B210" );
    if ( !gesuchNr.equals( "" ) )
    {
      int pos = gesuchNr.indexOf( "." );
      gesuchNr = gesuchNr.substring( 0, pos ) + gesuchNr.substring( pos + 1 );
    }

    if ( isCHPatent() )
    {
      // Für Liste der neuen Gesuche bzw. neuen Patente ist "WO" nicht massgebend
      if ( !xml.getValueOf( "B190" ).equalsIgnoreCase( "WO" ) )
      {
        // Liste der neuen Gesuche EP_001
        prepareListeGesuch();

        // Liste der neuen Patente EP_004
        prepareListePatent();
      }
      // Liste Mutation eingetragener EP EP_003
      prepareListeMutation();

      // Liste Spezial EP_002
      prepareListeSpezial();
    }
  }

  /**
   * XML an JIPPS übergeben zum ausdrucken
   */
  public void sendJIPPS()
  {
    // XML der neuen Gesuche ausgeben
    writeListeGesuch();
    // XML der neuen patente ausgeben
    writeListePatent();
    // XML der Mutation eingetragener EP abschliessen und ausgeben
    writeListeMutation();
    // XML der Liste Spezial abschliessen und ausgeben
    writeListeSpezial();
  }

  /**
   * Test ob CH für das Patent benannt ist
   * 
   * @return boolean true falls CH für das Patent benannt
   */
  private boolean isCHPatent()
  {
    List<String> list = xml.getValuesOf( "B840", "ctry" );
    return ( list.contains( "CH" ) || list.contains( "LI" ) );
  }

  /**
   * Datum der Form yyyymmdd in Printforamt umwandelnybr> Die Exception IndexOutOfBoundsException wir abgefangen damit
   * bei evtl Fehler im SGML nicht abgerochen wird.
   * 
   * @param datum String mit datum der Form yyyymmdd
   * @return String der Form mit dd.mm.yyyy
   */
  protected String printDate( String datum )
  {

    try
    {
      return sdfnew.format( sdfold.parse( datum ) );
    }
    catch ( ParseException pex )
    {
      return datum;
    }
  }

  /**
   * PatentNummer für Druck aufbereiten
   * 
   * @param patNr String mit patNr ohne Formatierung
   * @return String mit formatierter PatentNummer
   */
  private String printpatNr( String patNr )
  {
    try
    {
      return patNr.substring( 0, 2 ) + " " + patNr.substring( 2, 5 ) + " " + patNr.substring( 5, 8 ) + "." + patNr.substring( 8, 9 );
    }
    catch ( IndexOutOfBoundsException ouex )
    {
      return patNr;
    }
  }

  /**
   * PatentNummer für Druck aufbereiten
   * 
   * @param gesuchnr String mit patNr ohne Formatierung
   * @return String mit formatierter PatentNummer
   */
  private String printgesuchNr( String gesuchnr )
  {
    try
    {
      return gesuchnr.substring( 0, 8 ) + "." + gesuchnr.substring( 8, 9 );
    }
    catch ( IndexOutOfBoundsException ouex )
    {
      return gesuchnr;
    }
  }

  /**
   * Sammeln der PatentNummern und GesuchNummern bei neu angemldeten Gesuche
   */
  private void prepareListeGesuch()
  {
    if ( epkind.compareTo( "A3" ) < 0 && epstatus.equalsIgnoreCase( "N" ) )
    {
      listGesuchMP.put( schutitNr, gesuchNr );
    }
  }

  /**
   * Sammeln der PatentNummern bei neu eingetragenen Patente
   */
  private void prepareListePatent()
  {
    if ( epkind.equals( "B1" ) && epstatus.equalsIgnoreCase( "N" ) )
    {
      listPatAL.add( schutitNr );
    }
  }

  /**
   * XML für die Liste "Mutation" aufbereiten,<br>
   * d.h. alle <BCHG..... <ECHG> suchen und auswerten
   */
  private void prepareListeMutation()
  {
    // keine neue eingetragene Patente
    // und keine Gesuche
    if ( ( epkind.equals( "B1" ) && epstatus.equalsIgnoreCase( "N" ) ) || epkind.compareTo( "B1" ) < 0 )
    {
      return;
    }
    StringBuffer sbInhalt = new StringBuffer( "" );

    String grpTag = "";
    String tag = "";
    String tagInhalt = "";

    int number = xml.getNumberOf( "ep-chg" );
    for ( int i = 1; i <= number; i++ )
    {
      grpTag = xml.getAttribute( "ep-chg[" + i + "]", "btag" );

      // TODO B050EP übersetzen mit Text
      // TODO B510EP Ipc-Klassen String aufsplitten
      if ( !grpTag.equals( "B078EP" ) && // date of 'No opposition filed'
          !grpTag.equals( "B080EP" ) && // nicht relevant für IGE
          !grpTag.equals( "B451EP" ) && // nicht relevant für IGE
          !grpTag.equals( "B452EP" ) && // nicht relevant für IGE
          !grpTag.equals( "B475" ) && // nicht relevant für IGE
          !grpTag.equals( "B477" ) && // nicht relevant für IGE
          !grpTag.equals( "B540" ) && // wegen Schachtelung
          !grpTag.equals( "B620" ) && // nicht relevant für IGE
          !grpTag.equals( "B620EP" ) && // nicht relevant für IGE
          !grpTag.equals( "B740" ) && // Attorney/agent of the applicant(s)
          !grpTag.equals( "B844EP" ) && // nicht relevant
          !grpTag.equals( "B880" ) ) // Recherchenbericht
      {
        sbInhalt.append( IH );
        if ( grpTag.equals( "B130" ) )
        {
          sbInhalt.append( "<ktext>Dokumentart</ktext>" );
        }
        else if ( grpTag.equals( "B091EP" ) )
        {
          sbInhalt.append( "<ktext>Mutation Beschränkung</ktext>" );
        }
        else if ( grpTag.equals( "B092EP" ) )
        {
          sbInhalt.append( "<ktext>Mutation Beschränkung Zurückweisung</ktext>" );
        }
        else if ( grpTag.equals( "B093EP" ) )
        {
          sbInhalt.append( "<ktext>Mutation Beschränkung Gutheissung</ktext>" );
        }
        else if ( grpTag.equals( "B4530EP" ) )
        {
          sbInhalt.append( "<ktext>Mutation Beschränkung Entscheiddatum</ktext>" );
        }
        else if ( grpTag.equals( "B239" ) )
        {
          sbInhalt.append( "<ktext>Widerruf</ktext>" );
        }
        else if ( grpTag.equals( "B243" ) )
        {
          sbInhalt.append( "<ktext>Aufrechterhaltung in geänderter Form</ktext>" );
        }
        else if ( grpTag.equals( "B246" ) )
        {
          sbInhalt.append( "<ktext>Unterbrechung des Verfahrens</ktext>" );
        }
        else if ( grpTag.equals( "B250" ) )
        {
          sbInhalt.append( "<ktext>Sprache der Einreichung</ktext>" );
        }
        else if ( grpTag.equals( "B251EP" ) )
        {
          sbInhalt.append( "<ktext>Verfahrenssprache</ktext>" );
        }
        else if ( grpTag.equals( "B300" ) )
        {
          sbInhalt.append( "<ktext>Priorität</ktext>" );
        }
        else if ( grpTag.equals( "B450" ) )
        {
          sbInhalt.append( "<ktext>Widerruf der Erteilung</ktext>" );
        }
        else if ( grpTag.equals( "B510EP" ) )
        {
          sbInhalt.append( "<ktext>IPC-Klassen</ktext>" );
        }
        else if ( grpTag.equals( "B542" ) )
        {
          sbInhalt.append( "<ktext>Erfindungsbezeichnung</ktext>" );
        }
        else if ( grpTag.equals( "B720" ) )
        {
          sbInhalt.append( "<ktext>Mutation Erfinder</ktext>" );
        }
        else if ( grpTag.equals( "B730" ) )
        {
          sbInhalt.append( "<ktext>Übertragung</ktext>" );
        }
        else if ( grpTag.equals( "B781" ) )
        {
          sbInhalt.append( "<ktext>Einspruch</ktext>" );
        }
        else if ( grpTag.equals( "B787" ) )
        {
          sbInhalt.append( "<ktext>Zurückweisung des Einspruchs</ktext>" );
        }
        else if ( grpTag.equals( "B788" ) )
        {
          sbInhalt.append( "<ktext>Einstellung des Einspruchs</ktext>" );
        }
        else if ( grpTag.equals( "B790" ) )
        {
          sbInhalt.append( "<ktext>Lizenz</ktext>" );
        }
        else if ( grpTag.equals( "B840" ) )
        {
          sbInhalt.append( "<ktext>Bestimmungsstaaten</ktext>" );
        }
        else
        {
          // Keine Uebersetzung
          sbInhalt.append( "<ktext>" + grpTag + "</ktext>" );
        }
        sbInhalt.append( "<text>" );

        tagInhalt = xml.getAttribute( "ep-chg[" + i + "]", "date" );
        sbInhalt.append( "date=" + printDate( tagInhalt ) );
        tagInhalt = xml.getAttribute( "ep-chg[" + i + "]", "status " );
        sbInhalt.append( " status=" + tagInhalt );
        sbInhalt.append( "</text>" + IHE );

        String idref = xml.getAttribute( "ep-chg[" + i + "]", "idref " );
        List<Pair> list = xml.getText( idref );

        for ( Pair pair: list )
        {
          tag = pair.getKey();
          tagInhalt = pair.getValue();

          if ( tag.equals( "kind" ) && grpTag.startsWith( "B781" ) )
          {
            if ( tagInhalt.equals( "2" ) )
            {
              sbInhalt.append( IH );
              sbInhalt.append( "<text>Unzulässiger Einspruch</text>" );
              sbInhalt.append( IHE );
            }
            else if ( tagInhalt.equals( "3" ) )
            {
              sbInhalt.append( IH );
              sbInhalt.append( "<text>Einspruch gilt als nicht eingereicht</text>" );
              sbInhalt.append( IHE );
            }
          }
          else if ( tag.equals( "date" ) )
          {
            if ( !tagInhalt.equals( "" ) && !tagInhalt.equals( "00000000" ) )
            {
              sbInhalt.append( IH );
              sbInhalt.append( "<text>" + printDate( tagInhalt ) + "</text>" );
              sbInhalt.append( IHE );
            }
          }
          else if ( tag.equals( "B784" ) )
          {
            sbInhalt.append( IH );
            sbInhalt.append( "<ktext>Vertreter</ktext>" );
            sbInhalt.append( IHE );
          }
          else
          {
            if ( !tagInhalt.equals( "" ) )
            {
              sbInhalt.append( IH );
              sbInhalt.append( "<text>" );
              sbInhalt.append( StringEscapeUtils.escapeXml( tagInhalt ) );
              sbInhalt.append( "</text>" );
              sbInhalt.append( IHE );
            }
          }
        }
      }
    }

    // nun werden noch spezielle tag gesucht
    String value = xml.getValueOf( "B235", "date" );
    if ( value != null && !value.equals( "" ) )
    {
      sbInhalt.append( IH );
      sbInhalt.append( "<ktext>Zurückweisung</ktext>" );
      sbInhalt.append( "<text>" + printDate( value ) + "</text>" );
      sbInhalt.append( IHE );
    }

    value = xml.getValueOf( "B236", "date" );
    if ( value != null && !value.equals( "" ) )
    {
      sbInhalt.append( IH );
      sbInhalt.append( "<ktext>Zurücknahme</ktext>" );
      sbInhalt.append( "<text>" + printDate( value ) + "</text>" );
      sbInhalt.append( IHE );
    }

    value = xml.getValueOf( "B237", "date" );
    if ( value != null && !value.equals( "" ) )
    {
      sbInhalt.append( IH );
      sbInhalt.append( "<ktext>Zurücknahme Fiktion</ktext>" );
      sbInhalt.append( "<text>" + printDate( value ) + "</text>" );
      sbInhalt.append( IHE );
    }

    value = xml.getValueOf( "B243", "date" );
    if ( value != null && !value.equals( "" ) )
    {
      sbInhalt.append( IH );
      sbInhalt.append( "<ktext>Aufrechterhaltung in geänderter Form</ktext>" );
      sbInhalt.append( "<text>" + printDate( value ) + "</text>" );
      sbInhalt.append( IHE );
    }

    /*
     * B090EP Beschränkungs-Verfahren wird als "spezielle Tags" behandelt
     * B012EP Widerruf-Verfahren wird als "spezielle Tags" behandelt
     * d.h. die Daten werden ausgegeben sofern sie enthalten sind und nicht wie bei den 
     * anderen Tags welche nur ausgegeben werden sofern als changed markiert
        <B002EP>
          <ep-chg-info>
            <ep-chg idref="ep-chg0001" btag="B010EP" date="19950503" status="r"/>
          </ep-chg-info>
        </B002EP>
     */
    listeMutationLimitation.genLimitationText( sbInhalt, xml );
    listeMutationRevocation.genRevocationText( sbInhalt, xml );

    // gab es bei diesem EP <BCHG>-Tags?
    if ( sbInhalt.length() > 0 )
    {
      sbXMLMutataion.append( "<ep><head><datum>" + tagesDatum + "</datum>" );
      sbXMLMutataion.append( "<epnr>" + printpatNr( schutitNr ) + " " + epkind + "</epnr>" );
      sbXMLMutataion.append( "<epblatt>" + publDat + "</epblatt>" );
      sbXMLMutataion.append( "<erteilt>" + printDate( xml.getValueOf( "B450", "date" ) ) + "</erteilt></head>" );
      sbXMLMutataion.append( sbInhalt + "</ep>" );
    }

  }

  /**
   * XML aufbereiten für die Liste "EP Spezial"
   * Beinhaltet Inhalt des Elements B052EP = Free text
   */
  private void prepareListeSpezial()
  {
    if ( xml.exists( "B052EP" ) )
    {
      sbXMLSpezial.append( "<ep><head>" );
      sbXMLSpezial.append( "<datum>" + tagesDatum + "</datum>" );
      sbXMLSpezial.append( "<epnr>" + printpatNr( schutitNr ) + " " + epkind + "</epnr></head>" );

      List<String> values = xml.getValuesOf( "B052EP" );
      for ( int i = 0; i < values.size(); i++ )
      {
        sbXMLSpezial.append( "<inhalt><ktext>Text</ktext><text>" + StringEscapeUtils.escapeXml( values.get( i ) ) + "</text></inhalt>" );
      }

      sbXMLSpezial.append( "</ep>" );
    }
  }

  /**
   * XML der Liste "Gesuche nach patentnummern aufbereiten
   */
  private void writeListeGesuch()
  {
    try
    {
      StringBuffer sbXML = new StringBuffer();
      String filePath = new String( "c:/temp/ep_001.xml" );
      BufferedWriter out = new BufferedWriter( new FileWriter( filePath ) );
      sbXML.append( XMLHEADER );
      sbXML.append( "<liste>" );
      sbXML.append( "<head>" );
      sbXML.append( "<titel>Gesuche nach Patent Nummern</titel>" );
      sbXML.append( "<datum>" + tagesDatum + "</datum>" );
      sbXML.append( "<publtxt>Veröffentlichungsdatum</publtxt>" );
      sbXML.append( "<publdat>" + publDat + "</publdat>" );
      sbXML.append( "</head>" );

      for ( String patNr: listGesuchMP.keySet() )
      {
        sbXML.append( "<inhalt>" );
        sbXML.append( "<patnr>" + printpatNr( patNr ) + "</patnr>" );
        sbXML.append( "<gesuchnr>" + printgesuchNr( listGesuchMP.get( patNr ) ) + "</gesuchnr>" );
        sbXML.append( "</inhalt>" );
      }

      sbXML.append( "</liste>" );
      out.write( sbXML.toString() );
      out.flush();
      out.close();

      JippsXmlSender sender = new JippsXmlSender();
      try
      {
        sender.sendText( sbXML.toString(), "ep_001", dirDat );
        sender.close();
      }
      catch ( Exception ex )
      {
        LOG.error( "Fehler beim versenden von EP001. Das XML befindet sich auf c:\\temp " );
        LOG.error( ex );
      }
    }
    catch ( IOException ex )
    {
      LOG.error( ex );
    }
  }

  /**
   * XML der Liste "Patente nach Patentnummer" aufbereiten
   */
  private void writeListePatent()
  {
    try
    {
      StringBuffer sbXML = new StringBuffer();
      String filePath = new String( "c:/temp/ep_004.xml" );
      BufferedWriter out = new BufferedWriter( new FileWriter( filePath ) );
      sbXML.append( XMLHEADER );
      sbXML.append( "<liste>" );
      sbXML.append( "<head>" );
      sbXML.append( "<titel>Eingetragene Patente nach Nummern</titel>" );
      sbXML.append( "<datum>" + tagesDatum + "</datum>" );
      sbXML.append( "<publtxt>Veröffentlichungsdatum</publtxt>" );
      sbXML.append( "<publdat>" + publDat + "</publdat>" );
      sbXML.append( "</head>" );

      Collections.sort( listPatAL );
      for ( String patNr: listPatAL )
      {
        sbXML.append( "<inhalt>" );
        sbXML.append( "<patnr>" + printpatNr( patNr ) + "</patnr>" );
        sbXML.append( "</inhalt>" );
      }

      sbXML.append( "</liste>" );
      out.write( sbXML.toString() );
      out.flush();
      out.close();

      JippsXmlSender sender = new JippsXmlSender();
      try
      {
        sender.sendText( sbXML.toString(), "ep_004", dirDat );
        sender.close();
      }
      catch ( Exception ex )
      {
        LOG.error( "Fehler beim versenden von EP004. Das XML befindet sich auf c:\\temp" );
        LOG.error( ex );
      }
    }
    catch ( IOException ex )
    {
      LOG.error( ex );
    }
  }

  /**
   * XML der Liste "Mutation" abschliessen und auf HD schreiben
   */
  private void writeListeMutation()
  {
    sbXMLMutataion.append( "</liste>" );
    try
    {
      String filePath = new String( "c:/temp/ep_003.xml" );
      BufferedWriter out = new BufferedWriter( new FileWriter( filePath ) );
      out.write( sbXMLMutataion.toString() );
      out.flush();
      out.close();

      JippsXmlSender sender = new JippsXmlSender();
      try
      {
        sender.sendText( sbXMLMutataion.toString(), "ep_003", dirDat );
        sender.close();
      }
      catch ( Exception ex )
      {
        LOG.error( "Fehler beim versenden von EP003. Das XML befindet sich auf c:\\temp" );
        LOG.error( ex );
      }
    }
    catch ( IOException ex )
    {
      LOG.error( ex );
    }
  }

  /**
   * XML der Liste "Spezial" abschliessen und auf HD schreiben
   */
  private void writeListeSpezial()
  {
    if ( sbXMLSpezial != null && sbXMLSpezial.length() > 0 )
    {
      sbXMLSpezial.insert( 0, XMLHEADER + "<liste>" );
      sbXMLSpezial.append( "</liste>" );
      try
      {
        String filePath = new String( "c:/temp/ep_002.xml" );
        BufferedWriter out = new BufferedWriter( new FileWriter( filePath ) );
        out.write( sbXMLSpezial.toString() );
        out.flush();
        out.close();

        JippsXmlSender sender = new JippsXmlSender();
        try
        {
          sender.sendText( sbXMLSpezial.toString(), "ep_002", dirDat );
          sender.close();
        }
        catch ( Exception ex )
        {
          LOG.error( "Fehler beim versenden von EP002. Das XML befindet sich auf c:\\temp" );
          LOG.error( ex );
        }
      }
      catch ( IOException ex )
      {
        LOG.error( ex );
      }
    }
  }
}
