package ch.ige.epband.server;

import java.rmi.RemoteException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.ejb.EJBException;

import org.apache.log4j.Logger;

import ch.ige.adr.server.AdresseEpFacade;
import ch.ige.epband.exception.EpbandDelException;
import ch.ige.epband.exception.ProcessingException;
import ch.ige.epband.server.dao.BlandDAO;
import ch.ige.epband.server.dao.EreigTxtDAO;
import ch.ige.epband.server.dao.EreignisDAO;
import ch.ige.epband.server.dao.IpcDAO;
import ch.ige.epband.server.dao.MappingUnicodeDAO;
import ch.ige.epband.server.dao.PatDAO;
import ch.ige.epband.server.dao.PatIpcDAO;
import ch.ige.epband.server.dao.PrioritaetDAO;
import ch.ige.epband.server.dao.SchutitDAO;
import ch.ige.epband.server.dao.SchutitEmpfadrDAO;
import ch.ige.epband.server.dao.SchutitRegadrDAO;
import ch.ige.epband.vo.EpbandVO;
import ch.ige.epband.vo.EreigTxtVO;
import ch.ige.epband.vo.EreignisVO;
import ch.ige.epband.vo.IpcVO;
import ch.ige.epband.vo.PatIpcVO;
import ch.ige.epband.vo.PrioritaetVO;
import ch.ige.epband.vo.SchutitEmpfadrVO;
import ch.ige.epband.vo.SchutitRegadrVO;
import ch.ige.epband.vo.SchutitVO;
import ch.ige.util.vo.AdrRegadrNrVO;
import ch.ige.util.vo.RegadrVO;

/**
 * <p>
 * Überschrift: Europäische Patente (EP) einlesen
 * </p>
 * <p>
 * Beschreibung: Business Object für EP ab Band
 * </p>
 * <p>
 * Copyright: Copyright (c) 2002, Eidgenössisches Institut für geistiges Eigentum
 * </p>
 * <p>
 * Organisation: IGE
 * </p>
 * 
 * @author M. Trachsel
 * @version $Id: EpbandBO.java,v 1.39 2009/01/12 14:21:07 blank_blt Exp $
 */

public class EpbandBO
{
  private static final Logger LOG = Logger.getLogger( EpbandBO.class );

  private long tmp = 0; // Zeit für DB-Aufrufe - Testzwecke

  /**
   * Kreiert mit dem ubergebenen Value-Object ein EP in der Datenbank<br>
   * inkl. lösen des Primarykey (schutitKey) DBTimeoutExcpetion (Retry) wird als ApplicationDBTimeoutException
   * zurückgegeben. ApplicationDBTimeoutException ist eine ApplicationException somit erfolgt nicht wie bei der
   * SystemException automatisch ein Rollback der Trankaktion. Mit setRollbackOnly() wird ein Flag gesetzt, sodass ein
   * Rollback der Transaktion erfolgt. Bei ApplicationExcpetion wird BeanInstanz(AdresseEpFacade) nicht zerstört, was
   * bei einer SystemException der Fall wäre (CORBA.OBJECT_NOT_EXIST)
   * 
   * @param epbandVO ValueObject mit allen Daten des EP
   * @param adresseEpFacade Facade
   * @param epbandXxIdentNumberLocal
   * @throws EpbandDelException
   */
  public void createEp( EpbandVO epbandVO, AdresseEpFacade adresseEpFacade, EpbandXxIdentNumberLocal epbandXxIdentNumberLocal ) throws EpbandDelException
  {

    try
    {
      // Test ob EP in DB vorhanden mit schutitNr oder gesuchNr
      // schutit_key nur lösen, wenn das EP neu ist, also nicht in DB vorhanden
      // ist das EP vorhanden, muss alter schutitKey benutzt werden (wegen RegdDB)
      SchutitDAO schutitDAO = new SchutitDAO();
      EreignisDAO ereignisDAO = new EreignisDAO();

      int schutitKey = getSchutitKey( epbandVO );
      if ( schutitKey == 0 )
      {
        // Primärschlüssel lösen da neues EP
        // TODO Konstante definieren für Schutit_key; Im Interface darf man das nicht mehr machen. (xDoclet)
        schutitKey = epbandXxIdentNumberLocal.getNextId( "schutit_key" );
        LOG.debug( "getNextId Schutitkey: " + schutitKey );
      }
      else
      {
        // löschen EP mit Zwischenspeicherung bestimmter Ereignisse, welche
        // wiederum eingefügt werden müssen
        ArrayList<EreignisVO> oldEreignisAL = ereignisDAO.getoldEreignis( schutitKey );

        // Löschen des existierenden EP,
        // kann evt. fehlschlage, was richtig ist
        try
        {
          tmp = System.currentTimeMillis();
          schutitDAO.deleteSchutit( schutitKey );
          LOG.debug( "Time: " + ( System.currentTimeMillis() - tmp ) + " \tEP gelöscht\t\tSchutit-Nr/Gesuch-Nr: " + epbandVO.schutitVO.schutitNr + "/"
              + epbandVO.schutitVO.gesuchNr );
        }
        catch ( SQLException ex )
        {
          // ApplicationExcpetion kein automatischer Rollback
          // auch nicht erforderlich da erster Datenbankzugriff in dieser Transation
          LOG.debug( "EbpandBO throw EpbandDelExcpetion" );
          throw new EpbandDelException( ex.toString() );
        }

        // Ereignisse mit alten ergänzen und Laufnummer von ereigIdat neu setzen
        if ( !oldEreignisAL.isEmpty() )
        {
          epbandVO.ereigAL.addAll( oldEreignisAL );
        }
        EreignisVO ereigiVO = null;
        EreignisVO ereigjVO = null;
        for ( int i = 0; i < epbandVO.ereigAL.size(); i++ )
        {
          ereigiVO = epbandVO.ereigAL.get( i );
          for ( int j = i + 1; j < epbandVO.ereigAL.size(); j++ )
          {
            ereigjVO = epbandVO.ereigAL.get( j );
            if ( ereigiVO.ereigIdat == ereigjVO.ereigIdat )
            {
              ereigjVO.ereigIdat += 5;
              epbandVO.ereigAL.set( j, ereigjVO );
            }
          }
        }
      }

      epbandVO.schutitVO.schutitKey = schutitKey;
      tmp = System.currentTimeMillis();
      schutitDAO.insert( epbandVO.schutitVO );
      LOG.debug( "Time: " + ( System.currentTimeMillis() - tmp ) + " \tInsert Tabelle Schutit\tSchutit-Nr/Gesuch-Nr: " + epbandVO.schutitVO.schutitNr + "/"
          + epbandVO.schutitVO.gesuchNr );

      epbandVO.patVO.schutitKey = schutitKey;
      tmp = System.currentTimeMillis();
      new PatDAO().insert( epbandVO.patVO );
      LOG.debug( "Time: " + ( System.currentTimeMillis() - tmp ) + " \tInsert Tabelle Pat\tSchutit-Nr/Gesuch-Nr: " + epbandVO.schutitVO.schutitNr + "/"
          + epbandVO.schutitVO.gesuchNr );

      EreignisVO ereignisVO = null;
      for ( int i = 0; i < epbandVO.ereigAL.size(); i++ )
      {
        ereignisVO = epbandVO.ereigAL.get( i );
        ereignisVO.schutitKey = schutitKey;
        tmp = System.currentTimeMillis();
        ereignisDAO.insert( ereignisVO );
        LOG.debug( "Time: " + ( System.currentTimeMillis() - tmp ) + " \tInsert Tabelle Ereignis\tSchutit-Nr/Gesuch-Nr: " + epbandVO.schutitVO.schutitNr + "/"
            + epbandVO.schutitVO.gesuchNr + " EreigIdat: " + ereignisVO.ereigIdat + " Ereignis: " + ereignisVO.artNr );
      }

      EreigTxtVO ereigTxtVO = null;
      EreigTxtDAO ereigTxtDAO = new EreigTxtDAO();
      for ( int i = 0; i < epbandVO.ereigTxtAL.size(); i++ )
      {
        ereigTxtVO = epbandVO.ereigTxtAL.get( i );
        ereigTxtVO.schutitKey = schutitKey;
        tmp = System.currentTimeMillis();
        ereigTxtDAO.insert( ereigTxtVO );
        LOG.debug( "Time: " + ( System.currentTimeMillis() - tmp ) + " \tInsert Tabelle EreigTxt\tSchutit-Nr/Gesuch-Nr: " + epbandVO.schutitVO.schutitNr + "/"
            + epbandVO.schutitVO.gesuchNr + " EreigIdat: " + ereigTxtVO.ereigIdat );
      }

      PrioritaetVO prioVO = null;
      PrioritaetDAO prioDAO = new PrioritaetDAO();
      for ( int i = 0; i < epbandVO.prioAL.size(); i++ )
      {
        prioVO = epbandVO.prioAL.get( i );
        prioVO.schutitKey = schutitKey;
        prioVO.prioLnr = ( i + 1 );
        tmp = System.currentTimeMillis();
        prioDAO.insert( prioVO );
        LOG.debug( "Time: " + ( System.currentTimeMillis() - tmp ) + " \tInsert Tabelle Prio\tSchutit-Nr/Gesuch-Nr: " + epbandVO.schutitVO.schutitNr + "/"
            + epbandVO.schutitVO.gesuchNr + " Prio-Laufnummer: " + prioVO.prioLnr );
      }

      /*
      PatIpcVO patIpcVO = null;
      PatIpcDAO patIpcDAO = new PatIpcDAO();
      for ( int i = 0; i < epbandVO.patIpcAL.size(); i++ )
      {
        patIpcVO = epbandVO.patIpcAL.get( i );
        patIpcVO.schutitKey = schutitKey;
        tmp = System.currentTimeMillis();
        patIpcDAO.insert( patIpcVO );
        LOG.debug( "Time: " + ( System.currentTimeMillis() - tmp ) + " \tInsert Tabelle PatIpc\tSchutit-Nr/Gesuch-Nr: " + epbandVO.schutitVO.schutitNr + "/"
            + epbandVO.schutitVO.gesuchNr + " IPC-Symbol: " + patIpcVO.patIpcCd );
      }
      */
//  
      List<PatIpcVO> ipcClasses = epbandVO.patIpcAL;
      if ( !ipcClasses.isEmpty() )
      {
        Set<String> uniques = new HashSet<String>( 8 );
        PatIpcDAO patIpcDAO = new PatIpcDAO();
        for ( PatIpcVO patIpcVO: ipcClasses )
        {
          patIpcVO.schutitKey = schutitKey;
          if ( uniques.add( patIpcVO.createIdentifierForInsertUniqueness() ) )
          {
            tmp = System.currentTimeMillis();
            patIpcDAO.insert( patIpcVO );
            LOG.debug( "Time: " + ( System.currentTimeMillis() - tmp ) + " \tInsert Tabelle PatIpc\tSchutit-Nr/Gesuch-Nr: " + epbandVO.schutitVO.schutitNr
                + "/" + epbandVO.schutitVO.gesuchNr + " IPC-Symbol: " + patIpcVO.ipcSymb );
          }
          else
          {
            LOG.info( "Duplizierte IPC Klasse gefunden: Schutit-Nr/Gesuch-Nr: " + epbandVO.schutitVO.schutitNr + "/" + epbandVO.schutitVO.gesuchNr
                + " IPC-Symbol: " + patIpcVO.ipcSymb );
          }
        }
      }

      // Aufruf Adresskomponente
      createAdresse( epbandVO, adresseEpFacade, schutitKey );
    }
    catch ( RemoteException ex )
    {
      // SystemException bewirkt ein Rollback der Transaktion
      // Epband-Verarbeitung wird abgebrochen
      LOG.error( ex.getMessage() );
      throw new EJBException( ex );
    }
    catch ( SQLException ex )
    {
      // SystemException bewirkt ein Rollback der Transaktion
      // Epband-Verarbeitung wird abgebrochen
      LOG.error( ex.getMessage() );
      throw new EJBException( ex );
    }
  }

  /**
   * Da IGE nicht zwischen Applicant und Grantee unterscheidet, kann es vorkommen, dass gleiche Adressen vorhanden sind.
   * Zudem ist zu berücksichtigen, dass spezielle Adressen verknüpft werden müssen wie "Verzicht auf Erfindernennung"
   * Alle Registeradressen inkl. evtl. Adressen durch die Applikation "adr" erstellen, bzw. suchen lassen. Applikation
   * gibt bestehende oder neu erstellte regadrNr zurück die so erhaltenen regadrNr in den betr. schutitRegadr eintragen.
   * Bei Verzicht Erfindernennung wird im Client regadrNr = 4069999 bzw. 4069998 gesetzt, deshalb nur Adresse suchen wo
   * regadrNr = 0 ist. und dabei entscheiden, ob schutitEmpfadr nötig und welche doppelte schutitRegadr eliminieren
   * indem auf gleiche regadrNr UND adrRollenCd gesucht wird laufNr setzen
   */
  private void createAdresse( EpbandVO epbandVO, AdresseEpFacade adresseEpFacade, int schutitKey ) throws SQLException, RemoteException
  {
    SchutitEmpfadrDAO schutitEmpfadrDAO = new SchutitEmpfadrDAO();
    SchutitEmpfadrVO schutitEmpfadrVO = null;

    AdrRegadrNrVO adrRegadrNrVO = new AdrRegadrNrVO();
    ArrayList<AdrRegadrNrVO> regadrAdrAL = new ArrayList<AdrRegadrNrVO>();

    for ( int i = 0, sr = 0; i < epbandVO.regAdrAL.size(); i++, sr++ )
    {
      RegadrVO regadrVO = epbandVO.regAdrAL.get( i );
      SchutitRegadrVO schutitRegadrVO = epbandVO.schutitregAdrAL.get( sr );
      String sprachCd = "USA";

      if ( epbandVO.patVO.epApublSpra.equalsIgnoreCase( "DE" ) )
      {
        sprachCd = "D";
      }
      else if ( epbandVO.patVO.epApublSpra.equalsIgnoreCase( "FR" ) )
      {
        sprachCd = "F";
      }

      // Bei Verzicht Erfindernennung wird im Client regadrNr = 4069999 bzw. 4069998 gesetzt
      // Diese SchutitRegadr überspringen
      while ( schutitRegadrVO.regadrNr != 0 )
      {
        sr++;
        schutitRegadrVO = epbandVO.schutitregAdrAL.get( sr );
      }
      if ( schutitRegadrVO.adrRolleCd == SchutitRegadrVO.ERFINDER || epbandVO.schutitVO.registerKz == SchutitVO.GESUCH )
      {
        // nur Registeradresse kreieren bei Erfinder oder EP im Gesuchstatium
        tmp = System.currentTimeMillis();
        schutitRegadrVO.regadrNr = adresseEpFacade.createRegadrEp( regadrVO );
        LOG.debug( "Time: " + ( System.currentTimeMillis() - tmp ) + " \tInsert Tabelle RegAdr\tSchutit-Nr/Gesuch-Nr: " + epbandVO.schutitVO.schutitNr + "/"
            + epbandVO.schutitVO.gesuchNr + " Adresse: " + schutitRegadrVO.regadrNr );
      }
      else
      {
        tmp = System.currentTimeMillis();
        adrRegadrNrVO = adresseEpFacade.createRegadrVersadrEp( regadrVO, sprachCd );
        LOG.debug( "Time: " + ( System.currentTimeMillis() - tmp ) + " \tInsert Tabelle Versadr\tSchutit-Nr/Gesuch-Nr: " + epbandVO.schutitVO.schutitNr + "/"
            + epbandVO.schutitVO.gesuchNr + " Adresse: " + adrRegadrNrVO.regadrNr );

        schutitRegadrVO.regadrNr = adrRegadrNrVO.regadrNr;
        regadrAdrAL.add( adrRegadrNrVO );
      }
      epbandVO.schutitregAdrAL.set( sr, schutitRegadrVO );
    }

    // doppelte eleiminieren
    SchutitRegadrVO schutitRegadrVO = new SchutitRegadrVO();
    SchutitRegadrVO schutitRegadrjVO = new SchutitRegadrVO();

    for ( int i = 0; i < epbandVO.schutitregAdrAL.size(); i++ )
    {
      schutitRegadrVO = epbandVO.schutitregAdrAL.get( i );
      for ( int j = i + 1; j < epbandVO.schutitregAdrAL.size(); j++ )
      {
        schutitRegadrjVO = epbandVO.schutitregAdrAL.get( j );
        if ( schutitRegadrVO.regadrNr == schutitRegadrjVO.regadrNr && schutitRegadrVO.adrRolleCd == schutitRegadrjVO.adrRolleCd )
        {
          epbandVO.schutitregAdrAL.remove( j );
          break;
        }
      }
    }
    // Durchnummerieren und schreiben von schutitRegader inkl schutitEmpfadr
    SchutitRegadrDAO schutitRegadrDAO = new SchutitRegadrDAO();
    for ( int i = 0; i < epbandVO.schutitregAdrAL.size(); i++ )
    {
      schutitRegadrVO = epbandVO.schutitregAdrAL.get( i );
      schutitRegadrVO.schutadrLnr = i + 1;
      schutitRegadrVO.schutitKey = schutitKey;

      tmp = System.currentTimeMillis();
      schutitRegadrDAO.insert( schutitRegadrVO );
      LOG.debug( "Time: " + ( System.currentTimeMillis() - tmp ) + " \tInsert Tabelle SchutitRegAdr\tSchutit-Nr/Gesuch-Nr: " + epbandVO.schutitVO.schutitNr
          + "/" + epbandVO.schutitVO.gesuchNr + " Adresse: " + adrRegadrNrVO.regadrNr );

      // Empfadr schrieben wenn EP eingetragen und nicht Erfinderadresse
      schutitEmpfadrVO = new SchutitEmpfadrVO();
      schutitEmpfadrVO.schutitKey = schutitKey;
      if ( epbandVO.schutitVO.registerKz == SchutitVO.REGISTER && schutitRegadrVO.adrRolleCd != SchutitRegadrVO.ERFINDER )
      {
        // Heraussuchen der Adr_nr
        for ( int j = 0; j < regadrAdrAL.size(); j++ )
        {
          adrRegadrNrVO = regadrAdrAL.get( j );
          if ( schutitRegadrVO.regadrNr == adrRegadrNrVO.regadrNr )
          {
            schutitEmpfadrVO.adrNr = adrRegadrNrVO.adrNr;
            break;
          }
        }
        if ( schutitRegadrVO.adrRolleCd == SchutitRegadrVO.LIZENZNEHMER ) // Lizenznehmer nur Löschanzeige
        {
          schutitEmpfadrVO.empfTyp = 3;
          tmp = System.currentTimeMillis();
          schutitEmpfadrDAO.insert( schutitEmpfadrVO );
          LOG.debug( "Time: " + ( System.currentTimeMillis() - tmp ) + "\tInsert Tabelle SchutitEmpfAdr Empfänger-Adresse erstellt Key: "
              + schutitEmpfadrVO.schutitKey + " Adresse: " + schutitEmpfadrVO.adrNr );
        }
        else
        {
          for ( int k = 1; k <= 3; k++ ) // Inhaber empf_typ = 1,2 und 3
          {
            schutitEmpfadrVO.empfTyp = k;
            tmp = System.currentTimeMillis();
            schutitEmpfadrDAO.insert( schutitEmpfadrVO );
            LOG.debug( "Time: " + ( System.currentTimeMillis() - tmp ) + " \tInsert Tabelle SchutitEmpfAdr Empfänger-Adresse erstellt Key: "
                + schutitEmpfadrVO.schutitKey + " Adresse: " + schutitEmpfadrVO.adrNr );
          }
        }
      }
    }
  }

  /**
   * Nicht Eintreten in europ. Phase Lesen von PCT mit schutitNr, mit schutitKey dann ereignis 30060 einfügen falls
   * dieses nicht schon vorhanden ist
   * 
   * @param epbandVO mit daten für das betroffenen PCT
   * @exception ProcessingException
   */
  public void createEreigWipo( EpbandVO epbandVO ) throws ProcessingException
  {
    try
    {
      SchutitDAO schutitDAO = new SchutitDAO();
      int schutitKey = schutitDAO.selectSchutitKeyWithSchutitNr( epbandVO.schutitVO.schutitNr, SchutitVO.SCHUTIT_TYP_PCT );
      if ( schutitKey == 0 )
      {
        throw new ProcessingException( "PCT nicht gefunden" );
      }

      EreignisDAO ereignisDAO = new EreignisDAO();

      boolean exist = ereignisDAO.existArtNr( schutitKey, EreignisVO.WIPO );
      if ( !exist )
      {
        EreignisVO ereignisVO = epbandVO.ereigAL.get( 0 );
        ereignisVO.schutitKey = schutitKey;
        ereignisVO.ereigIdat = ereignisDAO.getNextereigIdat( ereignisVO.schutitKey, ereignisVO.ereigIdat );
        tmp = System.currentTimeMillis();
        ereignisDAO.insert( ereignisVO );
        LOG.debug( "Time: " + ( System.currentTimeMillis() - tmp ) + " \tEreignis 30060 erstellt Schutit-Nr/Gesuch-Nr: " + epbandVO.schutitVO.schutitNr + "/"
            + epbandVO.schutitVO.gesuchNr );
      }
    }
    catch ( SQLException ex )
    {
      LOG.error( ex.getMessage() );
      throw new EJBException( ex );
    }
  }

  /**
   * Eine Zurücknahme wird im Normalfall nur bei Gesuchen gemacht Es kann jedoch vorkommen dass die Zurücknahme zu einem
   * verzögerten Zeitpunkt eintritt und das Gesuch in der Zwischenzeit schon zu einem Patent wurde. Bei erteilten
   * Patenten (Ereignis 20010 - Erteilung mit Wirkung CH/LI - vorhanden) darf das Ereignis 20250 Zurücknahme Benennung
   * CH/LI nicht automatisch geschrieben werden. Es müssen manuelle Anpassungen vorgenommen werden. Bei den Gesuchen
   * Ereignis 20250 automatisch schreiben. Falls Schutztitel nicht vorhanden keine Aktion ausführen Kein Fehler, da CH
   * nicht mehr benannt für ein Gesuch das gar nicht exitstiert in DB
   * 
   * @param epbandVO Aufbereitete Daten
   * @throws ProcessingException 20250 bei erteiltem EP
   */
  public void createCHzurueck( EpbandVO epbandVO ) throws ProcessingException
  {
    SchutitDAO schutitDAO = new SchutitDAO();

    try
    {
      int schutitKey = getSchutitKey( epbandVO );
      if ( schutitKey != 0 )
      {
        EreignisDAO ereignisDAO = new EreignisDAO();

        // Ereignis "20010" suchen
        boolean exist = ereignisDAO.existArtNr( schutitKey, EreignisVO.ERTEILUNG );
        if ( !exist )
        {
          // Ereignis "20250" suchen
          exist = ereignisDAO.existArtNr( schutitKey, EreignisVO.CH_ZURUECKNAHME );
          if ( !exist )
          {
            EreignisVO ereignisVO = epbandVO.ereigAL.get( 0 );
            ereignisVO.schutitKey = schutitKey;
            ereignisVO.ereigIdat = ereignisDAO.getNextereigIdat( schutitKey, ereignisVO.ereigIdat );
            ereignisDAO.insert( ereignisVO );

            // Update des Schutztitels (löschdatum setzen)
            epbandVO.schutitVO.schutitKey = schutitKey;

            tmp = System.currentTimeMillis();
            schutitDAO.updateSchutit( epbandVO.schutitVO );
            LOG.debug( "Time: " + ( System.currentTimeMillis() - tmp ) + " \tEreignis 20250 erstellt Schutit-Nr/Gesuch-Nr: " + epbandVO.schutitVO.schutitNr
                + "/" + epbandVO.schutitVO.gesuchNr );
          }
        }
        else
        {
          throw new ProcessingException( "Zurücknahme Benennung CH/LI bei erteiltem EP" );
        }
      }
    }
    catch ( SQLException ex )
    {
      LOG.error( ex.getMessage() );
      throw new EJBException( ex );
    }
  }

  /**
   * Falls Ereignis Kein Einspruch (20450) nicht vorhanden ist, dieses schreiben
   * 
   * @param epbandVO ValueObject mit Daten
   * @throws ProcessingException Abbruch des Programms
   */
  public void createkeinEinspruch( EpbandVO epbandVO ) throws ProcessingException

  {
    try
    {
      int schutitKey = getSchutitKey( epbandVO );
      if ( schutitKey == 0 )
      {
        throw new ProcessingException( "EP nicht gefunden" + schutitKey );
      }

      EreignisVO ereignisVO = new EreignisVO();
      EreignisDAO ereignisDAO = new EreignisDAO();
      ereignisVO = epbandVO.ereigAL.get( 0 );

      // Ereignis 20450 suchen
      boolean exist = ereignisDAO.existArtNr( schutitKey, ereignisVO.artNr );
      if ( !exist )
      {
        ereignisVO.schutitKey = schutitKey;
        ereignisVO.ereigIdat = ereignisDAO.getNextereigIdat( schutitKey, ereignisVO.ereigIdat );
        tmp = System.currentTimeMillis();
        ereignisDAO.insert( ereignisVO );
        LOG.debug( "Time: " + ( System.currentTimeMillis() - tmp ) + " \tEreignis 20450 erstellt Schutit-Nr/Gesuch-Nr: " + epbandVO.schutitVO.schutitNr + "/"
            + epbandVO.schutitVO.gesuchNr );
      }
    }
    catch ( SQLException ex )
    {
      LOG.error( ex.getMessage() );
      throw new EJBException( ex );
    }
  }

  /**
   * Überprüft ob der Schutztitel vorhanden ist. Falls vorhanden wird der Schutztitelkey zurückgegeben, ansonsten 0<br>
   * 
   * @param epbandVO Aufbereitete Daten
   * @return int Schutztitel-Key
   */
  private int getSchutitKey( EpbandVO epbandVO )
  {
    try
    {
      SchutitDAO schutitDAO = new SchutitDAO();

      // EP lesen mit schuiti_nr
      int schutitKey = schutitDAO.selectSchutitKeyWithSchutitNr( epbandVO.schutitVO.schutitNr, SchutitVO.SCHUTIT_TYP_EP );
      if ( schutitKey == 0 )
      {
        // Falls EP über schutit_nr nicht gefunden über gersuch_nr versuchen
        schutitKey = schutitDAO.selectSchutitKeyWithGesuchNr( epbandVO.schutitVO.gesuchNr, SchutitVO.SCHUTIT_TYP_EP );
      }

      return schutitKey;
    }
    catch ( SQLException ex )
    {
      LOG.error( ex.getMessage() );
      throw new EJBException( ex );
    }
  }

  /**
   * Selektiert alle BLänder und gibt sie in einer ArrayList zurück.
   * 
   * @return Value Object aller BLänder
   */
  public ArrayList<String> getAllBland()
  {
    try
    {
      return new BlandDAO().getAllBland();
    }
    catch ( SQLException ex )
    {
      LOG.error( ex.getMessage() );
      throw new EJBException( ex );
    }
  }

  /**
   * Selektiert die Daten einer IPC Klasse
   * 
   * @param ipcSymb
   * @param gueltigVon
   * @return Value Object einer IPC Klasse
   */
  public IpcVO getIpc( String ipcSymb, Date gueltigVon )
  {
    try
    {
      return new IpcDAO().select( ipcSymb, gueltigVon );
    }
    catch ( SQLException ex )
    {
      LOG.error( ex.getMessage() );
      throw new EJBException( ex );
    }
  }

  /**
   * Selektiert die Daten einer IPC Klasse
   * 
   * @param ipcSymb
   * @return Value Object einer IPC Klasse
   */
  public IpcVO getYoungestIpc( String ipcSymb )
  {
    try
    {
      return new IpcDAO().selectYoungest( ipcSymb );
    }
    catch ( SQLException ex )
    {
      LOG.error( ex.getMessage() );
      throw new EJBException( ex );
    }
  }

  /**
   * Selektiert alle Unicode Mappings und gibt sie in einer Hashmap zurück.
   * 
   * @return Map HashMap mit unicode iso8859 mapping
   */
  public HashMap<String, String> getAllMappingUnicode()
  {
    try
    {
      return new MappingUnicodeDAO().selectAll();
    }
    catch ( SQLException ex )
    {
      LOG.error( ex.getMessage() );
      throw new EJBException( ex );
    }
  }

}
