package ch.ige.image;

import java.awt.Graphics;
import java.awt.MediaTracker;
import java.awt.image.BufferedImage;
import java.awt.image.RenderedImage;

import ch.ige.image.format.Format;

/**
 * Wrapper around Java2D images.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 *
 * @author Michael Locher
 * @version $Id: BufferedImageWrapper.java,v 1.17 2009/03/19 13:26:22 locher Exp $
 */
public class BufferedImageWrapper extends BitmapImage implements Cloneable
{
  private static final long serialVersionUID = 1L;

  private transient BufferedImage source;

  /** For deserialization only. */
  public BufferedImageWrapper()
  {
    super();
  }

  public BufferedImageWrapper( RenderedImage source, Metadata metadata )
  {
    this.setBufferedImage( source );
    this.setMetadata( metadata );
  }

  @Override
  protected void finalize() throws Throwable
  {
    this.dispose();
    super.finalize();
  }

  public void dispose()
  {
    source.flush();
    source = null;
  }

  public Image cloneImage() throws CloneNotSupportedException
  {
    BufferedImageWrapper clone = (BufferedImageWrapper) this.clone();
    clone.source = BitmapImage.copyBufferedImage( this.source );
    return clone;
  }

  @Override
  protected void setMetadata( Metadata metadata )
  {
    MetadataBean temp = new MetadataBean( metadata );
    temp.setFormat( Format.fromTypeString( "java" ) );
    super.setMetadata( temp );
    this.extractMetadataFromImage();
  }

  private void extractMetadataFromImage()
  {
    WritableMetadata temp = this.getWritableMetadata();

    try
    {
      switch( this.source.getType() )
      {
        case BufferedImage.TYPE_CUSTOM:
          temp.setColorspaceType( "CUSTOM" );
          break;

        case BufferedImage.TYPE_INT_RGB:
        case BufferedImage.TYPE_INT_ARGB:
        case BufferedImage.TYPE_INT_ARGB_PRE:
        case BufferedImage.TYPE_USHORT_565_RGB:
        case BufferedImage.TYPE_USHORT_555_RGB:
          temp.setColorspaceType( "RGB" );
          break;

        case BufferedImage.TYPE_INT_BGR:
        case BufferedImage.TYPE_3BYTE_BGR:
        case BufferedImage.TYPE_4BYTE_ABGR:
        case BufferedImage.TYPE_4BYTE_ABGR_PRE:
          temp.setColorspaceType( "BGR" );
          break;

        case BufferedImage.TYPE_BYTE_GRAY:
        case BufferedImage.TYPE_USHORT_GRAY:
          temp.setColorspaceType( "GRAY" );
          break;

        case BufferedImage.TYPE_BYTE_BINARY:
          temp.setColorspaceType( "BINARY" );
          break;

        case BufferedImage.TYPE_BYTE_INDEXED:
          temp.setColorspaceType( "INDEXED" );
          break;
        default:
          temp.setColorspaceType( null );
      }
    }
    catch ( Exception e )
    {
      throw new RuntimeException( "Colorspace metadata could not be set", e );
    }

    // Get the dimensions from the BufferedImage
    try
    {
      temp.setWidth( this.source.getWidth() );
      temp.setHeight( this.source.getHeight() );
    }
    catch ( MetadataException e )
    {
      throw new RuntimeException( "Width/Height metadata could not be set", e );
    }
  }

  private void setBufferedImage( RenderedImage source )
  {
    this.source = BitmapImage.asBufferedImage( source );
  }

  @Override
  public RenderedImage asRenderedImage()
  {
    return this.source;
  }

  /**
   * Wraps an AWT image for usage with this library.
   *
   * @see ImageFactory#wrapImage(java.awt.Image, Metadata)
   *
   * @param image
   *          the image to wrap
   * @param metadata
   *          the metadata for the resulting image
   * @return an image with the same appearance
   */
  public static BufferedImageWrapper constructFromAWTImage( java.awt.Image image, Metadata metadata )
  {
    if ( image instanceof RenderedImage )
    {
      // hmm, for rendered images we have a direct path...
      return new BufferedImageWrapper( (RenderedImage) image, metadata );
    }
    BufferedImageWrapper.waitForAWTImage( image );

    BufferedImage converted = new BufferedImage( image.getWidth( null ), image.getHeight( null ), BufferedImage.TYPE_INT_ARGB );
    Graphics g = converted.getGraphics();
    g.drawImage( image, 0, 0, null );
    g.dispose();
    return new BufferedImageWrapper( converted, metadata );
  }

  private static void waitForAWTImage( java.awt.Image image )
  {
    if ( java.awt.GraphicsEnvironment.isHeadless() )
    {
      return;
    }
    MediaTracker mt = new MediaTracker( new java.awt.Label() );
    mt.addImage( image, 0 );
    try
    {
      mt.waitForID( 0 );
    }
    catch ( InterruptedException ie )
    {
      // ignore
    }
  }

}
