package ch.ige.image;

import java.awt.Dimension;
import java.awt.image.BufferedImage;
import java.awt.image.RenderedImage;

import javax.swing.JFrame;

import ch.ige.image.color.GrayScale.Algorithm;

/**
 * An Image.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 *
 * @author Michael Locher
 * @version $Id: Image.java,v 1.15 2008/05/21 08:17:54 locher Exp $
 */
public interface Image
{

  /**
   * Unit Constant for pixels.
   */
  public static final String UNIT_PIXEL = "px";

  /**
   * Unit Contant for millimeters.
   */
  public static final String UNIT_MILLIMETER = "mm";

  /**
   * Displays the image in a JFrame.
   *
   * @return the frame containing the image.
   */
  public JFrame show();

  /**
   * Returns the image as a RenderedImage.
   *
   * @return the image as RenderedImage
   */
  public RenderedImage asRenderedImage();

  /**
   * Returns the image as a BufferedImage.
   *
   * @return the image as BufferedImage
   */
  public BufferedImage asBufferedImage();

  /**
   * Gives the Metadata of the image.
   *
   * @return the associated metadata.
   */
  public Metadata getMetadata();

  /**
   * Gives a writable version of the image metadata.
   *
   * @return the associated metadata.
   */
  public WritableMetadata getWritableMetadata();

  /**
   * Release all resources of this image. The image can not be used after that.
   * @see EncodedImage#flushCache()
   */
  public void dispose();

  /**
   * Returns a copy of the image.
   *
   * @return a copy of the same (dynamic) type.
   * @throws CloneNotSupportedException
   *           if cloning is not supported.
   */
  public Image cloneImage() throws CloneNotSupportedException;

  /**
   * Returns a grayscaled clone of the image.
   *
   * @return the grayscaled clone
   */
  public Image grayscale();

  /**
   * Returns a grayscaled clone of the image.
   *
   * @param algorithm the grayscale algorithm to use 
   * @return the grayscaled clone
   */
  public Image grayscale( Algorithm algorithm );

  /**
   * Returns a grayscaled clone of the image and removes the alpha channel.
   *
   * @return the grayscaled clone
   */
  public Image grayscaleAndEliminateAlpha();

  /**
   * Returns a clone of the image in sRGB colorspace.
   *
   * @return the image in sRGB
   */
  public Image asRGB();

  /**
   * Returns a resized clone of the image.
   *
   * @param dimension
   *          the size of the clone in pixels.
   * @return the resized clone
   */
  public Image resize( Dimension dimension );

  /**
   * Returns a resized clone of the image while keeping the aspect ratio of the
   * image. At least one of the width and height will be set to the width and
   * height of the requested maxDimension.
   *
   * @param maxDimension
   *          the maximum size of the clone in pixels.
   * @return the resized clone
   */
  public Image resizeKeepAspect( Dimension maxDimension );

}
