package ch.ige.image;

import java.awt.Dimension;
import java.awt.color.ICC_ColorSpace;
import java.awt.color.ICC_Profile;
import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;
import java.awt.image.ComponentColorModel;
import java.awt.image.Raster;
import java.awt.image.RenderedImage;
import java.awt.image.WritableRaster;
import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.nio.ByteBuffer;
import java.util.List;

import javax.imageio.IIOImage;
import javax.imageio.ImageIO;
import javax.imageio.ImageReader;
import javax.imageio.event.IIOReadWarningListener;
import javax.imageio.metadata.IIOMetadata;
import javax.imageio.metadata.IIOMetadataFormatImpl;
import javax.imageio.stream.ImageInputStream;

import org.apache.log4j.Logger;
import org.w3c.dom.Node;

import ch.ige.image.format.Format;

/**
 * Wrapper for javax.imageio Images.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @author Michael Locher
 * @version $Id: ImageIOImage.java,v 1.31 2009/03/19 13:26:22 locher Exp $
 */
public class ImageIOImage extends EncodedBitmapImage implements Externalizable
{
  private static final long serialVersionUID = 2L;

  private static final Logger LOG = Logger.getLogger( ImageIOImage.class );

  private transient IIOMetadata streamMetadata = null;

  private transient IIOImage parsedImage = null;

  private static final IIOReadWarningListener READER_WARNING_HANDLER = new IIOReadWarningListener()
  {
    public void warningOccurred( ImageReader source, String warning )
    {
      LOG.info( warning );
    }
  };

  /**
   * Sets the usage of the disk cache.
   * 
   * @param useDisk whether disk cache should be used.
   */
  public static void setUseDiskCache( boolean useDisk )
  {
    ImageIO.setUseCache( useDisk );
  }

  /**
   * Parses a byte buffer with the javax.imageio library.
   * 
   * @param imageData the image to parse
   * @return the parsed image
   */
  public static ImageIOImage parse( ByteBuffer imageData ) throws IOException
  {
    final ImageReader reader = ImageIOImage.createReader( imageData );
    ImageIOImage result;
    try
    {
      result = new ImageIOImage( imageData, reader );
    }
    finally
    {
      reader.dispose();
    }
    return result;
  }

  private static ImageReader createReader( ByteBuffer imageData ) throws IOException
  {
    ImageInputStream iis = new ByteBufferImageInputStream( imageData );
    ImageReader reader = ImageIO.getImageReaders( iis ).next();
    reader.setInput( iis );
    return reader;
  }

  /** For deserialization only. */
  public ImageIOImage()
  {
    super();
  }

  private ImageIOImage( ByteBuffer imageData, javax.imageio.ImageReader reader ) throws IOException, IllegalArgumentException
  {
    super( imageData );
    this.readImage( reader );
  }

  public synchronized void flushCache()
  {
    this.parsedImage = null;
    this.streamMetadata = null;
  }

  // @see ch.ige.image.EncodedBitmapImage#readExternal(java.io.ObjectInput)
  @Override
  public void readExternal( ObjectInput in ) throws IOException, ClassNotFoundException
  {
    super.readExternal( in );
    this.rereadImage();
  }

  @Override
  protected void releaseClonedLazyReferences()
  {
    this.flushCache();
  }

  private void rereadImage()
  {
    try
    {
      ByteBuffer imgData = this.getDataBuffer();
      if ( imgData == null )
      {
        throw new NullPointerException( "image data missing" );
      }
      final ImageReader reader = ImageIOImage.createReader( imgData );
      try
      {
        this.readImage( reader );
      }
      finally
      {
        reader.dispose();
      }
    }
    catch ( IOException e )
    {
      throw new RuntimeException( "reparsing of image failed", e );
    }
  }

  private void readImage( ImageReader reader ) throws IOException
  {
    reader.addIIOReadWarningListener( READER_WARNING_HANDLER );
    try
    {
      synchronized ( this )
      {
        this.parsedImage = reader.readAll( 0, null );
        this.streamMetadata = reader.getStreamMetadata();
      }
      Metadata currentMetadata = this.getMetadata();
      if ( currentMetadata == null || !( currentMetadata instanceof WritableMetadata ) )
      {
        this.setMetadata( new MetadataWrapper( Format.fromTypeString( reader.getFormatName() ) ) );
      }
    }
    finally
    {
      reader.removeIIOReadWarningListener( READER_WARNING_HANDLER );
      ( (ImageInputStream) reader.getInput() ).close();
    }
  }

  private synchronized IIOImage getParsedImage()
  {
    if ( this.parsedImage == null )
    {
      this.rereadImage();
    }
    return this.parsedImage;
  }

  private synchronized IIOMetadata getStreamMetadata()
  {
    if ( this.streamMetadata == null )
    {
      this.rereadImage();
    }
    return this.streamMetadata;
  }

  @Override
  public RenderedImage asRenderedImage()
  {
    return fixColorSpace( this.getParsedImage().getRenderedImage() );
  }

  private RenderedImage fixColorSpace( RenderedImage rendered )
  {
    ColorModel cm = rendered.getColorModel();
    if ( cm.getColorSpace().getClass().getName().indexOf( "Bogus" ) >= 0 )
    {
      // rt.jar, from jdk5: com.sun.imageio.plugins.common.BogusColorSpace
      // jai_imageio.jar: com.sun.media.imageioimpl.common.BogusColorSpace

      LOG.warn( "Bogus ColorSpace detected: " + cm );
      try
      {
        if ( this.getMetadata().hasEmbeddedICCProfile() )
        {
          ICC_Profile profile = this.getMetadata().getEmbeddedICCProfile();
          if ( profile != null )
          {
            BufferedImage img = BitmapImage.asBufferedImage( rendered );
            ICC_ColorSpace cs = new ICC_ColorSpace( profile );
            WritableRaster imgData = img.getRaster();
            boolean alpha = imgData.getNumBands() > cs.getNumComponents();
            ColorModel fixedColorModel = new ComponentColorModel( cs, alpha, false, cm.getTransparency(), imgData.getTransferType() );
            BufferedImage fixed = new BufferedImage( fixedColorModel, imgData, cm.isAlphaPremultiplied(), null );
            LOG.warn( "replaced BogusColorSpace with ICC_ColorSpace from Embedded Profile" );
            return fixed;
          }
        }
        else
        {
          LOG.warn( "image has a BogusColorSpace, but no ICC Profile found" );
        }
      }
      catch ( Exception e )
      {
        LOG.error( "failed to fix bogus colorspace", e );
      }
    }
    return rendered;
  }

  // --------------------------------------------------------------------------

  /** Inner Class */
  private class MetadataWrapper implements Metadata
  {
    private static final long serialVersionUID = 2L;

    private String embeddedColorProfileDefinition;

    private Format format;

    /**
     * @param basicFormatInformation
     */
    public MetadataWrapper( Format basicFormatInformation )
    {
      this.completeFormatInformation( basicFormatInformation );
    }

    private void completeFormatInformation( Format basicFormatInformation )
    {
      this.format = basicFormatInformation;
      try
      {
        this.format.setFormatVersion( this.getFormatVersion() );
      }
      catch ( MetadataException e )
      {
        // ignore
      }
      try
      {
        this.format.setCompressionType( this.getCompressionType() );
      }
      catch ( MetadataException e )
      {
        // ignore
      }
      try
      {
        this.format.setLossless( this.isLossless() );
      }
      catch ( MetadataException e )
      {
        // ignore
      }
    }

    private IIOMetadata getStreamMeta()
    {
      return ImageIOImage.this.getStreamMetadata();
    }

    private IIOMetadata getImageMeta()
    {
      return ImageIOImage.this.getParsedImage().getMetadata();
    }

    public Format getFormat()
    {
      return this.format;
    }

    @Override
    public String toString()
    {
      StringBuffer result = new StringBuffer();
      this.printMetadata( "streamMetadata", this.getStreamMeta(), result );
      this.printMetadata( "imageMetadata", this.getImageMeta(), result );
      return result.toString();
    }

    private void printMetadata( String tag, IIOMetadata meta, StringBuffer result )
    {
      if ( meta != null )
      {
        result.append( "<" + tag + ">\n" );
        result.append( IIOMetadataUtil.serializeMetadataNative( meta ) );
        result.append( "</" + tag + ">\n" );
      }
    }

    public Dimension getSize()
    {
      return new Dimension( this.getWidth(), this.getHeight() );
    }

    public Dimension getSize( String unit ) throws MetadataException
    {
      if ( Image.UNIT_PIXEL.equals( unit ) )
      {
        return this.getSize();
      }

      if ( Image.UNIT_MILLIMETER.equals( unit ) )
      {
        return BitmapImage.convertPixelToMillimeter( this.getSize(), this.getDPI() );
      }

      throw new MetadataException( unit + " not supported" );
    }

    public int getDPI() throws MetadataException
    {
      try
      {
        return Math.round( 25.4f / Float.parseFloat( this.queryDefaultImageMetadata( "/Dimension/HorizontalPixelSize/@value" ) ) );
      }
      catch ( Exception e )
      {
        throw new MetadataException( "DPI Metadata not available", e );
      }
    }

    public String getColorspaceType() throws MetadataException
    {
      try
      {
        return this.queryDefaultImageMetadata( "/Chroma/ColorSpaceType/@name" );
      }
      catch ( Exception e )
      {
        throw new MetadataException( "ColorspaceType Metadata not available", e );
      }
    }

    public ICC_Profile getEmbeddedICCProfile()
    {
      if ( !this.hasEmbeddedICCProfile() || this.embeddedColorProfileDefinition == null )
      {
        return null;
      }
      else
      {
        return constructICCProfileFromMetadataInformation( this.embeddedColorProfileDefinition );
      }
    }

    public boolean hasEmbeddedICCProfile()
    {
      if ( this.embeddedColorProfileDefinition == null )
      {
        final String query = "//TIFFField[@name='ICC Profile']/TIFFUndefined/@value";
        final String TIFF_METADATA_FORMAT = "com_sun_media_imageio_plugins_tiff_image_1.0";
        this.embeddedColorProfileDefinition = IIOMetadataUtil.queryMetadata( this.getImageMeta(), TIFF_METADATA_FORMAT, query );
      }
      return embeddedColorProfileDefinition != null;
    }

    private String queryDefaultImageMetadata( String xpathQuery )
    {
      return this.queryImageMetadata( IIOMetadataFormatImpl.standardMetadataFormatName, xpathQuery );
    }

    private String queryImageMetadata( String formatName, String xpathQuery )
    {
      return IIOMetadataUtil.queryMetadata( this.getImageMeta(), formatName, xpathQuery );
    }

    private ICC_Profile constructICCProfileFromMetadataInformation( String commaSeparatedUnsignedBytes )
    {
      String[] bytes = commaSeparatedUnsignedBytes.split( "," );
      byte[] iccData = new byte[bytes.length];
      for ( int i = 0; i < bytes.length; i++ )
      {
        iccData[i] = (byte) Integer.parseInt( bytes[i] );
      }
      return ICC_Profile.getInstance( iccData );
    }

    private int getWidth()
    {
      try
      {
        return Integer.parseInt( this.queryDefaultImageMetadata( "/Dimension/HorizontalScreenSize/@value" ) );
      }
      catch ( Exception e )
      {
        Raster raster = ImageIOImage.this.getParsedImage().getRaster();
        if ( raster != null )
        {
          return raster.getWidth();
        }
        return ImageIOImage.this.asBufferedImage().getWidth();
      }
    }

    private int getHeight()
    {
      try
      {
        return Integer.parseInt( this.queryDefaultImageMetadata( "/Dimension/VerticalScreenSize/@value" ) );
      }
      catch ( Exception e )
      {
        Raster raster = ImageIOImage.this.getParsedImage().getRaster();
        if ( raster != null )
        {
          return raster.getHeight();
        }
        return ImageIOImage.this.asBufferedImage().getHeight();
      }
    }

    private String getCompressionType() throws MetadataException
    {
      try
      {
        return this.queryDefaultImageMetadata( "/Compression/CompressionTypeName/@value" );
      }
      catch ( Exception e )
      {
        throw new MetadataException( "CompressionType Metadata not available", e );
      }
    }

    private boolean isLossless() throws MetadataException
    {
      try
      {
        return Boolean.valueOf( this.queryDefaultImageMetadata( "/Compression/Lossless/@value" ) ).booleanValue();
      }
      catch ( Exception e )
      {
        throw new MetadataException( "Lossless Metadata not available", e );
      }
    }

    private String getFormatVersion() throws MetadataException
    {
      try
      {
        return this.queryDefaultImageMetadata( "/Document/FormatVersion/@value" );
      }
      catch ( Exception e )
      {
        throw new MetadataException( "FormatVersion Metadata not available", e );
      }
    }

    private String getAttributValueFromNode( String keyword )
    {
      List<Node> nodes = this.queryImageMetadataNodes( this.format.getXMLRootTag(), this.format.getSignatureXQuery() );
      for ( Node n: nodes )
      {
        String value = n.getAttributes().getNamedItem( "value" ).getNodeValue();
        if ( value.contains( keyword ) )
        {
          return value.substring( value.indexOf( ":" ) + 1 );
        }
      }
      return "";
    }

    private List<Node> queryImageMetadataNodes( String formatName, String xpathQuery )
    {
      return IIOMetadataUtil.queryMetadataNodes( this.getImageMeta(), formatName, xpathQuery );
    }

    // @see ch.ige.image.Metadata#getConversionRuleName()
    public String getConversionRuleName()
    {
      return this.getAttributValueFromNode( Format.KEYWORD_ORIGINAL_RULE_NAME );
    }

    // @see ch.ige.image.Metadata#getMAC()
    public String getMAC()
    {
      return this.getAttributValueFromNode( Format.KEYWORD_MAC );
    }

    // @see ch.ige.image.Metadata#getOrigMetadata()
    public String getOrigMetadata()
    {
      return this.getAttributValueFromNode( Format.KEYWORD_ORIGINAL_METADATA );
    }
  } // inner-class
}
