package ch.ige.image;

import java.awt.Dimension;
import java.awt.color.ICC_Profile;
import java.awt.image.BufferedImage;
import java.awt.image.RenderedImage;
import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.nio.ByteBuffer;

import javax.media.jai.PlanarImage;

import ch.ige.image.IOUtil.ReadableByteBuffer;
import ch.ige.image.format.Format;

import com.sun.media.jai.codec.ImageCodec;
import com.sun.media.jai.codec.ImageDecoder;
import com.sun.media.jai.codec.MemoryCacheSeekableStream;
import com.sun.media.jai.codec.SeekableStream;

/**
 * Wrapper for JAI Images.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 *
 * @author Michael Locher
 * @version $Id: JAIImage.java,v 1.22 2009/03/18 16:37:02 locher Exp $
 */
public class JAIImage extends EncodedBitmapImage implements Externalizable
{
  private static final long serialVersionUID = 1L;

  private transient PlanarImage parsedImage;

  /**
   * Parses a byte buffer with the JAI library.
   *
   * @param imageData the image to parse
   * @return the parsed image
   */
  public static JAIImage parse( ByteBuffer imageData ) throws IOException
  {
    Object[] readerParams = JAIImage.createReader( imageData );
    return new JAIImage( imageData, (String) readerParams[0], (ImageDecoder) readerParams[1] );
  }

  /** For deserialization only. */
  public JAIImage()
  {
    super();
  }

  private JAIImage( ByteBuffer imageData, String format, ImageDecoder imageDecoder ) throws IOException
  {
    super( imageData );
    this.readImage( format, imageDecoder );
  }

  @Override
  protected void finalize() throws Throwable
  {
    this.flushCache();
    super.finalize();
  }

  public void flushCache()
  {
    if ( this.parsedImage != null )
    {
      this.parsedImage.dispose();
      this.parsedImage = null;
    }
  }

  // @see ch.ige.image.EncodedBitmapImage#readExternal(java.io.ObjectInput)
  @Override
  public void readExternal( ObjectInput in ) throws IOException, ClassNotFoundException
  {
    super.readExternal( in );
    this.rereadImage();
  }

  @Override
  protected void releaseClonedLazyReferences()
  {
    this.parsedImage = null;
  }

  private void rereadImage()
  {
    try
    {
      Object[] readerParams = JAIImage.createReader( this.getDataBuffer() );
      this.readImage( (String) readerParams[0], (ImageDecoder) readerParams[1] );
    }
    catch ( IOException e )
    {
      throw new RuntimeException( "reparsing of image failed", e );
    }
  }

  private static Object[] createReader( ByteBuffer imageData ) throws IOException
  {
    SeekableStream img = new MemoryCacheSeekableStream( new ReadableByteBuffer( imageData.duplicate() ) );
    String type = ImageCodec.getDecoderNames( img )[0];
    return new Object[] { type, ImageCodec.createImageDecoder( type, img, null ) };
  }

  private void readImage( String format, ImageDecoder imageDecoder ) throws IOException
  {
    final RenderedImage renderedImage = imageDecoder.decodeAsRenderedImage();
    this.parsedImage = PlanarImage.wrapRenderedImage( renderedImage );
    this.setMetadata( new MetadataWrapper( Format.fromTypeString( format ) ) );
  }

  private synchronized PlanarImage getParsedImage()
  {
    if ( this.parsedImage == null )
    {
      this.rereadImage();
    }
    return this.parsedImage;
  }

  @Override
  public PlanarImage asRenderedImage()
  {
    return this.getParsedImage();
  }

  @Override
  public BufferedImage asBufferedImage()
  {
    return this.getParsedImage().getAsBufferedImage();
  }

  // --------------------------------------------------------------------------

  /** Inner Class */
  private class MetadataWrapper implements Metadata
  {
    private static final long serialVersionUID = 1L;

    private Format format;

    public MetadataWrapper( Format basicFormatInformation )
    {
      this.completeFormatInformation( basicFormatInformation );
    }

    private void completeFormatInformation( Format basicFormatInformation )
    {
      this.format = basicFormatInformation;
      // here would be the place to get more information about the format from
      // the JAI reader
    }

    public Format getFormat()
    {
      return this.format;
    }

    public String getColorspaceType() throws MetadataException
    {
      throw new MetadataException( "ColorspaceType Metadata not available" );
    }

    public int getDPI() throws MetadataException
    {
      throw new MetadataException( "DPI Metadata not available" );
    }

    public Dimension getSize()
    {
      return new Dimension( JAIImage.this.getParsedImage().getWidth(), JAIImage.this.getParsedImage().getHeight() );
    }

    public Dimension getSize( String unit ) throws MetadataException
    {
      if ( !Image.UNIT_PIXEL.equals( unit ) )
      {
        throw new MetadataException( "Only pixel unit is supported" );
      }
      return this.getSize();
    }

    public boolean hasEmbeddedICCProfile()
    {
      /** TODO can we extract ICC Information from JAI Images */
      return false;
    }

    public ICC_Profile getEmbeddedICCProfile()
    {
      /** TODO can we extract ICC Information from JAI Images */
      return null;
    }

    // @see ch.ige.image.Metadata#getConversionRuleName()
    public String getConversionRuleName()
    {
      return null;
    }

    // @see ch.ige.image.Metadata#getMAC()
    public String getMAC()
    {
      return null;
    }

    // @see ch.ige.image.Metadata#getOrigMetadata()
    public String getOrigMetadata()
    {
      return null;
    }

  } // inner-class

}
