package ch.ige.image;

import java.awt.Transparency;
import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;
import java.awt.image.DataBuffer;
import java.awt.image.Raster;
import java.awt.image.RenderedImage;
import java.awt.image.SampleModel;
import java.awt.image.WritableRaster;

import org.apache.log4j.Logger;

/**
 * Detects which transparent grayscale images make trouble. 
 *
 * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
 * @author   Michael Locher
 * @version  $Id: TransparentGrayscaleProblem.java,v 1.1 2008/08/07 17:10:04 locher Exp $
 */
public class TransparentGrayscaleProblem
{

  private static final Logger LOG = Logger.getLogger( TransparentGrayscaleProblem.class );

  /**
   * Checks if the problem is present. Inspired by 410001_1472.
   * @param ri the image
   * @return true if the problem is present, false otherwise
   */
  public static boolean hasProblem( RenderedImage ri )
  {
    boolean expectProblem = false;
    try
    {
      final ColorModel cm = ri.getColorModel();
      final Raster data = ri.getData();

      //guess
      expectProblem = isProblemExpected( cm, data );

      //verify problem
      WritableRaster raster = data.createCompatibleWritableRaster( 2, 2 );
      BufferedImage target = new BufferedImage( cm, raster, cm.isAlphaPremultiplied(), null );
      target.createGraphics(); // this throws a NullPointerException sometimes
      return false;
    }
    catch ( RuntimeException e )
    {
      if ( expectProblem )
      {
        LOG.warn( "image operation failed as expected", e );
      }
      else
      {
        LOG.error( "image operation failed unexpectedly", e );
      }
      return true;
    }
  }

  private static boolean isProblemExpected( ColorModel cm, Raster data )
  {
    return rasterHasProblemCriteria( data ) && colorModelHasProblemCriteria( cm );
  }

  private static boolean colorModelHasProblemCriteria( ColorModel cm )
  {
    return ( 2 == cm.getNumComponents() ) && ( DataBuffer.TYPE_BYTE == cm.getTransferType() ) && cm.hasAlpha() && !cm.isAlphaPremultiplied()
        && ( Transparency.TRANSLUCENT == cm.getTransparency() ) && ( 8 == cm.getComponentSize( 0 ) ) && ( 8 == cm.getComponentSize( 1 ) );
  }

  private static boolean rasterHasProblemCriteria( Raster data )
  {
    SampleModel sm = data.getSampleModel();
    return ( 2 == sm.getNumBands() ) && ( DataBuffer.TYPE_BYTE == data.getTransferType() ) && ( DataBuffer.TYPE_BYTE == sm.getTransferType() );
  }

}
