package ch.ige.image.format;

import java.util.HashMap;
import java.util.Map;

import javax.imageio.ImageWriteParam;
import javax.imageio.metadata.IIOMetadata;

import org.apache.log4j.Logger;

import ch.ige.image.Image;
import ch.ige.image.MetadataException;

import com.sun.media.imageio.plugins.tiff.BaselineTIFFTagSet;

/**
 * TIFF Format.
 * <p>
 * Supported Compressions:
 * <ul>
 * <li>"CCITT RLE", bilevel images only</li>
 * <li>"CCITT T.4", bilevel images only</li>
 * <li>"CCITT T.6", bilevel images only</li>
 * <li>"JPEG", lossy</li>
 * <li>"LZW"</li>
 * <li>"PackBits"</li>
 * <li>"Deflate"</li>
 * <li>"ZLib"</li>
 * </ul>
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @author Michael Locher
 * @version $Id: TIFFFormat.java,v 1.21 2009/03/19 13:26:20 locher Exp $
 */
public class TIFFFormat extends Format
{
  /** The serialization version. */
  private static final long serialVersionUID = "$Revision: 1.21 $".hashCode();

  private static final Logger LOG = Logger.getLogger( TIFFFormat.class );

  private static final Map<String, String> SUPPORTED_COMPRESSIONS = new HashMap<String, String>();

  public static final String IDENTIFIER = "tif";

  public static final String IDENTIFIER2 = "tiff";

  public static final String MIME = "image/tiff";

  /*
   * The following Compressions support lossless compression of all image types: LZW - PackBits - Deflate - ZLib
   */
  protected static final String DEFAULT_COMPRESSION = "Deflate";

  static
  {
    SUPPORTED_COMPRESSIONS.put( "none", "keine Komprimierung" );
    SUPPORTED_COMPRESSIONS.put( "CCITT RLE", "CCITT RLE" );
    SUPPORTED_COMPRESSIONS.put( "CCITT T.4", "CCITT Group 3 Fax" );
    SUPPORTED_COMPRESSIONS.put( "CCITT T.6", "CCITT Group 4 Fax" );
    SUPPORTED_COMPRESSIONS.put( "LZW", "LZW" );
    SUPPORTED_COMPRESSIONS.put( "Old JPEG", "6.0 JPEG" );
    SUPPORTED_COMPRESSIONS.put( "JPEG", "JPEG DCT" );
    SUPPORTED_COMPRESSIONS.put( "PackBits", "PACKBITS (Macintosh)" );
    SUPPORTED_COMPRESSIONS.put( "Deflate", "Deflate" );
    SUPPORTED_COMPRESSIONS.put( "ZLib", "ZLib" );
  }

  public TIFFFormat()
  {
    super( IDENTIFIER );
    this.setSignatureXQuery( "/TIFFIFD/TIFFField[@number='270']/TIFFAsciis/TIFFAscii" );
    this.setXMLRootTag( "com_sun_media_imageio_plugins_tiff_image_1.0" );
  }

  @Override
  public String getMIMEType()
  {
    return MIME;
  }

  @Override
  public EncodingAdapter getEncodingAdapter( final Image image )
  {
    return new Format.EncodingAdapter()
    {

      @Override
      public void adaptWriteParams( ImageWriteParam params )
      {
        boolean setDefaultCompression = false;
        try
        {
          setDefaultCompression = TIFFFormat.this.getCompressionType() == null;
        }
        catch ( MetadataException e )
        {
          // if no compression metadata is available
          setDefaultCompression = true;
        }
        if ( setDefaultCompression )
        {
          TIFFFormat.this.setCompressionType( DEFAULT_COMPRESSION );
          TIFFFormat.this.applyCompressionType( params );
        }
      }

      @Override
      public void adaptMetadata( ConversionContext conversionContext, IIOMetadata streamMetadata, IIOMetadata imageMetadata )
      {
        try
        {
          float dpi = image.getMetadata().getDPI();
          this.setPixelSizeViaDefaultMetadataSchema( imageMetadata, 25.4f / dpi );
        }
        catch ( MetadataException e )
        {
          // no DPI information available... nothing to do
        }

        catch ( Exception e )
        {
          LOG.info( "Setting of Resolution Metadata failed", e );
        }
        // --------------------------------------
        // now merge the conversion context
        // TODO solve this with a single merge
        if ( conversionContext != null )
        {
          try
          {
            this.mergeMetadata( imageMetadata, "com_sun_media_imageio_plugins_tiff_image_1.0", createConversionContext( conversionContext ) );
          }
          catch ( Exception e )
          {
            LOG.info( "merge of conversion metadata failed", e );
            throw new RuntimeException( e );
          }
        }
      }

      private String createConversionContext( ConversionContext conversionContext )
      {
        StringBuilder mergeDoc = new StringBuilder(
            "<com_sun_media_imageio_plugins_tiff_image_1.0><TIFFIFD tagSets='com.sun.media.imageio.plugins.tiff.BaselineTIFFTagSet'><TIFFField number='" );
        mergeDoc.append( BaselineTIFFTagSet.TAG_IMAGE_DESCRIPTION );
        mergeDoc.append( "' name='ImageDescription'><TIFFAsciis><TIFFAscii value='" );
        mergeDoc.append( Format.KEYWORD_ORIGINAL_METADATA );
        mergeDoc.append( ":" );
        mergeDoc.append( conversionContext.getOriginalMetadata() );
        mergeDoc.append( "'/><TIFFAscii value='" );
        mergeDoc.append( Format.KEYWORD_MAC );
        mergeDoc.append( ":" );
        mergeDoc.append( EmbeddedHashes.getTemplate( null ) );
        mergeDoc.append( "'/><TIFFAscii value='" );
        mergeDoc.append( Format.KEYWORD_ORIGINAL_RULE_NAME );
        mergeDoc.append( ":" );
        mergeDoc.append( conversionContext.getRulesName() );
        mergeDoc.append( "'/></TIFFAsciis></TIFFField></TIFFIFD></com_sun_media_imageio_plugins_tiff_image_1.0>" );
        return mergeDoc.toString();
      }
    };
  }
}
