package ch.ige.image.format.png;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.ReadableByteChannel;
import java.nio.channels.WritableByteChannel;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;

import ch.ige.image.IOUtil.WritableByteBuffer;

/**
 * Moves chunks to the front.
 *
 * <p>Copyright (c) 2009, Swiss Federal Institute of Intellectual Property</p>
 * @author   locher
 * @version  $Id: ReorderChunksFilter.java,v 1.1 2009/04/07 11:31:18 locher Exp $
 */
public class ReorderChunksFilter extends PNGChunkFilter
{

  private static final Set<String> DEFAULT_TYPES = new HashSet<String>( Arrays.asList( "IHDR", "tIME", "tEXt" ) );

  private WritableByteBuffer delayed;

  private final Set<String> typesToFront;

  /**
   * Moves IHDR, tIME and tEXt to the front.
   */
  public ReorderChunksFilter()
  {
    this( DEFAULT_TYPES );
  }

  /**
   * Moves the specified chunk types to the front.
   * @param typesToFront the types to reorder
   */
  public ReorderChunksFilter( Collection<String> typesToFront )
  {
    super();
    this.typesToFront = typesToFront instanceof Set ? (Set<String>) typesToFront : new HashSet<String>( typesToFront );
  }

  // @see ch.ige.image.format.png.PNGChunkFilter#filter(java.nio.ByteBuffer)
  @Override
  public ByteBuffer filter( ByteBuffer img )
  {
    this.delayed = new WritableByteBuffer( img.isDirect(), img.limit() );
    return super.filter( img );
  }

  // @see ch.ige.image.format.png.PNGChunkFilter#filter(java.nio.channels.ReadableByteChannel, java.nio.channels.WritableByteChannel)
  @Override
  public void filter( ReadableByteChannel in, WritableByteChannel out ) throws IOException
  {
    try
    {
      if ( this.delayed == null )
      {
        this.delayed = new WritableByteBuffer( false );
      }
      super.filter( in, out );
    }
    finally
    {
      this.delayed = null;
    }
  }

  // @see ch.ige.image.format.png.PNGChunkFilter#handleChunk(ch.ige.image.format.png.PNGChunkFilter.EmitableChunk)
  @Override
  protected void handleChunk( EmitableChunk in ) throws IOException
  {
    if ( isDelayed( in ) )
    {
      in.emit( delayed );
    }
    else
    {
      in.emit();
    }
  }

  private boolean isDelayed( EmitableChunk c )
  {
    return !this.typesToFront.contains( c.getType() ) && !c.isIEND();
  }

  // @see ch.ige.image.format.png.PNGChunkFilter#beforeLast(java.nio.channels.WritableByteChannel)
  @Override
  protected void beforeLast( WritableByteChannel out ) throws IOException
  {
    writeRemaining( delayed.getBuffer(), out );
  }

}
