package ch.ige.image.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GraphicsConfiguration;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.awt.image.RenderedImage;
import java.awt.image.VolatileImage;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.atomic.AtomicInteger;

import javax.swing.JComponent;
import javax.swing.SwingUtilities;
import javax.swing.TransferHandler;

import org.apache.log4j.Logger;

import ch.ige.image.Image;
import ch.ige.image.MetadataException;

/**
 * Image Display Component.
 * <p>
 * Respects the DPI setting of the image. Allows zooming.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @see #setZoomFactor(double)
 * @author Michael Locher
 * @version $Id: ImageDisplay.java,v 1.23 2009/04/16 13:08:00 locher Exp $
 */
public class ImageDisplay extends JComponent
{
  /** The serialization version. */
  private static final long serialVersionUID = "$Revision: 1.23 $".hashCode();

  private static final Logger LOG = Logger.getLogger( ImageDisplay.class );

  protected final AtomicInteger ticketNumber = new AtomicInteger( 0 );

  protected volatile transient RenderedImage source = null;

  private volatile AffineTransform transform = null;

  private volatile int dpi = 72;

  private volatile double zoomFactor = 1.0;

  private transient volatile BufferedImage transformedImage;

  private transient volatile VolatileImage volatileImage;

  private static final ExecutorService renderer = Executors.newSingleThreadExecutor();

  public ImageDisplay()
  {
    super();
    this.setDoubleBuffered( false );
    setLayout( null );
    this.setTransferHandler( new TransferHandler( null )
    {
      /** The serialization version. */
      private static final long serialVersionUID = 1L;

      // @see javax.swing.TransferHandler#createTransferable(javax.swing.JComponent)
      @Override
      protected Transferable createTransferable( JComponent c )
      {
        return new Transferable()
        {
          public Object getTransferData( DataFlavor flavor )
          {
            if ( isDataFlavorSupported( flavor ) )
            {
              return source;
            }
            return null;
          }

          public DataFlavor[] getTransferDataFlavors()
          {
            return new DataFlavor[] { DataFlavor.imageFlavor };
          }

          public boolean isDataFlavorSupported( DataFlavor flavor )
          {
            return DataFlavor.imageFlavor.equals( flavor );
          }
        };
      }

      // @see javax.swing.TransferHandler#getSourceActions(javax.swing.JComponent)
      @Override
      public int getSourceActions( JComponent c )
      {
        return COPY;
      }
    } );
    MouseListener listener = new MouseAdapter()
    {
      @Override
      public void mousePressed( MouseEvent e )
      {
        if ( ImageDisplay.this.source != null )
        {
          JComponent c = (JComponent) e.getSource();
          c.getTransferHandler().exportAsDrag( c, e, TransferHandler.COPY );
        }
      }
    };
    this.addMouseListener( listener );
  }

  public ImageDisplay( Image image )
  {
    this();
    this.setImage( image );
  }

  // @see javax.swing.JComponent#setBackground(java.awt.Color)
  @Override
  public void setBackground( Color bg )
  {
    super.setBackground( bg );
    ImageDisplay.renderer.execute( new Runnable()
    {
      public void run()
      {
        ImageDisplay.this.handleImageChange();
      }
    } );
  }

  /** use to display a new image */
  public synchronized void setImage( final Image im )
  {
    final int ticket = ticketNumber.incrementAndGet();
    if ( im == null )
    {
      this.setImage( null, 72 );
    }
    else
    {
      int DPI = 72;
      try
      {
        DPI = im.getMetadata().getDPI();
      }
      catch ( MetadataException e )
      {
        // ignore
      }
      final int fDPI = DPI;
      ImageDisplay.renderer.execute( new Runnable()
      {
        public void run()
        {
          if ( ticket == ticketNumber.get() )
          {
            setImage( im, fDPI );
          }
          else
          {
            LOG.debug( "image skipped" );
          }
        }
      } );
    }
  }

  void setImage( final Image im, int DPI )
  {
    RenderedImage img;
    if ( im == null )
    {
      img = null;
    }
    else
    {
      try
      {
        img = im.asRenderedImage();
      }
      catch ( RuntimeException e )
      {
        //hmmm...
        LOG.error( "could not get image", e );
        img = null;
      }
    }

    final RenderedImage oldSrc = this.source;
    if ( oldSrc != null && ( oldSrc instanceof BufferedImage ) && oldSrc != img )
    {
      ( (BufferedImage) oldSrc ).flush();
    }

    this.source = img;
    this.dpi = Math.max( 1, DPI );
    this.transform = null;

    this.handleImageChange();

  }

  private void handleImageChange()
  {
    if ( this.source != null )
    {
      this.calcPreferedSize();
      this.transformImage();
      this.prepareVolatileImage();
    }
    else
    {
      this.transformedImage = null;
      this.volatileImage = null;
    }
    SwingUtilities.invokeLater( new Runnable()
    {
      public void run()
      {
        ImageDisplay.this.revalidate();
        if ( ImageDisplay.this.isVisible() )
        {
          ImageDisplay.this.repaint();
        }
      }
    } );
  }

  private void prepareVolatileImage()
  {
    BufferedImage trans = this.transformedImage;
    if ( trans == null )
    {
      this.volatileImage = null;
    }
    else
    {
      this.volatileImage = this.createVolatileImage( trans.getWidth(), trans.getHeight() );
      this.renderOffscreen();
    }
  }

  private void renderOffscreen()
  {
    // TODO Convert back to trace logging
    LOG.debug( "[TRACE] render offscreen:start" );
    final GraphicsConfiguration ctx = this.getGraphicsConfiguration();
    VolatileImage img = this.volatileImage;
    BufferedImage trans = this.transformedImage;
    if ( img == null || trans == null )
    {
      return;
    }
    do
    {
      if ( img.validate( ctx ) == VolatileImage.IMAGE_INCOMPATIBLE )
      {
        img = this.createVolatileImage( trans.getWidth(), trans.getHeight() );
        this.volatileImage = img;
      }
      Graphics2D g = img.createGraphics();
      g.setRenderingHint( RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC );
      g.setRenderingHint( RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY );
      g.setRenderingHint( RenderingHints.KEY_COLOR_RENDERING, RenderingHints.VALUE_COLOR_RENDER_QUALITY );
      g.setRenderingHint( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );
      g.setRenderingHint( RenderingHints.KEY_ALPHA_INTERPOLATION, RenderingHints.VALUE_ALPHA_INTERPOLATION_QUALITY );
      g.drawImage( trans, null, 0, 0 );
      g.dispose();
    } while ( img.contentsLost() );
    // TODO Convert back to trace logging
    LOG.debug( "[TRACE] render offscreen:finish" );
  }

  private void transformImage()
  {
    final GraphicsConfiguration gc = this.getGraphicsConfiguration();
    if ( gc != null )
    {
      BufferedImage t = gc.createCompatibleImage( this.calcZoomedImageWidth(), this.calcZoomedImageHeight() );
      Graphics2D g2d = t.createGraphics();
      this.setRenderingHints( g2d );
      // paint the image
      try
      {
        LOG.debug( "transform image" );
        g2d.setColor( getBackground() );
        g2d.fillRect( 0, 0, t.getWidth(), t.getHeight() );
        g2d.drawRenderedImage( this.source, this.getTransform() );
        BufferedImage old = this.transformedImage;
        if ( old != null )
        {
          old.flush();
        }
        this.transformedImage = t;
      }
      catch ( OutOfMemoryError e )
      {
        LOG.error( "Display of image failed", e );
      }
    }
  }

  private void calcPreferedSize()
  {
    int w = this.calcZoomedImageWidth();
    int h = this.calcZoomedImageHeight();
    Insets insets = this.getInsets();
    Dimension dim = new Dimension( w + insets.left + insets.right, h + insets.top + insets.bottom );
    this.setPreferredSize( dim );
  }

  private int calcZoomedImageHeight()
  {
    RenderedImage src = this.source;
    return (int) ( src == null ? 1 : ( src.getHeight() * this.zoomFactor * 72.0 / this.dpi ) );
  }

  private int calcZoomedImageWidth()
  {
    RenderedImage src = this.source;
    return (int) ( src == null ? 1 : ( src.getWidth() * this.zoomFactor * 72.0 / this.dpi ) );
  }

  public synchronized double getZoomFactor()
  {
    return this.zoomFactor;
  }

  /**
   * Sets the zoom factor of the component.
   * 
   * @param zoomFactor scaling factor (1.0 = 100%)
   */
  public synchronized void setZoomFactor( final double zoomFactor )
  {
    ImageDisplay.renderer.execute( new Runnable()
    {
      public void run()
      {
        ImageDisplay.this.zoomFactor = zoomFactor;
        ImageDisplay.this.transform = null;
        ImageDisplay.this.handleImageChange();
      }
    } );
  }

  /**
   * Calculates the zoom factor that the entire image is displayed.
   * @return the zoom factor between 0.0 and 1.0
   */
  public double calcZoomShrinkToFit( Dimension targetSize )
  {
    RenderedImage src = this.source;
    if ( src == null )
    {
      return 1.0;
    }
    double w = src.getWidth();
    double h = src.getHeight();
    double zHorizontal = targetSize.width / w;
    double zVertical = targetSize.height / h;
    return Math.max( 1E-6, Math.min( 1.0, Math.min( zHorizontal, zVertical ) * this.dpi / 72.0 ) );

  }

  private synchronized AffineTransform getTransform()
  {
    AffineTransform t = this.transform;
    if ( t == null )
    {
      Insets insets = this.getInsets();
      t = getBaseTransform();
      if ( this.dpi != 72 )
      {
        double dpiScaleFactor = 72.0 / this.dpi;
        t.concatenate( AffineTransform.getScaleInstance( dpiScaleFactor, dpiScaleFactor ) );
      }
      if ( this.zoomFactor != 1.0 )
      {
        t.concatenate( AffineTransform.getScaleInstance( this.zoomFactor, this.zoomFactor ) );
      }
      t.concatenate( AffineTransform.getTranslateInstance( insets.left, insets.top ) );
      this.transform = t;
    }
    return t;
  }

  private AffineTransform getBaseTransform()
  {
    //Note getting the gc requires the Swing Tree Lock
    GraphicsConfiguration gc = this.getGraphicsConfiguration();
    AffineTransform t = gc.getDefaultTransform();
    t.concatenate( gc.getNormalizingTransform() );
    return t;
  }

  /** paint routine */
  @Override
  public void paintComponent( Graphics g )
  {
    assert SwingUtilities.isEventDispatchThread() : "processing in EDT required";

    Graphics2D g2d = (Graphics2D) g;

    VolatileImage img = this.volatileImage;

    if ( img == null )
    {
      g2d.setColor( getBackground() );
      g2d.fillRect( 0, 0, getWidth(), getHeight() );
    }
    else
    {
      this.setRenderingHints( g2d );

      // clear damaged component area
      Rectangle clipBounds = g2d.getClipBounds();
      g2d.setColor( getBackground() );
      g2d.fillRect( clipBounds.x, clipBounds.y, clipBounds.width, clipBounds.height );

      // paint the image
      try
      {
        do
        {
          int returnCode = img.validate( getGraphicsConfiguration() );
          if ( returnCode == VolatileImage.IMAGE_RESTORED )
          {
            renderOffscreen();
          }
          else if ( returnCode == VolatileImage.IMAGE_INCOMPATIBLE )
          {
            this.prepareVolatileImage();
          }
          g2d.drawImage( img, 0, 0, this );
        } while ( img.contentsLost() );

        BufferedImage trans = this.transformedImage;
        if ( trans != null )
        {
          g2d.drawRenderedImage( trans, null );
        }
      }
      catch ( OutOfMemoryError e )
      {
        LOG.error( "Display of image failed", e );
      }
    }
  }

  private void setRenderingHints( Graphics2D g2d )
  {
    g2d.setRenderingHint( RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BICUBIC );
    g2d.setRenderingHint( RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY );
    g2d.setRenderingHint( RenderingHints.KEY_COLOR_RENDERING, RenderingHints.VALUE_COLOR_RENDER_QUALITY );
    g2d.setRenderingHint( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON );
    g2d.setRenderingHint( RenderingHints.KEY_ALPHA_INTERPOLATION, RenderingHints.VALUE_ALPHA_INTERPOLATION_QUALITY );
  }

}
