package ch.ige.imagehandler;

import ch.ige.image.Image;
import ch.ige.image.ImageException;
import ch.ige.image.ImageFactory;
import ch.ige.image.MetadataException;
import ch.ige.imagehandler.rule.RuleException;

/**
 * IGE ImageHandler.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @see ch.ige.imagehandler.ImageHandlerFactory
 * 
 * @author    Michael Locher
 * @version   $Id: ImageHandler.java,v 1.18 2008/05/20 15:14:20 locher Exp $
 */
public interface ImageHandler
{

  /**
   * Gives an ImageFactory for image creation.
   * 
   * @return an ImageFactory for creating images.
   */
  public ImageFactory getImageFactory();

  /**
   * Throws an exception which describes the reason for failing the check.
   * 
   * @see ch.ige.imagehandler.rule.Checker#checkImage(Image)
   * 
   * @param image
   *          the image to be checked.
   * @param rulesName
   *          name of the rule set of the profile.
   * @throws MetadataException
   *           is thrown if not all needed metadata is present.
   * @throws RuleException
   *           is thrown if the image does not conform to the rules.
   */
  public void checkWithFeedback( final Image image, String rulesName ) throws MetadataException, RuleException;

  /**
   * Checks if the image conforms to the given rules.
   * 
   * @see ch.ige.imagehandler.rule.Checker#checkImage(Image)
   * 
   * @param image
   *          the image to be checked.
   * @param rulesName
   *          name of the rule set of the profile.
   * @return true, if the image conforms to the rules.
   * @throws MetadataException
   *           is thrown if not all needed metadata is present.
   */
  public boolean check( Image image, String rulesName ) throws MetadataException;

  /**
   * Converts the image according to the choosen RuleList of the profile.
   * 
   * @see ch.ige.imagehandler.rule.Converter#convert(Image)
   * 
   * @param image
   *          the image to be converted.
   * @param rulesName
   *          name of the rule set of the profile.
   * @return a new image with the results of the conversion.
   * @throws RuleException
   *           if a problem with a rule is detected.
   * @throws ImageException
   *           if an image manipulation operation fails.
   */
  public Image convert( Image image, String rulesName ) throws RuleException, ImageException;

}