package ch.ige.imagehandler.batch;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.Callable;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.Semaphore;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

import org.apache.log4j.Logger;

import ch.ige.image.EncodedImage;
import ch.ige.image.Image;
import ch.ige.imagehandler.ImageHandler;

/**
 * Batch Conversion.
 *
 * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
 * @author   Michael Locher
 * @version  $Id: BatchConvert.java,v 1.12 2009/04/14 09:57:51 locher Exp $
 */
public class BatchConvert implements Runnable
{
  static final Logger LOG = Logger.getLogger( BatchConvert.class );

  protected final ImageHandler handler;

  private final BlockingQueue<JobDescription> jobs;

  private final ExecutorService converter;

  private final ExecutorService deliverer;

  private volatile boolean running = false;

  private volatile boolean finished = false;

  private volatile Semaphore transit = new Semaphore( 1 );

  private final Semaphore busy = new Semaphore( 1 );

  public static BatchConvert createBatch( ImageHandler handler, int numJobs )
  {
    return new BatchConvert( handler, numJobs );
  }

  public static BatchConvert createBatch( ImageHandler handler, JobDescription job ) throws InterruptedException
  {
    BatchConvert batch = createBatch( handler, job.getRequestedConversions().size() );
    batch.submitJob( job );
    return batch;
  }

  public static void runAndWait( BatchConvert batch ) throws InterruptedException
  {
    batch.startWorker();
    batch.joinAndStop();
  }

  private BatchConvert( ImageHandler handler, int numConverters )
  {
    this.handler = handler;
    this.jobs = new LinkedBlockingQueue<JobDescription>();
    this.deliverer = new ThreadPoolExecutor( 1, 1, 0L, TimeUnit.MILLISECONDS, new ArrayBlockingQueue<Runnable>( 2 ) );
    if ( numConverters > 1 )
    {
      this.converter = Executors.newFixedThreadPool( numConverters );
    }
    else
    {
      this.converter = Executors.newSingleThreadExecutor();
    }
  }

  public synchronized void startWorker()
  {
    try
    {
      this.transit.acquire();
      if ( !this.finished && !this.running )
      {
        ScheduledExecutorService ex = Executors.newSingleThreadScheduledExecutor();
        ex.execute( this );
        ex.shutdown();
      }
      else
      {
        this.transit.release();
      }
    }
    catch ( InterruptedException e )
    {
      LOG.error( "start worker failed", e );
    }
  }

  private void stopWorker() throws InterruptedException
  {
    this.busy.acquire();
    this.finished = true;
    this.converter.shutdown();
    this.deliverer.shutdown();
    this.busy.release();
  }

  public void joinAndStop() throws InterruptedException
  {
    this.join();
    this.stopWorker();
  }

  public void join() throws InterruptedException
  {
    aquireReleaseCycle();
    while ( !this.jobs.isEmpty() )
    {
      aquireReleaseCycle();
      Thread.sleep( 1000 );
    }
    aquireReleaseCycle();
  }

  private void aquireReleaseCycle() throws InterruptedException
  {
    this.transit.acquire();
    this.transit.release();
    this.busy.acquire();
    this.busy.release();
  }

  public synchronized void submitJob( JobDescription jobDesc ) throws InterruptedException
  {
    if ( this.finished )
    {
      throw new IllegalStateException( "batch already finished" );
    }
    this.jobs.put( jobDesc );
  }

  // @see java.lang.Runnable#run()
  public void run()
  {
    try
    {
      running = true;
      this.transit.release();
      while ( !this.finished )
      {
        pollOnceAndRun();
      }
    }
    catch ( InterruptedException e )
    {
      LOG.warn( "loop interrupted", e );
    }
    finally
    {
      running = false;
    }
  }

  private void pollOnceAndRun() throws InterruptedException
  {
    transit.acquire();
    JobDescription job = this.jobs.poll( 1, TimeUnit.SECONDS );
    if ( job != null )
    {
      try
      {
        this.processJob( job );
      }
      catch ( Throwable e )
      {
        LOG.error( "job failed", e );
      }
    }
    else
    {
      transit.release();
    }
  }

  private void processJob( JobDescription job ) throws Throwable
  {
    this.busy.acquire();
    this.transit.release();
    LOG.info( "start job" );
    long startMillis = System.currentTimeMillis();
    final ImageSource src = job.getSource();
    try
    {
      final Semaphore deliverySync = new Semaphore( 1 );
      final Consumer dest = job.getConsumer();
      final boolean wantsMetadata = dest.wantMetadata();
      final Collection<String> requestedConversions = job.getRequestedConversions();
      int numConversions = requestedConversions.size();
      Future<Void> receipt = null;
      int cnt = 0;
      for ( ImageWithContext<EncodedImage> img: src )
      {
        cnt++;
        if ( wantsMetadata && !img.hasMetadata() )
        {
          LOG.debug( "get metadata for " + cnt );
          img.putMetadataContext();
        }

        CountDownLatch readyFlag = new CountDownLatch( numConversions );
        Map<String, Future<ImageWithContext<Image>>> result = convert( readyFlag, cnt, img, requestedConversions, wantsMetadata );
        handleLastReceiptAndNotifyNextReady( receipt, readyFlag, dest );
        receipt = deliverResult( cnt, deliverySync, dest, job, img, result );
      }
      LOG.info( String.format( "processed %d images", cnt ) );
    }
    catch ( AbortBatchException e )
    {
      LOG.info( "batch aborted" );
      src.abort();
    }
    finally
    {
      LOG.info( String.format( "job finished after %dms", System.currentTimeMillis() - startMillis ) );
      this.busy.release();
    }
  }

  private Future<Void> deliverResult( final int cnt, final Semaphore deliverySync, final Consumer dest, final JobDescription job,
      final ImageWithContext<EncodedImage> img, final Map<String, Future<ImageWithContext<Image>>> result )
  {
    LOG.debug( "post result for image " + cnt );

    return this.deliverer.submit( new Callable<Void>()
    {
      // @see java.util.concurrent.Callable#call()
      public Void call() throws Exception
      {
        try
        {
          try
          {
            LOG.debug( "wait for delivery of " + cnt );
            deliverySync.acquire();
            LOG.debug( "wait over, handle result " + cnt );
            dest.handleConverted( job, img, result );
            return null;
          }
          catch ( InterruptedException e )
          {
            throw new RuntimeException( e );
          }
        }
        finally
        {
          deliverySync.release();
          img.dispose();
          for ( Future<ImageWithContext<Image>> converted: result.values() )
          {
            try
            {
              ImageWithContext<Image> ic = converted.get();
              Image i = ic.getImage();
              if ( i != null )
              {
                i.dispose();
              }
            }
            catch ( Exception e )
            {
              LOG.debug( "dispose failed", e );
            }
          }
        }
      }
    } );
  }

  private void handleLastReceiptAndNotifyNextReady( Future<Void> receipt, CountDownLatch nextReady, Consumer dest ) throws AbortBatchException
  {
    boolean notifyDone = false;
    try
    {
      if ( receipt != null )
      {
        LOG.debug( "wait for result handle to finish" );
        boolean receiptTimeout = true;
        while ( receiptTimeout )
        {
          try
          {
            receipt.get( 500, TimeUnit.MILLISECONDS );
            receiptTimeout = false;
          }
          catch ( InterruptedException e )
          {
            receiptTimeout = false;
            throw new RuntimeException( e );
          }
          catch ( ExecutionException e )
          {
            receiptTimeout = false;
            Throwable cause = e.getCause();
            if ( cause instanceof AbortBatchException )
            {
              throw (AbortBatchException) cause;
            }
            else if ( cause instanceof RuntimeException )
            {
              throw (RuntimeException) cause;
            }
            else
            {
              throw new RuntimeException( cause );
            }
          }
          catch ( TimeoutException e )
          {
            try
            {
              if ( nextReady.await( 0, TimeUnit.MILLISECONDS ) )
              {
                dest.notifyNextReady();
                notifyDone = true;
              }
            }
            catch ( InterruptedException ie )
            {
              throw new RuntimeException( ie );
            }
          }
        }
      }
    }
    finally
    {
      LOG.debug( "result handle finished" );
      if ( !notifyDone )
      {
        try
        {
          if ( nextReady.await( 0, TimeUnit.MILLISECONDS ) )
          {
            dest.notifyNextReady();
          }
        }
        catch ( InterruptedException ie )
        {
          throw new RuntimeException( ie );
        }
      }
    }
  }

  private Map<String, Future<ImageWithContext<Image>>> convert( CountDownLatch readyFlag, int cnt, ImageWithContext<EncodedImage> img,
      final Collection<String> requestedConversions, boolean wantsMetadata )
  {
    LOG.debug( "submit image " + cnt );
    final int numConversions = requestedConversions.size();
    Map<String, Future<ImageWithContext<Image>>> result = Collections
        .synchronizedMap( new HashMap<String, Future<ImageWithContext<Image>>>( 2 * numConversions ) );
    for ( String conversionRule: requestedConversions )
    {
      result.put( conversionRule, convert( readyFlag, cnt, img, conversionRule, wantsMetadata ) );
    }
    return result;
  }

  private Future<ImageWithContext<Image>> convert( CountDownLatch readyFlag, int cnt, ImageWithContext<EncodedImage> img, String conversionRule,
      boolean wantsMetadata )
  {
    return this.converter.submit( this.convertJob( readyFlag, cnt, img, conversionRule, wantsMetadata ) );
  }

  private Callable<ImageWithContext<Image>> convertJob( final CountDownLatch readyFlag, final int cnt, final ImageWithContext<EncodedImage> img,
      final String conversionRule, final boolean wantsMetadata )
  {
    return new Callable<ImageWithContext<Image>>()
    {
      public ImageWithContext<Image> call() throws Exception
      {
        LOG.debug( "convert " + cnt + " with rule " + conversionRule );
        ImageWithContext<Image> i = new ImageWithContext<Image>( BatchConvert.this.handler.convert( img.getImage(), conversionRule ) );
        if ( wantsMetadata )
        {
          LOG.debug( "get metadata for " + cnt + " with rule " + conversionRule );
          i.putMetadataContext();
        }
        readyFlag.countDown();
        return i;
      }
    };
  }

  //---------------------------------------------

  public static void abortBatch() throws AbortBatchException
  {
    throw new AbortBatchException();
  }

  public static class AbortBatchException extends Exception
  {
    /** The serialization version. */
    private static final long serialVersionUID = "$Revision: 1.12 $".hashCode();

    public AbortBatchException()
    {
      super();

    }
  }

}