package ch.ige.imagehandler.batch;

import java.io.File;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.Map;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;

import org.apache.log4j.Logger;

import ch.ige.image.EncodedImage;
import ch.ige.image.Image;
import ch.ige.image.format.Format;

/**
 * Stores the converted images.
 *
 * <p>Copyright (c) 2009, Swiss Federal Institute of Intellectual Property</p>
 * @author   locher
 * @version  $Id: ResultsOnlyConsumer.java,v 1.3 2009/04/23 17:39:40 locher Exp $
 */
public class ResultsOnlyConsumer extends AbstractFileConsumer
{

  private static final Logger LOG = Logger.getLogger( ResultsOnlyConsumer.class );

  private final String baseDirectoryPath;

  private final boolean fixExtension;

  public ResultsOnlyConsumer( File destLocation, boolean fixExtension )
  {
    this( destLocation, null, fixExtension );
  }

  public ResultsOnlyConsumer( File destLocation, File baseDirectory, boolean fixExtension )
  {
    super( destLocation );
    this.baseDirectoryPath = ( baseDirectory == null ? null : baseDirectory.getAbsolutePath() );
    this.fixExtension = fixExtension;
  }

  // @see ch.ige.imagehandler.batch.Consumer#wantMetadata()
  public boolean wantMetadata()
  {
    return false;
  }

  // @see ch.ige.imagehandler.batch.Consumer#handleConverted(ch.ige.imagehandler.batch.JobDescription, ch.ige.imagehandler.batch.ImageWithContext, java.util.Map)
  public void handleConverted( JobDescription job, ImageWithContext<EncodedImage> img, Map<String, Future<ImageWithContext<Image>>> results )
  {
    final boolean multiple = results.size() > 1;
    for ( Map.Entry<String, Future<ImageWithContext<Image>>> conv: results.entrySet() )
    {
      String rule = conv.getKey();
      File originalName = (File) img.getContextValue( FileListSource.FILE_CONTEXT );
      try
      {
        try
        {
          ImageWithContext<Image> converted = conv.getValue().get();
          if ( converted.isEncoded() )
          {
            try
            {
              Format fmt = this.fixExtension ? determineFormat( converted ) : null;
              File destination = determineFilename( multiple, rule, originalName, fmt, false );
              writeImage( destination, converted.asEncoded() );
            }
            catch ( IOException e )
            {
              LOG.error( rule + " failed", e );
            }
          }
        }
        catch ( ExecutionException e )
        {
          File destination = determineFilename( multiple, rule, originalName, null, true );
          writeException( destination, e.getCause() );
        }
      }
      catch ( Exception e )
      {
        LOG.error( rule + " failed", e );
      }
    }
  }

  private Format determineFormat( ImageWithContext<Image> converted )
  {
    try
    {
      return converted.getImage().getMetadata().getFormat();
    }
    catch ( Exception e )
    {
      return null;
    }
  }

  private File determineFilename( boolean multiple, String rule, File originalName, Format format, boolean forException )
  {
    StringBuilder filename = new StringBuilder();
    if ( multiple )
    {
      filename.append( rule );
      filename.append( '_' );
    }
    final String name = originalName == null ? "unknown.img" : originalName.getName();
    filename.append( name );
    if ( format != null )
    {
      String extension = getExtension( format );
      if ( !name.endsWith( extension ) )
      {
        filename.append( extension );
      }
    }
    if ( forException )
    {
      filename.append( ".txt" );
    }
    String extraDirs = null;
    if ( this.baseDirectoryPath != null && originalName != null )
    {
      String originalPath = originalName.getAbsolutePath();
      if ( originalPath.startsWith( this.baseDirectoryPath ) )
      {
        File originalRelative = new File( originalPath.substring( this.baseDirectoryPath.length() ) );
        extraDirs = originalRelative.getParent();
      }
    }
    File result = relativeFile( extraDirs, filename.toString() );
    int i = 1;
    while ( result.exists() )
    {
      i++;
      StringBuilder next = new StringBuilder( "__" );
      next.append( i );
      next.append( "__" );
      next.append( filename );
      result = relativeFile( extraDirs, next.toString() );
    }
    return result;
  }

  private String getExtension( Format format )
  {
    String extension = format.getName().toLowerCase();
    if ( "jpeg".equals( extension ) )
    {
      extension = "jpg";
    }
    else if ( "tiff".equals( extension ) )
    {
      extension = "tif";
    }
    return "." + extension;
  }

  private void writeImage( File destination, ImageWithContext<EncodedImage> img ) throws IOException
  {
    EncodedImage i = img.getImage();
    if ( i != null )
    {
      i.write( destination );
    }
  }

  private void writeException( File destination, Throwable cause ) throws IOException
  {
    OutputStreamWriter out = createWriter( destination );
    try
    {
      out.write( cause == null ? "null" : cause.toString() );
    }
    finally
    {
      out.close();
    }
  }

}
