package ch.ige.imagehandler.rule;

import java.util.List;

import org.apache.log4j.Logger;

import ch.ige.image.Image;
import ch.ige.imagehandler.profile.Profile;
import ch.ige.imagehandler.rule.parameter.Environment;
import ch.ige.imagehandler.rule.parameter.ListBinding;
import ch.ige.imagehandler.rule.parameter.Parameter;
import ch.ige.imagehandler.rule.parameter.SingleBinding;

/**
 * Base Class for Rules.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @author Michael Locher
 * @version $Id: AbstractRule.java,v 1.13 2009/04/14 11:38:03 locher Exp $
 */
public abstract class AbstractRule<I extends Image> implements Rule<I>
{
  private static final String MESSAGE_PARAMETER = "message";

  private static final Logger LOG = Logger.getLogger( Rule.class );

  private Environment parameters;

  private Profile profile;

  public AbstractRule()
  {
    this.parameters = new Environment();
  }

  /**
   * Gives the name of this rule.
   * 
   * @return the name
   */
  protected abstract String getRuleName();

  /*
   * @see ch.ige.imagehandler.rule.Rule#checkParameters()
   */
  public void checkParameters() throws RuleException
  {
    // nothing to check per default
  }

  /*
   * @see ch.ige.imagehandler.rule.Rule#accept(ch.ige.imagehandler.rule.RuleVisitor)
   */
  public void accept( RuleVisitor visitor ) throws RuleException
  {
    visitor.handleRule( this );
  }

  /**
   * Sets the message parameter.
   * 
   * @param message
   *          the message.
   */
  public void setMessage( String message )
  {
    this.parameters.addBinding( new SingleBinding( MESSAGE_PARAMETER, message ) );
  }

  @Override
  public String toString()
  {
    StringBuffer result = new StringBuffer( this.getRuleName() + ":\n" );
    if ( !this.parameters.isEmpty() )
    {
      result.append( this.parameters.toString() );
    }
    return result.toString();
  }

  // ------------------------------------------------------------------------------------------
  // Profile

  /*
   * @see ch.ige.imagehandler.rule.Rule#setProfile(ch.ige.imagehandler.Profile)
   */
  public void setProfile( Profile profile )
  {
    this.profile = profile;
  }

  /*
   * @see ch.ige.imagehandler.rule.Rule#getProfile()
   */
  public Profile getProfile()
  {
    return this.profile;
  }

  // ------------------------------------------------------------------------------------------
  // Parameters

  /*
   * @see ch.ige.imagehandler.rule.Rule#addParameter(ch.ige.imagehandler.rule.parameter.Parameter)
   */
  public void addParameter( Parameter binding )
  {
    this.parameters.addBinding( binding );
  }

  /**
   * Gives the value of the given parameter.
   * 
   * @param name
   *          the parameter name
   * @return the value or UNDEFINED_PARAM if not defined
   * @see Rule#UNDEFINED_PARAM
   */
  public String getParameterValue( String name )
  {
    Parameter param = this.getParameter( name );
    if ( param == null )
    {
      return UNDEFINED_PARAM;
    }
    return ( (SingleBinding) param ).getValue();
  }

  /**
   * Gives the values of the given parameter list.
   * 
   * @param name
   *          the parameter name
   * @return the values or UNDEFINED_PARAMLIST if not defined
   * @see Rule#UNDEFINED_PARAMLIST
   */
  public List<String> getParameterList( String name )
  {
    Parameter param = this.getParameter( name );
    if ( param == null )
    {
      return UNDEFINED_PARAMLIST;
    }
    return ( (ListBinding) param ).getValues();
  }

  private Parameter getParameter( String name )
  {
    return this.parameters.lookup( name );
  }

  protected boolean hasParameter( String name )
  {
    return this.getParameter( name ) != null;
  }

  // ------------------------------------------------------------------------------------------
  // Utils

  protected boolean isUnitMillimeter()
  {
    return Image.UNIT_MILLIMETER.equals( this.getParameterValue( CONFIG_UNIT ).toLowerCase() );
  }

  protected boolean isUnitPixel()
  {
    return Image.UNIT_PIXEL.equals( this.getParameterValue( CONFIG_UNIT ).toLowerCase() );
  }

  protected void assertParameterAvailable( String name )
  {
    if ( !this.hasParameter( name ) )
    {
      throw new IllegalArgumentException( "'" + name + "' parameter is missing" );
    }
  }

  protected void assertParameterPredicate( String message, boolean predicate )
  {
    if ( !predicate )
    {
      throw new IllegalArgumentException( message );
    }
  }

  protected int parsePositiveInteger( String parameterValue )
  {
    int dim = Integer.parseInt( parameterValue );
    this.assertParameterPredicate( "positive integer expected", dim > 0 );
    return dim;
  }

  protected void assertAcceptedUnit()
  {
    this.assertParameterAvailable( CONFIG_UNIT );
    this.assertParameterPredicate( "Unit must be '" + Image.UNIT_MILLIMETER + "' or '" + Image.UNIT_PIXEL + "'", this.isUnitMillimeter() || this.isUnitPixel() );
  }

  protected void logCheck()
  {
    this.logAction( "check" );
  }

  protected void logConvert()
  {
    this.logAction( "convert" );
  }

  private void logAction( String action )
  {
    LOG.debug( action + ": " + this.getRuleName() );
    String msg = this.getParameterValue( MESSAGE_PARAMETER );
    if ( msg != UNDEFINED_PARAM ) // this reference comparision is intended!
    {
      LOG.info( msg );
    }
  }

}
