package ch.ige.imagehandler.rule;

import java.awt.Dimension;

import ch.ige.image.Image;
import ch.ige.image.MetadataException;

/**
 * Checks the dimension of an image.
 * <p>
 * <em>Parameters</em> <table border="1">
 * <tr>
 * <td>Name</td>
 * <td>Value</td>
 * </tr>
 * <tr>
 * <td>maxWidth (required)</td>
 * <td>allowed width, positive integer</td>
 * </tr>
 * <tr>
 * <td>maxHeight (required)</td>
 * <td>allowed height, positive integer</td>
 * </tr>
 * <tr>
 * <td>unit (required)</td>
 * <td>"mm" or "px"</td>
 * </tr>
 * </table>
 * <p>
 * <em>Example</em>
 * <pre>
 *     &lt;rule class=&quot;ch.ige.imagehandler.rule.DimensionChecker&quot;&gt;
 *      &lt;param name=&quot;maxWidth&quot;&gt;800&lt;/param&gt;
 *      &lt;param name=&quot;maxHeight&quot;&gt;800&lt;/param&gt;
 *      &lt;param name=&quot;unit&quot;&gt;mm&lt;/param&gt;
 *     &lt;/rule&gt;
 * </pre>
 * 
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @see ch.ige.image.Metadata#getSize(String)
 * @author Michael Locher
 * @version $Id: DimensionChecker.java,v 1.12 2009/04/14 11:38:02 locher Exp $
 */
public class DimensionChecker<I extends Image> extends AbstractRule<I> implements Checker<I>
{

  /* @see ch.ige.imagehandler.rule.AbstractRule#getRuleName() */
  @Override
  protected String getRuleName()
  {
    return "DimensionChecker";
  }

  /*
   * @see ch.ige.imagehandler.rule.Rule#checkParameters()
   */
  @Override
  public void checkParameters() throws RuleException
  {
    this.assertParameterAvailable( CONFIG_MAX_WIDTH );
    this.assertParameterAvailable( CONFIG_MAX_HEIGHT );
    this.assertAcceptedUnit();
    this.getMaxWidth();
    this.getMaxHeight();
  }

  private int getMaxHeight()
  {
    return this.parsePositiveInteger( this.getParameterValue( CONFIG_MAX_HEIGHT ) );
  }

  private int getMaxWidth()
  {
    return this.parsePositiveInteger( this.getParameterValue( CONFIG_MAX_WIDTH ) );
  }

  // --------------------------------------------------------------------------------------------
  // Check

  /* @see ch.ige.imagehandler.rule.Checker#checkImage(ch.ige.image.Image) */
  public void checkImage( I image ) throws RuleException
  {
    this.logCheck();
    Dimension size = null;
    try
    {
      if ( this.isUnitMillimeter() )
      {
        size = image.getMetadata().getSize( Image.UNIT_MILLIMETER );
      }
      if ( this.isUnitPixel() )
      {
        size = image.getMetadata().getSize( Image.UNIT_PIXEL );
      }
    }
    catch ( MetadataException e )
    {
      throw new RuleException( "could not determine size of image" );
    }
    if ( size == null )
    {
      throw new RuleException( "size of the image could not be determined" );
    }
    if ( size.width > this.getMaxWidth() )
    {
      throw new RuleException( "image width too big" );
    }
    if ( size.height > this.getMaxHeight() )
    {
      throw new RuleException( "image height too big" );
    }
  }

}
