package ch.ige.imagehandler.rule;

import java.util.ArrayList;
import java.util.List;

import ch.ige.image.Image;
import ch.ige.image.MetadataException;
import ch.ige.image.format.Format;

/**
 * Checks the encoding of an image.
 * <p>
 * <em>Parameters</em> <table border="1">
 * <tr>
 * <td>Name</td>
 * <td>Value</td>
 * </tr>
 * <tr>
 * <td>allowedFormats (required)</td>
 * <td>list of accepted formats<br>
 * jpeg,png,tif,gif,bmp</td>
 * </tr>
 * <tr>
 * <td>allowedCompressions (optional)</td>
 * <td>codec specific compression names</td>
 * </tr>
 * </table>
 * <p>
 * <em>Example</em>
 * <pre>
 *     &lt;rule class=&quot;ch.ige.imagehandler.rule.FormatChecker&quot;&gt;
 *       &lt;paramlist name=&quot;allowedFormats&quot;&gt;
 *         &lt;element&gt;PNG&lt;/element&gt;
 *         &lt;element&gt;GIF&lt;/element&gt;
 *       &lt;/paramlist&gt;
 *       &lt;paramlist name=&quot;allowedCompressions&quot;&gt;
 *         &lt;element&gt;deflate&lt;/element&gt;
 *       &lt;/paramlist&gt;
 *     &lt;/rule&gt;
 * </pre>
 * 
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @see ch.ige.image.Metadata#getFormat()
 * @author Michael Locher
 * @version $Id: FormatChecker.java,v 1.11 2009/04/14 11:38:04 locher Exp $
 */
public class FormatChecker<I extends Image> extends AbstractRule<I> implements Checker<I>
{

  private static final String CONFIG_ALLOWED_COMPRESSIONS = "allowedCompressions";

  private static final String CONFIG_ALLOWED_FORMATS = "allowedFormats";

  /* @see ch.ige.imagehandler.rule.AbstractRule#getRuleName() */
  @Override
  protected String getRuleName()
  {
    return "Format Checker";
  }

  /* @see ch.ige.imagehandler.rule.Rule#checkParameters() */
  @Override
  public void checkParameters() throws RuleException
  {
    this.assertParameterPredicate( CONFIG_ALLOWED_FORMATS + " parameter invalid", !this.getAllowedFormats().isEmpty() );
  }

  private List<Format> getAllowedFormats()
  {
    List<Format> result = new ArrayList<Format>();
    for ( String string: this.getParameterList( CONFIG_ALLOWED_FORMATS ) )
    {
      result.add( Format.fromTypeString( string ) );
    }
    return result;
  }

  private List<String> getAllowedCompressions()
  {
    // TODO check the allowed codecs for possible compressions
    return this.getParameterList( CONFIG_ALLOWED_COMPRESSIONS );
  }

  // --------------------------------------------------------------------------------------------
  // Check

  /* @see ch.ige.imagehandler.rule.Checker#checkImage(ch.ige.image.Image) */
  public void checkImage( Image image ) throws RuleException
  {
    this.logCheck();
    Format format = image.getMetadata().getFormat();
    this.checkFormat( format );
    this.checkCompression( format );
  }

  private void checkFormat( Format format ) throws RuleException
  {
    boolean formatINVALID = true;
    for ( Format allowed: this.getAllowedFormats() )
    {
      if ( allowed.isSameKind( format ) )
      {
        formatINVALID = false;
        break;
      }
    }
    if ( formatINVALID )
    {
      throw new RuleException( "format invalid: " + format );
    }
  }

  private void checkCompression( Format format ) throws RuleException
  {
    List<String> allowedCompressions = this.getAllowedCompressions();
    if ( !allowedCompressions.isEmpty() )
    {
      String compressionType;
      try
      {
        compressionType = format.getCompressionType();
      }
      catch ( MetadataException e )
      {
        throw new RuleException( "compression unknown" );
      }

      boolean compressionINVALID = true;

      for ( String allowed: allowedCompressions )
      {
        if ( allowed.equals( compressionType ) )
        {
          compressionINVALID = false;
          break;
        }
      }
      if ( compressionINVALID )
      {
        throw new RuleException( "compression invalid: " + compressionType );
      }
    }
  }

}
