package ch.ige.imagehandler.rule;

import java.io.IOException;

import ch.ige.image.EncodedImage;
import ch.ige.image.Image;
import ch.ige.image.ImageException;
import ch.ige.image.format.Format;
import ch.ige.image.format.JPEGFormat;
import ch.ige.image.format.TIFFFormat;

/**
 * Encodes the image with the specified codec.
 * <p>
 * <em>Parameters</em> <table border="1">
 * <tr>
 * <td>Name</td>
 * <td>Value</td>
 * </tr>
 * <tr>
 * <td>format (required)</td>
 * <td>name of the codec</td>
 * </tr>
 * <tr>
 * <td>enforceColorspace (required)</td>
 * <td>"true" or "false", if true the colors are converted to sRGB or standard grayscale</td>
 * </tr>
 * <tr>
 * <td>compression (required for TIFF)</td>
 * <td>name of the compression scheme</td>
 * </tr>
 * <tr>
 * <td>quality (optional)</td>
 * <td>JPEG quality, float between 0.0 and 1.0</td>
 * </tr>
 * <tr>
 * <td>adjustQualityForFilesize (optional)</td>
 * <td>JPEG target filesize in kB</td>
 * </tr>
 * <tr>
 * <td>sign (optional)</td>
 * <td><code>true</code>(default) or <code>false</code></td>
 * </tr>
 * </table>
 * <p>
 * <em>Examples</em>
 * <pre>
 *    &lt;rule class=&quot;ch.ige.imagehandler.rule.FormatConverter&quot;&gt;
 *      &lt;param name=&quot;format&quot;&gt;JPEG&lt;/param&gt;
 *      &lt;param name=&quot;forceRGB&quot;&gt;false&lt;/param&gt;
 *      &lt;param name=&quot;adjustQualityForFilesize&quot;&gt;32&lt;/param&gt;
 *    &lt;/rule&gt;
 * </pre>
 * <pre>
 *    &lt;rule class=&quot;ch.ige.imagehandler.rule.FormatConverter&quot;&gt;
 *      &lt;param name=&quot;format&quot;&gt;TIFF&lt;/param&gt;
 *      &lt;param name=&quot;forceRGB&quot;&gt;false&lt;/param&gt;
 *      &lt;param name=&quot;compression&quot;&gt;LZW&lt;/param&gt;
 *    &lt;/rule&gt;
 * </pre>
 * <pre>
 *    &lt;rule class=&quot;ch.ige.imagehandler.rule.FormatConverter&quot;&gt;
 *      &lt;param name=&quot;format&quot;&gt;PNG&lt;/param&gt;
 *      &lt;param name=&quot;forceRGB&quot;&gt;false&lt;/param&gt;
 *    &lt;/rule&gt;
 * </pre>
 * 
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @see ch.ige.image.format.Format#encode(Image, boolean)
 * @author Michael Locher
 * @version $Id: FormatConverter.java,v 1.17 2009/04/14 11:38:03 locher Exp $
 */
public class FormatConverter<I extends Image> extends AbstractRule<I> implements Converter<I, EncodedImage>
{
  private static final Converter<Image, Image> STANDARDIZE_COLORSPACES = new StandardizeColorspaceConverter<Image>();

  private static final String CONFIG_FORMATNAME = "format";

  private static final String CONFIG_ENFORCE_COLORSPACE = "enforceColorspace";

  private static final String CONFIG_COMPRESSION = "compression";

  private static final String CONFIG_JPEG_ADJUST_QUALITY = "adjustQualityForFilesize";

  private static final String CONFIG_JPEG_QUALITY = "quality";

  private static final String CONFIG_SIGN = "sign";

  /* @see ch.ige.imagehandler.rule.AbstractRule#getRuleName() */
  @Override
  protected String getRuleName()
  {
    return "Format Converter";
  }

  /* @see ch.ige.imagehandler.rule.Rule#checkParameters() */
  @Override
  public void checkParameters() throws RuleException
  {
    this.assertParameterAvailable( CONFIG_FORMATNAME );
    this.assertParameterAvailable( CONFIG_ENFORCE_COLORSPACE );
    this.getFormat();
    this.enforceColorspace();
    this.sign();
  }

  private boolean enforceColorspace()
  {
    return Boolean.valueOf( this.getParameterValue( CONFIG_ENFORCE_COLORSPACE ) ).booleanValue();
  }

  private Format getFormat()
  {
    Format format = Format.fromTypeString( this.getParameterValue( CONFIG_FORMATNAME ) );
    if ( this.hasParameter( CONFIG_COMPRESSION ) )
    {
      format.setCompressionType( this.getParameterValue( CONFIG_COMPRESSION ) );
    }

    if ( format.isKindOf( TIFFFormat.IDENTIFIER ) )
    {
      this.assertParameterAvailable( CONFIG_COMPRESSION );
    }

    if ( format.isKindOf( JPEGFormat.IDENTIFIER ) )
    {
      extractJPEGParameters( (JPEGFormat) format );
    }

    return format;
  }

  private void extractJPEGParameters( JPEGFormat jpg )
  {
    boolean qualitySet = false, targetSizeSet = false;

    if ( this.hasParameter( CONFIG_JPEG_ADJUST_QUALITY ) )
    {
      jpg.setTargetSize( this.parsePositiveInteger( this.getParameterValue( CONFIG_JPEG_ADJUST_QUALITY ) ) );
      targetSizeSet = true;
    }
    if ( this.hasParameter( CONFIG_JPEG_QUALITY ) )
    {
      jpg.setQuality( Float.parseFloat( this.getParameterValue( CONFIG_JPEG_QUALITY ) ) );
      qualitySet = true;
    }
    if ( qualitySet && targetSizeSet )
    {
      throw new IllegalArgumentException( "dont give both '" + CONFIG_JPEG_ADJUST_QUALITY + "' and '" + CONFIG_JPEG_QUALITY + "'" );
    }
    this.assertParameterPredicate( "either give '" + CONFIG_JPEG_ADJUST_QUALITY + "' or '" + CONFIG_JPEG_QUALITY + "'", qualitySet || targetSizeSet );
  }

  private boolean sign()
  {
    if ( this.hasParameter( CONFIG_SIGN ) )
    {
      return Boolean.valueOf( this.getParameterValue( CONFIG_SIGN ) ).booleanValue();
    }
    return true;
  }

  // --------------------------------------------------------------------------------------------
  // Convert

  /* @see ch.ige.imagehandler.rule.Converter#convert(ch.ige.image.Image) */
  public EncodedImage convert( I source ) throws RuleException, ImageException
  {
    this.logConvert();
    try
    {
      Image in = this.enforceColorspace() ? STANDARDIZE_COLORSPACES.convert( source ) : source;
      return this.getFormat().encode( in, this.sign() );
    }
    catch ( IOException e )
    {
      throw new RuleException( e.toString() );
    }
  }

}
