package ch.ige.imagehandler.test;

import java.net.URL;
import java.nio.ByteBuffer;

import junit.framework.TestCase;
import ch.ige.image.EncodedImage;
import ch.ige.image.EncodedImageWrapper;
import ch.ige.image.ImageException;
import ch.ige.image.Metadata;
import ch.ige.image.MetadataException;
import ch.ige.image.test.ImageFactoryTest;
import ch.ige.imagehandler.ImageHandler;
import ch.ige.imagehandler.ImageHandlerFactory;
import ch.ige.imagehandler.rule.RuleException;

/**
 * Tests IGE Signature.
 *
 * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
 * @author   galli
 * @version  $Id: ImageHandlerMetadataTest.java,v 1.11 2009/04/15 07:23:36 locher Exp $
 */
public class ImageHandlerMetadataTest extends TestCase
{
  private ImageHandler imageHandler = null;

  public ImageHandlerMetadataTest( String name )
  {
    super( name );
  }

  @Override
  protected void setUp() throws Exception
  {
    super.setUp();
    imageHandler = ImageHandlerFactory.createLocalInstance( getProfile() );
  }

  private URL getProfile()
  {
    return ImageHandlerTest.class.getResource( "test_profile.xml" );
  }

  @Override
  protected void tearDown() throws Exception
  {
    imageHandler = null;
    super.tearDown();
  }

  private EncodedImage loadImage( String name ) throws Exception
  {
    assertNotNull( ImageFactoryTest.class.getResource( name ) );
    return this.imageHandler.getImageFactory().loadURL( ImageFactoryTest.class.getResource( name ) );
  }

  private EncodedImage testConvert( EncodedImage img, String rule ) throws RuleException, ImageException
  {
    EncodedImage result = (EncodedImage) this.imageHandler.convert( img, rule );
    Metadata meta = result.getMetadata();
    String mac = meta.getMAC();
    String orig = meta.getOrigMetadata();
    String conversionRule = meta.getConversionRuleName();
    assertNotNull( mac );
    assertNotNull( orig );
    assertNotNull( conversionRule );
    assertTrue( !"".equals( mac ) );
    assertTrue( !"".equals( orig ) );
    assertEquals( rule, conversionRule );
    return result;
  }

  public void testWritePNGMetadata() throws Exception
  {
    testConvert( this.loadImage( "circle.png" ), "metadataWrite" );
  }

  public void testWriteGIFMetadata() throws Exception
  {
    testConvert( this.loadImage( "circle.gif" ), "metadataWrite" );
  }

  public void testWriteTIFFMetadata() throws Exception
  {
    testConvert( this.loadImage( "a6_300.tiff" ), "metadataWrite" );
  }

  public void testWriteJPEGMetadata() throws Exception
  {
    testConvert( this.loadImage( "circle.jpg" ), "metadataWrite" );
  }

  public void testWriteTIFFWithoutMetadata() throws Exception
  {
    testConvert( this.loadImage( "a6_300.tiff" ), "writeWithoutMetadata" );
  }

  public void testWriteJPEGWithoutMetadata() throws Exception
  {
    testConvert( this.loadImage( "circle.jpg" ), "writeWithoutMetadata" );
  }

  //---------------------------------------------

  private void assertValidSignature( EncodedImage img ) throws MetadataException, RuleException
  {
    ByteBuffer pre = img.getDataBuffer();
    ByteBuffer defensiveCopy = ByteBuffer.allocate( pre.limit() );
    defensiveCopy.put( pre );
    defensiveCopy.rewind();

    this.imageHandler.checkWithFeedback( img, "signatureTest" );
    assertEquals( defensiveCopy, img.getDataBuffer() );
  }

  public void testPNGRoudtrip() throws Exception
  {
    EncodedImage converted = testConvert( this.loadImage( "circle.png" ), "metadataWrite" );
    assertValidSignature( converted );
  }

  public void testReadPNGMetadata() throws Exception
  {
    assertValidSignature( this.loadImage( "circle_IGEMetadata.png" ) );
  }

  public void testReadPNGMetadataReadonlyBuffer() throws Exception
  {
    EncodedImage img = this.loadImage( "circle_IGEMetadata.png" );
    EncodedImage readOnly = new EncodedImageWrapper( img.getDataBuffer().asReadOnlyBuffer(), img.getWritableMetadata() );
    assertValidSignature( readOnly );
  }

  public void testReadGIFMetadata() throws Exception
  {
    assertValidSignature( this.loadImage( "circle_IGEMetadata.gif" ) );
  }

  public void testReadTIFFMetadata() throws Exception
  {
    assertValidSignature( this.loadImage( "a6_300_IGEMetadata.tiff" ) );
  }

  public void testReadJPEGMetadata() throws Exception
  {
    assertValidSignature( this.loadImage( "circle_IGEMetadata.jpg" ) );
  }

  public void testReadJPEGMetadataReadonlyBuffer() throws Exception
  {
    EncodedImage img = this.loadImage( "circle_IGEMetadata.jpg" );
    EncodedImage readOnly = new EncodedImageWrapper( img.getDataBuffer().asReadOnlyBuffer(), img.getWritableMetadata() );
    assertValidSignature( readOnly );
  }

  public void testReadTooOldJPEGMetadata() throws Exception
  {
    assertEquals( false, this.imageHandler.check( this.loadImage( "circle_IGEMetadata.jpg" ), "signatureTestFail" ) );
  }

}
