package ch.ige.ipcloadscheme.client.file;

import java.io.File;
import java.util.Date;

import ch.ige.ipcloadscheme.client.SchemeProperties;

/**
 * A file which can be processed and thus moved to a directory that shows that it's processed.
 *
 * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
 * @author   lewis_lee
 * @version  $Id: ProcessableFile.java,v 1.1 2008/07/01 06:27:02 lewis_lee Exp $
 */
public class ProcessableFile extends File
{
  /**
   * Shows the date of the file's data.
   */
  private Date processingDate = null;

  /**
   * Files are moved to this subdirectory after processing.
   */
  private static final String DONE_SUBDIRECTORY = "done\\";

  /**
   * Creates a new <code>File</code> instance by converting the given pathname string into an abstract pathname.
   * If the given string is the empty string, then the result is the empty abstract pathname.
   * @param pathname A pathname string
   */
  public ProcessableFile( String pathname )
  {
    this( pathname, (Date) null );
  }

  /**
   * Creates a new <code>File</code> instance by converting the given pathname string into an abstract pathname.
   * If the given string is the empty string, then the result is the empty abstract pathname.
   * @param pathname A pathname string
   * @param processingDate Shows the date of the file's data
   */
  public ProcessableFile( String pathname, Date processingDate )
  {
    super( pathname );
    initialize( processingDate );
  }

  /**
   * Creates a new <code>File</code> instance from a parent abstract pathname and a child pathname string.
   * @param parent The parent abstract pathname
   * @param child The child pathname string
   */
  public ProcessableFile( String parent, String child )
  {
    this( parent, child, null );
  }

  /**
   * Creates a new <code>File</code> instance from a parent abstract pathname and a child pathname string.
   * @param parent The parent abstract pathname
   * @param child The child pathname string
   * @param processingDate Shows the date of the file's data
   */
  public ProcessableFile( String parent, String child, Date processingDate )
  {
    super( parent, child );
    initialize( processingDate );
  }

  /**
   * Initializes this.
   * @param theProcessingDate Shows the date of the file's data
   */
  private void initialize( Date theProcessingDate )
  {
    this.processingDate = theProcessingDate;
    this.getParentFile().mkdirs();
  }

  /**
    * Moves the file to a <code>done</code> subdirectory after processing.
    * @return <code>true</code> if the move was ok, otherwise <code>false</code>
    */
  public boolean moveToDoneDirectory()
  {
    File doneDirectory = new File( this.getParent(), DONE_SUBDIRECTORY
        + ( this.processingDate != null ? SchemeProperties.getFormattedProcessingDate( this.processingDate ) : "" ) );
    doneDirectory.mkdirs();

    File doneFile = new File( doneDirectory.getPath(), this.getName() );
    if ( doneFile.exists() )
    {
      doneFile.delete();
    }

    return this.renameTo( doneFile );
  }
}
