package ch.ige.ipcloadvalidity.client.batch;

import java.util.Map;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.apache.log4j.Logger;

import ch.ige.ipcloadvalidity.BuildInfo;
import ch.ige.ipcloadvalidity.client.ValidityFile;
import ch.ige.ipcloadvalidity.client.ValidityZipFile;
import ch.ige.ipcloadvalidity.client.download.Download;
import ch.ige.ipcloadvalidity.client.process.ReadFile;
import ch.ige.ipcloadvalidity.util.ValidityProperties;
import ch.ige.util.client.ClientLogInitalizer;
import ch.ige.util.client.batch.BatchException;
import ch.ige.util.client.batch.BatchReport;
import ch.ige.util.client.batch.BatchReportMail;
import ch.ige.util.client.batch.BatchReturnCode;
import ch.ige.util.client.batch.NewAbstractBatch;
import ch.ige.util.client.batch.OptionWithDefault;

/**
 * Startet Verarbeitung zum Laden der IPC-Symbole in die Bagis-Datenbank
 * Die IPC-Symbole werden in einem XML-File (Validity File) vom OMPI geliefert
 * Ein neues Validity File kann höchstens alle drei Monate geliefert werden
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version $Id: IpcLoadValidity.java,v 1.21 2008/07/01 06:32:18 kuenzli Exp $
 */
public class IpcLoadValidity extends NewAbstractBatch
{

  private static final Logger LOG = Logger.getLogger( "ch.ige.ipcloadvalidity.client.batch" );

  private static final String APPLICATION_NAME = "ipcloadvalidty";

  // Logging
  static
  {
    ClientLogInitalizer.initalizeLogging( ClientLogInitalizer.BATCH_CONFIG );
  }

  /**
   * The short option for the program mode.
   */
  private static final String PROGRAM_MODE = "m";

  /**
   * The long option for the program mode.
   */
  private static final String PROGRAM_MODE_LONG = "mode";

  /**
   * The short option for the file.
   */
  private static final String FILE_NAME = "f";

  /**
   * The long option for the file.
   */
  private static final String FILE_NAME_LONG = "file";

  /**
   * The mode of the program.
   */
  private ProgramMode mod = ProgramMode.ALL;

  /**
   * Used when the batch run can't be initialized.
   */
  private static final BatchReturnCode INITIALIZATION_FAILED = new BatchReturnCode( 3, "Fehler bei der Initialisierung des Batch Job: {0} / Properties: {1}" );

  /**
   * Name of the file to download.
   */
  private String filename = null;

  /**
   * Process Statistics.
   */
  private String statistik = null;

  /**
   * Possible values for <code>PROGRAM_MODE</code>.
   */
  public static enum ProgramMode
  {
    /**
     * Download the file.
     */
    DOWNLOAD,

    /**
     * Process a downloaded file.
     */
    PROCESS,

    /**
     * Download and process a file.
     */
    ALL
  }

  /**
   * The CLI options to use.
   */
  protected static final Options OPTIONS;
  /**
   * Initialize program command line options.
   */
  static
  {
    // define and set options
    OptionWithDefault programModeOption = new OptionWithDefault( PROGRAM_MODE, PROGRAM_MODE_LONG, true, "program mode" );
    programModeOption.setType( ProgramMode.class );
    programModeOption.setRequired( false );
    programModeOption.setDefaultValue( ProgramMode.ALL );

    Option fileNameOption = new Option( FILE_NAME, FILE_NAME_LONG, true, "file to download" );
    fileNameOption.setType( String.class );
    fileNameOption.setRequired( true );

    OPTIONS = new Options();
    OPTIONS.addOption( programModeOption );
    OPTIONS.addOption( fileNameOption );

  }

  /**
   * Creates this.
   */
  public IpcLoadValidity()
  {
    super( APPLICATION_NAME, OPTIONS );
  }

  /**
   * Creates the batch run <strong>for tests</strong>.
   * @param applicationName The application name of this batch, used in logs etc.
   * @param options The command-line options or <code>null</code> if the command-line is ignored
   * @param endWithSystemExit If this is <code>true</code>, the batch run ends with a <code>System.exit()</code>,
   *        otherwise it just returns.
   */
  public IpcLoadValidity( String applicationName, Options options, boolean endWithSystemExit )
  {
    super( applicationName, options, endWithSystemExit );
  }

  /**
   * Liest die Programm Parameter ein und setzt (falls ein Wert vorhanden) die
   * entsprechenden Attribute.
   *
   * @param  args             programm arguments
   * @throws BatchException   in case of wrong arguments
   */
  private void loadParams( Map<String, Object> parsedCommandLine ) throws BatchException
  {
    LOG.info( "loadParams - begin" );
    this.filename = (String) parsedCommandLine.get( FILE_NAME );
    this.mod = (ProgramMode) parsedCommandLine.get( PROGRAM_MODE );
    LOG.info( "loadParams - done" );
  }

  private void startAll() throws BatchException
  {
    LOG.info( "start all" );
    ValidityZipFile zipfile = startDownload();
    startProcess( zipfile );
  }

  private ValidityZipFile startDownload()
  {
    LOG.info( "start download" );
    ValidityZipFile zipfile = new ValidityZipFile( this.filename );
    if ( zipfile.exists() )
    {
      zipfile.delete();
    }
    try
    {
      new Download().download( zipfile );
    }
    catch ( Exception e )
    {
      LOG.error( "startProcess(): Error downloading ZIP file. " + zipfile.getName() );
      throw new BatchException( BatchReturnCode.UNEXPECTED_ERROR, e );
    }

    LOG.info( "finsished download to " + zipfile.getPath() );
    return zipfile;
  }

  /**
   * @throws BatchException
   */
  public void startProcess() throws BatchException
  {
    ValidityZipFile zipfile = new ValidityZipFile( this.filename );
    if ( !zipfile.exists() )
    {
      throw new BatchException( BatchReturnCode.RESOURCE_NOT_AVAILABLE.formatMessage( " Zipfile: " + zipfile.getName() + " does not exist " ) );
    }
    startProcess( zipfile );
  }

  public void startProcess( ValidityZipFile zipFile )
  {
    LOG.info( "start process" );

    String unzipDir = "";
    try
    {
      unzipDir = zipFile.unzip();
    }
    catch ( Exception e )
    {
      LOG.error( "startProcess(): Error unzipping file. " + zipFile.getPath() + " unzip to " + unzipDir );
      throw new BatchException( BatchReturnCode.UNEXPECTED_ERROR, e );
    }
    LOG.info( zipFile.getPath() + " unzip to " + unzipDir );

    ValidityFile directory = new ValidityFile( unzipDir );
    ReadFile readFile;
    try
    {
      readFile = new ReadFile();
    }
    catch ( Exception e )
    {
      LOG.error( "startProcess(): Error reading validity file. " );
      throw new BatchException( BatchReturnCode.UNEXPECTED_ERROR, e );
    }

    if ( directory.exists() && directory.isDirectory() )
    {
      // alle Files in diesem Verzeichnis einlesen die dem Format ipcr_validity_yyyymmdd_core_yyyy.xml entsprechen
      ValidityFile[] files = directory.listValidityFiles();

      for ( int i = 0; i < files.length; i++ )
      {
        readFile.process( files[i] );
        this.statistik = readFile.getStatistik();
      }

      if ( readFile.isErrorOccured() )
      {
        throw new BatchException( BatchReturnCode.UNEXPECTED_ERROR.formatMessage( readFile.getLogMessage() ) );
      }
    }
    else
    {
      throw new BatchException( BatchReturnCode.RESOURCE_NOT_AVAILABLE.formatMessage( " Directory: " + ValidityProperties.getInstance().getProperty( ValidityProperties.UNZIP_DIRECTORY )
          + " not found" ) );
    }

    zipFile.moveToDoneDirectory();
    zipFile.deleteUnzippedFiles();
  }

  // @see ch.ige.util.client.batch.NewAbstractBatch#getBatchReports()
  @Override
  public BatchReport[] getBatchReports() throws BatchException
  {
    String smtpServer = ValidityProperties.getInstance().getProperty( ValidityProperties.MAIL_SMTP_HOST );
    String from = ValidityProperties.getInstance().getProperty( ValidityProperties.MAIL_SEND_FROM );
    String to = ValidityProperties.getInstance().getProperty( ValidityProperties.MAIL_SEND_TO );
    String cc = ValidityProperties.getInstance().getProperty( ValidityProperties.MAIL_SEND_CC );
    String subject = BuildInfo.BUILD_ENV_NAME + " - " + "Batch Job: " + APPLICATION_NAME;

    return new BatchReport[] { new BatchReportMail( smtpServer, from, to, cc, subject ) };
  }

  // @see ch.ige.util.client.batch.NewAbstractBatch#getPossibleReturnCodes()
  @Override
  protected BatchReturnCode[] getPossibleReturnCodes()
  {
    return BatchReturnCode.getBatchReturnCodes( IpcLoadValidity.class );
  }

  // @see ch.ige.util.client.batch.NewAbstractBatch#run(org.apache.commons.cli.CommandLine, java.util.Map)
  @Override
  @SuppressWarnings( "unused" )
  protected void run( CommandLine commandLine, Map<String, Object> parsedCommandLine ) throws BatchException
  {
    LOG.info( "### Load IPC Validity - main... ###" );

    this.loadParams( parsedCommandLine );

    switch( this.mod )
    {
      case ALL:
        this.startAll();
        break;
      case DOWNLOAD:
        this.startDownload();
        break;
      case PROCESS:
        this.startProcess();
        break;
    }

    LOG.info( "### Load IPC Validity - main successfully done ###" );
    LOG.info( "Configurations: " + ValidityProperties.getInstance().getProperties() );
    writeBatchReportLine( "Statistik:" );
    writeBatchReportLine( this.statistik );
  }

  public static void main( String[] args )
  {
    IpcLoadValidity ipcLoadValidity = new IpcLoadValidity();
    NewAbstractBatch.start( ipcLoadValidity, args );
  }

  @Override
  protected void checkCommandLine( CommandLine commandLine, Map<String, Object> parsedCommandLine ) throws IllegalArgumentException
  {
  }

  @Override
  protected String getBuildInfo()
  {
    return BuildInfo.BUILD_ID;
  }

  @Override
  protected String getCvsTag()
  {
    // TODO Auto-generated method stub
    return BuildInfo.CVS_TAG;
  }

}
