package ch.ige.jipps;

import java.sql.SQLException;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import ch.ige.dbutils.vo.AbstractVO;
import ch.ige.ipobjects.ServicesFactory;
import ch.ige.ipobjects.common.DetailTerminatedException;
import ch.ige.ipobjects.event.Event;
import ch.ige.ipobjects.key.vo.IPKeyVO;
import ch.ige.ipobjects.services.Services;
import ch.ige.ipobjects.time.vo.EreigIDat;
import ch.ige.ipobjects.trademark.Trademark;
import ch.ige.jipps.constants.StatusConstants;
import ch.ige.jipps.dao.BfaktPosDAO;
import ch.ige.jipps.dao.BfakturDAO;
import ch.ige.jipps.vo.AnswerVO;
import ch.ige.jipps.vo.AuszugVO;
import ch.ige.jipps.vo.BfakturVO;
import ch.ige.jipps.vo.PrintJobVO;
import ch.ige.util.constants.EventNumberConstants;
import ch.ige.util.constants.InvoiceConstants;
import ch.ige.util.constants.IpRegisterStateConstants;
import ch.ige.util.constants.PrintStatusConstants;
import ch.ige.util.enums.IPType;
import ch.ige.util.enums.printer.ReportPaper;
import ch.ige.util.enums.report.ReportType;

/**
 * Controls the flow of the registration confirmation (<code>Eintragungsbescheinigung</code> (report) and
 * <code>Eintragungsbestätigung</code> (letter) in German).
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Johannes Lang
 * @author Eric Lewis
 * @version $Id: EintrBeschControl.java,v 1.93 2009/02/17 13:51:26 lang Exp $
 */
public class EintrBeschControl extends AuszugControl
{
  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( EintrBeschControl.class );

  /**
   * The name for the registration confirmation reports.
   */
  private static final String REGISTRATION_CONFIRMATION_REPORT_NAME = "Eintragungsbescheinigungen";

  /**
   * The printjobname for the registration confirmation reports.
   */
  private static final String REGISTRATION_CONFIRMATION_REPORT_PRINTJOB_NAME = "EintrBesch";

  /**
   * The prefix for the configuration of the registration confirmation reports.
   */
  private static final String REGISTRATION_CONFIRMATION_REPORT_PREFIX = "eintrbesch";

  /**
   * The printjobname for the registration confirmation letters.
   */
  private static final String REGISTRATION_CONFIRMATION_LETTER_NAME = "Eintragungsbestätigungen";

  /**
   * The name for the registration confirmation letters.
   */
  private static final String REGISTRATION_CONFIRMATION_LETTER_PRINTJOB_NAME = "EintrBest";

  /**
   * The prefix for the configuration of the registration confirmation letters.
   */
  private static final String REGISTRATION_CONFIRMATION_LETTER_PREFIX = "eintrbest";

  private static final String NO_REG_EVENT_MESSAGE = "Kein Eintragungsereignis oder Publikationsdatum in Ereignis {0} für SchutitKey {1} gefunden; BfaktId: {2}";

  /**
   * The current date, per instance.
   */
  private final Date TODAY = new Date();

  /**
   * Shows how many registration confirmations were transformed to reports.
   */
  private int numberOfRegistrationConfirmationReports = 0;

  /**
   * The number of errors that occurred during processing of reports.
   */
  private int numberOfReportErrors = 0;

  /**
   * If this is <code>true</code>, the print jobs are sent to the print server and a
   * confirmation mail is sent after processing. When testing locally, use <code>false</code>.
   */
  private boolean printAndMailConfirmation = true;

  /**
   * converts a list of invoices to a {@link java.util.TreeMap}, therefore the order of the list is maintained in the map;
   * @param invoices
   * @return map ipkey is the key, the bfakturVO is the value
   */
  private Map<Integer, BfakturVO> convertListToMapByIPKey( List<BfakturVO> invoices )
  {
    if ( invoices.isEmpty() )
    {
      return Collections.emptyMap();
    }
    else
    {
      // this has to be a LinkedHashMap to maintain the insertion order an therefore the print-order.
      Map<Integer, BfakturVO> map = new LinkedHashMap<Integer, BfakturVO>();
      for ( BfakturVO invoice: invoices )
      {
        map.put( invoice.ipKey, invoice );
      }
      return map;
    }
  }

  /**
   * Prints all registration confirmations that are due <strong>for electronic publication, not paper publication</strong>.
   * It first gets the invoices that were created, but not yet printed, adds a registration confirmation for each and
   * prints the whole lot.
   * @param printAndMailConfirmation If this is <code>true</code>, the print jobs are sent to the print server and a
   * confirmation mail is sent after processing. When testing locally, use <code>false</code>.
   * @return {@link ch.ige.jipps.constants.StatusConstants#OK} if processing was correct, otherwise
   * {@link ch.ige.jipps.constants.StatusConstants#ERROR}
   */
  @SuppressWarnings( "hiding" )
  public int printEintrBesch( boolean printAndMailConfirmation )
  {
    this.printAndMailConfirmation = printAndMailConfirmation;
    int status = StatusConstants.OK;

    try
    {
      List<BfakturVO> invoices = new BfakturDAO().selectBFakturByForm( InvoiceConstants.FORM_ELECTRONIC_PUBLICATION_CONFIRMATION,
          PrintStatusConstants.READY_FOR_PRINTING_STRING );
      logInvoices( invoices, "registration confirmations" );

      JippsConfiguration configuration = getConfiguration( REGISTRATION_CONFIRMATION_REPORT_PRINTJOB_NAME, REGISTRATION_CONFIRMATION_REPORT_PREFIX );

      if ( invoices.isEmpty() )
      {
        // Nothing to print
        LOG.info( "There are no registration confirmations." );
        if ( this.printAndMailConfirmation )
        {
          processAnswers( REGISTRATION_CONFIRMATION_REPORT_NAME, false, this.answers, 0, configuration, 0, 0 );
        }
        status = StatusConstants.OK;
      }
      else
      {
        Map<Integer, BfakturVO> invoicesByIpKey = convertListToMapByIPKey( invoices );
        if ( invoicesByIpKey.size() != invoices.size() )
        {
          throw new IllegalStateException( "Mapping invoices to IP keys failed" );
        }
        invoices = null; // Use invoices by IP key!

        // Check that the invoice really triggered the correct event and that this event was published.
        List<Integer> registrationIpKeys = new BfaktPosDAO().getRegistrationIpKeys( LetterControl.getInvoiceIds( invoicesByIpKey.values() ) );
        Services ipObjectsServices = ServicesFactory.getInstance();
        for ( Iterator<Integer> iterator = registrationIpKeys.iterator(); iterator.hasNext(); )
        {
          Integer ipKey = iterator.next();

          if ( !invoicesByIpKey.containsKey( ipKey ) )
          {
            throw new IllegalStateException( "Mismatch between IP keys and invoices" );
          }

          Trademark trademark = ipObjectsServices.findTrademark( IPKeyVO.create( ipKey ) );
          trademark.applyRegisterTransformation( EreigIDat.max( TODAY ) );
          Event registrationEvent = null;
          for ( Event event: trademark.getEvents() )
          {
            if ( event.getArt_nr().equals( EventNumberConstants.REGISTRATION_TRADEMARK_CH_INTEGER.toString() ) )
            {
              registrationEvent = event;
              break;
            }
          }

          if ( registrationEvent == null || registrationEvent.getPubl_dat() == null )
          {
            BfakturVO invoice = invoicesByIpKey.remove( ipKey );
            iterator.remove();
            LOG.error( "Found no registration event or no publication date in event " + registrationEvent + " for trademark with IP key " + ipKey
                + ", ignoring invoice with ID " + invoice.bfaktId );
            // in this case, the corresponding ipKey has must also be removed to avoid printing of this report
            registrationIpKeys.remove( ipKey );
            this.numberOfReportErrors++;
            addAnswer( 0, REGISTRATION_CONFIRMATION_REPORT_PRINTJOB_NAME, ipKey.intValue(), MessageFormat.format( NO_REG_EVENT_MESSAGE, registrationEvent
                .getEreig_idat(), ipKey.toString(), Integer.toString( invoice.bfaktId ) ), "", "" );
          }
        }
        int numberOfMissingPublications = this.numberOfReportErrors;

        // The status has two possible outcomes:
        // OK     Means that everything before the call was ok (no missing publications) and that all reports were created correctly.
        // ERROR  Has two reasons:
        //        a) There was at least one missing publication before.
        //           Now, if there are *only* missing publications (which means that no invoices are left to process),
        //           it doesn't make sense to continue.
        //           But if there's a mix of missing publications and correct publications, where the reports were created
        //           correctly, then continue, because we want the letters for those correct cases.
        //        b) There was no missing publication before. In that case, the reporting went wrong, so stop processing.
        status = printRegistrationConfirmationReports( registrationIpKeys, configuration );
        if ( !invoicesByIpKey.isEmpty() )
        {
          if ( status == StatusConstants.OK || numberOfMissingPublications > 0 )
          {
            configuration = getConfiguration( REGISTRATION_CONFIRMATION_LETTER_PRINTJOB_NAME, REGISTRATION_CONFIRMATION_LETTER_PREFIX );
            status = printRegistrationConfirmationLetters( invoicesByIpKey.values(), configuration );
          }
        }
      }
    }
    catch ( SQLException e )
    {
      LOG.error( "Error while getting invoices for registration confirmations", e );
      status = StatusConstants.ERROR;
    }
    catch ( DetailTerminatedException e )
    {
      LOG.error( "Couldn't check trademarks for registration confirmations", e );
      status = StatusConstants.ERROR;
    }
    return status;
  }

  /**
   * Prints all registration confirmation reports corresponding to the given invoices.
   * @param ipKeys The keys of the IPs whose registration confirmation reports should be printed
   * @param configuration The configuration to use
   * @return {@link ch.ige.jipps.constants.StatusConstants#OK} if processing was correct, otherwise
   * {@link ch.ige.jipps.constants.StatusConstants#ERROR}
   */
  private int printRegistrationConfirmationReports( Collection<Integer> ipKeys, JippsConfiguration configuration )
  {
    LOG.info( "Starting processing of '" + REGISTRATION_CONFIRMATION_REPORT_NAME + "'" );

    int status = StatusConstants.OK;

    try
    {
      Integer batchId = Integer.valueOf( getNextBatchId() );

      // Create the reports
      List<PrintJobVO> printJobs = createPrintJobs( ipKeys, batchId, configuration.printerId );
      // Note: An answer may have been created before because the report and publication were not synchronized.
      // This is an awful hack...
      if ( !printJobs.isEmpty() || !this.answers.isEmpty() )
      {
        // This creates the answers
        makePrint( printJobs, configuration );
        if ( this.printAndMailConfirmation )
        {
          status = processAnswers( REGISTRATION_CONFIRMATION_REPORT_NAME, !printJobs.isEmpty(), this.answers, batchId, configuration,
              this.numberOfRegistrationConfirmationReports, this.numberOfReportErrors );
        }
      }
    }
    catch ( Exception ex )
    {
      LOG.error( "Couldn't print registration confirmation reports (" + REGISTRATION_CONFIRMATION_REPORT_NAME + ")", ex );
      status = StatusConstants.ERROR;
    }
    LOG.info( "Finished batch for '" + REGISTRATION_CONFIRMATION_REPORT_NAME + "' with status " + status );
    return status;
  }

  /**
   * Prints all registration confirmation letters corresponding to the given invoices.
   * @param invoices The invoices which are printed in a batch run
   * @param configuration The configuration to use
   * @return {@link ch.ige.jipps.constants.StatusConstants#OK} if processing was correct, otherwise
   * {@link ch.ige.jipps.constants.StatusConstants#ERROR}
   */
  private int printRegistrationConfirmationLetters( Collection<BfakturVO> invoices, JippsConfiguration configuration )
  {
    LOG.info( "Starting processing of '" + REGISTRATION_CONFIRMATION_LETTER_NAME + "'" );

    int status = StatusConstants.OK;

    try
    {
      Integer batchId = Integer.valueOf( getNextBatchId() );

      // The issue date on the top of the letter is the same as the ePublication date.
      for ( BfakturVO invoice: invoices )
      {
        invoice.printDate = invoice.publicationDate;
      }

      // Create the letters
      LetterBatchControl letterBatchControl = new LetterBatchControl();
      List<PrintJobVO> printJobs = letterBatchControl.createPrintJobs( invoices, batchId, false, configuration.printerId );
      // Don't let the LetterBatchControl send the batch job to JMS, because the configuration is wrong.
      List<AnswerVO> letterAnswers = letterBatchControl.makePrint( printJobs, false );
      int numberOfSuccesses = 0;
      int numberOfErrors = 0;
      for ( AnswerVO letterAnswer: letterAnswers )
      {
        this.answers.add( letterAnswer );
        if ( letterAnswer.printStatus.equals( AnswerVO.STATUS_OK ) )
        {
          numberOfSuccesses++;
        }
        else
        {
          numberOfErrors++;
        }
      }

      // Create the reports
      // Note: An answer may have been created before because the report and publication were not synchronized.
      // This is an awful hack...
      if ( !printJobs.isEmpty() || !this.answers.isEmpty() )
      {
        sendBatchJobToJms( printJobs.get( 0 ), configuration );

        if ( this.printAndMailConfirmation )
        {
          status = processAnswers( REGISTRATION_CONFIRMATION_LETTER_NAME, !printJobs.isEmpty(), this.answers, printJobs.get( 0 ).batchId, configuration,
              numberOfSuccesses, numberOfErrors );
        }
      }
    }
    catch ( Exception ex )
    {
      LOG.error( "Couldn't print registration confirmation letters (" + REGISTRATION_CONFIRMATION_LETTER_NAME + ")", ex );
      status = StatusConstants.ERROR;
    }
    LOG.info( "Finished batch for '" + REGISTRATION_CONFIRMATION_LETTER_NAME + "' with status " + status );
    return status;
  }

  /**
   * Creates the print jobs for the registration confirmations.
   * @param registeredIpKeys The IP keys for which a registration confirmation is printed (one for each)
   * @param batchId The batch ID to use
   * @param printerId The ID of the printer to print on
   * @return The print jobs to print
   */
  private List<PrintJobVO> createPrintJobs( Collection<Integer> registeredIpKeys, Integer batchId, int printerId )
  {
    List<PrintJobVO> printJobs = new ArrayList<PrintJobVO>( registeredIpKeys.size() );
    for ( Integer registeredIpKey: registeredIpKeys )
    {
      PrintJobVO printJobVO = new PrintJobVO();
      printJobVO.batchId = batchId;
      printJobVO.formTyp = PrintJobVO.AUSZUG;
      printJobVO.druckerId = printerId;
      printJobVO.regauszTyp = ReportType.REGISTRATION_CONFIRMATION.toDB();
      printJobVO.registerKz = IpRegisterStateConstants.REGISTRATION_STATE;
      printJobVO.reportId = registeredIpKey.intValue();
      printJobVO.paperMode = ReportPaper.PAPER_WITH_IPI_LOGO_INCLUDING_COPIES.toDB();
      printJobVO.anzKopien = 1;
      printJobVO.schutitTyp = IPType.TRADEMARK_CH.toDB();
      printJobVO.druckStatus = PrintStatusConstants.READY_FOR_PRINTING;
      printJobVO.senderUser = PrintJobVO.SYSTEM;
      printJobVO.senderApp = PrintJobVO.SYSTEM;
      printJobVO.standDatum = TODAY;
      printJobVO.sprache = PrintJobVO.USE_IP_LANGUAGE;

      printJobs.add( printJobVO );
    }

    return printJobs;
  }

  /**
   * Creates the reports and maybe sends them to the print server.
   * @param printJobs The print jobs to render into report PDFs
   * @return The corresponding answer for every print job
   * @see #makePrint(List)
   */
  @Override
  public List<AnswerVO> makePrint( List<PrintJobVO> printJobs )
  {
    // This is only done to be compatible with the superclass.
    LOG.warn( "The direct makePrint() method should not be called." );
    return makePrint( printJobs, getConfiguration( REGISTRATION_CONFIRMATION_REPORT_PRINTJOB_NAME, REGISTRATION_CONFIRMATION_REPORT_PREFIX ) );
  }

  /**
   * Creates the reports and maybe sends them to the print server.
   * @param printJobs The print jobs to render into report PDFs
   * @param configuration The configuration to use
   * @return The corresponding answer for every print job
   * @see #makePrint(List)
   */
  private List<AnswerVO> makePrint( List<PrintJobVO> printJobs, JippsConfiguration configuration )
  {
    LOG.debug( "Printing registration confirmations" );
    this.assembler = new AuszugAssembler();
    String status;
    PrintJobVO printJob = null;

    for ( Iterator<PrintJobVO> iter = printJobs.iterator(); iter.hasNext(); )
    {
      printJob = iter.next();

      status = AnswerVO.STATUS_OK;
      AuszugVO report = null;
      String formattedApplicationNumber = "";
      String formattedIpNumber = "";
      try
      {
        try
        {
          // This assumes that all print jobs have the same data source!
          IPKeyVO ipKey = IPKeyVO.create( printJob.reportId );
          ReportType reportType = ReportType.parse( printJob.regauszTyp );
          report = this.assembler.loadReport( ipKey, reportType, printJob.getBagisLanguage(), printJob.standDatum, printJob.maNr, printJob.printIgekopf );
          formattedApplicationNumber = report.getIpHeader().getFormattedApplicationNumber();
          formattedIpNumber = report.getIpHeader().getFormattedIpNumber();
        }
        catch ( Exception ex )
        {
          throw new Exception( "Couldn't load formatted application number", ex );
        }

        String reportXml = null;
        try
        {
          reportXml = this.writeXml.toXml( report );
        }
        catch ( Exception ex )
        {
          throw new Exception( "Couldn't create registration confirmation XML", ex );
        }
        LOG.debug( "Created registration confirmation XML" );

        try
        {
          String xslName = getXslName( printJob );
          printJob.printJobFile = createPdf( reportXml, xslName );
        }
        catch ( Exception ex )
        {
          throw new Exception( "Couldn't transform confirmation XML", ex );
        }

        int printJobId = getNextPrintJobId();
        LOG.debug( "Print job ID for report confirmation: " + printJobId );
        printJob.printJobId = printJobId;

        storePdf( printJob );
        this.numberOfRegistrationConfirmationReports++;

        LOG.debug( "Saved report confirmation print job" );
      }
      catch ( Exception ex )
      {
        LOG.error( ex.getMessage(), ex );
        status = ex.getMessage();
        this.numberOfReportErrors++;
      }
      finally
      {
        addAnswer( printJob.printJobId, printJob.formTyp, printJob.reportId, status, formattedApplicationNumber, formattedIpNumber );
      }
    }

    sendBatchJobToJms( printJob, configuration );

    return this.answers;
  }

  /**
   * Sends the batch job to JMS.
   * @param printJob One of the print jobs in the batch job
   * @param configuration The JIPPS configuration
   */
  private void sendBatchJobToJms( PrintJobVO printJob, JippsConfiguration configuration )
  {
    if ( this.printAndMailConfirmation && printJob != null && printJob.druckStatus == PrintStatusConstants.READY_FOR_PRINTING )
    {
      sendBatchJob( printJob.batchId, null, printJob.druckerId, true, configuration.printJobName, configuration.pdfFilePath );
    }
  }

  /**
   * Processes the answers from the printing of the print jobs.
   * @param reportName The name of the report
   * @param areTherePrintJobs <code>true</code> if there are print jobs, otherwise <code>false</code>
   * @param printJobAnswers The answers to each of the print jobs
   * @param batchId The ID of the batch run in which the print jobs were printed
   * @param configuration The configuration to use
   * @param numberOfSuccesses The number of registration confirmations that were created successfully
   * @param numberOfErrors The number of errors that occured during processing
   * @return {@link ch.ige.jipps.constants.StatusConstants#OK} if processing was correct, otherwise
   * {@link ch.ige.jipps.constants.StatusConstants#ERROR}
   */
  private int processAnswers( String reportName, boolean areTherePrintJobs, List<AnswerVO> printJobAnswers, int batchId, JippsConfiguration configuration,
      int numberOfSuccesses, int numberOfErrors )
  {
    int status = StatusConstants.OK;
    StringBuilder errorMessages = new StringBuilder();

    if ( printJobAnswers != null )
    {
      for ( AnswerVO answer: printJobAnswers )
      {
        if ( !answer.printStatus.equals( AnswerVO.STATUS_OK ) )
        {
          if ( answer.listPrintId.get( 0 ) == 0 )
          {
            status = StatusConstants.ERROR;
            errorMessages.append( answer.printStatus );
            errorMessages.append( '\n' );
            LOG.error( "Couldn't create '" + reportName + "':" + answer.printStatus );
          }
          else
          {
            errorMessages.append( "Schutztitel Key: " );
            errorMessages.append( answer.schutitKey );
            errorMessages.append( " / GesuchNr: " );
            errorMessages.append( answer.gesuchNr );
            errorMessages.append( '\n' );
          }
        }
      }
    }

    try
    {
      JIPPSPropertiesServices jippsProperties = JIPPSPropertiesFactory.getInstance();
      if ( areTherePrintJobs )
      {
        jippsProperties.doMail( batchId, reportName, null, errorMessages.toString(), configuration.propertyPrefix, numberOfSuccesses, numberOfErrors,
            configuration.printerId, configuration.pdfFilePath );
        LOG.debug( " *** Sent confirmation mail for '" + reportName + "'." );
      }
      else
      {
        jippsProperties.doMailNoData( reportName, null, errorMessages.toString(), configuration.propertyPrefix, numberOfErrors );
        LOG.debug( " *** There is no data for '" + reportName + "', sent confirmation mail." );
      }
    }
    catch ( Exception ex )
    {
      LOG.error( "Couldn't prepare answers for registration confirmations", ex );
      status = StatusConstants.ERROR;
    }
    return status;
  }

  /**
   * Gets the configuration for a certain type of print job.
   * @param printJobName The name of the print job
   * @param propertyPrefix The prefix of the property keys for the print job
   * @return The configuration for a certain type of print job
   */
  private JippsConfiguration getConfiguration( String printJobName, String propertyPrefix )
  {
    try
    {
      JIPPSPropertiesServices jippsProperties = JIPPSPropertiesFactory.getInstance();

      JippsConfiguration configuration = new JippsConfiguration();
      configuration.printJobName = printJobName;
      configuration.propertyPrefix = propertyPrefix;
      configuration.printerId = jippsProperties.getDrucker( propertyPrefix );
      configuration.pdfFilePath = jippsProperties.getDirectoryPath( propertyPrefix );

      LOG.debug( "Configuration: " + configuration );
      return configuration;
    }
    catch ( Exception ex )
    {
      throw new IllegalStateException( "Getting JIPPS properties (printer, PDF file path etc.) failed", ex );
    }
  }

  /**
   * Contains the configuration for a batch run.
   */
  private static class JippsConfiguration extends AbstractVO
  {
    /**
     * The name of the print job to print.
     */
    private String printJobName = "";

    /**
     * The prefix of all the keys for this configuration.
     */
    private String propertyPrefix = "";

    /**
     * The ID of the printer to print on
     */
    private int printerId = 0;

    /**
     * The path to the directory where PDF files are saved
     */
    private String pdfFilePath = "";
  }
}
