package ch.ige.jipps;

import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import ch.ige.dbutils.dao.AbstractDAO;
import ch.ige.dbutils.dao.DAO;
import ch.ige.jipps.configuration.PathConfiguration;
import ch.ige.jipps.dao.ReferenceXMLDAO;
import ch.ige.jipps.vo.PrintJobVO;
import ch.ige.jipps.vo.ReferenceXMLVO;
import ch.ige.util.constants.IpRegisterStateConstants;
import ch.ige.util.date.DateConverter;
import ch.ige.util.date.DateFormatter;
import ch.ige.util.enums.BAGISLanguageDomain;
import ch.ige.util.enums.IPType;
import ch.ige.util.enums.report.ReportType;

/**
 * Fills the table <code>reference_xml</code> with predefined reports; this is only used to import test data
 * for all tests extending {@link ch.ige.jipps.test.AbstractTestFacadeXml}.
 * 
 * <p>To update all test cases in the database, do first <code>delete from referenz_xml where ref_xml_id > 500</code>, 
 * do <code>update referenz_xml set ref_xml_id = (ref_xml_id + 500)</code>, and then let the import run.</p> 
 * <p>
 * Copyright (c) 2006, Eidgenössisches Institut für Geistiges Eigentum.
 * </p>
 * 
 * @author Esther Bandschapp
 * @version 1.0 $Id: ReferenceXMLImport.java,v 1.39 2008/04/14 10:45:00 bandschapp Exp $
 */
public class ReferenceXMLImport
{
  static
  {
    AbstractDAO.setRunningInJ2EEServer( false );
    System.setProperty( DAO.STANDALONE_CONFIG_PROPERTY_NAME, "true" );
  }

  /**
   * Some user...
   */
  private static final int USER_HW = 3;

  /**
   * Used for getting the reports
   */
  private JIPPSFacadeServicesImpl jippsFacade = new JIPPSFacadeServicesImpl();

  /**
   * Defines at what ID (<code>ref_xml_id</code>) the import should start. If it's <code>null</code> it starts at
   * the beginning. <strong>This is only active if {@link #idsToBeInserted} is empty!</strong>
   */
  private Integer startId = null;

  /**
   * Defines at what ID (<code>ref_xml_id</code>) the import should end, <strong>including this</strong>. If it's
   * <code>null</code> it runs till the last test case. <strong>This is only active if {@link #idsToBeInserted} is
   * empty!</strong>
   */
  private Integer endId = null;

  /**
   * These are the IDs that must be inserted, regardless of {@link #startId} and {@link #endId}.
   */
  private int[] idsToBeInserted = {};

  /**
   * These are the report types that must be inserted.
   */
  private ReportType[] reportTypesToInsert = {};

  /**
   * Runs this.
   * 
   * @param argv The Arguments (which are ignored)
   * @throws Exception
   */
  public static void main( String argv[] ) throws Exception
  {
    PathConfiguration.setPathsForLocalTest();

    new ReferenceXMLImport().fillData();
  }

  /**
   * Creates the XML documents and inserts them into the database.
   * 
   * @throws Exception if the reading/inserting fails
   */
  private void fillData() throws Exception
  {
    ReferenceXMLDAO referenceXMLDAO = new ReferenceXMLDAO();

    List<ReferenceXMLVO> referenceVOs = getReferenceVOs();
    for ( ReferenceXMLVO reference: referenceVOs )
    {
      if ( isToBeInserted( reference ) )
      {
        System.out.println( "*************************************************************************" );
        System.out.println( reference.getRefXmlId() + ": report type " + reference.getReportType() + " for IP type " + reference.getIpType() + " at "
            + DateFormatter.shortFormat( reference.getReportDate() ) + ", IP key " + reference.getIpKey() );
        System.out.println( "*************************************************************************" );

        //TODO Make this smarter, after all it calls the same business logic three times
        List<PrintJobVO> printJobs = getPrintJobVOs( reference );
        reference.setXmlData( getXml( printJobs ) );
        reference.setPdfFile( getMergePdf( printJobs ) );
        reference.setXslFoData( getXslFo( printJobs ) );

        referenceXMLDAO.insert( reference );
      }
    }
  }

  /**
   * Reads the XML file with the reference items and creates the corresponding value objects.
   * 
   * @return The reference objects from the XML file
   * @throws ParserConfigurationException if the XML reading fails
   * @throws SAXException if the XML reading fails
   * @throws IOException if the XML file can't be read
   * @throws ParseException if the XML date can't be parsed
   */
  public static List<ReferenceXMLVO> getReferenceVOs() throws ParserConfigurationException, SAXException, IOException, ParseException
  {
    DocumentBuilderFactory documentBuilderFactory = DocumentBuilderFactory.newInstance();
    DocumentBuilder documentBuilder = documentBuilderFactory.newDocumentBuilder();
    Document document = documentBuilder.parse( ReferenceXMLImport.class.getResourceAsStream( "referenceXMLItems.xml" ) );

    // normalize text representation
    document.getDocumentElement().normalize();

    NodeList listOfIps = document.getElementsByTagName( "ip" );

    List<ReferenceXMLVO> nodes = new ArrayList<ReferenceXMLVO>( listOfIps.getLength() );
    Map<Integer, List<ReferenceXMLVO>> nodeMap = new HashMap<Integer, List<ReferenceXMLVO>>( listOfIps.getLength() );

    for ( int i = 0; i < listOfIps.getLength(); i++ )
    {
      Node ipNode = listOfIps.item( i );
      if ( ipNode.getNodeType() == Node.ELEMENT_NODE )
      {
        Element ipElement = (Element) ipNode;

        ReferenceXMLVO referenceVO = new ReferenceXMLVO();
        referenceVO.setRefXmlId( getNodeIntegerValue( ipElement, "refXmlId" ) );
        referenceVO.setIpKey( getNodeIntegerValue( ipElement, "ipKey" ) );
        referenceVO.setIpType( IPType.parse( getNodeIntegerValue( ipElement, "ipType" ) ) );
        referenceVO.setReportType( ReportType.parse( getNodeIntegerValue( ipElement, "reportType" ) ) );
        referenceVO.setReportDate( DateConverter.xmlStringToSqlDate( getNodeValue( ipElement, "reportDate" ) ) );

        referenceVO.setLocale( Locale.GERMAN );
        Integer ipLanguageCode = getNodeIntegerValue( ipElement, "ipLanguage" );
        if ( ipLanguageCode != null )
        {
          referenceVO.setLocale( BAGISLanguageDomain.parse( ipLanguageCode ).toLocale() );
        }

        // Check whether this reference case already exists
        if ( !nodeMap.containsKey( referenceVO.getIpKey() ) )
        {
          nodeMap.put( referenceVO.getIpKey(), new ArrayList<ReferenceXMLVO>() );
        }
        List<ReferenceXMLVO> ipReferences = nodeMap.get( referenceVO.getIpKey() );
        int indexForDuplicate = ipReferences.indexOf( referenceVO );
        if ( indexForDuplicate > -1 )
        {
          ReferenceXMLVO duplicate = ipReferences.get( indexForDuplicate );
          throw new IllegalStateException( "Reference item " + referenceVO.getRefXmlId() + " is a duplicate of item " + duplicate.getRefXmlId() + "\n"
              + referenceVO + "\n" + duplicate + "\n<!-- TODO Use " + referenceVO.getRefXmlId() + ", it was a duplicate of " + duplicate.getRefXmlId() + " -->" );
        }
        else
        {
          ipReferences.add( referenceVO );
          nodes.add( referenceVO );
        }
      }
    }
    return nodes;
  }

  /**
   * Gets the value of an element's node.
   * 
   * @param element The element whose node is taken
   * @param nodeName The node's name
   * @return The node's value or <code>null</code> if it doesn't exist
   */
  private static String getNodeValue( Element element, String nodeName )
  {
    NodeList nodeList = element.getElementsByTagName( nodeName );
    Element subElement = (Element) nodeList.item( 0 );
    if ( subElement == null )
    {
      return null;
    }
    else
    {
      NodeList subNodeList = subElement.getChildNodes();
      return subNodeList.item( 0 ).getNodeValue().trim();
    }
  }

  /**
   * Gets the value of an element's node as an Integer.
   * 
   * @param element The element whose node is taken
   * @param nodeName The node's name
   * @return The node's value as an Integer or <code>null</code> if it doesn't exist
   */
  private static Integer getNodeIntegerValue( Element element, String nodeName )
  {
    String nodeValue = getNodeValue( element, nodeName );
    if ( nodeValue == null )
    {
      return null;
    }
    else
    {
      return new Integer( getNodeValue( element, nodeName ) );
    }
  }

  /**
   * Gets the report's XML document.
   * 
   * @param printJobs The print jobs to execute
   * @return The XML document containing the report
   * @throws Exception if the XML can't be created
   */
  private String getXml( List<PrintJobVO> printJobs ) throws Exception
  {
    String[] xmlDocuments = this.jippsFacade.getXml( printJobs );
    return xmlDocuments[0];
  }

  /**
   * Gets the report's PDF file.
   * 
   * @param printJobs The print jobs to execute
   * @return The PDF file containing the report
   * @throws Exception if the PDF can't be created
   */
  private byte[] getMergePdf( List<PrintJobVO> printJobs ) throws Exception
  {
    return this.jippsFacade.getMergePdf( printJobs );
  }

  /**
   * Gets the report's XSL-FO document.
   * 
   * @param printJobs The print jobs to execute
   * @return The XSL-FO document containing the report
   * @throws Exception if the XSL-FO document can't be created
   */
  private String getXslFo( List<PrintJobVO> printJobs ) throws Exception
  {
    PrintJobVO printJob = printJobs.get( 0 );
    String[] xslFoDocuments;
    if ( printJob.formTyp == PrintJobVO.DESIGN_BILDER )
    {
      xslFoDocuments = new DesignBilderControl().makeXslFo( printJobs );
    }
    else
    {
      xslFoDocuments = new AuszugControl().makeXslFo( printJobs );
    }
    return xslFoDocuments[0];
  }

  /**
   * Gets a print job for the given IP.
   * 
   * @param reference The reference XML container
   * @return A <code>List</code> with one print job
   */
  private static List<PrintJobVO> getPrintJobVOs( ReferenceXMLVO reference )
  {
    PrintJobVO printJob = new PrintJobVO();
    if ( reference.getReportType().equals( ReportType.DESIGN_IMAGE_LIST ) )
    {
      printJob.formTyp = PrintJobVO.DESIGN_BILDER;
    }
    else
    {
      printJob.formTyp = PrintJobVO.AUSZUG;
    }
    printJob.schutitTyp = reference.getIpType().toDB();
    printJob.printIgekopf = false;
    printJob.reportId = reference.getIpKey().intValue();
    printJob.maNr = USER_HW;
    printJob.regauszTyp = reference.getReportType().toDB();
    printJob.sprache = BAGISLanguageDomain.parse( reference.getLocale() ).toDB();
    printJob.standDatum = reference.getReportDate();
    // Designs have their current state only in application state
    if ( reference.getIpType().equals( IPType.DESIGN ) && reference.getReportType().equals( ReportType.CURRENT_STATE ) )
    {
      printJob.registerKz = IpRegisterStateConstants.APPLICATION_STATE;
    }
    else
    {
      printJob.registerKz = IpRegisterStateConstants.REGISTRATION_STATE;
    }

    List<PrintJobVO> printJobs = new ArrayList<PrintJobVO>();
    printJobs.add( printJob );
    return printJobs;
  }

  /**
   * Defines whether a reference XML document should be inserted into the database.
   * 
   * @param referenceXML The reference XML to check
   * @return <code>true</code> if the referenced XML document must be inserted, otherwise <code>false</code>
   * @see #startId
   * @see #endId
   * @see #idsToBeInserted
   * @see #reportTypesToInsert
   */
  private boolean isToBeInserted( ReferenceXMLVO referenceXML )
  {
    boolean insertIt = false;
    if ( this.idsToBeInserted.length > 0 )
    {
      for ( int idToInsert: this.idsToBeInserted )
      {
        if ( referenceXML.getRefXmlId() == idToInsert )
        {
          insertIt = true;
        }
      }
    }
    else
    {
      insertIt = true;
      if ( this.startId != null && referenceXML.getRefXmlId() < this.startId )
      {
        insertIt = false;
      }
      if ( this.endId != null && referenceXML.getRefXmlId() > this.endId )
      {
        insertIt = false;
      }
    }

    if ( this.reportTypesToInsert != null && this.reportTypesToInsert.length > 0 )
    {
      boolean reportTypeOk = false;
      for ( ReportType reportTypeToInsert: this.reportTypesToInsert )
      {
        if ( referenceXML.getReportType().equals( reportTypeToInsert ) )
        {
          reportTypeOk = true;
          break;
        }
      }
      insertIt = insertIt && reportTypeOk;
    }
    return insertIt;
  }
}
