package ch.ige.jipps;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Date;
import java.util.Iterator;
import java.util.List;

import javax.activation.DataHandler;
import javax.activation.DataSource;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.AddressException;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;

import org.apache.log4j.Logger;

import ch.ige.jipps.vo.MailAttachementVO;

/**
 * Mit dieser Klasse wird ein Email versendet. Den Methoden können unterschiedliche Parameter übergeben werden.
 * Copyright (c) 2003, Eidgenössisches Institut für Geistiges Eigentum
 * @author Marc Bouquet
 * @version $Id: SendMail.java,v 1.26 2008/10/01 10:33:32 lang Exp $
 */
public class SendMail
{
  // Attribute für IGELogging
  private static final Logger LOG = Logger.getLogger( SendMail.class );

  private String adressTo;

  private String adressCC;

  private String adressFrom;

  private String betreff;

  private String meldung;

  private List byteList = null;

  private Session session;

  /**
   * Default-Konstruktor.
   * @param session the javax.mail.Session-object to use
   * @throws Exception Klasse konnte nicht inizialisiert werden, Parameter in DB fehlen.
   */
  public SendMail( Session session ) throws Exception
  {
    this.session = session;
    session.setDebug( false );
  }

  /**
   * Mit dieser Methode wird ein Mail mit den übergebenen Parameter an einen frei definierbaren Empfänger gesendet.
   * @param adressTo Empfängeradresse
   * @param adressCC Zusätzliche Empfänger
   * @param adressFrom Absenderadresse
   * @param betreff Betreff des Mails
   * @param meldung Mitteilung
   */
  public void sendMail( String adressTo, String adressCC, String adressFrom, String betreff, String meldung )
  {
    this.adressTo = adressTo;
    this.adressCC = adressCC;
    this.adressFrom = adressFrom;
    setBetreff( betreff );
    this.meldung = meldung;

    send();
  }

  /**
   * @param betreff
   */
  private void setBetreff( String betreff )
  {
    this.betreff = ch.ige.util.BuildInfo.BUILD_ENV_NAME + " - " + betreff;
  }

  /**
   * Mit dieser Methode wird ein Mail mit Attachment an einen frei definierbaren Empfänger gesendet.
   * @param adressTo Empfängeradresse
   * @param adressCC Zusätzliche Empfänger
   * @param adressFrom Absenderadresse
   * @param betreff Betreff des Mails
   * @param byteList Liste mit einem oder mehreren MailAttachment-Value-Objects
   */
  public void sendMail( String adressTo, String adressCC, String adressFrom, String betreff, List byteList )
  {
    this.adressTo = adressTo;
    this.adressCC = adressCC;
    this.adressFrom = adressFrom;
    setBetreff( betreff );
    this.byteList = byteList;

    send();
  }

  /**
   * Mitteilung wird gesendet.
   */
  private void send()
  {
    Iterator itByteList;
    MimeBodyPart mbp2;

    try
    {
      // Mitteilung erstellen.
      MimeMessage msg = new MimeMessage( session );
      msg.setFrom( new InternetAddress( adressFrom ) );
      msg.setRecipients( Message.RecipientType.TO, InternetAddress.parse( adressTo, false ) );
      // allfällige CC Empfänger setzen
      if ( adressCC != null )
      {
        msg.setRecipients( Message.RecipientType.CC, adressCC );
      }

      msg.setSubject( betreff, "iso8859-1" );
      msg.setSentDate( new Date() );

      if ( byteList != null )
      {
        // Erstellen und füllen der ersten Mitteilung.
        MimeBodyPart mbp1 = new MimeBodyPart();
        mbp1.setText( "Dieses Mail wurde automatisch generiert! \n ", "iso8859-1" );

        // create the Multipart and its parts to it
        Multipart mp = new MimeMultipart();
        mp.addBodyPart( mbp1 );

        itByteList = byteList.iterator();
        while ( itByteList.hasNext() )
        {
          MailAttachementVO mailVO = null;

          try
          // tryCatch wegen ClassCast
          {
            mailVO = (MailAttachementVO) itByteList.next();
            // Erstellen und füllen der zweiten Mitteilung als Attachement.
            mbp2 = new MimeBodyPart();
            // Use setText(text, charset), to show it of
            mbp2.setFileName( mailVO.fileName );
            mbp2.setDataHandler( new DataHandler( new ByteArrayDataSource( mailVO.file, mailVO.mimeType ) ) );
            mp.addBodyPart( mbp2 );
          }
          catch ( Exception ex )
          {
            LOG.error( "Es wurde kein MailAttachmentVO übergeben.", ex );
          }
        }
        // add the Multipart to the message
        msg.setContent( mp );
      }
      else
      {
        msg.setText( "Dieses Mail wurde automatisch generiert! \n \n" + meldung, "iso8859-1" );
      }
      Transport.send( msg );
    }
    catch ( AddressException aex )
    {
      LOG.error( "AddressException", aex );
    }
    catch ( MessagingException mex )
    {
      LOG.error( "MessageException", mex );
    }
  }
}

/**
 * Mit dieser Klasse kann bei einem Mail das Attachment als ByteArray übergeben werden. Diese Klasse ist eine
 * Hilfsklasse für die Methode send(byte[] attachment).
 */
class ByteArrayDataSource implements DataSource
{
  private byte[] byteArray;

  private String mimeType;

  /**
   * Konstruktor.
   * @param byteArray übergebener ByteArray
   * @param mimeType übergebener mimeType
   */
  public ByteArrayDataSource( byte[] byteArray, String mimeType )
  {
    this.mimeType = mimeType;
    this.byteArray = byteArray;
  }

  /**
   * Gibt den InputStream zurück.
   * @return ByteArrayInputStream des ByteArray
   * @throws IOException Unerwarteter IO-Fehler
   */
  public InputStream getInputStream() throws IOException
  {
    return ( new ByteArrayInputStream( byteArray ) );
  }

  /**
   * Gibt den OutputStream zurück.
   * @return null
   * @throws IOException Unerwarteter IO-Fehler
   */
  public OutputStream getOutputStream() throws IOException
  {
    return ( null );
  }

  /**
   * Gibt den ContentType zurück.
   * @return String des mimeType
   */
  public String getContentType()
  {
    return ( mimeType );
  }

  /**
   * Gibt den Namen zurück.
   * @return null
   */
  public String getName()
  {
    return ( null );
  }
}
