package ch.ige.jipps.currentstate;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Locale;

import ch.ige.ipobjects.Types;
import ch.ige.ipobjects.common.GenericIP;
import ch.ige.ipobjects.report.dao.RegInhaltDAO;
import ch.ige.ipobjects.time.PointInTime;
import ch.ige.jipps.text.ReportTextLoader;
import ch.ige.jipps.vo.comparator.EventCancellationComparator;
import ch.ige.jipps.vo.comparator.ReportTextNumberComparator;
import ch.ige.util.enums.IPType;
import ch.ige.util.enums.report.ReportArea;
import ch.ige.util.enums.report.ReportAreaUsage;
import ch.ige.util.enums.report.ReportType;

/**
 * Builds the objects for the current state in the report.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: CurrentState.java,v 1.21 2007/12/19 17:54:48 lewis_lee Exp $
 */
public class CurrentState
{
  /**
   * Gets the current state for the report.
   * @param ip The IP with all required data
   * @param currentState The state is built for this date
   * @param language The language of the report
   * @param reportType The report type
   * @return The current state elements in the correct sequence for printing
   * @throws SQLException if the report texts can't be fetched
   */
  @SuppressWarnings( "unchecked" )
  public static List<CurrentStateElement> getCurrentState( GenericIP ip, PointInTime currentState, Locale language, ReportType reportType ) throws SQLException
  {
    List<CurrentStateElement> stateElements = new ArrayList<CurrentStateElement>();

    CurrentStateBuilder builder = getCurrentStateBuilder( ip );

    // By invoking the methods with reflection, it isn't necessary to change
    // this every time the interface changes
    try
    {
      Method[] builderMethods = CurrentStateBuilder.class.getMethods();
      Object[] parameters = new Object[] { ip, reportType, currentState, language };
      for ( int i = 0; i < builderMethods.length; i++ )
      {
        Method builderMethod = builderMethods[i];
        List<CurrentStateElement> newElements = (List<CurrentStateElement>) builderMethod.invoke( builder, parameters );
        if ( newElements != null )
        {
          stateElements.addAll( newElements );
        }
      }
    }
    catch ( IllegalAccessException ex )
    {
      throw new IllegalStateException( "Couldn't invoke the current state builder", ex );
    }
    catch ( IllegalArgumentException ex )
    {
      throw new IllegalStateException( "Couldn't invoke the current state builder", ex );
    }
    catch ( InvocationTargetException ex )
    {
      throw new IllegalStateException( "Couldn't invoke the current state builder", ex );
    }

    Collections.sort( stateElements, createReportTextNumberComparator( ip.getBAGISType(), ReportAreaUsage.CURRENT_STATE ) );

    // Normally, the first event shown in the state is the payment. But for designs,
    // the event showing the deletion of the IP is shown at the top - if there is one, of course.
    if ( Types.DESIGN.isInstance( ip ) )
    {
      Collections.sort( stateElements, new EventCancellationComparator() );
    }

    ReportTextLoader.insertReportText( stateElements, ip.getType(), language, reportType, ReportArea.CURRENT_STATE, true );
    return stateElements;
  }

  /**
   * Creates a new report text number comparator which is used to sort events for printing.
   * @param ipType The IP type
   * @param reportAreaUsage The report area usage, which defines in what report area the text is used
   * @return The new comparator used for <code>Collections.sort(...)</code>
   * @throws SQLException if the database query fails
   */
  private static ReportTextNumberComparator createReportTextNumberComparator( IPType ipType, ReportAreaUsage reportAreaUsage ) throws SQLException
  {
    List<Integer> numberSequence = new RegInhaltDAO().findReportTextSequence( ipType, reportAreaUsage );
    return new ReportTextNumberComparator( numberSequence );
  }

  /**
   * Gets the implementation of <code>CurrentStateBuilder</code> depending on the IP type.
   * @param ip The IP whose type defines the implementation
   * @return The object that builds the current state for the IP type
   */
  private static CurrentStateBuilder getCurrentStateBuilder( GenericIP ip )
  {
    CurrentStateBuilder builder = null;
    switch( ip.getBAGISType() )
    {
      case TRADEMARK_CH:
        builder = new TrademarkBuilder();
        break;
      case PATENT_CH:
      case PATENT_EP:
      case PATENT_PCT:
        builder = new PatentBuilder();
        break;
      case SPC:
        builder = new SpcBuilder();
        break;
      case DESIGN:
        builder = new DesignBuilder();
        break;
      default:
        throw new IllegalArgumentException( "Found unknown IP type " + ip.getBAGISType() );
    }
    return builder;
  }
}
