package ch.ige.jipps.currentstate;

import java.sql.SQLException;
import java.util.List;
import java.util.Locale;

import ch.ige.ipobjects.common.GenericIP;
import ch.ige.ipobjects.time.vo.EreigIDat;
import ch.ige.util.enums.report.ReportType;

/**
 * Builds the objects for the current state in the report, depending on the IP type.
 *
 * <p>Copyright (c) 2006, Swiss Federal Institute of Intellectual Property</p>
 * @author    Eric Lewis
 * @version   $Id: CurrentStateBuilder.java,v 1.11 2007/05/04 08:56:36 lewis_lee Exp $
 */
interface CurrentStateBuilder
{
  /**
   * Gets an IP's description.
   * @param ip The IP containing all data
   * @param reportType The report type
   * @param currentState The data is filtered for this date
   * @param language The language to use for texts
   * @return The current state elements containing this part of the current state in the report
   * @throws SQLException if the retrieval of other data fails
   */
  public List<CurrentStateElement> getDescription( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException;

  /**
   * Gets the IP's registration.
   * @param ip The IP containing all data
   * @param reportType The report type
   * @param currentState The data is filtered for this date
   * @param language The language to use for texts
   * @return The current state elements
   *         containing this part of the current state in the report
   * @throws SQLException if the retrieval of other data fails
   */
  public List<CurrentStateElement> getRegistration( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException;

  /**
   * Gets the IP's cancellation.
   * @param ip The IP containing all data
   * @param reportType The report type
   * @param currentState The data is filtered for this date
   * @param language The language to use for texts
   * @return The current state elements
   *         containing this part of the current state in the report
   * @throws SQLException if the retrieval of other data fails
   */
  public List<CurrentStateElement> getCancellation( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException;

  /**
   * Gets the IP's publication.
   * @param ip The IP containing all data
   * @param reportType The report type
   * @param currentState The data is filtered for this date
   * @param language The language to use for texts
   * @return The current state elements
   *         containing this part of the current state in the report
   * @throws SQLException if the retrieval of other data fails
   */
  public List<CurrentStateElement> getPublication( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException;

  /**
   * Gets the IP's addresses.
   * @param ip The IP containing all data
   * @param reportType The report type
   * @param currentState The data is filtered for this date
   * @param language The language to use for texts
   * @return The current state elements
   *         containing this part of the current state in the report
   * @throws SQLException if the retrieval of other data fails
   */
  public List<CurrentStateElement> getAddresses( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException;

  /**
   * Gets the latest fee, when this IP was payed.
   * @param ip The IP containing all data
   * @param reportType The report type
   * @param currentState The data is filtered for this date
   * @param language The language to use for texts
   * @return The current state elements
   *         containing this part of the current state in the report
   * @throws SQLException if the retrieval of other data fails
   */
  public List<CurrentStateElement> getLatestFee( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException;

  /**
   * Gets the IP's prioritites.
   * @param ip The IP containing all data
   * @param reportType The report type
   * @param currentState The data is filtered for this date
   * @param language The language to use for texts
   * @return The current state elements
   *         containing this part of the current state in the report
   * @throws SQLException if the retrieval of other data fails
   */
  public List<CurrentStateElement> getPriorities( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException;

  /**
   * Gets the classification of a IP.
   * <ul>
   * <li>For a patent, this is the IPC classification.</li>
   * <li>For a design, it's a list of classes.</li>
   * <li>For a trademark it's the list of goods and services (WDL). This contains:
   *   <ul>
   *     <li>The free-form text describing the goods and services</li>
   *     <li>The list of Nice classes describing the trademark</li>
   *   </ul>
   * </li>
   * </ul>
   * @param ip The IP containing all data
   * @param reportType The report type
   * @param currentState The data is filtered for this date
   * @param language The language to use for texts
   * @return The current state elements
   *         containing this part of the current state in the report
   * @throws SQLException if the retrieval of other data fails
   */
  public List<CurrentStateElement> getClassification( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException;

  /**
   * Gets the PCT application and publication of a patent.
   * @param ip The IP containing all data
   * @param reportType The report type
   * @param currentState The data is filtered for this date
   * @param language The language to use for texts
   * @return The current state elements
   *         containing this part of the current state in the report
   * @throws SQLException if the retrieval of other data fails
   */
  public List<CurrentStateElement> getPct( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException;

  /**
   * Gets the start of the protection of an SPC.
   * @param ip The IP containing all data
   * @param reportType The report type
   * @param currentState The data is filtered for this date
   * @param language The language to use for texts
   * @return The current state elements
   *         containing this part of the current state in the report
   * @throws SQLException if the retrieval of other data fails
   */
  public List<CurrentStateElement> getProtectionStart( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException;

  /**
   * Gets the maximum term of protection for an SPC.
   * @param ip The IP containing all data
   * @param reportType The report type
   * @param currentState The data is filtered for this date
   * @param language The language to use for texts
   * @return The current state elements
   *         containing this part of the current state in the report
   * @throws SQLException if the retrieval of other data fails
   */
  public List<CurrentStateElement> getProtectionEnd( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException;

  /**
   * Gets the renewal of the IP's protection.
   * @param ip The IP containing all data
   * @param reportType The report type
   * @param currentState The data is filtered for this date
   * @param language The language to use for texts
   * @return The current state elements
   *         containing this part of the current state in the report
   * @throws SQLException if the retrieval of other data fails
   */
  public List<CurrentStateElement> getRenewal( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException;

  /**
   * Gets the trademark's kind, for instance visual trademark, text trademark etc.
   * @param ip The IP containing all data
   * @param reportType The report type
   * @param currentState The data is filtered for this date
   * @param language The language to use for texts
   * @return The current state elements
   *         containing this part of the current state in the report
   * @throws SQLException if the retrieval of other data fails
   */
  public List<CurrentStateElement> getTrademarkKind( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException;

  /**
   * Gets the design's items and number of items.
   * @param ip The IP containing all data
   * @param reportType The report type
   * @param currentState The data is filtered for this date
   * @param language The language to use for texts
   * @return The current state elements
   *         containing this part of the current state in the report
   * @throws SQLException if the retrieval of other data fails
   */
  public List<CurrentStateElement> getItems( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException;

  /**
   * Gets the design's deposit.
   * @param ip The IP containing all data
   * @param reportType The report type
   * @param currentState The data is filtered for this date
   * @param language The language to use for texts
   * @return The current state elements
   *         containing this part of the current state in the report
   * @throws SQLException if the retrieval of other data fails
   */
  public List<CurrentStateElement> getDeposit( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException;

  /**
   * Gets the design's type.
   * @param ip The IP containing all data
   * @param reportType The report type
   * @param currentState The data is filtered for this date
   * @param language The language to use for texts
   * @return The current state elements
   *         containing this part of the current state in the report
   * @throws SQLException if the retrieval of other data fails
   */
  public List<CurrentStateElement> getType( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException;

  /**
   * Gets the design's exhibition immunity.
   * This protects the design even before the application, if it's shown at
   * an international exhibition for which it can be protected.
   * @param ip The IP containing all data
   * @param reportType The report type
   * @param currentState The data is filtered for this date
   * @param language The language to use for texts
   * @return The current state elements
   *         containing this part of the current state in the report
   * @throws SQLException if the retrieval of other data fails
   */
  public List<CurrentStateElement> getExhibitionImmunity( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException;

  /**
   * Gets the design's unsealing information.
   * @param ip The IP containing all data
   * @param reportType The report type
   * @param currentState The data is filtered for this date
   * @param language The language to use for texts
   * @return The current state elements
   *         containing this part of the current state in the report
   * @throws SQLException if the retrieval of other data fails
   */
  public List<CurrentStateElement> getUnsealing( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException;

  /**
   * Gets the remarks.
   * @param ip The IP containing all data
   * @param reportType The report type
   * @param currentState The data is filtered for this date
   * @param language The language to use for texts
   * @return The current state elements
   *         containing this part of the current state in the report
   * @throws SQLException if the retrieval of other data fails
   */
  public List<CurrentStateElement> getRemark( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException;

  /**
   * Gets the trademark's color claim.
   * @param ip The IP containing all data
   * @param reportType The report type
   * @param currentState The data is filtered for this date
   * @param language The language to use for texts
   * @return The current state elements
   *         containing this part of the current state in the report
   * @throws SQLException if the retrieval of other data fails
   */
  public List<CurrentStateElement> getColorClaim( GenericIP ip, ReportType reportType, EreigIDat currentState, Locale language ) throws SQLException;
}
