package ch.ige.jipps.currentstate.test;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.w3c.dom.Node;

import ch.ige.jipps.configuration.PathConfiguration;
import ch.ige.jipps.test.AbstractTestFacadeXml;
import ch.ige.jipps.vo.ReferenceXMLVO;

/**
 * Gets the reference XML documents and compares their state to what is generated on the fly. Note that some data
 * contains spaces at the end, so <code>String</code>s are always trimmed.
 * <p>
 * Copyright (c) 2006, Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * @author Eric Lewis
 * @version FacadeXmlCurrentStateTest.java,v 1.4 2006/09/06 13:14:37 lewis_lee Exp
 */
public class FacadeXmlCurrentStateTest extends AbstractTestFacadeXml
{
  /**
   * The usual constructor
   * @param args Any parameters
   */
  public FacadeXmlCurrentStateTest( String args )
  {
    super( args );
  }

  // @see junit.framework.TestCase#setUp()
  @Override
  public void setUp() throws Exception
  {
    super.setUp();
    this.checkAllIpTypes = true;
    PathConfiguration.setPathsForLocalTest();
  }

  /**
   * Gets the XML and tests part of it in a subclass. This is required because otherwise JUnit doesn't find any test.
   * @throws Exception if the test fails
   */
  public void testGetXml() throws Exception
  {
    super.testGetXml( false );
  }

  // @see ch.ige.jipps.test.AbstractTestFacadeXml#compareXml(java.lang.String, java.lang.String,
  // ch.ige.jipps.vo.ReferenceXMLVO)
  @Override
  public void compareXml( String expectedXml, String actualXml, ReferenceXMLVO referenceXmlVO ) throws Exception
  {
    final String xPath = "/auszug/standData/*[local-name()='standEreigTitel' or local-name()='standAuszTxt']";
    List<Node> expectedNodes = getNodes( expectedXml, xPath );
    Map<String, List<List<String>>> expectedNodeMap = createStateNodeMap( expectedNodes );
    List<Node> actualNodes = getNodes( actualXml, xPath );
    Map<String, List<List<String>>> actualNodeMap = createStateNodeMap( actualNodes );

    // Note: Create a new Set, because the Map will be changed by removing elements
    Set<String> titles = new HashSet<String>( expectedNodeMap.keySet() );
    for ( String title: titles )
    {
      if ( !actualNodeMap.containsKey( title ) )
      {
        fail( "Found no title <" + title + "> in generated XML", expectedXml, actualXml, getInformation( referenceXmlVO ) );
      }
      else
      {
        List<List<String>> expectedNodeList = expectedNodeMap.remove( title );
        List<List<String>> actualNodeList = actualNodeMap.remove( title );
        for ( Iterator<List<String>> expectedIterator = expectedNodeList.iterator(); expectedIterator.hasNext(); )
        {
          boolean foundText = false;
          List<String> expectedTextLines = expectedIterator.next();
          expectedIterator.remove();

          String expectedFirstLine = getFirstLine( expectedTextLines );
          List<String> actualFirstLines = new ArrayList<String>();
          if ( expectedFirstLine != null )
          {
            for ( List<String> actualTextLines: actualNodeList )
            {
              String actualFirstLine = getFirstLine( actualTextLines );
              actualFirstLines.add( actualFirstLine );
              if ( linesMatch( expectedFirstLine, actualFirstLine ) )
              {
                foundText = true;
                compareLines( expectedTextLines, actualTextLines );
              }
            }

            if ( !foundText )
            {
              fail( "Found no text lines or wrong lines in generated XML for title <" + title + ">, expected <" + expectedFirstLine
                  + "> but it wasn't found in " + actualFirstLines );
            }
          }
        }
      }
    }

    if ( !actualNodeMap.isEmpty() )
    {
      fail( "Generated XML contains extra titles: " + actualNodeMap.keySet() );
    }

    flattenAndCheckNode( expectedXml, actualXml, "standData", referenceXmlVO );
  }

  /**
   * Creates a <code>Map</code> of state data.
   * @param nodes The nodes containing state data, below <code>standData</code>
   * @return A <code>Map</code> where the key is the title of a group of state data entries and the value is a
   * <code>List</code> containing one entry for each paragraph, which again is a <code>List</code> of
   * <code>String</code>s containing one line.
   * @throws Exception if the nodes can't be read
   */
  private Map<String, List<List<String>>> createStateNodeMap( List<Node> nodes ) throws Exception
  {
    Map<String, List<List<String>>> nodeMap = new HashMap<String, List<List<String>>>();
    String currentTitle = null;
    for ( Node node: nodes )
    {
      if ( node.getNodeName().equals( "standEreigTitel" ) )
      {
        currentTitle = getFirstText( node );

      }
      if ( currentTitle != null && !nodeMap.containsKey( currentTitle ) )
      {
        nodeMap.put( currentTitle, new ArrayList<List<String>>() );
      }
      List<List<String>> stateTexts = nodeMap.get( currentTitle );
      if ( stateTexts == null )
      {
        fail( "Found no state title and thus no list of nodes" );
      }
      else if ( node.getNodeName().equals( "standAuszTxt" ) )
      {
        stateTexts.add( getTextLines( node, "standTxt" ) );
      }
    }
    return nodeMap;
  }
}
