package ch.ige.jipps.dao;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import ch.ige.dbutils.dao.AbstractJippsDAO;
import ch.ige.dbutils.runner.IGEQueryRunner;
import ch.ige.dbutils.sql.SQL;
import ch.ige.dbutils.sql.SQLStatement;
import ch.ige.jipps.vo.DruckPeriVO;
import ch.ige.jipps.vo.LoeschungLaufVO;
import ch.ige.jipps.vo.LoeschungVO;
import ch.ige.util.constants.CancellationProcessingConstants;
import ch.ige.util.validation.Check;

/**
 * Accesses the table <code>LOESCHUNG</code>.
 * 
 * <p>Copyright (c) 2001, Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author Priska Steiner
 * @author Eric Lewis
 * @version $Id: LoeschungDAO.java,v 1.45 2008/05/22 11:43:29 lewis_lee Exp $
 */

public class LoeschungDAO extends AbstractJippsDAO
{
  /**
   * The type of value object to return
   */
  private static final Class<LoeschungVO> VO_TYPE = LoeschungVO.class;

  /**
   * The maximum number of individual print stati which can be selected
   */
  private static final int MAXIMUM_NUMBER_OF_PRINT_STATI = 5;

  /**
   * Since print stati are set into a PreparedStatement, but must also be variable in number
   * at the same time, this is inserted as a dummy print status which will never exist - hopefully!  ;-)
   */
  private static final int DUMMY_PRINT_STATUS = -932;

  /**
   * Selects the cancellations
   */
  private static final SQLStatement<LoeschungVO> SELECT = SQL.select( "SELECT loeschung_lauf_id, bfakt_id, adr_nr, druckperi AS druck_peri, fach_nr, "
      + "schutit_typ, adr_nr_vtr, sort_txt, TO_NUMBER(verarb_stat) AS verarb_stat, loeschung_xml, created_dat AS create_date, changed_dat AS changed_date "
      + "FROM loeschung " + "WHERE druckperi = ? AND loeschung_lauf_id = ? AND verarb_stat IN(?, ?, ?, ?, ?) "
      + "ORDER BY fach_nr, adr_nr, schutit_typ, adr_nr_vtr, sort_txt" );

  /**
   * Selects the latest cancellation run
   */
  private static final SQLStatement<Integer> SELECT_LATEST_CANCELLATION_RUN = SQL
      .select( "SELECT MAX(loeschung_lauf_id) AS loeschung_lauf_id FROM loeschung_lauf " + "WHERE druck_art = ?" );

  /**
   * Selects the latest few cancellation runs
   */
  private static final SQLStatement<LoeschungLaufVO> SELECT_CANCELLATION_RUNS = SQL
      .select( "SELECT * FROM loeschung_lauf WHERE druck_art = ? ORDER BY loeschung_lauf_id DESC" );

  /**
   * Selects the print recipient groups
   */
  private static final SQLStatement<DruckPeriVO> SELECT_PRINT_RECIPIENT_GROUPS = SQL
      .select( "SELECT loeschung_lauf_id, druckperi AS druck_peri, verarb_stat AS is_printed, COUNT(*) AS anzahl " + "FROM loeschung "
          + "WHERE loeschung_lauf_id IN " + "(SELECT loeschung_lauf_id FROM loeschung_lauf WHERE druck_art = ?) " + "AND loeschung_lauf_id = ? "
          + "GROUP BY loeschung_lauf_id, druckperi, verarb_stat " + "ORDER BY druckperi" );

  /**
   * Inserts a cancellation.
   */
  private static final SQLStatement<Integer> INSERT = SQL.update( "INSERT INTO loeschung "
      + "(loeschung_lauf_id, bfakt_id, adr_nr, druckperi, fach_nr, schutit_typ, "
      + "adr_nr_vtr, sort_txt, verarb_stat, loeschung_xml, created_dat, changed_dat) " + "VALUES (?,?,?,?,?,?,?,?,?,?,sysdate,sysdate)" );

  /**
   * Updates one cancellation
   */
  private static final SQLStatement<Integer> UPDATE_ONE = SQL.update( "UPDATE loeschung SET " + "verarb_stat = ?, " + "changed_dat = sysdate "
      + "WHERE loeschung_lauf_id = ? " + "AND bfakt_id = ? " + "AND adr_nr = ?" );

  /**
   * Updates all cancellation with a certain print recipient group
   */
  private static final SQLStatement<Integer> UPDATE_PRINT_RECIPIENT_GROUP = SQL.update( "UPDATE loeschung SET " + "verarb_stat = ?, "
      + "changed_dat = sysdate " + "WHERE loeschung_lauf_id = ? " + "AND druckperi = ?" );

  /**
   * Deletes one cancellation
   */
  private static final SQLStatement<Integer> DELETE_ONE = SQL.update( "DELETE FROM loeschung WHERE loeschung_lauf_id = ? AND bfakt_id = ? AND adr_nr = ?" );

  /**
   * Selects cancellations for a given print recipient group which are new or should be printed again.
   * @param printRecipientGroup The print recipient group
   * @param numberOfCancellations The number of cancellations to fetch
   * @param cancellationRunId The ID of the cancellation run
   * @return The cancellations that match the parameters
   * @exception SQLException if the query fails
   */
  public List<LoeschungVO> selectSorted( String printRecipientGroup, int numberOfCancellations, int cancellationRunId ) throws SQLException
  {
    return selectSorted( printRecipientGroup, numberOfCancellations, cancellationRunId, new int[] { CancellationProcessingConstants.NEW,
        CancellationProcessingConstants.PRINT_AGAIN } );
  }

  /**
   * Selects cancellations for a given print recipient group.
   * @param printRecipientGroup The print recipient group
   * @param numberOfCancellations The number of cancellations to fetch
   * @param cancellationRunId The ID of the cancellation run
   * @param printStati The print stati to look for
   * @return The cancellations that match the parameters
   * @exception SQLException if the query fails
   */
  public List<LoeschungVO> selectSorted( String printRecipientGroup, int numberOfCancellations, int cancellationRunId, int[] printStati ) throws SQLException
  {
    if ( printStati.length > MAXIMUM_NUMBER_OF_PRINT_STATI )
    {
      throw new IllegalArgumentException( "Too many print stati!" );
    }

    List<Object> parameters = new ArrayList<Object>( MAXIMUM_NUMBER_OF_PRINT_STATI + 2 );
    // Limits the number of rows that are returned. The query still loads everything from the DB, though....
    parameters.add( IGEQueryRunner.maxRows( numberOfCancellations ) );

    parameters.add( printRecipientGroup );
    parameters.add( Integer.valueOf( cancellationRunId ) );
    // Add the print stati, defaulting them to the dummy
    for ( int i = 0; i < MAXIMUM_NUMBER_OF_PRINT_STATI; i++ )
    {
      if ( i < printStati.length )
      {
        parameters.add( Integer.toString( printStati[i] ) );
      }
      else
      {
        parameters.add( Integer.toString( DUMMY_PRINT_STATUS ) );
      }
    }

    return selectVOList( 100, VO_TYPE, SELECT, parameters.toArray() );
  }

  /**
   * Gets print recipients for the given cancellation type and run.
   * @param cancellationType The cancellationType, one of {@link ch.ige.util.constants.CancellationConstants}
   * @param cancellationRunId The ID of the cancellation run
   * @return All print receipients for the given cancellation type and run
   * @throws SQLException if the query fails
   */
  public List<DruckPeriVO> selectDruckPeriByDruckArt( int cancellationType, int cancellationRunId ) throws SQLException
  {
    return selectVOList( 5, DruckPeriVO.class, SELECT_PRINT_RECIPIENT_GROUPS, Integer.valueOf( cancellationType ), Integer.valueOf( cancellationRunId ) );
  }

  /**
   * Gets the latest cancellation run ID for the given print status.
   * @param cancellationType The cancellationType, one of {@link ch.ige.util.constants.CancellationConstants}
   * @return The latest cancellation run ID or <code>0</code> if none was found
   * @throws SQLException
   */
  public int getLoeschungLaufId( int cancellationType ) throws SQLException
  {
    return selectInt( getQueryRunner(), SELECT_LATEST_CANCELLATION_RUN, 0, Integer.valueOf( cancellationType ) );
  }

  /**
   * Gets all cancellation run IDs for the given print status.
   * @param cancellationType The cancellationType, one of {@link ch.ige.util.constants.CancellationConstants}
   * @return All cancellation run IDs or <code>0</code> if none was found
   * @throws SQLException
   */
  public List<LoeschungLaufVO> getLoeschungLaeufe( int cancellationType ) throws SQLException
  {
    return selectVOList( 100, LoeschungLaufVO.class, SELECT_CANCELLATION_RUNS, Integer.valueOf( cancellationType ) );
  }

  /**
   * Inserts a cancellation into the database.
   * @param cancellation The cancellation to insert
   */
  public void insert( final LoeschungVO cancellation )
  {
    Check.notNull( cancellation, "cancellation" );

    this.execute( new SingleDAOUnitOfWork<Integer>()
    {
      // @see ch.ige.dbutils.dao.AbstractSingleDAOUnitOfWork#getTransactionName()
      @Override
      protected String getTransactionName()
      {
        return "loeschung_insert";
      }

      // @see ch.ige.dbutils.dao.UnitOfWork#execute(ch.ige.dbutils.runner.IGEQueryRunner)
      public void execute( IGEQueryRunner runner ) throws SQLException
      {
        String processingStatus = "";
        if ( cancellation.verarbStat != null )
        {
          processingStatus = cancellation.verarbStat.toString();
        }
        int count = runner.update( INSERT, cancellation.loeschungLaufId, cancellation.bfaktId, cancellation.adrNr, cancellation.druckPeri, cancellation.fachNr,
            cancellation.schutitTyp, cancellation.adrNrVtr, cancellation.sortTxt, processingStatus, cancellation.loeschungXml );
        verifyUpdateCount( count, 1, "update count must be 1" );
      }
    } );
  }

  /**
   * Updates the processing status of a cancellation.
   * @param cancellationRunId The ID of the cancellation run
   * @param invoiceId The ID of the invoice to which the cancellation belongs
   * @param addressNumber The number of the address
   * @param processingStatus The processing status to set, one of {@link CancellationProcessingConstants}
   * @return The number of updated rows, which can be <code>0</code>; but it should be <code>1</code>, since
   *         a cancellation is identified by <code>cancellationRunId</code>, <code>invoiceId</code> and <code>addressNumber</code> 
   */
  public Integer update( final int cancellationRunId, final int invoiceId, final int addressNumber, final Integer processingStatus )
  {
    Check.notNull( processingStatus, "processing status" );

    return this.execute( new SingleDAOUnitOfWork<Integer>()
    {
      // @see ch.ige.dbutils.dao.AbstractSingleDAOUnitOfWork#getTransactionName()
      @Override
      protected String getTransactionName()
      {
        return "loeschung_update_one";
      }

      // @see ch.ige.dbutils.dao.UnitOfWork#execute(ch.ige.dbutils.runner.IGEQueryRunner)
      public void execute( IGEQueryRunner runner ) throws SQLException
      {
        int count = runner.update( UPDATE_ONE, processingStatus.toString(), cancellationRunId, invoiceId, addressNumber );
        setResult( Integer.valueOf( count ) );
      }
    } );
  }

  /**
   * Updates the processing status for a print recipient group.
   * @param cancellationRunId The ID of the cancellation run
   * @param printRecipientGroup The print recipient group within the cancellation run
   * @param processingStatus The processing status to set, one of {@link CancellationProcessingConstants}
   * @return The number of updated rows, which can be <code>0</code> 
   */
  public Integer updateDruckPeri( final int cancellationRunId, final String printRecipientGroup, final Integer processingStatus )
  {
    return this.execute( new SingleDAOUnitOfWork<Integer>()
    {
      // @see ch.ige.dbutils.dao.AbstractSingleDAOUnitOfWork#getTransactionName()
      @Override
      protected String getTransactionName()
      {
        return "loeschung_update";
      }

      // @see ch.ige.dbutils.dao.UnitOfWork#execute(ch.ige.dbutils.runner.IGEQueryRunner)
      public void execute( IGEQueryRunner runner ) throws SQLException
      {
        int count = runner.update( UPDATE_PRINT_RECIPIENT_GROUP, processingStatus.toString(), cancellationRunId, printRecipientGroup );
        setResult( Integer.valueOf( count ) );
      }
    } );
  }

  /**
   * Deletes the cancellation.
   * @param cancellationRunId The ID of the cancellation run
   * @param invoiceId The ID of the invoice to which the cancellation belongs
   * @param addressNumber The number of the address
   */
  @SuppressWarnings( "unchecked" )
  public void delete( final int cancellationRunId, final int invoiceId, final int addressNumber )
  {
    this.execute( new SingleDAOUnitOfWork()
    {
      // @see ch.ige.dbutils.dao.AbstractSingleDAOUnitOfWork#getTransactionName()
      @Override
      protected String getTransactionName()
      {
        return "loeschung_delete";
      }

      // @see ch.ige.dbutils.dao.UnitOfWork#execute(ch.ige.dbutils.runner.IGEQueryRunner)
      public void execute( IGEQueryRunner runner ) throws SQLException
      {
        int numberOfUpdates = runner.update( DELETE_ONE, Integer.valueOf( cancellationRunId ), Integer.valueOf( invoiceId ), Integer.valueOf( addressNumber ) );
        verifyUpdateCount( numberOfUpdates, 1, "Couldn't delete cancellation " + cancellationRunId + " with invoice " + invoiceId + " and address "
            + addressNumber );
      }
    } );
  }
}
