package ch.ige.jipps.dao.test;

import java.math.BigDecimal;
import java.sql.SQLException;
import java.text.ParseException;

import ch.ige.dbutils.dao.test.BaseDAOTestTemplate;
import ch.ige.jipps.dao.JoinSchreibenDAO;
import ch.ige.jipps.vo.JippsFakturVO;
import ch.ige.util.date.DateConverter;
import ch.ige.util.enums.BAGISLanguage;
import ch.ige.util.enums.IPType;

/**
 * Tests {@link ch.ige.jipps.dao.JoinSchreibenDAO}.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: JoinSchreibenDAOTest.java,v 1.5 2008/05/19 11:16:01 lewis_lee Exp $
 */
public class JoinSchreibenDAOTest extends BaseDAOTestTemplate<JoinSchreibenDAO, JippsFakturVO>
{
  /**
   * An existing invoice
   */
  private static final int INVOICE_ID = 6031380;

  /**
   * The invoice for an SPC base on an EP patent
   */
  private static final int SPC_EP_INVOICE_ID = 5270627;

  /**
   * The invoice for an SPC base on an Swiss patent
   */
  private static final int SPC_CH_INVOICE_ID = 5906581;

  /**
   * The only constructor
   * @param name The test's name
   */
  public JoinSchreibenDAOTest( String name )
  {
    super( name );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#getDAOClass()
  @Override
  protected Class<JoinSchreibenDAO> getDAOClass()
  {
    return JoinSchreibenDAO.class;
  }

  /**
   * Tests {@link ch.ige.jipps.dao.JoinSchreibenDAO#select(int)}.
   * @throws Exception if the test fails
   */
  public void testSelect() throws Exception
  {
    JippsFakturVO invoice = getDAO().select( 13 );
    assertNull( "Checking unknown invoice", invoice );

    invoice = getDAO().select( INVOICE_ID );
    assertEquals( "Checking invoice ID", INVOICE_ID, invoice.bfaktId );
    assertEquals( "Checking invoice date", "2006-07-11", invoice.bfaktDat.toString() );
    assertEquals( "Checking form type", "406", invoice.formartCd );
    assertEquals( "Checking language", BAGISLanguage.FR.toDB(), invoice.sprachCd );
    assertEquals( "Checking address number", 4500561, invoice.adrNr );
    assertEquals( "Checking IP key", 1200016048, invoice.schutitKey );
    assertEquals( "Checking IP number", "P-283157", invoice.schutitNr );
    assertEquals( "Checking IP type", IPType.TRADEMARK_CH.toDB().intValue(), invoice.schutitTyp );
    assertEquals( "Checking owner", "Pro-Mark Corporation, Houston/TX", invoice.inhaber );
    assertEquals( "Checking signature", "Christine Bove", invoice.untName );
    assertEquals( "Checking customer's reference", "30421/CH", invoice.ihrZeichen );
    assertEquals( "Checking IPI's reference", "MAREG sc/ P-283157", invoice.unsereRef );
    assertEquals( "Checking phone number", "+41 31 322 48 25", invoice.telNr );
    assertEquals( "Checking registered mail", "0", invoice.kzEinschr );
    assertEquals( "Checking type of correction", "0", invoice.kzKorrektur );
    assertEquals( "Checking letter attachment", 0, invoice.txtNrBeilage );
    assertEquals( "Checking final sentences", 0, invoice.txtSchluss );
    assertEquals( "Checking variable text", "FOSC no. 131 du 10.07.2006", invoice.varText );
    assertEquals( "Checking additional address text", 0, invoice.txtAdrZusatz );
    assertEquals( "Checking date when the invoice must be paid", null, invoice.verfallDat );
    assertEquals( "Checking amount to pay", null, invoice.faktBetrag );
    // Publication date is set by the publication confirmation, but not used in the DB
    assertEquals( "Checking description", "pro-mark ((fig.))", invoice.beschreibung );
    assertEquals( "Checking protection start", "1976-06-02", invoice.schutzbeginn.toString() );
    assertEquals( "Checking protection end", "2016-06-02", invoice.schutzende.toString() );
    assertEquals( "Checking application number", "02788/1976", invoice.gesuchNr );
  }

  /**
   * Tests selecting an SPC because their application number contains a slash,
   * which may lead to problems.
   * @throws Exception
   */
  public void testSelectSpcWithBaseEpPatent() throws Exception
  {
    JippsFakturVO invoice = getDAO().select( SPC_EP_INVOICE_ID );
    assertEquals( "Checking invoice ID", SPC_EP_INVOICE_ID, invoice.bfaktId );
    assertEquals( "Checking invoice date", "2004-05-10", invoice.bfaktDat.toString() );
    assertEquals( "Checking form type", "197", invoice.formartCd );
    assertEquals( "Checking language", BAGISLanguage.FR.toDB(), invoice.sprachCd );
    assertEquals( "Checking address number", 500442, invoice.adrNr );
    assertEquals( "Checking IP key", 1200107399, invoice.schutitKey );
    assertEquals( "Checking IP number", "C00012401/01", invoice.schutitNr );
    assertEquals( "Checking IP type", IPType.SPC.toDB().intValue(), invoice.schutitTyp );
    assertEquals( "Checking owner", "Merck & Co., Inc., Rahway/NJ", invoice.inhaber );
    assertEquals( "Checking signature", null, invoice.untName );
    assertEquals( "Checking customer's reference", "0.0251 SPC 2", invoice.ihrZeichen );
    assertEquals( "Checking IPI's reference", "PAdmin jgb/C00012401/01", invoice.unsereRef );
    assertEquals( "Checking phone number", "+41 31 322 49 46", invoice.telNr );
    assertEquals( "Checking registered mail", "0", invoice.kzEinschr );
    assertEquals( "Checking type of correction", "0", invoice.kzKorrektur );
    assertEquals( "Checking letter attachment", 0, invoice.txtNrBeilage );
    assertEquals( "Checking final sentences", 0, invoice.txtSchluss );
    assertEquals( "Checking variable text", "10.12.1999", invoice.varText );
    assertEquals( "Checking additional address text", 0, invoice.txtAdrZusatz );
    assertEquals( "Checking date when the invoice must be paid", null, invoice.verfallDat );
    assertEquals( "Checking amount to pay", null, invoice.faktBetrag );
    // Publication date is set by the publication confirmation, but not used in the DB
    assertEquals( "Checking description", "Lisinopril", invoice.beschreibung );
    assertEquals( "Checking protection start", "1999-12-10", invoice.schutzbeginn.toString() );
    assertEquals( "Checking protection end", "2004-05-07", invoice.schutzende.toString() );
    assertEquals( "Checking application number", "C00012401/01", invoice.gesuchNr );
  }

  /**
   * Tests selecting an SPC because their application number contains a slash,
   * which may lead to problems.
   * @throws Exception
   */
  public void testSelectSpcWithBaseChPatent() throws Exception
  {
    JippsFakturVO invoice = getDAO().select( SPC_CH_INVOICE_ID );
    assertEquals( "Checking invoice ID", SPC_CH_INVOICE_ID, invoice.bfaktId );
    assertEquals( "Checking invoice date", "2006-02-20", invoice.bfaktDat.toString() );
    assertEquals( "Checking form type", "197", invoice.formartCd );
    assertEquals( "Checking language", BAGISLanguage.DE.toDB(), invoice.sprachCd );
    assertEquals( "Checking address number", 500629, invoice.adrNr );
    assertEquals( "Checking IP key", 1200110256, invoice.schutitKey );
    assertEquals( "Checking IP number", "C644615/01", invoice.schutitNr );
    assertEquals( "Checking IP type", IPType.SPC.toDB().intValue(), invoice.schutitTyp );
    assertEquals( "Checking owner", "Glaxo Group Limited, GB- Greenford (Middx UB6 ONN)", invoice.inhaber );
    assertEquals( "Checking signature", null, invoice.untName );
    assertEquals( "Checking customer's reference", "D16913/C0207", invoice.ihrZeichen );
    assertEquals( "Checking IPI's reference", "PAdmin jgb/C644615/01", invoice.unsereRef );
    assertEquals( "Checking phone number", "+41 31 322 49 46", invoice.telNr );
    assertEquals( "Checking registered mail", "0", invoice.kzEinschr );
    assertEquals( "Checking type of correction", "0", invoice.kzKorrektur );
    assertEquals( "Checking letter attachment", 0, invoice.txtNrBeilage );
    assertEquals( "Checking final sentences", 0, invoice.txtSchluss );
    assertEquals( "Checking variable text", "13.02.2001", invoice.varText );
    assertEquals( "Checking additional address text", 0, invoice.txtAdrZusatz );
    assertEquals( "Checking date when the invoice must be paid", null, invoice.verfallDat );
    assertEquals( "Checking amount to pay", null, invoice.faktBetrag );
    // Publication date is set by the publication confirmation, but not used in the DB
    assertEquals( "Checking description", "Fluticasonpropionat", invoice.beschreibung );
    assertEquals( "Checking protection start", "2001-02-13", invoice.schutzbeginn.toString() );
    assertEquals( "Checking protection end", "2006-02-12", invoice.schutzende.toString() );
    assertEquals( "Checking application number", "C644615/01", invoice.gesuchNr );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#getSerializableVO()
  @Override
  protected JippsFakturVO getSerializableVO() throws SQLException
  {
    JippsFakturVO invoice = new JippsFakturVO();
    invoice.bfaktId = 8147;
    try
    {
      invoice.bfaktId = 8147;
      invoice.bfaktDat = DateConverter.stringToDate( "03.02.1997" );
      invoice.formartCd = "ABC";
      invoice.sprachCd = BAGISLanguage.DE.toDB();
      invoice.adrNr = 57818;
      invoice.schutitKey = 117514;
      invoice.schutitNr = "11751/4";
      invoice.schutitTyp = IPType.DESIGN.toDB();
      invoice.inhaber = "Joe Jackson";
      invoice.untName = "Napoleon III";
      invoice.ihrZeichen = "JJ";
      invoice.unsereRef = "Nap";
      invoice.telNr = "031 333 44 55";
      invoice.kzEinschr = "1";
      invoice.kzKorrektur = "2";
      invoice.txtNrBeilage = 51;
      invoice.txtSchluss = 88;
      invoice.varText = "Whatever...";
      invoice.txtAdrZusatz = 889;
      invoice.verfallDat = DateConverter.stringToDate( "11.11.2011" );
      invoice.faktBetrag = new BigDecimal( 0 );
      invoice.publicationDate = DateConverter.stringToDate( "08.08.1988" );
      invoice.beschreibung = "Ein schönes Design ist das, jawoll!";
      invoice.schutzbeginn = DateConverter.stringToDate( "16.06.1966" );
      invoice.schutzende = DateConverter.stringToDate( "19.09.1999" );
      invoice.gesuchNr = "bla/57123/x";
    }
    catch ( ParseException e )
    {
      throw new SQLException( e.getMessage() );
    }
    return invoice;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#assertVOEqual(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected boolean assertVOEqual( JippsFakturVO originalVO, JippsFakturVO serializedVO )
  {
    return originalVO.bfaktId == serializedVO.bfaktId && originalVO.bfaktDat.equals( serializedVO.bfaktDat )
        && originalVO.formartCd.equals( serializedVO.formartCd ) && originalVO.sprachCd.equals( serializedVO.sprachCd )
        && originalVO.adrNr == serializedVO.adrNr && originalVO.schutitKey == serializedVO.schutitKey && originalVO.schutitNr.equals( serializedVO.schutitNr )
        && originalVO.schutitTyp == serializedVO.schutitTyp && originalVO.inhaber.equals( serializedVO.inhaber )
        && originalVO.untName.equals( serializedVO.untName ) && originalVO.ihrZeichen.equals( serializedVO.ihrZeichen )
        && originalVO.unsereRef.equals( serializedVO.unsereRef ) && originalVO.telNr.equals( serializedVO.telNr )
        && originalVO.kzEinschr.equals( serializedVO.kzEinschr ) && originalVO.kzKorrektur.equals( serializedVO.kzKorrektur )
        && originalVO.txtNrBeilage == serializedVO.txtNrBeilage && originalVO.txtSchluss == serializedVO.txtSchluss
        && originalVO.varText.equals( serializedVO.varText ) && originalVO.txtAdrZusatz == serializedVO.txtAdrZusatz
        && originalVO.verfallDat.equals( serializedVO.verfallDat ) && originalVO.faktBetrag.equals( serializedVO.faktBetrag )
        && originalVO.publicationDate.equals( serializedVO.publicationDate ) && originalVO.beschreibung.equals( serializedVO.beschreibung )
        && originalVO.schutzbeginn.equals( serializedVO.schutzbeginn ) && originalVO.schutzende.equals( serializedVO.schutzende )
        && originalVO.gesuchNr.equals( serializedVO.gesuchNr );
  }
}
