package ch.ige.jipps.dao.test;

import java.sql.SQLException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import ch.ige.dbutils.dao.UnitOfWorkException;
import ch.ige.dbutils.dao.test.BaseDAOTestTemplate;
import ch.ige.jipps.dao.SearchDAO;
import ch.ige.jipps.vo.PrintJobVO;
import ch.ige.jipps.vo.PrintjobSearchVO;
import ch.ige.util.constants.PrintStatusConstants;
import ch.ige.util.date.DateConverter;

/**
 * Tests {@link ch.ige.jipps.dao.SearchDAO}.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: SearchDAOTest.java,v 1.9 2008/05/21 15:26:23 lewis_lee Exp $
 */
public class SearchDAOTest extends BaseDAOTestTemplate<SearchDAO, PrintjobSearchVO>
{
  /**
   * A known and existing batch ID
   */
  private static final int BATCH_ID = 1140;

  /**
   * A known and existing print job ID
   */
  private static final int PRINT_JOB_ID = 47474;

  /**
   * The only constructor
   * @param name The test's name
   */
  public SearchDAOTest( String name )
  {
    super( name );
  }

  /**
   * Tests {@link ch.ige.jipps.dao.SearchDAO#select(ch.ige.jipps.vo.PrintjobSearchVO)}.
   * @throws Exception if the test fails
   */
  public void testSelect() throws Exception
  {
    try
    {
      getDAO().select( null );
      fail( "Null search must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    // Batch ID
    PrintjobSearchVO search = createPrintJobSearch();
    search.batchId = BATCH_ID;
    List<PrintjobSearchVO> printJobs = getDAO().select( search );
    assertEquals( "Checking search for batch ID", 8, printJobs.size() );
    for ( PrintjobSearchVO printJob: printJobs )
    {
      assertEquals( "Checking batch ID", BATCH_ID, printJob.batchId );
    }

    // Print job ID
    search = createPrintJobSearch();
    search.printjobId = PRINT_JOB_ID;
    printJobs = getDAO().select( search );
    assertEquals( "Checking for print job ID", 1, printJobs.size() );
    assertEquals( "Checking the print job's batch ID", BATCH_ID, printJobs.get( 0 ).batchId );

    // Report ID
    search = createPrintJobSearch();
    search.reportId = 5000036;
    printJobs = getDAO().select( search );
    assertTrue( "Checking for report ID", printJobs.size() > 0 );
    boolean foundPrintJob = false;
    for ( PrintjobSearchVO printJob: printJobs )
    {
      if ( printJob.batchId == BATCH_ID )
      {
        foundPrintJob = true;
      }
    }
    assertTrue( "Checking the print job's batch ID", foundPrintJob );

    // Form type
    search = createPrintJobSearch();
    search.formtyp = "letter"; // Note the capital letters!
    printJobs = getDAO().select( search );
    assertTrue( "Checking for form type", printJobs.size() >= 240 );
    for ( PrintjobSearchVO printJob: printJobs )
    {
      assertEquals( "Checking form type", "letter", printJob.formtyp.toLowerCase() );
    }

    // Print status
    search = createPrintJobSearch();
    search.druckStatus = PrintStatusConstants.PREVIEW_PRINT;
    printJobs = getDAO().select( search );
    assertTrue( "Checking for print status", printJobs.size() >= 1 );
    for ( PrintjobSearchVO printJob: printJobs )
    {
      assertEquals( "Checking print status", PrintStatusConstants.PREVIEW_PRINT, printJob.druckStatus );
    }

    // Creation date
    search = createPrintJobSearch();
    search.erstDat = DateConverter.stringToSqlDate( "01.12.2007" );
    printJobs = getDAO().select( search );
    assertEquals( "Checking for creation date", 14, printJobs.size() );
    for ( PrintjobSearchVO printJob: printJobs )
    {
      assertTrue( "Checking creation date", printJob.erstDat.toString().startsWith( "2007-12-01" ) );
    }

    // Creation date and batch ID
    search = createPrintJobSearch();
    search.erstDat = DateConverter.stringToSqlDate( "01.12.2007" );
    search.batchId = 1242;
    printJobs = getDAO().select( search );
    assertEquals( "Checking for creation date and batch ID", 7, printJobs.size() );
    for ( PrintjobSearchVO printJob: printJobs )
    {
      assertTrue( "Checking creation date", printJob.erstDat.toString().startsWith( "2007-12-01" ) );
      assertEquals( "Checking batch ID", 1242, printJob.batchId );
    }

    // Form type and sender
    search = createPrintJobSearch();
    search.formtyp = "LeTTeR";
    search.sender = "bolzli";
    printJobs = getDAO().select( search );
    assertEquals( "Checking for form type and sender", 1, printJobs.size() );
    for ( PrintjobSearchVO printJob: printJobs )
    {
      assertEquals( "Checking form type", "letter", printJob.formtyp.toLowerCase() );
      assertEquals( "Checking sender", "bolzli", printJob.sender );
    }
  }

  /**
   * Creates a new print job search with the print status set
   * to {@link ch.ige.jipps.vo.PrintjobSearchVO#DUMMY_PRINT_STATUS}.
   * @return A new print job
   */
  private PrintjobSearchVO createPrintJobSearch()
  {
    PrintjobSearchVO search = new PrintjobSearchVO();
    search.druckStatus = PrintjobSearchVO.DUMMY_PRINT_STATUS;
    search.batchId = PrintjobSearchVO.DUMMY_BATCH_ID;
    return search;
  }

  /**
   * Tests {@link ch.ige.jipps.dao.SearchDAO#selectPrintJobTypes()}.
   * @throws Exception if the test fails
   */
  public void testSelectPrintJobTypes() throws Exception
  {
    List<String> knownPrintJobTypes = new ArrayList<String>();
    for ( String knownPrintJobType: PrintJobVO.FORM_TYPES )
    {
      knownPrintJobTypes.add( knownPrintJobType.toLowerCase() );
    }

    Set<String> printJobTypes = getDAO().selectPrintJobTypes();
    for ( String printJobType: printJobTypes )
    {
      assertTrue( "Checking print job type '" + printJobType + "'", knownPrintJobTypes.contains( printJobType ) );
    }
  }

  /**
   * Tests {@link ch.ige.jipps.dao.SearchDAO#update(ch.ige.jipps.vo.PrintjobSearchVO)}.
   * @throws Exception if the test fails
   */
  public void testUpdate() throws Exception
  {
    try
    {
      getDAO().update( null );
      fail( "Null search must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      PrintjobSearchVO search = createPrintJobSearch();
      search.printjobId = -1;
      getDAO().update( search );
      fail( "Invalid print job ID must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      PrintjobSearchVO search = createPrintJobSearch();
      search.printjobId = PRINT_JOB_ID;
      search.druckerId = 999;
      getDAO().update( search );
      fail( "Invalid printer ID must fail" );
    }
    catch ( UnitOfWorkException e )
    {
      // This is expected
    }

    final int NEW_PRINTER_ID = 16;

    PrintjobSearchVO search = createPrintJobSearch();
    search.printjobId = PRINT_JOB_ID;
    PrintjobSearchVO printJob = getDAO().select( search ).get( 0 );

    assertTrue( "Checking printer ID", printJob.druckerId != NEW_PRINTER_ID );

    int savedPrinterId = printJob.druckerId;

    printJob.druckerId = NEW_PRINTER_ID;
    getDAO().update( printJob );

    PrintjobSearchVO loadedPrintJob = getDAO().select( search ).get( 0 );
    assertTrue( "Checking printer ID", loadedPrintJob.druckerId == NEW_PRINTER_ID );
    assertTrue( "Checking print status", loadedPrintJob.druckStatus == PrintStatusConstants.READY_FOR_PRINTING );

    // Reset to old state
    printJob.druckerId = savedPrinterId;
    getDAO().update( printJob );

    loadedPrintJob = getDAO().select( search ).get( 0 );
    assertTrue( "Checking printer ID", loadedPrintJob.druckerId == savedPrinterId );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#getDAOClass()
  @Override
  protected Class<SearchDAO> getDAOClass()
  {
    return SearchDAO.class;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#getSerializableVO()
  @Override
  protected PrintjobSearchVO getSerializableVO() throws SQLException
  {
    try
    {
      PrintjobSearchVO printJobSearch = new PrintjobSearchVO();
      printJobSearch.printjobId = 234234;
      printJobSearch.reportId = 99;
      printJobSearch.batchId = 100;
      printJobSearch.druckerId = 101;
      printJobSearch.druckStatus = 33;
      printJobSearch.sender = "Return to sender";
      printJobSearch.formtyp = "form XY/13";
      printJobSearch.erstDat = DateConverter.stringToSqlDate( "03.02.2005" );
      printJobSearch.coverCd = false;
      return printJobSearch;
    }
    catch ( ParseException e )
    {
      throw new SQLException( e.getMessage() );
    }
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#assertVOEqual(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected boolean assertVOEqual( PrintjobSearchVO originalVO, PrintjobSearchVO serializedVO )
  {
    return originalVO.printjobId == serializedVO.printjobId && originalVO.reportId == serializedVO.reportId && originalVO.batchId == serializedVO.batchId
        && originalVO.druckStatus == serializedVO.druckStatus && originalVO.sender.equals( serializedVO.sender )
        && originalVO.formtyp.equals( serializedVO.formtyp ) && originalVO.erstDat.equals( serializedVO.erstDat ) && originalVO.coverCd == serializedVO.coverCd;
  }
}
