package ch.ige.jipps.history.description;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import ch.ige.ipobjects.Types;
import ch.ige.ipobjects.common.GenericIP;
import ch.ige.ipobjects.event.Event;
import ch.ige.ipobjects.report.history.HistoryDescription;
import ch.ige.ipobjects.report.history.HistoryItemDetailDescription;
import ch.ige.ipobjects.time.PointInTime;
import ch.ige.ipobjects.time.PointInTimeCollection;
import ch.ige.ipobjects.time.TimePeriod;
import ch.ige.jipps.history.detail.BaseHistoryItemDetail;
import ch.ige.jipps.history.detail.InvalidAddressDetail;
import ch.ige.jipps.history.detail.ValidAndInvalidAddressDetail;
import ch.ige.jipps.text.ConvertText;
import ch.ige.jipps.vo.HistoryItemVO;
import ch.ige.util.constants.EventNumberConstants;
import ch.ige.util.constants.NumericLiterals;
import ch.ige.util.date.DateFormatter;
import ch.ige.util.enums.report.ReportType;

/**
 * Configures how the history part of the report is built.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: BaseHistoryDescription.java,v 1.12 2007/12/20 08:24:54 lewis_lee Exp $
 */
public class BaseHistoryDescription implements HistoryDescription<List<HistoryItemVO>, HistoryItemVO>, NumericLiterals
{
  /**
   * The IP to use
   */
  protected GenericIP ip;

  /**
   * The current state date
   */
  protected PointInTime currentState;

  /**
   * The language to use
   */
  protected Locale language;

  /**
   * Shows whether events on the wait list must be included
   */
  protected boolean includeWaitListEvents;

  /**
   * Shows whether the history starts at application or at registration time
   */
  protected boolean startFromApplication;

  /**
   * The report groups, a <code>Map</code> where the keys show whether an event must be included on a report. The
   * values can be ignored.
   */
  private Map<Integer, String> allowedReportGroups;

  /**
   * The type of report
   */
  private ReportType reportType;

  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( BaseHistoryDescription.class );

  /**
   * Creates this with all required data.
   * @param ip The IP
   * @param currentState The history is built for this date
   * @param language The language of the report
   * @param includeWaitListEvents If this is <code>true</code>, events that are on the wait list are also shown
   * @param startFromApplication If this is <code>true</code>, events starting from the application are shown,
   * otherwise from the registration
   * @param allowedReportGroups The report groups, a <code>Map</code> where the keys show whether an event must be
   * included on a report. The values can be ignored.
   * @param reportType The report type
   */
  public BaseHistoryDescription( GenericIP ip, PointInTime currentState, Locale language, boolean includeWaitListEvents, boolean startFromApplication,
      Map<Integer, String> allowedReportGroups, ReportType reportType )
  {
    this.ip = ip;
    this.currentState = currentState;
    this.language = language;
    this.includeWaitListEvents = includeWaitListEvents;
    this.startFromApplication = startFromApplication;
    this.allowedReportGroups = allowedReportGroups;
    this.reportType = reportType;
  }

  /**
   * Gets the detail descriptions which are common to all IP types. This must be called before any IP type specific
   * detail descriptions are created.
   * @return The details for the given event.
   */
  @SuppressWarnings( "unchecked" )
  public final BaseHistoryItemDetail<? extends TimePeriod>[] getDetailDescriptions()
  {
    List<BaseHistoryItemDetail<? extends TimePeriod>> details = new ArrayList<BaseHistoryItemDetail<? extends TimePeriod>>();
    if ( Types.TRADEMARK.isInstance( this.ip ) )
    {
      details.add( new InvalidAddressDetail( this.ip, this.ip.getRegAdr(), this.ip.getType(), this.language ) );
    }
    else
    {
      details.add( new ValidAndInvalidAddressDetail( this.ip, this.ip.getRegAdr(), this.ip.getType(), this.language ) );
    }

    List<BaseHistoryItemDetail<? extends TimePeriod>> ipSpecificDetails = getIpSpecificDetailDescriptions();
    if ( ipSpecificDetails != null )
    {
      details.addAll( ipSpecificDetails );
    }
    return details.toArray( new BaseHistoryItemDetail[] {} );
  }

  /**
   * Gets the detail descriptions which are specific for a certain IP type.
   * @return The details for the given event or <code>null</code> if there are none
   */
  protected List<BaseHistoryItemDetail<? extends TimePeriod>> getIpSpecificDetailDescriptions()
  {
    return null;
  }

  // See interface
  public final boolean hasSingleItemPerEvent()
  {
    return false;
  }

  // See interface
  public final PointInTime getRequestedStand()
  {
    return this.currentState;
  }

  /**
   * Gets the first event that's used in the history.
   * @return The first event that's used in the history
   */
  public final Event getHistoryStart()
  {
    if ( this.startFromApplication )
    {
      // The first event is assumed to be the application event
      return this.ip.getEvents().get( 0 );
    }
    else
    {
      return this.ip.getRegistrationEvent();
    }
  }

  // See interface
  public final PointInTimeCollection<Event> getEventsForVisitor()
  {
    if ( this.includeWaitListEvents )
    {
      return this.ip.getEvents();
    }
    else
    {
      return this.ip.getEventsNotOnWaitlist();
    }
  }

  // See interface
  public final List<HistoryItemVO> createItemContainer()
  {
    return new ArrayList<HistoryItemVO>();
  }

  // See interface
  public boolean includeInHistory( Event event )
  {
    Event start = this.getHistoryStart();
    if ( start == null )
    {
      return false;
    }
    else
    {
      return ( event.sameTime( start ) || event.after( start ) ) && isAllowedAsReportGroup( event );
    }
  }

  // See interface
  public final HistoryItemVO createHistoryItem( Event event, int historyItemNumber )
  {
    try
    {
      HistoryItemVO historyItem = new HistoryItemVO();
      // The start date is always the event date, but the end date depends on the IP type
      historyItem.setStartDate( DateFormatter.shortFormat( event.getDate() ) );
      historyItem.setEventNumber( event.getNumericArt_nr() );

      return historyItem;
    }
    catch ( Exception ex )
    {
      throw new RuntimeException( "Couldn't create history item", ex );
    }
  }

  // @see
  // ch.ige.ipobjects.report.history.HistoryDescription#attachDetailToItem(ch.ige.ipobjects.report.history.HistoryItemDetailDescription,
  // java.lang.Object, java.lang.Object)
  public <D> void attachDetailToItem( HistoryItemDetailDescription<List<HistoryItemVO>, HistoryItemVO, D> detailDescription, D detail, HistoryItemVO item )
  {
    detailDescription.attachDetailToItem( item, detail );
  }

  // @see ch.ige.ipobjects.report.history.HistoryDescription#inspectAndFilterCreated(ch.ige.ipobjects.event.Event,
  // java.util.List)
  public List<HistoryItemVO> inspectAndFilterCreated( Event event, List<HistoryItemVO> createdItems )
  {
    if ( !createdItems.isEmpty() )
    {
      // Only the first history item gets the event text and start date,
      // and only the last history item gets the publication and event texts
      final int lastIndex = createdItems.size() - 1;
      for ( int i = 0; i <= lastIndex; i++ )
      {
        HistoryItemVO historyItem = createdItems.get( i );
        if ( i > 0 )
        {
          historyItem.setEventNumber( null );
          historyItem.setStartDate( null );
        }
        if ( i == lastIndex )
        {
          try
          {
            historyItem.setPublication( PublicationText.getPublicationText( this.ip, this.currentState, event, this.language, this.reportType, false ) );
          }
          catch ( Exception ex )
          {
            LOG.error( "Couldn't set publication for event " + event, ex );
          }
          // There are texts which consist only of a '.'
          if ( StringUtils.isNotEmpty( event.getEreig_txt() ) && event.getEreig_txt().length() >= 2 )
          {
            historyItem.setRemarks( ConvertText.convertNewlineToBreak( event.getEreig_txt() ) );
          }
        }
      }
    }
    return createdItems;
  }

  // @see ch.ige.ipobjects.report.history.HistoryDescription#getEventWithSpecialHistoryRequirements()
  public Event getEventWithSpecialHistoryRequirements()
  {
    return this.getHistoryStart();
  }

  /**
   * Checked if the article is in a) the allowed report groups or b) a first event which has to be included
   * @param event The Event
   * @return Returns <code>true</code> if the event is either part of the article group or a first event
   */
  private boolean isAllowedAsReportGroup( Event event )
  {
    // First check if in allowed report groups, then if it is a first event (report group 10 which is not included
    // before)
    if ( this.allowedReportGroups.containsKey( event.getRegausz_grp() )
        || Arrays.asList( EventNumberConstants.ALL_FIRST_EVENTS ).contains( new Integer( event.getArt_nr() ) ) )
    {
      return true;
    }
    return false;
  }
}
