package ch.ige.jipps.imageconv;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletResponse;

import org.apache.commons.httpclient.Header;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.io.IOUtils;
import org.apache.log4j.Logger;

import ch.ige.dbutils.dao.AbstractDAO;
import ch.ige.jipps.constants.FileConstants;
import ch.ige.jipps.constants.ImageTypeConstants;
import ch.ige.jipps.constants.PropertyConstants;
import ch.ige.jipps.constants.URLConstants;
import ch.ige.jipps.dao.PropertyJippsDAO;
import ch.ige.jipps.url.URLBuilder;

/**
 * Base class for all image writers.
 * 
 * @author Johannes Lang
 * @version $Id: ImageWriter.java,v 1.57 2009/05/05 15:47:55 lewis_lee Exp $
 */
public abstract class ImageWriter
{
  /**
   * The log.
   */
  private static final Logger LOG = Logger.getLogger( ImageWriter.class );

  /**
   * The default image size in centimeters.
   */
  private static final double DEFAULT_IMAGE_SIZE = 8.0;

  /**
   * The JIPPS property DAO.
   */
  private PropertyJippsDAO propertyJippsDAO;

  /**
   * The conversion rate from DPCM to DPI.
   */
  protected static final double CONVERSION_DPCM_TO_DPI = 2.54;

  /**
   * The image width.
   */
  protected String imageWidth;

  /**
   * The image height.
   */
  protected String imageHeigth;

  /**
   * The x density.
   */
  protected int xDensity;

  /**
   * The y density.
   */
  protected int yDensity;

  /**
   * The width in pixels.
   */
  protected int imageWidthInPixel;

  /**
   * The height in pixels.
   */
  protected int imageHeigthInPixel;

  /**
   * Processes the image.
   * @param pathWithoutFileType The path without the file type / ending
   * @param image The image itself (raw data)
   * @return The image's metadata, see {@link ImageProcessor#writeImage(String, byte[], boolean)}
   * @throws IOException If problems occur with writing or reading the file
   */
  public abstract Map<String, Object> processImage( String pathWithoutFileType, byte[] image ) throws IOException;

  /**
   * Rounds to 2 digits.
   * 
   * @param in double value
   * @return double value rounded to two digits
   */
  protected double roundTo2Digits( double in )
  {
    int decimalPlace = 2;
    BigDecimal bd = new BigDecimal( in );
    bd = bd.setScale( decimalPlace, BigDecimal.ROUND_HALF_UP );
    return bd.doubleValue();
  }

  /**
   * Writes an image to the disk.
   * @param image The image data
   * @param path The path where the image should be written to
   */
  protected void writeToFile( byte[] image, String path )
  {
    LOG.debug( "ImageWriter: writeToFile, path: " + path );
    OutputStream os = null;
    try
    {
      os = new FileOutputStream( path );
      os.write( image );
      os.flush();
    }
    catch ( IOException ex )
    {
      LOG.error( ex.getMessage(), ex );
    }
    finally
    {
      IOUtils.closeQuietly( os );
    }
  }

  /**
   * Gets the image's dimensions.
   * @return Map The dimension properties
   */
  protected Map<String, Object> getDimensions()
  {
    Map<String, Object> dimensions = new HashMap<String, Object>( 2 );
    dimensions.put( ImageProcessor.IMAGE_WIDTH, getDimension( this.xDensity, this.imageWidthInPixel ) );
    dimensions.put( ImageProcessor.IMAGE_HEIGHT, getDimension( this.yDensity, this.imageHeigthInPixel ) );
    return dimensions;
  }

  /**
   * Gets the image's dimention in centimeters.
   * 
   * @param density The image's density in DPI
   * @param sizeInPixel The size in pixels
   * @return The dimension in centimeters, defaulting to {@link #DEFAULT_IMAGE_SIZE}
   */
  protected Double getDimension( int density, int sizeInPixel )
  {
    if ( ( density > 0 ) && ( sizeInPixel > 0 ) )
    {
      return new Double( roundTo2Digits( ( (double) sizeInPixel / density ) * CONVERSION_DPCM_TO_DPI ) );
    }
    else
    {
      return new Double( DEFAULT_IMAGE_SIZE );
    }
  }

  /**
   * Calls the servlet that converts the image.
   * 
   * @param imagePath The full path of the original image file
   * @param imageType The type of image, see {@link ImageTypeConstants}
   * @return The type of the converted file, usually {@link ImageTypeConstants#IMAGE_TYPE_JPG}
   */
  protected String callServlet( String imagePath, String imageType )
  {
    // The servlet is only used when running on the application server.
    // When running locally for tests, it's cumbersome to have to build
    // the EAR and deploy it and start the application server just to
    // convert a couple of pictures.
    if ( AbstractDAO.staticIsRunningInJ2EEServer() )
    {
      return convertUsingServlet( imagePath, imageType );
    }
    else
    {
      return convertUsingLocalConverter( imagePath );
    }
  }

  /**
   * Converts the image using the servlet.
   * @param imagePath The full path of the original image file
   * @param sourceImageType The type of source image, see {@link ImageTypeConstants}
   * @return The file extension of the converted file, usually {@link ImageTypeConstants#IMAGE_TYPE_JPG}
   */
  private String convertUsingServlet( String imagePath, String sourceImageType )
  {
    String targetImageType = null;
    GetMethod get = null;
    String url = null;

    try
    {
      url = URLBuilder.buildURLString( getPropertyJippsDAO().select( PropertyConstants.IMAGE_CONVERTER_SERVLET_URL ),
          URLConstants.IMAGE_CONVERTER_SERVLET_PARAM_IMAGEPATH, URLConstants.IMAGE_CONVERTER_SERVLET_PARAM_IMAGETYPE, imagePath, sourceImageType );
      HttpClient client = new HttpClient();
      get = new GetMethod( url );
      client.executeMethod( get );

      if ( get.getStatusCode() == HttpServletResponse.SC_OK )
      {
        Header mimeType = get.getResponseHeader( "Content-Type" );
        LOG.debug( "MIME type of response is " + mimeType.getValue() );
        String characterSet = get.getResponseCharSet();
        LOG.debug( "Character set of response is " + characterSet );

        InputStream responseStream = get.getResponseBodyAsStream();
        StringBuffer response = new StringBuffer();
        byte[] buffer = new byte[4096];
        for ( int size; ( size = responseStream.read( buffer ) ) != -1; )
        {
          response.append( new String( buffer, 0, size ) );
        }

        LOG.debug( "Image type for converted image is " + response );
        targetImageType = response.toString();
      }
      else
      {
        throw new IOException( "No conversion possible: " + get.getStatusText() );
      }
    }
    catch ( Exception ex )
    {
      LOG.error( "Can't convert using " + url, ex );
      //TODO This save the image with the image *type*, not the image file extension!
      //TODO Which means that a TIFF image x.tif is referenced as x.tiff in the XML, which leads to an error.
      targetImageType = sourceImageType; // if no conversion could be done ...
    }
    finally
    {
      if ( get != null )
      {
        get.releaseConnection();
      }
    }
    return targetImageType;
  }

  /**
   * Converts the image without using the image converter servlet.
   * This is used for local testing, where creating an EAR and deploying it
   * and starting an application server is a bit too much just to convert
   * an image.
   * @param imagePath The full path of the original image file
   * @return The image type of the converted file, usually {@link ImageTypeConstants#IMAGE_TYPE_JPG}
   */
  private String convertUsingLocalConverter( String imagePath )
  {
    try
    {
      if ( imagePath.endsWith( FileConstants.EXTENSION_PNG ) )
      {
        return ImageTypeConstants.IMAGE_TYPE_PNG;
      }
      else
      {
        ImageMagickConverter.configureLocalImageMagick();
        ImageMagickConverter.convert( imagePath, ImageTypeConstants.IMAGE_TYPE_JPG );
        return ImageTypeConstants.IMAGE_TYPE_JPG;
      }
    }
    catch ( Exception e )
    {
      LOG.error( "Couldn't convert " + imagePath, e );
      return null;
    }
  }

  /**
   * Gets the PropertyJippsDAO, creating it if it doesn't exist.
   * @return The PropertyJippsDAO
   */
  private PropertyJippsDAO getPropertyJippsDAO()
  {
    if ( this.propertyJippsDAO == null )
    {
      this.propertyJippsDAO = new PropertyJippsDAO();
    }
    return this.propertyJippsDAO;
  }
}
