package ch.ige.jipps.pdf;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;

import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.sax.SAXResult;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.apache.fop.apps.FOUserAgent;
import org.apache.fop.apps.Fop;
import org.apache.fop.apps.FopFactory;
import org.apache.fop.apps.MimeConstants;
import org.apache.log4j.Logger;

import ch.ige.jipps.configuration.PathConfiguration;
import ch.ige.jipps.constants.FileConstants;
import ch.ige.jipps.constants.PropertyConstants;
import ch.ige.jipps.transformation.CachingTransformerFactory;

/**
 * Renders a PDF file using Apache FOP.
 * <p>
 * Copyright (c) 2006, Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * @author Johannes Lang
 * @version $Id: PdfRenderer.java,v 1.28 2007/12/19 18:00:58 lewis_lee Exp $
 */
public class PdfRenderer
{
  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( PdfRenderer.class );

  /**
   * Used to transform the XML document
   */
  private TransformerFactory transformerFactory;

  /**
   * The property to set the transformer factory
   */
  private static final String TRANSFORMER_FACTORY_PROPERTY_NAME = "javax.xml.transform.TransformerFactory";

  /**
   * The class of the transformer factory to use
   */
  private static final String TRANSFORMER_FACTORY_PROPERTY_VALUE = CachingTransformerFactory.class.getName();

  /**
   * The path where XSL files are found
   */
  private String xslPath = null;

  /**
   * The Fop factory which holds the caches and configuration for FOP.
   */
  protected FopFactory fopFactory;

  /**
   * The empty constructor, which configures the PDF renderer.
   */
  public PdfRenderer()
  {
    // Enforces the use of JIPPS's own TransformerFactory
    System.setProperty( TRANSFORMER_FACTORY_PROPERTY_NAME, TRANSFORMER_FACTORY_PROPERTY_VALUE );
    this.transformerFactory = TransformerFactory.newInstance();

    this.xslPath = PathConfiguration.getVerifiedPathAsUrl( PropertyConstants.JIPPS_PATH_XSL, false );

    this.fopFactory = FopFactory.newInstance();
    setupFopFactory( this.fopFactory );
  }

  /**
   * Sets up the FopFactory instance with custom configuration values. Override to set custom values. FopFactory is used
   * for the engine configuration.
   * @param fopFactory the FopFactory instance to configure
   */
  protected void setupFopFactory( FopFactory fopFactory )
  {
    fopFactory.setBase14KerningEnabled( true );
  }

  /**
   * Sets up the FOUserAgent instance with custom configuration values. Override to set custom values. FOUserAgent is
   * used to configure a single document processing run.
   * @param userAgent the FOUserAgent instance to configure
   */
  @SuppressWarnings( "unused" )
  protected void setupFOUserAgent( FOUserAgent userAgent )
  {
    // nop, override to add custom values
  }

  /**
   * Renders a PDF by applying an XSL stylesheet to an XML document.
   * @param reportXml The XML document
   * @param xslName The name of the XSL stylesheet
   * @return The rendered PDF
   */
  public byte[] renderPdf( String reportXml, String xslName )
  {
    ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
    try
    {
      FOUserAgent userAgent = this.fopFactory.newFOUserAgent();
      setupFOUserAgent( userAgent );

      Fop fop = this.fopFactory.newFop( MimeConstants.MIME_PDF, userAgent, outputStream );

      Transformer transformer = createTransformer( xslName );
      if ( LOG.isDebugEnabled() )
      {
        LOG.debug( "XML after XSL transformation, ready for XSL-FO:\n" + renderXslFo( transformer, reportXml, xslName ) );
      }
      StreamSource streamSource = new StreamSource( new StringReader( reportXml ) );
      LOG.debug( "Created StreamSource " + streamSource );
      SAXResult result = new SAXResult( fop.getDefaultHandler() );
      LOG.debug( "Created SAXResult " + result );
      LOG.debug( "Transforming to PDF" );
      transformer.transform( streamSource, result );
      LOG.debug( "Finished transforming" );
      byte[] resultPdf = outputStream.toByteArray();
      LOG.debug( "Got PDF with " + resultPdf.length + " bytes" );
      return resultPdf;
    }
    // Catch the exception to log exactly what's going on
    catch ( Exception e )
    {
      throw new RuntimeException( "Couldn't create PDF with XSL " + xslName, e );
    }
    finally
    {
      try
      {
        outputStream.close();
      }
      catch ( IOException e )
      {
        throw new RuntimeException( "Couldn't close PDF output stream", e );
      }
    }
  }

  /**
   * Creates the Transformer for XSLT.
   * @param xslName The XSL file (without <code>.xsl</code> to use
   * @return The Transformer for XSLT
   * @throws TransformerConfigurationException if the Transformer can't be created
   */
  private Transformer createTransformer( String xslName ) throws TransformerConfigurationException
  {
    return this.transformerFactory.newTransformer( new StreamSource( this.xslPath + xslName + FileConstants.EXTENSION_XSL ) );
  }

  /**
   * Returns the result of the XSL Transformation which happens just before FOP is called. <strong>This must only be
   * used for debugging and testing!</strong>
   * @param transformer The XSLT Transformer to use
   * @param reportXml The XML document
   * @param xslName The name of the XSL stylesheet
   * @return The transformed <code>reportXml</code> which now only consist of FO tags
   * @throws TransformerException if the transformation fail
   */
  public String renderXslFo( final Transformer transformer, final String reportXml, final String xslName ) throws TransformerException
  {
    Transformer transformerToUse = transformer;
    if ( transformer == null )
    {
      transformerToUse = createTransformer( xslName );
    }
    StringWriter transformationWriter = new StringWriter();
    transformerToUse.transform( new StreamSource( new StringReader( reportXml ) ), new StreamResult( transformationWriter ) );
    return transformationWriter.getBuffer().toString();
  }
}
