package ch.ige.jipps.signature;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Locale;

import ch.ige.ipobjects.Types;
import ch.ige.ipobjects.common.GenericIP;
import ch.ige.ipobjects.report.dao.RegAuszTypSTypDAO;
import ch.ige.ipobjects.report.dao.RegFormTextDAO;
import ch.ige.ipobjects.report.vo.RegAuszTypSTypVO;
import ch.ige.ipobjects.report.vo.RegFormTextVO;
import ch.ige.ipobjects.user.dao.MaDAO;
import ch.ige.jipps.signature.stamp.IgeBlackStamp;
import ch.ige.jipps.signature.stamp.Stamp;
import ch.ige.util.constants.HtmlConstants;
import ch.ige.util.enums.report.ReportArea;
import ch.ige.util.enums.report.ReportType;
import ch.ige.util.enums.report.SignatureType;

/**
 * Builds the objects for the signature in the report.
 *
 * <p>Copyright (c) 2006, Swiss Federal Institute of Intellectual Property</p>
 * @author    Eric Lewis
 * @version   $Id: Signature.java,v 1.10 2008/05/21 08:30:11 bandschapp Exp $
 */
public class Signature
{
  /**
   * Gets the signature for the report.
   * @param ip The IP with all required data
   * @param language The language of the report
   * @param reportType The report type
   * @param userNumber The number of the user printing the report
   * @return The signature lines
   * @throws SQLException if the signature can't be fetched
   */
  public static List<String> getSignature( GenericIP ip, Locale language, ReportType reportType, Integer userNumber ) throws SQLException
  {
    RegAuszTypSTypVO ipTypeReportType = new RegAuszTypSTypDAO().findIpTypeReportType( ip.getBAGISType(), reportType );

    if ( ipTypeReportType == null )
    {
      throw new IllegalArgumentException( "Found no signature configuration for IP type " + ip.getType() + " and report type " + reportType );
    }

    // Only do something if there actually is a signature
    if ( ipTypeReportType.getSignatureType().equals( SignatureType.NO_SIGNATURE ) )
    {
      return Collections.emptyList();
    }
    else
    {
      RegFormTextVO departmentSignature = null;
      // There is a special department signature for the trademark search. All other just get the standard signature.
      if ( Types.TRADEMARK.isInstance( ip ) && reportType.equals( ReportType.TRADEMARK_SEARCH ) )
      {
        List<RegFormTextVO> signatures = new RegFormTextDAO().findReportTexts( language, reportType, ip.getBAGISType(), ReportArea.SIGNATURE );
        for ( RegFormTextVO signature: signatures )
        {
          if ( signature.getReportType().equals( ReportType.TRADEMARK_SEARCH ) )
          {
            departmentSignature = signature;
          }
        }
        if ( departmentSignature == null )
        {
          throw new IllegalStateException( "Found no signature for trademark search" );
        }
      }
      else
      {
        List<RegFormTextVO> signatures = new RegFormTextDAO().findReportTexts( language, ReportType.NO_TYPE, ip.getBAGISType(), ReportArea.SIGNATURE );
        departmentSignature = signatures.get( 0 );
      }

      List<String> signatureLines = new ArrayList<String>();

      // The department signature is always shown
      signatureLines.add( departmentSignature.getText() );

      // Get signature for user
      String userSignature = null;
      if ( ipTypeReportType.getSignatureType().equals( SignatureType.NOT_PRINTED )
          || ipTypeReportType.getSignatureType().equals( SignatureType.HAND_WRITTEN_SIGNATURE ) )
      {
        // Get the user's signature: A user may be predefined for the signature in the IP type report type.
        // This means that that user always signs the report. On the other hand, the user requesting the
        // report may also sign it.
        if ( ipTypeReportType.getEmployeeId() == null
            && ( ipTypeReportType.getSignatureType().equals( SignatureType.NOT_PRINTED ) || ipTypeReportType.getSignatureType().equals(
                SignatureType.HAND_WRITTEN_SIGNATURE ) ) )
        {
          userSignature = new MaDAO().findReportSignature( userNumber, ipTypeReportType.getSignatureType(), language );
        }
        else
        {
          userSignature = new MaDAO().findReportSignature( ipTypeReportType.getEmployeeId(), ipTypeReportType.getSignatureType(), language );
        }

        // The user actually has a signature.
        if ( userSignature != null )
        {
          if ( ipTypeReportType.getSignatureType().equals( SignatureType.NOT_PRINTED ) )
          {
            signatureLines.add( userSignature );
          }
          // If an original (hand-written) signature is required, some empty lines are inserted,
          // otherwise there's not enough space on the report.
          else if ( ipTypeReportType.getSignatureType().equals( SignatureType.HAND_WRITTEN_SIGNATURE ) )
          {
            signatureLines.add( HtmlConstants.BREAK_BREAK );
            signatureLines.add( HtmlConstants.BREAK_BREAK );
            signatureLines.add( userSignature );
          }
        }
      }
      return signatureLines;
    }
  }

  /**
   * Gets the signature stamp for the report.
   * @param ip The IP with all required data
   * @param reportType The report type
   * @return The filename of the signature stamp image to use; <code>null</code> if there's none
   */
  public static Stamp getStamp( GenericIP ip, ReportType reportType )
  {
    Stamp stamp = null;
    if ( Types.TRADEMARK.isInstance( ip )
        && ( reportType.equals( ReportType.CORRECTION_CONFIRMATION ) || reportType.equals( ReportType.REGISTRATION_CONFIRMATION ) ) )
    {
      stamp = new IgeBlackStamp();
    }
    return stamp;
  }
}
