package ch.ige.jipps.snapshot;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import ch.ige.ipobjects.Types;
import ch.ige.ipobjects.common.GenericIP;
import ch.ige.ipobjects.common.vo.ImageVO;
import ch.ige.ipobjects.design.Design;
import ch.ige.ipobjects.time.PointInTime;
import ch.ige.ipobjects.trademark.Trademark;
import ch.ige.util.text.Replacer;

/**
 * Builds the objects for the snapshot in the report.
 * For instance, the trademarks may have their state at registration as a snapshot.
 *
 * <p>Copyright (c) 2006, Swiss Federal Institute of Intellectual Property</p>
 * @author    Eric Lewis
 * @version   $Id: Snapshot.java,v 1.3 2008/05/19 13:41:02 lewis_lee Exp $
 */
public class Snapshot
{
  /**
   * The regular expression for getting an image reference
   */
  public static final Pattern IMAGE_REFERENCE_PATTERN = Pattern.compile( "(<reference type=\"image\">)\\s*(\\d+)\\s*(</reference>)" );

  /**
   * Throws out the stuff that's not needed from the XML document
   */
  private static String CLEAN_XML_PATTERN = ".*(<snapshot)[^>]+(>)";

  /**
   * Gets the snapshot for the report.
   * @param ip The IP with all required data
   * @param snapshotDate The date for which the snapshot is searched
   * @return The snapshot
   */
  public static SnapshotVO getSnapshot( GenericIP ip, PointInTime snapshotDate )
  {
    SnapshotVO snapshotVO = null;
    ch.ige.ipobjects.common.Snapshot snapshot = ip.findSnapshot( snapshotDate );
    if ( snapshot != null )
    {
      snapshotVO = new SnapshotVO();
      snapshotVO.setXml( Replacer.replaceFirst( snapshot.getXml(), CLEAN_XML_PATTERN, "$1$2" ) );
      snapshotVO.setImages( getImages( ip, snapshot ) );
    }
    return snapshotVO;
  }

  /**
   * Gets the images which are references within the snapshot.
   * @param ip The IP with all required data
   * @param snapshot The snapshot containing possible image references
   * @return A <code>Map</code> of all references images, where the image number is the key, and the corresponding image is the value
   */
  private static Map<Integer, ImageVO> getImages( GenericIP ip, ch.ige.ipobjects.common.Snapshot snapshot )
  {
    // This may seem a bit primitive... that's because it is!  ;-)
    // Seriously though, the overhead for unmarshaling seems high for a couple of image references
    // when the XML is just taken as is.
    Map<Integer, ImageVO> images = new HashMap<Integer, ImageVO>();

    Matcher matcher = IMAGE_REFERENCE_PATTERN.matcher( snapshot.getXml() );
    while ( matcher.find() )
    {
      Integer imageNumber = new Integer( matcher.group( 2 ) );
      List<ImageVO> ipImages = getImages( ip );
      ImageVO image = null;
      for ( ImageVO imageCandidate: ipImages )
      {
        if ( imageCandidate.getImageNumber().equals( imageNumber ) )
        {
          image = imageCandidate;
          break;
        }
      }
      if ( image == null )
      {
        throw new IllegalArgumentException( "Found no image for image number " + imageNumber );
      }

      images.put( imageNumber, image );
    }
    return images;
  }

  /**
   * Gets the IP's images.
   * @param ip The IP to get the images from
   * @return A <code>List</code> of images, which may be empty
   */
  private static List<ImageVO> getImages( GenericIP ip )
  {
    List<ImageVO> images = new ArrayList<ImageVO>();
    if ( Types.PATENT_GENERIC.isInstance( ip ) )
    {
      // There are no images in the patent,
      // only in the patent document
    }
    else if ( Types.TRADEMARK.isInstance( ip ) )
    {
      Trademark trademark = (Trademark) ip;
      images = trademark.getImages();
    }
    else if ( Types.DESIGN.isInstance( ip ) )
    {
      Design design = (Design) ip;
      images = design.getImages();
    }
    return images;
  }
}
