package ch.ige.jipps.test;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;

import org.apache.commons.io.FileUtils;

import ch.ige.ipobjects.ServicesFactory;
import ch.ige.jipps.LetterControl;
import ch.ige.jipps.configuration.PathConfiguration;
import ch.ige.jipps.constants.FileConstants;
import ch.ige.jipps.vo.PrintJobVO;
import ch.ige.util.enums.BAGISLanguageDomain;

/**
 * Tests {@link ch.ige.jipps.LetterControl}.
 * <p>
 * Copyright (c) 2007, Swiss Federal Institute of Intellectual Property
 * </p>
 * 
 * @author Eric Lewis
 * @version $Id: LetterControlTest.java,v 1.5 2008/07/22 11:46:51 lewis_lee Exp $
 */
public class LetterControlTest extends TestCase
{
  static
  {
    ServicesFactory.initStandaloneClient();
  }

  /**
   * The invoice/letter ID and address ID for a trademark renewal.
   */
  @SuppressWarnings( "unused" )
  private static final int[] RENEWAL = { 6521458, 4671259 };

  /**
   * The invoice/letter ID and address ID for an application confirmation
   * (<code>Hinterlegungsbescheinigung</code> in German) for an image trademark.
   */
  @SuppressWarnings( "unused" )
  private static final int[] APPLICATION_CONFIRMATION_IMAGE_TRADEMARK = { 6402024, 6152288 };

  /**
   * The invoice/letter ID and address ID for a letter that asks for a priority.
   */
  @SuppressWarnings( "unused" )
  private static final int[] PRIORITY_REQUEST = { 5879677, 500469 };

  /**
   * The unit under test
   */
  private LetterControl letterControl;

  /**
   * The print jobs to execute
   */
  private List<PrintJobVO> printJobs;

  /**
   * The only constructor.
   * 
   * @param name The test's name
   */
  public LetterControlTest( String name )
  {
    super( name );
    PathConfiguration.setPathsForLocalTest();
    this.letterControl = new LetterControl();
  }

  // @see junit.framework.TestCase#setUp()
  @Override
  protected void setUp() throws Exception
  {
    super.setUp();

    int[] ids = PRIORITY_REQUEST; // Change this to get a different kind of letter

    assertTrue( "Letter ID and address ID must come in pairs", ids.length % 2 == 0 );

    PrintJobVO printJob = new PrintJobVO();
    printJob.hasPotZahl = false;
    printJob.reportId = ids[0];
    printJob.adrNr = ids[1];
    printJob.printIgekopf = true;

    this.printJobs = new ArrayList<PrintJobVO>();
    this.printJobs.add( printJob );
  }

  /**
   * Tests {@link ch.ige.jipps.LetterControl#makePdf(List)}.
   * 
   * @throws Exception if the test fails
   */
  public void testMakePdf() throws Exception
  {
    byte[] pdfContent = this.letterControl.makePdf( this.printJobs );
    assertNotNull( "Checking that a PDF exists", pdfContent );
    writePdfFile( pdfContent );
  }

  /**
   * Tests {@link ch.ige.jipps.LetterControl#makePdf(List)} for a publication
   * confirmation, where the publication date must be found within the PDF.
   * 
   * @throws Exception if the test fails
   */
  public void testMakePdfForPublicationConfirmation() throws Exception
  {
    PrintJobVO publicationConfirmation = new PrintJobVO();
    publicationConfirmation.druckerId = 1;
    publicationConfirmation.senderApp = "BAGIS";
    publicationConfirmation.senderUser = "lewis_lee";
    publicationConfirmation.formTyp = "Letter";
    publicationConfirmation.reportId = 6757462;
    publicationConfirmation.schutitTyp = 417; // The form code!
    publicationConfirmation.sprache = BAGISLanguageDomain.DE.toDB();
    publicationConfirmation.maNr = 992;
    publicationConfirmation.hasPotZahl = true;
    publicationConfirmation.overwrite = true;

    List<PrintJobVO> publicationConfirmations = new ArrayList<PrintJobVO>();
    publicationConfirmations.add( publicationConfirmation );

    byte[] pdfContent = this.letterControl.makePdf( publicationConfirmations );
    assertNotNull( "Checking that a PDF exists", pdfContent );
    writePdfFile( pdfContent );
  }

  /**
   * Writes a PDF to a file.
   * @param pdfContent The PDF's content
   * @throws IOException if the file can't be written
   */
  private void writePdfFile( byte[] pdfContent ) throws IOException
  {
    File pdfFile = File.createTempFile( "letter", FileConstants.EXTENSION_PDF );
    FileUtils.writeByteArrayToFile( pdfFile, pdfContent );
    System.out.print( "PDF written to " + pdfFile );
  }
}
