package ch.ige.jipps.test;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;

import ch.ige.dbutils.test.TestCaseWithDBInit;
import ch.ige.ipobjects.ServicesFactory;
import ch.ige.jipps.ListenControl;
import ch.ige.jipps.configuration.PathConfiguration;
import ch.ige.jipps.constants.FileConstants;
import ch.ige.jipps.vo.PrintJobVO;

/**
 * Tests {@link ch.ige.jipps.ListenControl}.
 *
 * <p>Copyright (c) 2009, Swiss Federal Institute of Intellectual Property</p>
 * @author   lewis_lee
 * @version  $Id: ListenControlTest.java,v 1.2 2009/01/23 13:21:06 lewis_lee Exp $
 */
public class ListenControlTest extends TestCaseWithDBInit
{
  /**
   * The only constructor.
   * @param name The test's name
   */
  public ListenControlTest( String name )
  {
    super( name );
    ServicesFactory.getStandaloneServices(); // Initialize
    PathConfiguration.setPathsForLocalTest();
  }

  // @see junit.framework.TestCase#setUp()
  @Override
  protected void setUp() throws Exception
  {
    super.setUp();
  }

  /**
   * Tests the upper limit for search lists.
   * @throws Exception if the search list example can't be read 
   */
  public void testUpperSearchListLimit() throws Exception
  {
    // This contains 5 search hits.
    String searchList = IOUtils.toString( ListenControlTest.class.getResourceAsStream( "/res/test/search_list.xml" ), "UTF-8" );

    checkListLength( 6, true, searchList );
    checkListLength( 5, true, searchList );
    checkListLength( 4, false, searchList );
  }

  /**
   * Tests the rednering of the search list.
   * @throws Exception
   */
  public void testRendering() throws Exception
  {
    String searchList = IOUtils.toString( ListenControlTest.class.getResourceAsStream( "/res/test/long_search_list.xml" ), "UTF-8" );
    List<PrintJobVO> printJobs = new ArrayList<PrintJobVO>();
    printJobs.add( createPrintJob( searchList ) );
    byte[] pdfContent = new ListenControl().makePdf( printJobs );
    assertNotNull( "Checking that a PDF exists", pdfContent );
    assertTrue( "Checking that a PDF exists", pdfContent.length > 6000 );

    File pdfFile = File.createTempFile( "search_list", FileConstants.EXTENSION_PDF );
    FileUtils.writeByteArrayToFile( pdfFile, pdfContent );
    System.out.print( "PDF written to " + pdfFile );
  }

  /**
   * Checks the existing list, setting a maximum limit and testing.
   * @param upperLimit The upper limit of search hits
   * @param isValidUpperLimit If this is {@code true}, a PDF is expected, if it's {@code false}, an Exception is expected
   * @param searchList The XML document containing the search list
   * @throws Exception if the PDF rendering fails
   */
  private void checkListLength( int upperLimit, boolean isValidUpperLimit, String searchList ) throws Exception
  {
    System.setProperty( ListenControl.UPPER_LIST_LIMIT, Integer.toString( upperLimit ) );
    List<PrintJobVO> printJobs = new ArrayList<PrintJobVO>();
    printJobs.add( createPrintJob( searchList ) );
    if ( isValidUpperLimit )
    {
      byte[] pdf = new ListenControl().makePdf( printJobs );
      assertNotNull( "Checking that a PDF exists", pdf );
      assertTrue( "Checking that a PDF exists", pdf.length > 6000 );
    }
    else
    {
      try
      {
        new ListenControl().makePdf( printJobs );
        fail( "Search hits over limit must fail" );
      }
      catch ( IllegalArgumentException e )
      {
        assertTrue( "Checking error message", e.getMessage().contains( "Suchtreffer" ) );
      }
    }
  }

  /**
   * Creates a print job for search lists.
   * @param xmlDocument The XML document containing the search list
   * @return The print job containing the search list
   */
  private PrintJobVO createPrintJob( String xmlDocument )
  {
    PrintJobVO printJob = new PrintJobVO();
    printJob.listData = xmlDocument;
    printJob.formTyp = PrintJobVO.LISTEN;
    printJob.printIgekopf = true;
    return printJob;
  }

  // @see junit.framework.TestCase#tearDown()
  @Override
  protected void tearDown() throws Exception
  {
    System.setProperty( ListenControl.UPPER_LIST_LIMIT, "" );
    super.tearDown();
  }
}
