package ch.ige.jipps.vo;

import java.sql.Blob;

import org.apache.log4j.Logger;

import ch.ige.ipobjects.common.vo.ImageVO;
import ch.ige.ipobjects.design.model.DesignEvent;
import ch.ige.ipobjects.design.vo.ModOrdnrVO;
import ch.ige.ipobjects.exceptions.ImageDiscardedException;
import ch.ige.ipobjects.time.vo.TimePeriodVO;
import ch.ige.jipps.text.EventTextContainer;

/**
 * Contains one image for a design.
 * Note that a design consists of several order number which in return consist
 * of several images, like this.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author Esther Bandschapp
 * @author Eric Lewis
 * @version  $Id: DesignImageVO.java,v 1.6 2009/03/23 09:51:15 locher Exp $
 */
public class DesignImageVO extends TimePeriodVO implements EventTextContainer
{
  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( DesignImageVO.class );

  /**
   * @see #getImageId()
   */
  private Integer imageId;

  /**
   * @see #getNumberOfImagesForOrderNumber()
   */
  private Integer numberOfImagesForOrderNumber;

  /**
   * @see #getImageNumberForOrderNumber()
   */
  private Integer imageNumberForOrderNumber;

  /**
   * The order number containing all the information
   * about an order number which contains images.
   */
  private ModOrdnrVO orderNumberVO;

  /**
   * The image containing the image data
   */
  private ImageVO image;

  /**
   * The event which contains {@link #orderNumberVO} and thus this image
   */
  private DesignEvent event;

  /**
   * The event's text corresponding to the event number
   */
  private String eventText;

  /**
   * Constructor for Serializable.
   */
  public DesignImageVO()
  {
    // Nothing to do
  }

  /**
   * Creates a design image.
   * @param imageId The IP's unique image ID
   * @param imageNumberForOrderNumber
   * @param orderNumberVO The order number containing all the information
   *        about an order number which contains images
   * @param event The event to which this image belongs
   * @param image The image containing the image data
   */
  public DesignImageVO( Integer imageId, Integer imageNumberForOrderNumber, ModOrdnrVO orderNumberVO, ImageVO image, DesignEvent event )
  {
    this.imageId = imageId;
    if ( orderNumberVO.getImages() != null )
    {
      this.numberOfImagesForOrderNumber = new Integer( orderNumberVO.getImages().size() );
    }
    else
    {
      this.numberOfImagesForOrderNumber = new Integer( 0 );
    }
    this.imageNumberForOrderNumber = imageNumberForOrderNumber;
    this.orderNumberVO = orderNumberVO;
    this.image = image;
    this.event = event;
  }

  // See superclass
  public int getPeriodStartTime()
  {
    return this.orderNumberVO.getPeriodStartTime();
  }

  // See superclass
  public int getPeriodStopTime()
  {
    return this.orderNumberVO.getPeriodStopTime();
  }

  /**
   * Gets the order number for the image,
   * which identifies a design's group of images.
   * Note that it's <em>called</em> number, but it
   * doesn't have to be - the designer can call it
   * anything he likes.
   * @return The order number for the image
   */
  public String getOrderNumber()
  {
    return this.orderNumberVO.getOrdn_nr();
  }

  /**
   * Gets the technical identifier for an order number.
   * As long as this is the same for several images,
   * they belong to the same order number.
   * @return The technical identifier for an order number
   */
  public Integer getOrderNumberId()
  {
    return this.orderNumberVO.getModordnr_lnr();
  }

  /**
   * Gets the image number relative to an order number.
   * Every order number contains a number of images,
   * as returned by {@link #getNumberOfImagesForOrderNumber()}.
   * This returns the image number within the total number of
   * images, as in <code>1 of 3</code>, <code>2 of 3</code> and
   * <code>3 of 3</code>. 
   * @return The relative image number within an order number
   */
  public Integer getImageNumberForOrderNumber()
  {
    return this.imageNumberForOrderNumber;
  }

  /**
   * Gets the total number of images that a certain order number
   * has.
   * @return The total number of images for an order number
   */
  public Integer getNumberOfImagesForOrderNumber()
  {
    return this.numberOfImagesForOrderNumber;
  }

  /**
   * Gets the image itself, the image data.
   * @return The image data
   */
  public Blob getImage()
  {
    try
    {
      return this.image.getImage();
    }
    catch ( ImageDiscardedException e )
    {
      throw e.throwAsSystemException();
    }
  }

  /**
   * Gets the image ID as it's defined within
   * the database: Every IP has a number of images,
   * and each of them must have a image ID which
   * is unique for that IP.
   * @return The IP's unique image ID
   */
  public Integer getImageId()
  {
    return this.imageId;
  }

  /**
   * Gets the event which contains this image.
   * @return The event which contains this image
   */
  public DesignEvent getEvent()
  {
    return this.event;
  }

  // @see ch.ige.jipps.text.EventTextContainer#getEventNumber()
  public Integer getEventNumber()
  {
    if ( this.event == null )
    {
      return null;
    }
    else
    {
      return this.event.getNumericArt_nr();
    }
  }

  /**
   * <strong>This is ignored!</strong>
   * It only exist to implement {@link EventTextContainer}.
   */
  public void setEventNumber( Integer eventNumber )
  {
    LOG.warn( "setEventNumber called with value " + eventNumber );
  }

  // @see ch.ige.jipps.text.EventTextContainer#getEventText()
  public String getEventText()
  {
    return this.eventText;
  }

  // @see ch.ige.jipps.text.EventTextContainer#setEventText(java.lang.String)
  public void setEventText( String eventText )
  {
    this.eventText = eventText;
  }
}
