package ch.ige.pctedi.client.batch;

import java.util.Map;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.Options;

import ch.ige.pctedi.BuildInfo;
import ch.ige.pctedi.client.PctFile;
import ch.ige.pctedi.client.PctProperties;
import ch.ige.pctedi.client.PctZipFile;
import ch.ige.pctedi.client.gazette.Gazette;
import ch.ige.pctedi.client.wpa.Wpa;
import ch.ige.util.client.ClientLogInitalizer;
import ch.ige.util.client.batch.BatchException;
import ch.ige.util.client.batch.BatchReport;
import ch.ige.util.client.batch.BatchReportMail;
import ch.ige.util.client.batch.BatchReturnCode;
import ch.ige.util.client.batch.NewAbstractBatch;
import ch.ige.util.client.batch.OptionWithDefault;

/**
 * <p>
 * Copyright (c) 2007, Swiss Federal Institute of Intellectual Property
 * </p>
 * 
 * @author ruegsegger
 * @version $Id: Pctedi.java,v 1.8 2008/07/01 07:42:30 kuenzli Exp $
 */
public class Pctedi extends NewAbstractBatch
{
//  private static final Logger LOG = Logger.getLogger( "ch.ige.pctedi.client.batch.Pctedi" );

  /**
   * Used when the batch run can't be initialized.
   */
  private static final BatchReturnCode INITIALIZATION_FAILED = new BatchReturnCode( 1, "Fehler bei der Initialisierung des Batch Job: {0} / Properties: {1}" );

  /**
   * Used when the batch run can't be initialized.
   */
  private static final BatchReturnCode PROCESS_ZIP_FILE_FAILED = new BatchReturnCode( 2, "Fehler beim Bearbeiten des Zip-Files: {0} / Properties: {1}" );

  /**
   * The short option for the program mode.
   */
  private static final String PROGRAM_MODE = "m";

  /**
   * The long option for the program mode.
   */
  private static final String PROGRAM_MODE_LONG = "mode";

  /**
   * Possible values for program mode.
   * @author kuenzli
   *
   */
  public static enum ProgramMode
  {
    /**
     * Download all files. 
     */
    ALL,
    /**
     * Download wo-published-applications only. 
     */
    WPA,
    /**
     * Download wo-gazette-section-ii only. 
     */
    GAZETTE
  }

  /**
   * Current program mod.
   */
  private ProgramMode mod = ProgramMode.ALL;

  /**
   * The CLI options to use.
   */
  protected static final Options OPTIONS;

  /**
   * Name of this application (e.g. for reports, file names, ...)
   */
  protected static final String APPLICATION_NAME = "pctedi";

  static
  {
    /**
     * Logging
     * 
     * Log4J configures itself with the config file provided with the 'log4j.configuration' property.
     * If property 'log4j.configuration' is not set BATCH_CONFIG is used for configuartion 
     * @see ClientLogInitalizer
     */
    ClientLogInitalizer.initalizeLogging( ClientLogInitalizer.BATCH_CONFIG );
    // define program option
    OptionWithDefault programModeOption = new OptionWithDefault( PROGRAM_MODE, PROGRAM_MODE_LONG, true, "program mode" );
    programModeOption.setType( ProgramMode.class );
    programModeOption.setRequired( false );
    programModeOption.setDefaultValue( ProgramMode.ALL );

    OPTIONS = new Options();
    OPTIONS.addOption( programModeOption );
  }

  public Pctedi()
  {
    super( APPLICATION_NAME, OPTIONS );
  }

  protected Pctedi( String applicationName, Options options, boolean endWithSystemExit )
  {
    super( applicationName, options, endWithSystemExit );
  }

  /**
   * Programmeigenschaften anhand Property Datei setzen
   * 
   * @throws Exception
   */
  public void init() throws BatchException
  {
    LOG.debug( "init - begin" );

    try
    {
      LOG.debug( "Properties \n" + PctProperties.getInstance().toString() );
    }
    catch ( Exception ex )
    {
      throw new BatchException( INITIALIZATION_FAILED.formatMessage( ex.getMessage(), PctProperties.getInstance().toString() ) );
    }

    LOG.debug( "init - done" );
  }

  /**
   * Liest die Programm Parameter ein und setzt (falls ein Wert vorhanden) die entsprechenden Attribute.
   * 
   * @param args programm arguments 
   * @throws BatchException in case of wrong arguments
   */
  private void loadParams( Map<String, Object> parsedCommandLine ) throws BatchException
  {
    LOG.debug( "loadParams - begin" );
    this.mod = (ProgramMode) parsedCommandLine.get( PROGRAM_MODE );
    LOG.debug( "loadParams - done" );
  }

  private void start()
  {
    LOG.debug( "start process..." );

    PctFile pctDir = new PctFile();
    LOG.info( "process files in directory: " + pctDir.getPath() );

    Process wpa = new Wpa();
    Process gazette = new Gazette();

    PctZipFile[] zipFiles = pctDir.listPctZipFiles();

    if ( zipFiles != null && zipFiles.length > 0 )
    {
      for ( int i = 0; i < zipFiles.length; i++ )
      {
        LOG.info( "start process of file: " + zipFiles[i].getPath() );

        try
        {
          zipFiles[i].unzip();

          switch( this.mod )
          {
            case ALL:
              wpa.start( zipFiles[i] );
              writeBatchReportLine( wpa.getLogMessage() );
              gazette.start( zipFiles[i] );
              writeBatchReportLine( gazette.getLogMessage() );
              break;
            case WPA:
              wpa.start( zipFiles[i] );
              writeBatchReportLine( wpa.getLogMessage() );
              break;
            case GAZETTE:
              gazette.start( zipFiles[i] );
              writeBatchReportLine( gazette.getLogMessage() );
              break;
          }
          LOG.debug( "pack XML in a tar.gz and move to done" );
          zipFiles[i].packAndMoveToDone();

          LOG.debug( "delete unzip folder: " + zipFiles[i].getUnzipDirectory() );
          zipFiles[i].deleteUnzipDirecotry();

          LOG.debug( "delete file: " + zipFiles[i].getPath() );
          zipFiles[i].delete();
        }
        catch ( Exception ex )
        {
          throw new BatchException( PROCESS_ZIP_FILE_FAILED.formatMessage( ex.getMessage(), PctProperties.getInstance().toString() ) );
        }

      }
    }
    else
    {
      throw new BatchException( PROCESS_ZIP_FILE_FAILED.formatMessage( "no file found to process in directory: " + pctDir.getPath(), PctProperties
          .getInstance().toString() ) );
    }

    if ( wpa.isErrorOccured() || gazette.isErrorOccured() )
    {
      throw new BatchException( PROCESS_ZIP_FILE_FAILED.formatMessage( wpa.getLogMessage() + gazette.getLogMessage(), PctProperties.getInstance().toString() ) );
    }

    LOG.debug( "process done..." );
  }

  @Override
  public BatchReport[] getBatchReports() throws ch.ige.util.client.batch.BatchException
  {
    String smtpServer = PctProperties.getInstance().getProperty( PctProperties.MAIL_SMTP_HOST );
    String from = PctProperties.getInstance().getProperty( PctProperties.MAIL_SEND_FROM );
    String to = PctProperties.getInstance().getProperty( PctProperties.MAIL_SEND_TO );
    String cc = PctProperties.getInstance().getProperty( PctProperties.MAIL_SEND_CC );
    String subject = ch.ige.pctedi.BuildInfo.BUILD_ENV_NAME + " - " + "Batch Job: " + APPLICATION_NAME;

    return new BatchReport[] { new BatchReportMail( smtpServer, from, to, cc, subject ) };
  }

  @Override
  protected BatchReturnCode[] getPossibleReturnCodes()
  {
    return BatchReturnCode.getBatchReturnCodes( Pctedi.class );
  }

  // @see ch.ige.util.client.batch.NewAbstractBatch#run(org.apache.commons.cli.CommandLine, java.util.Map)
  @Override
  @SuppressWarnings( "unused" )
  protected void run( CommandLine commandLine, Map<String, Object> parsedCommandLine ) throws BatchException
  {
    LOG.debug( "### PCTEDI - main started ###" );

    loadParams( parsedCommandLine );
    init();
    start();

    LOG.debug( "### PCTEDI - main successfully done ###" );

  }

  public static void main( String[] args )
  {
    Pctedi pctedi = new Pctedi();
    NewAbstractBatch.start( pctedi, args );
  }

  @Override
  protected void checkCommandLine( CommandLine commandLine, Map<String, Object> parsedCommandLine ) throws IllegalArgumentException
  {
  }

  @Override
  protected String getBuildInfo()
  {
    return BuildInfo.BUILD_ID;
  }

  @Override
  protected String getCvsTag()
  {
    return BuildInfo.CVS_TAG;
  }


}