package ch.ige.rbbagisexport.server.filetransfer;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.Serializable;
import java.net.SocketException;
import java.util.List;

import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPReply;
import org.apache.log4j.Logger;

import ch.ige.rbbagisexport.common.RBBagisExportConfiguration;
import ch.ige.rbbagisexport.server.config.RBBagisExportConfigReader;
import ch.ige.rbbagisexport.server.datacontainer.ExportDataContainer;

/**
 * Transfers data of ExportDataContainer objects via FTP to a defined location.
 * 
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author M. Künzli
 * @version $Id: ExportFileHelper.java,v 1.12 2008/04/29 06:26:24 kuenzli Exp $
 */
public class ExportFileHelper implements Serializable
{
  //----------------------------------------------------------------------------
  // Private Members
  //----------------------------------------------------------------------------
  private transient FTPClient ftp = null;

  private RBBagisExportConfiguration configuration = RBBagisExportConfigReader.instance().getConfiguration();

  private static final Logger LOG = Logger.getLogger( ExportFileHelper.class );

  private static final long serialVersionUID = 1L;

  //----------------------------------------------------------------------------
  // Constructors
  //----------------------------------------------------------------------------

  //----------------------------------------------------------------------------
  // Public Methods
  //----------------------------------------------------------------------------
  /**
   * Write file from dataContainer to configured directory.
   * @param dataContainer
   * @throws IOException
   */
  public void writeFile( ExportDataContainer dataContainer ) throws IOException
  {
    if ( LOG.isDebugEnabled() )
    {
      LOG.debug( "writeFile(): Start" );
    }
    connect();
    try
    {
      ftp.setFileType( FTPClient.BINARY_FILE_TYPE );
      ftp.changeWorkingDirectory( configuration.getExportDirectory() );
      LOG.info( "Changed Directory to: " + configuration.getExportDirectory() );
      ByteArrayInputStream contentStream = new ByteArrayInputStream( dataContainer.getXml().getBytes( "UTF8" ) );
      ftp.storeFile( dataContainer.getFileName(), contentStream );
      if ( LOG.isDebugEnabled() )
      {
        LOG.debug( "writeFile(): File written: " + dataContainer.getFileName() );
      }
    }
    finally
    {
      disconnect();
    }
  }

  /**
   * Write all passed ExportDataContainer as XML to the destination.
   * @param exportList List of ExportDataContainer
   * @throws IOException
   */
  public void writeFiles( List<ExportDataContainer> exportList ) throws IOException
  {
    if ( exportList.isEmpty() )
    {
      return;
    }
    connect();
    ftp.setFileType( FTPClient.BINARY_FILE_TYPE );
    ftp.changeWorkingDirectory( configuration.getExportDirectory() );
    removeFiles();
    ByteArrayInputStream contentStream = null;
    for ( ExportDataContainer exportContainer: exportList )
    {
      contentStream = new ByteArrayInputStream( exportContainer.getXml().getBytes( "UTF8" ) );
      ftp.storeFile( exportContainer.getFileName(), contentStream );
      LOG.info( "writeFiles(): File written: " + exportContainer );
    }
    disconnect();
  }

  //----------------------------------------------------------------------------
  // Private Methods
  //----------------------------------------------------------------------------
  /**
   * Disconnect FTP connection.
   */
  private void disconnect()
  {
    if ( ftp.isConnected() )
    {
      try
      {
        ftp.logout();
        ftp.disconnect();
      }
      catch ( Exception e )
      {
        LOG.warn( "disconnect(): Error disconnecting from FTP server " + configuration.getExportServerName() + ";" + e.getMessage() );
      }
    }
  }

  /**
   * Establish FTP connection.
   * @throws IOException
   */
  private void connect() throws IOException
  {
    this.ftp = new FTPClient();
    try
    {
      this.ftp.connect( configuration.getExportServerName() );
      this.ftp.login( configuration.getExportFtpLogin(), configuration.getExportFtpPassword() );
    }
    catch ( SocketException e )
    {
      throw new IOException( e.getMessage() );
    }
    catch ( IOException e )
    {
      throw e;
    }
    if ( FTPReply.isPositiveCompletion( ftp.getReplyCode() ) )
    {
      LOG.debug( "connect(): Connected to " + configuration.getExportServerName() + "." );
      LOG.debug( ftp.getReplyString() );
    }
    else
    {
      throw new IOException( "FTP-Connection cannot properly be established." );
    }
  }

  /**
   * Remove all previously exported files in current directory.
   * @throws IOException
   */
  private void removeFiles() throws IOException
  {
    String[] files = this.ftp.listNames( ExportDataContainer.EXP_FILE_NAME_PATTERN );

    for ( String fileName: files )
    {
      if ( LOG.isDebugEnabled() )
      {
        LOG.debug( "removeFiles(): File to remove: " + fileName );
      }
      this.ftp.deleteFile( fileName );
    }
  }
}
