package ch.ige.rbbagisexport.server.xml;

import java.io.CharArrayWriter;
import java.io.IOException;
import java.rmi.ServerException;

import javax.xml.bind.JAXBException;
import javax.xml.transform.stream.StreamResult;

import org.apache.log4j.Logger;

import ch.ige.xmlutil.bind.Bindings;

/**
 * Class for transforming a JAXB tree to XML string.
 * 
 * @author kuenzli
 * @version   $Id: JAXBTransformer.java,v 1.7 2008/05/06 11:48:36 kuenzli Exp $
 */
public class JAXBTransformer
{
  // -----------------------------------------------------------------------------
  // Private Members
  // -----------------------------------------------------------------------------
  private static final Logger LOG = Logger.getLogger( JAXBTransformer.class );

  private String jaxbPackage = null;

  private String jaxbRootClass = null;

  // -----------------------------------------------------------------------------
  // Constructor
  // -----------------------------------------------------------------------------
  /**
   * Constructor that enforces definition of what the transformer would accept and
   * produce.
   * <p/>
   * Example:<br/>
   * <Code>JAXBTransformer transformer = new JAXBTransformer("ch.ige.chsearchreport",
   *                                                         "SearchReportData", "ch-search-report-v1-2-ige0-4.xsd");<Code/>
   * @param jaxbPackage The JAXB Package where the root Class is defined.
   * @param jaxbRootClass The root JAXB class of which objects will be transformed.
   * @param schema The schema name the generated XML should adhere to.
   * @throws JAXBException
   */
  public JAXBTransformer( String jaxbPackage, String jaxbRootClass, String schema ) throws JAXBException
  {
    this.jaxbPackage = jaxbPackage;
    this.jaxbRootClass = this.jaxbPackage + "." + jaxbRootClass;
  }

  // -----------------------------------------------------------------------------
  // Public Methods
  // -----------------------------------------------------------------------------
  /**
   * Transforms the passed JAXB tree (obj: root object) to an XML string.<p/>
   * Note:<br/>
   * It will be checked whether the passed object <code>obj</code> is of the
   * appropriate type (see constructor).
   * @param obj The root JAXB object of the tree that should be transformed.
   * @return The XML as String.
   * @throws IOException
   * @throws XMLException
   * @throws ClassNotFoundException
   * @throws ServerException
   * @throws JAXBException 
   */
  public String transformToXml( Object obj ) throws IOException, ClassNotFoundException, JAXBException
  {
    validate( obj );
    CharArrayWriter cw = new CharArrayWriter();
    String xml = null;
    try
    {
      // perfrom marshalling
      StreamResult sr = new StreamResult(cw);
      Bindings.marshal( Bindings.CHSEARCHREPORT, obj, sr );
      cw.flush();

      xml = cw.toString();
    }
    catch ( JAXBException jaex )
    {
      LOG.error( "transformToXml(): " + jaex.getMessage(), jaex );
      throw jaex;
    }
    finally
    {
      cw.close();
    }
    return xml;
  }

  /**
   * Validate passed JAXB object against the XML schema.
   * @param obj JAXB object.
   * @throws JAXBException
   * @throws ClassNotFoundException
   * @throws ServerException
   */
  public void validate( Object obj ) throws JAXBException, ClassNotFoundException
  {
    assertJAXBInstance( obj );
    Bindings.validate( Bindings.CHSEARCHREPORT, obj );
  }

  // -----------------------------------------------------------------------------
  // Private Methods
  // -----------------------------------------------------------------------------
  /**
   * Assert that passed Object <code>obj</code> is of the same type as JAXBRootClass
   * defined within the current instance.
   * @param obj
   * @throws ClassNotFoundException
   * @throws ServerException
   */
  private void assertJAXBInstance( Object obj ) throws ClassNotFoundException
  {
    // assert that the passed object is instance of the
    if ( !( Class.forName( this.jaxbRootClass ).isInstance( obj ) ) )
    {
      throw new ClassNotFoundException( "transform(): Passed object is not of type " + this.jaxbRootClass );
    }
  }
}
