package ch.ige.st36xmlgenerator;

/**
 * The implementation of the xmlgenerator for ST36 bibliographic data. <p> Copyright (c) 2007, Swiss Federal Institute
 * of Intellectual Property </p>
 * @author Patrizia Fabrizioli
 * @version $Id: ST36BiblGeneratorImpl.java,v 1.42 2008/10/06 16:14:49 fabrizioli Exp $
 */

import java.rmi.RemoteException;
import java.sql.Date;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.Map.Entry;

import javax.ejb.CreateException;
import javax.naming.NamingException;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import ch.ige.dbutils.dao.DAOPool;
import ch.ige.dbutils.sql.SQL;
import ch.ige.dbutils.sql.SQLFragment;
import ch.ige.dbutils.sql.SQLStatement;
import ch.ige.ipobjects.ServicesFactory;
import ch.ige.ipobjects.address.Address;
import ch.ige.ipobjects.address.dao.SchutitRegadrDAO;
import ch.ige.ipobjects.address.filter.AddressRoleFilter;
import ch.ige.ipobjects.address.vo.SchutitRegadrVO;
import ch.ige.ipobjects.address.vo.SchutitRegadrWithWaiverTypeVO;
import ch.ige.ipobjects.common.DetailTerminatedException;
import ch.ige.ipobjects.common.vo.PrioritaetVO;
import ch.ige.ipobjects.event.Event;
import ch.ige.ipobjects.event.EventServices;
import ch.ige.ipobjects.event.vo.EreignisVO;
import ch.ige.ipobjects.exceptions.IPObjectsSystemException;
import ch.ige.ipobjects.key.vo.IPKeyVO;
import ch.ige.ipobjects.patent.IPCClassificationValue;
import ch.ige.ipobjects.patent.KindCode;
import ch.ige.ipobjects.patent.PatentCH;
import ch.ige.ipobjects.patent.dao.PatDocSchemaDAO;
import ch.ige.ipobjects.patent.dao.PatKindcodeDAO;
import ch.ige.ipobjects.patent.filter.PatentDocumentKindCodeFilter;
import ch.ige.ipobjects.patent.model.PatentEvent;
import ch.ige.ipobjects.patent.vo.IPCVO;
import ch.ige.ipobjects.patent.vo.PatentDocumentKindcodeVO;
import ch.ige.ipobjects.patent.vo.PatentDocumentSchemaVO;
import ch.ige.ipobjects.patent.vo.PatentDocumentVO;
import ch.ige.ipobjects.services.AbstractServiceImpl;
import ch.ige.ipobjects.time.BAGISTime;
import ch.ige.ipobjects.time.PointInTimeCollection;
import ch.ige.ipobjects.time.TimePeriodCollection;
import ch.ige.ipobjects.time.vo.EreigIDat;
import ch.ige.st36xmlgenerator.builder.Factory.ST36Factory;
import ch.ige.st36xmlgenerator.dao.ST36BiblGeneratorConfigDAO;
import ch.ige.st36xmlgenerator.exception.InconsistentEventException;
import ch.ige.st36xmlgenerator.exception.ST36GenerationException;
import ch.ige.st36xmlgenerator.exception.ST36GenerationRuntimeException;
import ch.ige.st36xmlgenerator.format.Formatter;
import ch.ige.st36xmlgenerator.format.InventorAddressTypeFilter;
import ch.ige.st36xmlgenerator.metadata.JAXBMetadataExtractorFactory;
import ch.ige.st36xmlgenerator.metadata.XMLAccessorContainer;
import ch.ige.st36xmlgenerator.metadata.XMLGeneratorMetadataExtractor;
import ch.ige.st36xmlgenerator.metadata.XPathXMLAccessor.AttributeDefinition;
import ch.ige.util.client.ClientLogInitalizer;
import ch.ige.util.enums.BAGISLanguageDomain;
import ch.ige.util.enums.address.AddressRole;
import ch.ige.util.enums.address.WaiverType;
import ch.ige.xmlutil.doc.Validator;
import ch.ige.xmlutil.doc.XMLDocument;

/**
 * Implementation of the generator
 * @author fabrizioli
 *
 */
public class ST36BiblGeneratorImpl extends AbstractServiceImpl<Object> implements ST36BiblGenerator
{

  protected static final Logger LOG = Logger.getLogger( ST36BiblGeneratorImpl.class );

  protected Map<String, KindCode> kindcodesPerArticlenumber;

  protected ST36Factory theBuilder;

  protected XMLGeneratorMetadataExtractor theBuilderLogicForTheActualSchema;

  private static final boolean TEST = false;

  /**
   * AddressRole mapping to assignee inventor elements
   */
  private static final AddressRole ROLE_FOR_APPLICANTS_AND_OWNERS = AddressRole.OWNER; // assignee in xml = owner

  /**
   * AddressRole mapping to agent element
   */
  private static final AddressRole ROLE_FOR_AGENTS = AddressRole.AGENT;


  /**
   * Designation element in german patent application
   */
  private static final String DESIGNATION_APPLICATION_DE = "PATENTANMELDUNG";

  /**
   * Designation element in french patent application
   */
  private static final String DESIGNATION_APPLICATION_FR = "DEMANDE DE BREVET";

  /**
   * Designation element in italian patent application
   */
  private static final String DESIGNATION_APPLICATION_IT = "DOMANDA DI BREVETTO";

  /**
   * Designation element in german patent specification
   */
  private static final String DESIGNATION_PATENT_DE = "PATENTSCHRIFT";

  /**
   * Designation element in french patent specification
   */
  private static final String DESIGNATION_PATENT_FR = "FASCICULE DU BREVET";

  /**
   * Designation element in italian patent specification
   */
  private static final String DESIGNATION_PATENT_IT = "FASCICOLO DEL BREVETTO";

  /**
   * Element for first symbol position in classifications-ipc element
   */
  private static String IPC_FIRST_POSITION = "first-symbol-position";

  /**
   * Element for later symbol position in classifications-ipc element
   */
  private static String IPC_LATER_POSITION = "later-symbol-position";

  /**
   * Element for classification level in classifications-ipc element
   */
  private static String IPC_CLASSIFICATION_LEVEL = "ipcr-advanced";

  /**
   * Element for classification value "invention" in classifications-ipc element
   */
  private static String IPC_CLASSIFICATION_VALUE_INVENTIVE = "invention";

  /**
   * Element for classification value "non invention" in classifications-ipc element
   */
  private static String IPC_CLASSIFICATION_VALUE_NON_INVENTIVE = "non-invention";

  /**
   * Code for Swiss country in ch-st36 XML
   */
  private static final String CH_BLAND_CD = "CH";

  // implementation specific constants
  private static final String WAIVERTYPE_ONE = "one";

  private static final String WAIVERTYPE_ALL = "all";

  private static final String WAIVERTYPE_THEOTHER = "the-other";

  private static final String WAIVERTYPE_ALLOTHERS = "all-others";

  private static final String WAIVERTYPE_XMLELEMENT = "ch-inventor-entitlement-waiver";

  private static final String WAIVERTYPE_ATTRIBUTE = "waiver-type";

  private static final String TYPE_ENUMERATION = "enumeration";
  
  private Map<Integer, Boolean> mandatoryChoiceElements = new HashMap<Integer, Boolean>( 2 );

  private Map<Integer, Boolean> nonMandatoryChoiceElements = new HashMap<Integer, Boolean>( 2 );
 
  
  // @see ch.ige.ipobjects.services.AbstractServiceImpl#initService()
  protected synchronized void initService()
  {
    super.initService();
    PatentDocumentSchemaVO productiveSchema = this.getProductiveSchema();
    LOG.info( "The productive schema was successfully read " + productiveSchema.getId_string() );
    JAXBMetadataExtractorFactory.initialize( productiveSchema );
    this.kindcodesPerArticlenumber = getKindcodesPerArtikelumber();
    if ( this.kindcodesPerArticlenumber == null || this.kindcodesPerArticlenumber.size() == 0 )
    {
      throw new ST36GenerationRuntimeException( "Kindcodes were not initialized " );
    }
    theBuilder = new ST36Factory();
    if ( !theBuilder.getPhysicalSchemaLocation().endsWith( productiveSchema.getId_string() ) )
    {
      throw new ST36GenerationRuntimeException( "The Builder [" + theBuilder.getPhysicalSchemaLocation() + "] is not for the current productive schema "
          + productiveSchema.getId_string() );
    }
    theBuilderLogicForTheActualSchema = JAXBMetadataExtractorFactory.getInstance().getExtractor( productiveSchema.getId_string() );
  }

  protected void initStandalone()
  {
    ClientLogInitalizer.initalizeLogging();
    ServicesFactory.initStandaloneClient();
    this.initService();
  }

  protected void initJ2EE() throws NamingException, CreateException
  {
    this.initService();
    ServicesFactory.getStandaloneServices();
    // Not available in IGE-EAR ServicesFactory.getEJBServices();
  }

  private static final SQLStatement<EreignisVO> SELECTOR = SQL
      .select( "select /*+ INDEX(e IDX_EREIGNIS_I7) */  schutit_key, ereig_idat, publ_stat, trim(art_nr) as art_nr from ereignis join ereig_stat using (ereig_stat_nr) join schutit using (schutit_key) where reg_faehig = 1 and schutit_typ in (1) and schutit_nr is not null" );

  //  failurehandling: Failures that occur in the generation of one doc are collected and application goes on generating docs
  // If a failure elsewhere generation is stopped: Info mail with failure is sent.
  // Event validation only by setting TEST =false 
  // regadr must contain plz  Nein, OK
  // Main group in IPC Klassen enthält leading 0 Nein, OK

  @Override
  protected void registerDAOs()
  {
    // Common DAOs
    this.registerAndFetchDAO( SchutitRegadrDAO.class );

    // PatentCH DAOs
    this.registerAndFetchDAO( PatDocSchemaDAO.class );
    this.registerAndFetchDAO( PatKindcodeDAO.class );

    // service specific
    this.registerAndFetchDAO( ST36BiblGeneratorConfigDAO.class );

  }

  protected PatentDocumentSchemaVO getProductiveSchema()
  {
    return voWithoutExceptions( new Worklet<PatentDocumentSchemaVO>( "select productive patdoc schema" )
    {
      protected PatentDocumentSchemaVO call() throws Exception
      {
        List<PatentDocumentSchemaVO> schemas = fromPool( PatDocSchemaDAO.class ).selectAll();
        PatentDocumentSchemaVO productiveSchema = null;
        for ( PatentDocumentSchemaVO schema: schemas )
        {
          if ( schema.isProductiveVersion() )
          {
            if ( productiveSchema != null )
            {
              throw new IllegalStateException( "There can be only one productive schema " );
            }
            productiveSchema = schema;
          }
        }
        return productiveSchema;
      }
    } );
  }

  protected List<PatentDocumentKindcodeVO> getKindCodes()
  {
    return listWithoutExceptions( new Worklet<List<PatentDocumentKindcodeVO>>( "select all patdoc kind codes" )
    {
      protected List<PatentDocumentKindcodeVO> call() throws Exception
      {
        return fromPool( PatKindcodeDAO.class ).selectAll();
      }
    } );
  }

  protected Map<String, KindCode> getKindcodesPerArtikelumber()
  {
    // TODO init lazily
    List<PatentDocumentKindcodeVO> voList = this.getKindCodes();
    Map<String, KindCode> map = new HashMap<String, KindCode>( voList.size() );
    for ( PatentDocumentKindcodeVO vo: voList )
    {
      String artNr = vo.getArt_nr();
      KindCode nextKindCode = vo.getKind_cd();
      if ( KindCode.TO_BE_CONSIDERED_FOR_BIBLIOGRAPHIC_DATA_GENERATION.contains( nextKindCode ) )
      {
        if ( map.get( artNr ) == null )
        {
          map.put( artNr, nextKindCode );
        }
        else
        {
          throw new IllegalArgumentException( "The article number " + artNr + " has to define unambigously the kindcode for the bibliographic data " );
        }
      }
    }
    return map;
  }

  public Map<Integer, List<EreignisVO>> getEventsToPublicate( final ST36BiblGeneratorProperties props ) throws RemoteException
  {
    return new Worklet<Map<Integer, List<EreignisVO>>>( "events to publicate" )
    {
      protected Map<Integer, List<EreignisVO>> call() throws Exception
      {
        final EventServices services = ServicesFactory.getInstance();
        LOG.debug( "selecting keys for regular update" );
        return services.getEventsMappedByKey( getQueryForPublishingEvents( props ), new Object[] {} );
      }

      private SQLStatement<EreignisVO> getQueryForPublishingEvents( final ST36BiblGeneratorProperties props ) throws IPObjectsSystemException
      {
        return SELECTOR.append( createArtikelSelection( props.getArtikels() ) ).append( createTimeRange( props ) );
      }

      private SQLFragment createTimeRange( ST36BiblGeneratorProperties props )
      {
        SQLFragment fragment = SQL.fragment( "and ereig_idat" );
        if ( props.getEndPoint() == 0 )
        {
          fragment = fragment.append( " > " ).append( Integer.toString( props.getStartPoint() ) );
        }
        else
        {
          fragment = fragment.append( " between " ).append( Integer.toString( props.getStartPoint() + 1 ) ).append( " and " ).append(
              Integer.toString( props.getEndPoint() ) );
        }
        return fragment;
      }

      private SQLFragment createArtikelSelection( List<String> artikel )
      {
        SQLFragment fragment = SQL.fragment( "and art_nr in (" );
        if ( !artikel.isEmpty() )
        {
          fragment = appendArtikelFragment( fragment, artikel.get( 0 ) );
        }
        for ( String artnr: artikel )
        {
          fragment = appendArtikelFragment( fragment.append( "," ), artnr );
        }
        return fragment.append( ")" );

      }

      private SQLFragment appendArtikelFragment( SQLFragment fragment, String artnr )
      {
        return fragment.append( artnr );
      }

    }.getResultWithoutExceptions();
  }
  
  public Map<Integer, List<XMLDocumentWithMetadata>> generateBibliographicData( final Map<Integer, List<EreignisVO>> eventsPerSchutitKey,
      final ST36BiblGeneratorProperties props ) throws RemoteException
  {
    return new Worklet<Map<Integer, List<XMLDocumentWithMetadata>>>( "generate bibliographic data" )
    {
      protected Map<Integer, List<XMLDocumentWithMetadata>> call() throws Exception
      {
        Map<Integer, List<XMLDocumentWithMetadata>> result = createBibliographicData( eventsPerSchutitKey, props );
        if ( result != null && !result.isEmpty() )
        {
          updateStartEreigidat( props );
        }
        return result;
      }

    }.getResultWithoutExceptions(); // enforce rollback ( by now the only update to the DB is the last command )
  }


  protected Map<Integer, List<XMLDocumentWithMetadata>> createBibliographicData( Map<Integer, List<EreignisVO>> eventsPerSchutitKey,
      ST36BiblGeneratorProperties props ) throws InconsistentEventException
  {
    int successCount = 0;
    int failureCount = 0;
    Set<Integer> schutitKeys = eventsPerSchutitKey.keySet();
    Map<Integer, List<XMLDocumentWithMetadata>> map = new HashMap<Integer, List<XMLDocumentWithMetadata>>( eventsPerSchutitKey.size() );
    for ( Integer schutitKey: schutitKeys )
    {
      List<EreignisVO> events = eventsPerSchutitKey.get( schutitKey );
      List<XMLDocumentWithMetadata> documentList = new ArrayList<XMLDocumentWithMetadata>( events.size() );
      PatentCH patent = ServicesFactory.getInstance().findCHPatent( IPKeyVO.create( schutitKey ) );
      for ( Event eventForPublication: events )
      {
        try
        {
          if ( !TEST )
          {
            validateEvent( patent, eventForPublication, props );
          }
          documentList.add( createDocumentForEvent( patent, kindcodesPerArticlenumber.get( eventForPublication.getArt_nr() ), eventForPublication
              .getEreig_idat() ) );
          successCount++;
        }
        catch ( ST36GenerationException e )
        {
          LOG.error( "Generation of document failed: " + e.getMessage() );
          failureCount++;
          XMLDocumentWithMetadata doc = new XMLDocumentWithMetadata( null, patent.getRegisterNumber(), patent.getLanguage().toString(),
              kindcodesPerArticlenumber.get( eventForPublication.getArt_nr() ).toDB(), eventForPublication.getEreig_idat().getCompareTime() );
          doc.setErrorMsg( e.getMessage() );
          documentList.add( doc );
        }
        catch ( DetailTerminatedException e )
        {
          LOG.error( "The patent " + patent.getRegisterNumber() + " has not terminated periods when removing negative publ stats" );
          failureCount++;
          XMLDocumentWithMetadata doc = new XMLDocumentWithMetadata( null, patent.getRegisterNumber(), patent.getLanguage().toString(),
              kindcodesPerArticlenumber.get( eventForPublication.getArt_nr() ).toDB(), eventForPublication.getEreig_idat().getCompareTime() );
          doc.setErrorMsg( e.getMessage() );
          documentList.add( doc );
        }
        catch ( Exception e )
        {
          LOG.error( "In the generation of the bibliographic data of the patent " + patent.getRegisterNumber() + " an unexpected error occurred", e );
          failureCount++;
          XMLDocumentWithMetadata doc = new XMLDocumentWithMetadata( null, patent.getRegisterNumber(), patent.getLanguage().toString(),
              kindcodesPerArticlenumber.get( eventForPublication.getArt_nr() ).toDB(), eventForPublication.getEreig_idat().getCompareTime() );
          doc.setErrorMsg( e.getMessage() );
          documentList.add( doc );
        }

      }
      map.put( schutitKey, documentList );
    }
    LOG.info( "Successfully generated documents : " + successCount + "; Failures : " + failureCount );
    return map;
  }

  /**
   * Create the document for the event 
   * @param patent
   * @param kindCode
   * @param ereigidat
   * @return the document with metadata for storage
   * @throws ST36GenerationException
   * @throws DetailTerminatedException
   */
  public synchronized XMLDocumentWithMetadata createDocumentForEvent( PatentCH patent, KindCode kindCode, EreigIDat ereigidat ) throws ST36GenerationException,
      DetailTerminatedException
  {
    // create new root object and set listfactory = null
    theBuilder.reset();
    resetApplicationConfiguration();
    theBuilder.setRootAttributes( patent.getLanguage().name().toLowerCase(), BAGISTime.convertIDatToDate( ereigidat ), getProductionDate(), patent
        .getIPKey() );
    Set<Integer> inidCodes = theBuilderLogicForTheActualSchema.getAllInidCodes();
    patent.removeAllEventsAfter( EreigIDat.max( ereigidat.getDate() ) );
    patent.removeEventsWithNegativePublStat();
    if ( kindCode == null )
    {
      throw new IllegalArgumentException( "the kindcode cannot be null, event " + ereigidat + " patent :  " + patent );
    }
    if ( !KindCode.CH_APPLICATION_RELATED_KINDCODES.contains( kindCode ) )
    {
      // the applicant is a  mandatory element, as this is not set for all patent documentations, we need to give a default value for this.
      XMLAccessorContainer xmlAccessors71 = theBuilderLogicForTheActualSchema.getXMLAccessors( 71, KindCode.A1.toDB() );
      try
      {
        xmlAccessors71.createPathAndSetValue( theBuilder, new Object[] { "", "", "", "", "", "", "", "", "", "" } );
      }
      catch ( ClassNotFoundException e )
      {
        throw new ST36GenerationRuntimeException( "Error in intializing the applicants element ", e );
      }
    }
    for ( Integer inidCode: inidCodes )
    {
      LOG.debug( "Writing inid code " + inidCode + " for patent " + patent.getRegisterNumber() );
      XMLAccessorContainer xmlAccessors = theBuilderLogicForTheActualSchema.getXMLAccessors( inidCode, kindCode.toDB() );
      if ( xmlAccessors == null )
      {
        LOG.info( "no inid code " + inidCode + " for kindcode " + kindCode );
        continue;
      }
      try
      {
        List<Object[]> objectList = getValuesForInidCode( patent, kindCode, inidCode, xmlAccessors.getExpectedNumberOfValues(), ereigidat );
        // for the attributes that depend on the data (they are defined in getAttributesForInidCode): values are the same for multiple object arrays in objectList
        Map<String, List<AttributeDefinition>>[] attributes = getAttributesForInidCode( patent, kindCode, inidCode, xmlAccessors.getExpectedNumberOfValues(),
            ereigidat );
        for ( Object[] array: objectList )
        {
          xmlAccessors.createPathAndSetValue( theBuilder, array, attributes );
        }
      }
      catch ( Exception e )
      {
        String errorMsg = "Generation of data for inidCode " + inidCode + " and kindCode " + kindCode + " failed for patent " + patent.getRegisterNumber();
        LOG.error( errorMsg, e );
        throw new ST36GenerationException( errorMsg, e );
      }
    }
    if ( !configurationValid() )
    {
      throw new InvalidPatentDocumentException( "Mandatory Choice Element Validation for inidCode 72 failed for document " + patent.getRegisterNumber()
          + " (Schutit " + patent.getIPKey() + ") and event " + ereigidat );
    }
    try
    {
      XMLDocument document = theBuilder.getMarshalledDocument();
      LOG.info( "Successfully generated document for kindCode " + kindCode + " for patent " + patent.getRegisterNumber() );
      this.validate( document );
      LOG.info( "Successfully validated document for kindCode " + kindCode + " for patent " + patent.getRegisterNumber() );
      return new XMLDocumentWithMetadata( document, patent.getRegisterNumber(), patent.getLanguage().toString(), kindCode.toDB(), ereigidat.getCompareTime() );
    }
    catch ( Exception e )
    {
      String errorMsg = "Document generation failed for kindCode " + kindCode + " for patent " + patent.getRegisterNumber();
      LOG.error( errorMsg, e );
      throw new ST36GenerationException( errorMsg, e );
    }
  }



  public void updateStartEreigidat( ST36BiblGeneratorProperties props ) throws RemoteException
  {
    this.getDAO( ST36BiblGeneratorConfigDAO.class ).updateStartTime( props.getStartPoint() );
    LOG.info( "Updated configuration " + props );
  }

  public ST36BiblGeneratorProperties getConfiguration() throws RemoteException
  {
    ST36BiblGeneratorProperties props;
    try
    {
      props = ST36BiblGeneratorProperties.initializeFromDatabase();
      LOG.info( "Retreived configruation from DB " + props );
    }
    catch ( SQLException e )
    {
      String msg = "Could not load configruation " + e.getMessage();
      LOG.error( msg );
      throw new ST36GenerationRuntimeException( msg, e );
    }
    return props;
  }
  
  /**
   * Validate the doc
   */
  public synchronized void validate( XMLDocument document ) throws InvalidPatentDocumentException
  {
    try
    {
      Validator.validate( theBuilderLogicForTheActualSchema.getVerifierSchema(), document, null );
    }
    catch ( Exception e )
    {
      throw new InvalidPatentDocumentException( "not valid document: " + e );
    }

  }
  
  protected Date getProductionDate()
  {
    return BAGISTime.todayAsSqlDate();
  }
  
  private void resetApplicationConfiguration()
  {
    mandatoryChoiceElements.put( 72, false );
    nonMandatoryChoiceElements.put( 60, false );
  }

  private boolean configurationValid()
  {
    for ( Entry<Integer, Boolean> mandatory: mandatoryChoiceElements.entrySet() )
    {
      //   sobald es mehrere gibt, sammle fehlerhafte mandatory elements und gebe sie zurück, zur Zeit nur eines vorhanden
      if ( !mandatory.getValue() )
      {
        return false;
      }
    }
    return true;

  }

  // return the list of attributes per element name (element name can occur only once in path)
  private Map<String, List<AttributeDefinition>>[] getAttributesForInidCode( PatentCH patent, KindCode kindCode, Integer inidCode, int expectedNumberOfValues,
      EreigIDat eventdate ) throws SQLException
  {
    Map<String, List<AttributeDefinition>> attributeMap = new HashMap<String, List<AttributeDefinition>>( 2 );
    Map<String, List<AttributeDefinition>>[] attributeMapList = new Map[1];
    attributeMapList[0] = attributeMap;
    switch( inidCode )
    {
      case 172:
      {
        /*
        NB: keine Überprüfung, dass der Eintrag gültig ist (also keine weiterer Erfinder besteht)
        NB: Sprache dieses Eintrages wird nicht überprüft (gleich Sprache des Patents)
        */
        TimePeriodCollection<SchutitRegadrWithWaiverTypeVO> addresses = DAOPool.get( SchutitRegadrDAO.class ).selectInventorAddressesWithWaiverType( patent )
            .getPeriodsContainingTime( EreigIDat.max( eventdate.getDate() ) ); //(TimePeriodCollection<SchutitRegadrVO>) AddressRoleFilter.filterSingle(ROLE_FOR_INVENTORS, patent.getRegAdr().getPeriodsContainingTime(EreigIDat.max( eventdate.getDate())));
        // not null guaranteed
        addresses = filterWaiverTypes( addresses );
        for ( Address address: addresses )
        {
          if ( address instanceof SchutitRegadrWithWaiverTypeVO )
          {
            SchutitRegadrWithWaiverTypeVO waiverAddress = (SchutitRegadrWithWaiverTypeVO) address;
            if ( WaiverType.ONE.contains( waiverAddress.getWaiverType() ) )
            {
              AttributeDefinition def = new AttributeDefinition( WAIVERTYPE_ATTRIBUTE, WAIVERTYPE_ONE, TYPE_ENUMERATION );
              attributeMap.put( WAIVERTYPE_XMLELEMENT, Collections.singletonList( def ) );
            }
            else if ( WaiverType.ALL.contains( waiverAddress.getWaiverType() ) )
            {
              AttributeDefinition def = new AttributeDefinition( WAIVERTYPE_ATTRIBUTE, WAIVERTYPE_ALL, TYPE_ENUMERATION );
              attributeMap.put( WAIVERTYPE_XMLELEMENT, Collections.singletonList( def ) );
            }
            else if ( WaiverType.THE_OTHER.contains( waiverAddress.getWaiverType() ) )
            {
              AttributeDefinition def = new AttributeDefinition( WAIVERTYPE_ATTRIBUTE, WAIVERTYPE_THEOTHER, TYPE_ENUMERATION );
              attributeMap.put( WAIVERTYPE_XMLELEMENT, Collections.singletonList( def ) );
            }
            else if ( WaiverType.ALL_OTHERS.contains( waiverAddress.getWaiverType() ) )
            {
              AttributeDefinition def = new AttributeDefinition( WAIVERTYPE_ATTRIBUTE, WAIVERTYPE_ALLOTHERS, TYPE_ENUMERATION );
              attributeMap.put( WAIVERTYPE_XMLELEMENT, Collections.singletonList( def ) );
            }

          }
          else
          {
            throw new IllegalArgumentException( "Address must be with waiver type after filtering" );
          }
        }

      }
    }
    return attributeMap.isEmpty() ? null : attributeMapList;
  }

  private List<Object[]> getValuesForInidCode( PatentCH patent, KindCode kindCode, Integer inidCode, int expectedNumberOfValues, EreigIDat eventdate )
      throws SQLException, InvalidPatentDocumentException
  {
    List<Object[]> valuesList = new ArrayList<Object[]>( 2 );
    Object[] values = new Object[expectedNumberOfValues];
    valuesList.add( values );
    switch( inidCode )
    {

      case 11:
      {
        exceptionForExpectedValueMismatch( 3, expectedNumberOfValues, inidCode );
        values[0] = Formatter.formatPatentNumber( patent.getRegisterNumber(), patent.getBAGISType() );
        values[1] = "CH";
        values[2] = kindCode.name();
        break;

      }
      case 12:
      {

        exceptionForExpectedValueMismatch( 1, expectedNumberOfValues, inidCode );
        if ( KindCode.CH_APPLICATION_RELATED_KINDCODES.contains( kindCode ) )
        {
          values[0] = resolveDesignationForApplication( patent.getLanguage() );
        }
        else
        {
          values[0] = resolveDesignationForRegistration( patent.getLanguage() );
        }
        break;

      }
      case 21:
      {
        exceptionForExpectedValueMismatch( 2, expectedNumberOfValues, inidCode );
        if ( patent.getApplicationNumber() != null )
        {
          values[0] = Formatter.formatApplicationNumber( patent.getApplicationNumber(), patent.getBAGISType() );
          values[1] = "CH";
        }
        break;
      }
      case 22:
      {
        exceptionForExpectedValueMismatch( 1, expectedNumberOfValues, inidCode );
        if ( patent.getApplicationDate() != null )
        {
          values[0] = patent.getApplicationDate();
        }
        break;
      }
      case 24:
      {
        exceptionForExpectedValueMismatch( 1, expectedNumberOfValues, inidCode );
        if ( patent.hasRegistrationEvent() )
        {
          values[0] = patent.getRegistrationEvent().getDate();
        }
        break;
      }
      case 30:
      {
        exceptionForExpectedValueMismatch( 3, expectedNumberOfValues, inidCode );
        // not null guaranteed
        TimePeriodCollection<PrioritaetVO> priorities = patent.getPrios().getPeriodsContainingTime( EreigIDat.max( eventdate.getDate() ) );
        iterateAndInsert( patent, priorities, expectedNumberOfValues, valuesList );
        break;
      }
      case 43:
      {
        exceptionForExpectedValueMismatch( 3, expectedNumberOfValues, inidCode );
        if ( patent.hasPublicationOfApplication() )
        {
          values[0] = "CH";
          values[1] = "";
          values[2] = patent.getPublicationOfApplication().getDate();
        }
        break;
      }
      case 45:
      {
        exceptionForExpectedValueMismatch( 1, expectedNumberOfValues, inidCode );
        if ( patent.hasRegistrationEvent() )
        {
          values[0] = patent.getRegistrationEvent().getDate();
        }
        break;
      }
      case 51:
      {
        // main-group without leading 0 is ok 
        exceptionForExpectedValueMismatch( 9, expectedNumberOfValues, inidCode );
        List<IPCVO> ipcs = new ArrayList<IPCVO>( patent.getIPC() );
        Iterator<IPCVO> ipcIterator = ipcs.iterator();
        while ( ipcIterator.hasNext() )
        {
          if ( ipcIterator.next().isPreClassification() )
          {
            ipcIterator.remove();
          }
        }
        iterateAndInsert( patent, ipcs, expectedNumberOfValues, valuesList );
        break;
      }
      case 54:
      {
        exceptionForExpectedValueMismatch( 1, expectedNumberOfValues, inidCode );
        if ( patent.getDescription() != null )
        {
          values[0] = patent.getDescription().trim();
        }
        break;
      }
      case 60: // choice with 62
      {
        exceptionForExpectedValueMismatch( 2, expectedNumberOfValues, inidCode );
        Long relatedDocumentNumber = patent.getPatentNumberOfRelatedPatent();
        if ( relatedDocumentNumber != null )
        {
          values[0] = "CH";
          values[1] = Formatter.formatPatentNumber( relatedDocumentNumber, patent.getBAGISType() );
          if ( nonMandatoryChoiceElements.get( 60 ).equals( Boolean.TRUE ) )
          {
            throw new InvalidPatentDocumentException( "Choice Element Validation for inidCode 60/62 failed in document " + patent.getRegisterNumber()
                + " (Schutit " + patent.getIPKey() + ") and event " + eventdate + ", when creating inid " + inidCode );
          }
          nonMandatoryChoiceElements.put( 60, true );
        }
        break;
      }
      case 62: // choice with 60
      {
        exceptionForExpectedValueMismatch( 2, expectedNumberOfValues, inidCode );
        String divisionAppNumber = patent.getApplicationNumberOfEarlierApplication();
        if ( divisionAppNumber != null )
        {
          values[0] = "CH";
          values[1] = Formatter.formatApplicationNumber( divisionAppNumber, patent.getBAGISType() );
          if ( nonMandatoryChoiceElements.get( 60 ).equals( Boolean.TRUE ) )
          {
            throw new InvalidPatentDocumentException( "Choice Element Validation for inidCode 60/62 failed in document " + patent.getRegisterNumber()
                + " (Schutit " + patent.getIPKey() + ") and event " + eventdate + ", when creating inid " + inidCode );
          }
          nonMandatoryChoiceElements.put( 60, true );
        }
        break;
      }
      case 71:
      {
        exceptionForExpectedValueMismatch( 10, expectedNumberOfValues, inidCode );
        TimePeriodCollection<Address> addresses = (TimePeriodCollection<Address>) AddressRoleFilter.filterSingle( ROLE_FOR_APPLICANTS_AND_OWNERS, patent
            .getRegAdr().getPeriodsContainingTime( EreigIDat.max( eventdate.getDate() ) ) );
        // not null guaranteed
        if ( addresses == null || addresses.isEmpty() )
        {
          values[0] = "";
          values[1] = "";
          values[2] = "";
          values[3] = "";
          values[4] = "";
          values[5] = "";
          values[6] = "";
          values[7] = "";
          values[8] = "";
          values[9] = "";
        }
        else
        {
          iterateAndInsert( patent, addresses, expectedNumberOfValues, valuesList );
          for ( Object[] array: valuesList )
          {
            array[8] = "";
            array[9] = array[7]; // ? "CH"
          }
        }
        break;
      }
      case 172: // choice with 72 within element in container, no collision guaranteed by procedure
      {
        exceptionForExpectedValueMismatch( 1, expectedNumberOfValues, inidCode );
        TimePeriodCollection<SchutitRegadrWithWaiverTypeVO> addresses = DAOPool.get( SchutitRegadrDAO.class ).selectInventorAddressesWithWaiverType( patent )
            .getPeriodsContainingTime( EreigIDat.max( eventdate.getDate() ) ); //(TimePeriodCollection<SchutitRegadrVO>) AddressRoleFilter.filterSingle(ROLE_FOR_INVENTORS, patent.getRegAdr().getPeriodsContainingTime(EreigIDat.max( eventdate.getDate())));
        // not null guaranteed
        addresses = filterWaiverTypes( addresses );
        if ( addresses.isEmpty() )
        {
          break;
        }
        iterateAndInsert( patent, addresses, expectedNumberOfValues, valuesList );
        mandatoryChoiceElements.put( 72, true );
        break;
      }
      case 72: // choice with 172 within element in container, no collision guaranteed by procedure
      {
        exceptionForExpectedValueMismatch( 8, expectedNumberOfValues, inidCode );
        TimePeriodCollection<SchutitRegadrWithWaiverTypeVO> addresses = DAOPool.get( SchutitRegadrDAO.class ).selectInventorAddressesWithWaiverType( patent )
            .getPeriodsContainingTime( EreigIDat.max( eventdate.getDate() ) ); //(TimePeriodCollection<SchutitRegadrVO>) AddressRoleFilter.filterSingle(ROLE_FOR_INVENTORS, patent.getRegAdr().getPeriodsContainingTime(EreigIDat.max( eventdate.getDate())));
        // not null guaranteed
        addresses = filterNonWaiverTypes( addresses );
        if ( addresses.isEmpty() )
        {
          break;
        }
        iterateAndInsert( patent, addresses, expectedNumberOfValues, valuesList );
        mandatoryChoiceElements.put( 72, true );
        break;
      }
      case 73:
      {
        exceptionForExpectedValueMismatch( 8, expectedNumberOfValues, inidCode );
        TimePeriodCollection<Address> addresses = AddressRoleFilter.filterSingle( ROLE_FOR_APPLICANTS_AND_OWNERS, patent.getRegAdr().getPeriodsContainingTime(
            EreigIDat.max( eventdate.getDate() ) ) );
        // not null guaranteed
        iterateAndInsert( patent, addresses, expectedNumberOfValues, valuesList );
        break;
      }
      case 74:
      {
        exceptionForExpectedValueMismatch( 8, expectedNumberOfValues, inidCode );
        TimePeriodCollection<Address> addresses = AddressRoleFilter.filterSingle( ROLE_FOR_AGENTS, patent.getRegAdr().getPeriodsContainingTime(
            EreigIDat.max( eventdate.getDate() ) ) );
        // not null guaranteed
        iterateAndInsert( patent, addresses, expectedNumberOfValues, valuesList );
        break;
      }
      case 86:
      {
        exceptionForExpectedValueMismatch( 2, expectedNumberOfValues, inidCode );
        String pctApplicationNumber = patent.getPCTApplicationNumber();
        if ( pctApplicationNumber != null )
        {
          String[] tmpvalues = Formatter.formatPCTApplicationNumber( pctApplicationNumber, patent.getBAGISType() );
          values[0] = tmpvalues[0];
          values[1] = tmpvalues[1];
        }
        break;
      }
      case 87:
      {
        exceptionForExpectedValueMismatch( 3, expectedNumberOfValues, inidCode );
        String pctPublicationNumber = patent.getPCTPublicationNumber();
        if ( pctPublicationNumber != null )
        {
          values[0] = "WO"; // WO  OK gemäss wml
          values[1] = Formatter.formatPCTPublicationNumber( pctPublicationNumber, patent.getBAGISType() );
          values[2] = patent.getPCTPublicationDate();
        }
        break;
      }
      case 88:
      {
        exceptionForExpectedValueMismatch( 3, expectedNumberOfValues, inidCode );
        EreigIDat eventDateForINID88 = null;
        if ( !kindCode.equals( KindCode.A3 ) )
        {
          // if the current publication is not the A3 publication, an A3 document that is already published has to exist for the inid 88
          TimePeriodCollection<PatentDocumentVO> vo = PatentDocumentKindCodeFilter.filterSingle( KindCode.A3, patent.getDocuments().getPeriodsContainingTime(
              EreigIDat.max( eventdate.getDate() ) ) );
          if ( vo != null && !vo.isEmpty() )
          {
            eliminateNotPublishedDocuments( vo, patent );
            if ( vo.size() != 1 )
            {
              throw new IllegalArgumentException( "There can be only one publication of the search report, but found " + vo.size() + " for patent "
                  + patent.getRegisterNumber() );
            }
            eventDateForINID88 = vo.get( 0 ).getEreig_idat();
          }
        }
        else
        {
          // already checked that this event has the article number for A3 publication
          eventDateForINID88 = eventdate;
        }
        if ( eventDateForINID88 != null )
        {
          Date date = BAGISTime.convertIDatToDate( eventDateForINID88 );
          values[0] = date;
          values[1] = date;
          values[2] = ""; // gazette -num
        }
        break;
      }
    }
    return valuesList;
  }

  private String resolveDesignationForApplication( BAGISLanguageDomain language )
  {
    if ( language.equals( BAGISLanguageDomain.DE ) )
    {
      return DESIGNATION_APPLICATION_DE;
    }
    else if ( language.equals( BAGISLanguageDomain.FR ) )
    {
      return DESIGNATION_APPLICATION_FR;
    }
    else if ( language.equals( BAGISLanguageDomain.IT ) )
    {
      return DESIGNATION_APPLICATION_IT;
    }
    else
      throw new IllegalArgumentException( "language not known " + language );
  }

  private String resolveDesignationForRegistration( BAGISLanguageDomain language )
  {
    if ( language.equals( BAGISLanguageDomain.DE ) )
    {
      return DESIGNATION_PATENT_DE;
    }
    else if ( language.equals( BAGISLanguageDomain.FR ) )
    {
      return DESIGNATION_PATENT_FR;
    }
    else if ( language.equals( BAGISLanguageDomain.IT ) )
    {
      return DESIGNATION_PATENT_IT;
    }
    else
      throw new IllegalArgumentException( "language not known " + language );
  }

  private TimePeriodCollection<SchutitRegadrWithWaiverTypeVO> filterNonWaiverTypes( TimePeriodCollection<SchutitRegadrWithWaiverTypeVO> addresses )
  {
    return InventorAddressTypeFilter.filterSingle( WaiverType.NONE, addresses );
  }

  private TimePeriodCollection<SchutitRegadrWithWaiverTypeVO> filterWaiverTypes( TimePeriodCollection<SchutitRegadrWithWaiverTypeVO> addresses )
  {

    return InventorAddressTypeFilter.filterAnyWaiverType( addresses );
  }

  private void eliminateNotPublishedEvents( PointInTimeCollection<? extends Event> events, PatentCH patent )
  {
    Iterator<? extends Event> eventIterator = events.iterator();
    while ( eventIterator.hasNext() )
    {
      Event event = eventIterator.next();
      if ( event != null && !PatentEvent.wrap( event ).isPublished() )
      {
        eventIterator.remove();
      }
    }
  }

  private void eliminateNotPublishedDocuments( TimePeriodCollection<PatentDocumentVO> volist, PatentCH patent )
  {
    Iterator<PatentDocumentVO> iterator = volist.iterator();
    while ( iterator.hasNext() )
    {
      PatentDocumentVO vo = iterator.next();
      if ( vo.getEreig_idat() == EreigIDat.NULL || !PatentEvent.wrap( patent.getEvents().get( vo.getEreig_idat() ) ).isPublished() )
      {
        iterator.remove();
      }
    }

  }

  private <X> void iterateAndInsert( PatentCH patent, List<X> list, int expectedNumberOfValues, List<Object[]> valuesList )
  {
    if ( !list.isEmpty() )
    {
      Iterator<X> iterator = list.iterator();
      X vo = iterator.next();
      Object[] values = valuesList.get( 0 );
      if ( vo instanceof IPCVO )
      {
        values[7] = IPC_FIRST_POSITION;
      }
      insertValues( values, vo );
      while ( iterator.hasNext() )
      {
        vo = iterator.next();
        values = new Object[expectedNumberOfValues];
        valuesList.add( values );
        if ( vo instanceof IPCVO )
        {
          values[7] = IPC_LATER_POSITION;
        }
        insertValues( values, vo );
      }
    }
  }

  private void insertValues( Object[] values, Object vo )
  {
    if ( vo instanceof PrioritaetVO )
    {
      PrioritaetVO priority = (PrioritaetVO) vo;
      values[0] = priority.getBland_cd();
      values[1] = priority.getPrio_nr();
      values[2] = priority.getPrio_dat();
    }

    else if ( vo instanceof IPCVO )
    {
      IPCVO ipc = (IPCVO) vo;
      
      String ipcSymb = ipc.getIpc_symb();
      if ( ipcSymb.length() < 8 || ipc.getIpc_gueltig_von() == null )
      {
        throw new IllegalArgumentException( "Ipc classes are not valid: " + ipcSymb + ", valid from " + ipc.getIpc_gueltig_von() );
      }
      values[0] = ipcSymb.substring( 0, 1 ); // section
      values[1] = ipcSymb.substring( 1, 3 ); // class
      values[2] = ipcSymb.substring( 3, 4 ); // subclass
      values[3] = Integer.valueOf( ipcSymb.substring( 4, 8 ) ); //main-group
      values[4] = ipcSymb.substring( 8 ); // subgroup
      values[5] = ipc.getIpc_gueltig_von(); //  ipc-version-indicator/date
      values[6] = IPC_CLASSIFICATION_LEVEL; // classification-level
      values[8] = ipc.getClass_value() == IPCClassificationValue.INVENTIVE ? IPC_CLASSIFICATION_VALUE_INVENTIVE : IPC_CLASSIFICATION_VALUE_NON_INVENTIVE;
    }
    else if ( vo instanceof SchutitRegadrWithWaiverTypeVO )
    {
      SchutitRegadrWithWaiverTypeVO address = (SchutitRegadrWithWaiverTypeVO) vo;
      if ( address.getWaiverType() == WaiverType.NONE )
      {
        formatAddress( values, (SchutitRegadrVO) vo );
      }
      else
      {
        int i = 0;
        for ( ; i < values.length - 1; i++ )
        {
          values[i] = "";
        }
        values[i] = address.getReg_name().trim();
      }
    }
    else if ( vo instanceof SchutitRegadrVO )
    {
      formatAddress( values, (SchutitRegadrVO) vo );
    }

  }



  protected void formatAddress( Object[] values, SchutitRegadrVO vo )
  {
    //  PLZ is not  always expected for CH adresses
    values[0] = vo.getReg_name() == null ? "" : vo.getReg_name().trim();
    String[] valuesForStreet = splitStreet( vo.getReg_strasse() );
    values[1] = valuesForStreet[0] == null ? "" : valuesForStreet[0];
    values[2] = valuesForStreet[1] == null ? "" : valuesForStreet[1];
    values[3] = valuesForStreet[2] == null ? "" : valuesForStreet[2];
    values[4] = valuesForStreet[3] == null ? "" : valuesForStreet[3];
    if ( vo.getBland_cd().equals( CH_BLAND_CD ) )
    {
      String[] valuesForCity = splitCity( vo.getReg_ort() );
      values[5] = valuesForCity[1] == null ? "" : valuesForCity[1];
      values[6] = valuesForCity[0] == null ? "" : valuesForCity[0];
    }
    else
    {
      values[5] = vo.getReg_ort() == null ? "" : vo.getReg_ort().trim();
      values[6] = "";
    }

    values[7] = vo.getBland_cd() == null ? "" : vo.getBland_cd().trim();
  }

  private String[] splitCity(String reg_ort) {
    // unfortunately the transformation to pdf foresees the 
    String[] plzCity = new String[2];
    if ( reg_ort == null || reg_ort.trim().equals("-") ) // - occurs with waivertype
    {
      return plzCity;
    }
    reg_ort = reg_ort.trim();
    int firstSpace = reg_ort.indexOf(' ');
    if (firstSpace < 0 || reg_ort.length() < 6)
    {
      plzCity[0] = "";
      plzCity[1] = reg_ort.trim(); 
      return plzCity;
    }
    
    // try to define the postcode
    plzCity[0] = reg_ort.substring( 0, firstSpace ).trim();

    // if the postcode is not composed by PLZ only
    if ( plzCity[0].length()  != 4  ||  !StringUtils.isNumeric(plzCity[0]))
    {
      plzCity[0] = "";
      plzCity[1] = reg_ort.trim(); 
      return plzCity;
    }
    plzCity[1] = reg_ort.substring( firstSpace + 1 ).trim();
    return plzCity;
  }

  private String[] splitStreet( String reg_strasse )
  {

    String[] streets = new String[4];
    streets[0] = "";
    streets[1] = "";
    streets[2] = "";
    if ( reg_strasse != null && reg_strasse.contains( "," ) )
    {
      StringTokenizer tokenizer = new StringTokenizer( reg_strasse.trim(), "," );
      int count = tokenizer.countTokens();
      if ( tokenizer.countTokens() > 1 )
      {
        StringBuilder reg_strasseNew = new StringBuilder();
        reg_strasseNew.append( tokenizer.nextElement() );
        int i = 1;
        while ( tokenizer.hasMoreElements() && count - i > 3 )
        {
          reg_strasseNew.append( ',' );
          reg_strasseNew.append( tokenizer.nextElement() );
          i++;
        }
        reg_strasse = reg_strasseNew.toString().trim();

        int j = 0;
        while ( tokenizer.hasMoreElements() )
        {
          streets[j++] = ( (String) tokenizer.nextElement() ).trim();
        }
      }
      else
      // only one token,  remove komma
      {
        reg_strasse = tokenizer.nextToken().trim();
      }
    }
    streets[3] = reg_strasse != null ? reg_strasse.trim() : "";
    return streets;
  }

  private void exceptionForExpectedValueMismatch( int availableNumberOfValues, int expectedNumberOfValues, int inidCode )
  {
    if ( expectedNumberOfValues != availableNumberOfValues )
    {
      throw new IllegalArgumentException( "Available values for inidCode " + inidCode + ": " + availableNumberOfValues + ", different from expected ones: "
          + expectedNumberOfValues );
    }
  }

  private void validateEvent( PatentCH patent, Event eventForPublication, ST36BiblGeneratorProperties props ) throws InconsistentEventException
  {
    int ereigIdat = eventForPublication.getCompareTime();
    Event eventFromPatentCH = patent.getEvents().get( eventForPublication );
    if ( !eventFromPatentCH.isOnWaitlist() )
    {
      throw new InconsistentEventException( "The event " + BAGISTime.formatForDebug( ereigIdat ) + " for patent " + patent.getRegisterNumber() + " ["
          + patent.getApplicationNumber() + "] is not on waitlist" );
    }
    if ( !BAGISTime.convertIDatToDate( ereigIdat ).equals( eventFromPatentCH.getPubl_dat() ) )
    {
      throw new InconsistentEventException( "The event " + BAGISTime.formatForDebug( ereigIdat ) + " for patent " + patent.getRegisterNumber() + " ["
          + patent.getApplicationNumber() + "] has an incorrect date for the publication" );
    }
    if ( patent.getDocuments() != null && patent.getDocuments().getTime( EreigIDat.from( eventFromPatentCH.getCompareTime() ) ).hasNext() )
    {
      throw new InconsistentEventException( "The event " + BAGISTime.formatForDebug( ereigIdat ) + " for patent " + patent.getRegisterNumber() + " ["
          + patent.getApplicationNumber() + "] already has a patent document attached" );
    }
    if ( kindcodesPerArticlenumber.get( eventFromPatentCH.getArt_nr() ) == null )
    {
      throw new InconsistentEventException( "The event " + BAGISTime.formatForDebug( ereigIdat ) + " for patent " + patent.getRegisterNumber() + " ["
          + patent.getApplicationNumber() + "] has an article number ( " + eventFromPatentCH.getArt_nr()
          + ") which is not relevant for the generation of ST36 documents" );
    }

    // update max event in properties to set new start_eretigidat at the end
    if ( ereigIdat > props.getStartPoint() )
    {
      props.setStartEreigidat( ereigIdat );
    }

  }


}
