package ch.ige.ses.client.sapinbound;

import java.util.*;
import com.sap.mw.jco.*;
import org.apache.log4j.*;
import ch.ige.ses.client.saputil.SapProperties;

/**
 * Oberklasse für alle SAP Remote Function Calls.<br>
 * Kappselt den JCo Verbindungsaufbau zum SAP System. Die abgeleiteten
 * RFC Klassen müssen sich somit nicht darum kümmern ob eine direkte Verbindung
 * aufgebaut wird oder ob mit einem Clientpool gearbeitet wird.
 * <p>
 * Achtung: Die Methoden rfcOpen, rfcExec und rfcClose sind nicht "thread safe"
 *          und dürfen nicht gleichzeitig von einem zweiten thread aufgerufen
 *          werden. Für diesen Fall stehen die Methoden rfcOpen2, rfcExec2 und
 *          rfcClose2 zur Verfügung.
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: RFC.java,v 1.13 2006/12/11 12:43:32 iten Exp $
 */
public class RFC
{
  //
  // const
  //
  private static final Logger   LOG                      = Logger.getLogger(RFC.class);

  public static final int       RET_CODE_SUCCESS         = 0;
  public static final int       RET_CODE_RETURN_FAILURE  = -1;
  public static final int       RET_CODE_EXCEPTION       = -2;


  //
  // fields
  //
  protected boolean       fInitialized   = false;   // module (client pool & repository)
                                                    // is initialized

  protected IRepository   fJCoRepository = null;    // jco repositiory
  protected String        fJCoPoolKey    = null;    // jco pool key
  protected String        fJCoUser       = null;    // jco user

  protected JCO.Client    fJCoClient     = null;    // jco clients (do not
  protected JCO.Client    fJCoClient2    = null;    // share between threads)

  //---------------------------------------------------------------------------
  // constructors & initialization
  //---------------------------------------------------------------------------

  public RFC() throws JCO.Exception
  {
    init();
  }


  private void init() throws JCO.Exception
  {
    Properties prop = SapInbound.getProperties();

    // set fields
    this.fJCoPoolKey = prop.getProperty("jco.clientpool.key");
    this.fJCoUser    = prop.getProperty("jco.clientpool.user");

    // add a connection pool to the specified system
    JCO.addClientPool
      ( this.fJCoPoolKey,                                     // alias for this pool
                                                              // max. number of connections
        Integer.parseInt(prop.getProperty("jco.clientpool.max_connections")),
        prop.getProperty("jco.clientpool.client"),            // sap client
        this.fJCoUser,                                        // userid
        SapProperties.decodeProperty                          // password
          (prop.getProperty("jco.clientpool.passwd")),
        prop.getProperty("jco.clientpool.lang"),              // language
        prop.getProperty("jco.clientpool.r3name"),            // host name
        prop.getProperty("jco.clientpool.group") );           // system number

    // create a new repository
    this.fJCoRepository = JCO.createRepository
      ( prop.getProperty("jco.repository.name"), this.fJCoPoolKey );

    // initialisation is done
    fInitialized = true;
  }

  //---------------------------------------------------------------------------
  // methods for connection handling
  //---------------------------------------------------------------------------

  public boolean isIntialized()
  {
    return fInitialized;
  }


  public void initialize()
  {
    try
    {
      if ( ! isIntialized() )
      {
        LOG.debug("initialize - setup rfc clientpool & repository");
        this.init();
      }
      else
      {
        LOG.warn("initialize - module is already initialised");
      }
    }
    catch (Exception ex)
    {
      LOG.error("initialize - exception occured: " + ex.toString() );
    }
  }


  public void reset()
  {
    try
    {
      LOG.debug("reset - remove rfc clientpool & repository");
      fInitialized = false;
      JCO.removeClientPool(this.fJCoPoolKey);
      fJCoRepository = null;
    }
    catch (Exception ex)
    {
      LOG.error("reset - exception occured: " + ex.toString() );
    }
  }

  //---------------------------------------------------------------------------
  // methods for rfc calls
  //---------------------------------------------------------------------------

  // caution:   rfcOpen(), rfcExec() and rfcClose() ar not thread safe
  //            and must be used only by on thread at a time !

  protected void rfcOpen() throws JCO.Exception
  {
    this.fJCoClient = JCO.getClient(this.fJCoPoolKey);
  }

  protected void rfcExec(JCO.Function function) throws JCO.Exception
  {
    this.fJCoClient.execute(function);
  }

  protected void rfcClose() throws JCO.Exception
  {
    JCO.releaseClient(this.fJCoClient);
  }


  // caution:   rfcOpen2(), rfcExec2() and rfcClose2() ar not thread safe
  //            and must be used only by on thread at a time !

  protected void rfcOpen2() throws JCO.Exception
  {
    this.fJCoClient2 = JCO.getClient(this.fJCoPoolKey);
  }

  protected void rfcExec2(JCO.Function function) throws JCO.Exception
  {
    this.fJCoClient2.execute(function);
  }

  protected void rfcClose2() throws JCO.Exception
  {
    JCO.releaseClient(this.fJCoClient2);
  }


} // end class

