package ch.ige.ses.client.sapoutbound;

import java.util.*;
import org.apache.log4j.*;

import ch.ige.ses.BuildInfo;
import ch.ige.ses.vo.ses.*;
import ch.ige.ses.client.saputil.*;
import ch.ige.ses.server.sapctrl.SapDelegateOutbound;

/**
 * SAP Outbound Server - Automatic Mode
 * Diese Klasse steuert den SAP Outbound Server im normalen Betrieb. Sie wartet
 * auf RCF Calls vom SAP System und gibt diese via EJB Aufruf an SES weiter.
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: AutoMode.java,v 1.24 2007/01/09 16:48:29 iten Exp $
 */
public class AutoMode
{
  //
  // const
  //
  private static final Logger   LOG = Logger.getLogger(AutoMode.class);
  private static final int      DEF_CONNECTION_RETRY_PERIOD   = 60;
  private static final int      DEF_CONNECTION_PING_INTERVAL  = 60;

  //
  // fields
  //
  private int   fSapConnRetryPeriod;        // sap & app server connection:
  private int   fAppSrvConnRetryPeriod;     // retry period & ping interval
  private int   fAppSrvConnPingInterval;

  private SapServerController fCtrl     = null;     // sap server controller
  private SapDelegateOutbound fDelegate = null;     // app server calls

  private boolean     fStopFlag     = false;        // stop application
  private Thread      fWaitThread   = null;         // used to interrupt waiting thread

  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------

  public AutoMode()
  {
    SapProperties sp = new SapProperties();

    // init params & properties
    logBuildInfo();
    logParams();
    logProperties("SAP Outbound Client - Properties",SapOutbound.getProperties());
    logProperties("System - Properties", System.getProperties());

    // init connection ping intervall & retry period
    LOG.info("Initialise App Server Connection Ping Intervall & Retry Period ...");
    fSapConnRetryPeriod     = sp.getInt(SapOutbound.getProperties(),"sap.connection.retry.period",DEF_CONNECTION_RETRY_PERIOD);
    fAppSrvConnPingInterval = sp.getInt(SapOutbound.getProperties(),"appsrv.connection.ping.intervall",DEF_CONNECTION_PING_INTERVAL);
    fAppSrvConnRetryPeriod  = sp.getInt(SapOutbound.getProperties(),"appsrv.connection.retry.period",DEF_CONNECTION_RETRY_PERIOD);

    // sap server control
    fCtrl = new SapServerController();
  }

  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  public void run()
  {
    try
    {
      //
      // init
      //
      setupAppServerConnection();       // setup app server connection

      cleanUpFailedErrorVOs();          // clean up error vo(s) (do this call
                                        // after app server connection ceck)

      setupSAPConnection();             // start sap servers

      LOG.info("### SAP Outbound Server - Setup finished, start listening ###");


      //
      // check connections
      //
      while ( ! this.getStopFlag() )
      {

        // check connection to app server
        // (check connection to sap is done by the serverExceptionOccurred handler)
        this.wait(this.fAppSrvConnPingInterval);

        // establish app server connection, , do clean up check
        if ( ! getStopFlag() && ! pingAppServer() )
        {
          LOG.info("run - Shutdown SAP Servers & restart establishing connections.");
          this.fCtrl.stopServers();         // stop sap servers
          setupAppServerConnection();       // setup app server connection
          cleanUpFailedErrorVOs();          // ckeck for error vo(s) to clean up
        }

        // establish sap connection
        if ( ! getStopFlag() && ! fCtrl.getServerAliveFlag() )
        {
          // LOG.warn("### run - lost connection to sap !");
          setupSAPConnection();
          LOG.info("### SAP Outbound Server - Setup reestablished, start listening ###");
        }

      } // while

    }
    catch (Exception ex)
    {
      LOG.fatal("run - service terminates with exception: " + ex.toString());
    }
    finally
    {
      //
      // exit application gracefully
      //
      this.fCtrl.stopServers();
      LOG.info("### SAP Outbound Server - Shutdown application im progress ...");
    }

  } // end method


  //
  // public - getter & setter
  // (synchronize because possible call's from diffrent tasks
  //

  public synchronized void setStopFlag(boolean state)
  {
    // set flag
    LOG.info("setStopFlag - called");
    this.fStopFlag = state;

    // interrupt waiting thread
    if ( (state) && (fWaitThread != null) )
    {
      LOG.info("setStopFlag - interrupt waiting thread");
      this.fWaitThread.interrupt();
    }
  }

  public synchronized boolean getStopFlag()
  {
    return this.fStopFlag;
  }

  //---------------------------------------------------------------------------
  // private methods - connection handling
  //---------------------------------------------------------------------------

  private void setupAppServerConnection()
  {
    boolean setup = false;

    while ( ! getStopFlag() && ! setup)
    {
      try
      {
        // init delegate (first call to app server)
        SapDelegateOutbound.removeInstance();         // explizit remove of the old instance!
        fDelegate = SapDelegateOutbound.getInstance();

        // check connection again
        // (should work properly, if initialising delegate succeeded already !)
        if ( ! this.pingAppServer() )
          throw new Exception();

        // all done
        setup = true;
        LOG.info("setupAppServerConnection - successfully done.");
      }
      catch (Exception ex)
      {
        LOG.error ( "setupAppServerConnection - failed, retry setup in " +
                    this.fAppSrvConnRetryPeriod + " seconds..." );

        this.wait( this.fAppSrvConnRetryPeriod );
      }
    } // while

  } // end method


  private boolean pingAppServer()
  {
    try
    {
      this.fDelegate.receivePing("Greetings from SapOutbound" + SapOutbound.getProgramId());

      LOG.info("pingAppServer - connection to AppServer is alive.");
      return true;
    }
    catch (Exception ex)
    {
      LOG.warn("### pingAppServer - lost connection to AppServer !");
      return false;
    }
  }


  private void setupSAPConnection()
  {
    boolean setup = false;

    while ( ! getStopFlag() && ! setup )
    {
      try
      {
        // start sap servers
        fCtrl.startServers();

        // check connection after 10 sec
        wait(10);
        if ( ! fCtrl.getServerAliveFlag() )
          throw new Exception();

        // all done
        setup = true;
        LOG.info("setupSAPConnection - successfully done.");
      }
      catch (Exception ex)
      {
        LOG.error ( "setupSAPConnection - failed, retry start servers in " +
                    this.fSapConnRetryPeriod + " seconds..." );

        this.wait( this.fSapConnRetryPeriod );
      }
    } // while

  } // end method


  private void wait(int seconds)
  {
    try
    {
      fWaitThread = Thread.currentThread();
      Thread.sleep(seconds * 1000);
    }
    catch (java.lang.InterruptedException ie)
    {
      // do nothing
    }
    finally
    {
      fWaitThread = null;
    }
  } // end method

  //---------------------------------------------------------------------------
  // private methods - misc
  //---------------------------------------------------------------------------

  /**
   * Diese Methode sucht im lokalen Verzeichnis nach nicht verarbeiteten
   * ErrorVO Meldungen und versucht, diese nochmals zu verarbeiten.
   *
   * Dieser Fall kann auftreten, falls beim Aufruf von saveError() (vgl. Klasse
   * SapServer) der AppServer nicht verfügbar ist und das ErrorVO nur noch
   * lokal gesichert wird.
   */
  private void cleanUpFailedErrorVOs()
  {
    ErrorFile   ef    = null;
    List        list  = null;
    Iterator    it    = null;
    String      fn    = null;
    ErrorVO     err   = null;

    try
    {
      // start clean up
      LOG.info ("Clean up error vo(s) ....");

      ef = new ErrorFile ( SapOutbound.getProperties().getProperty("sapoutbound.data.path"),
                           SapOutbound.getProgramIdentifier() );

      list = ef.getList();
      it   = list.iterator();
      LOG.info ("  " + list.size() + " pending error vo(s) found to clean up");

      while ( it.hasNext() )
      {
        try
        {
          fn  = (String) it.next();
          err = ef.readObjectFile(fn);
          SapDelegateOutbound.getInstance().receiveError(err);
          ef.rename(fn,ef.EXT_CLEANUP_OK);
          LOG.info("  process file " + fn + " ... ok");
        }
        catch (Exception ex)
        {
          ef.rename(fn,ef.EXT_CLEANUP_NOK);
          LOG.info("  process file " + fn + " ... nok");
        }
      } // end while

      LOG.info ("  >> Clean up error vo(s) successfully done");

    }
    catch (Exception ex)
    {
      LOG.error ( "cleanUpFailedErrorVOs - clean up failed with exception: "
                  + ex.getMessage() );
    }
  }

  private void logProperties(String title, Properties prop)
  {
    Enumeration eNames = prop.propertyNames();
    TreeSet     set  = new TreeSet();
    String      key;

    if ( LOG.isInfoEnabled() )
    {
      LOG.info(title);
      while ( eNames.hasMoreElements() )
      {
        key = eNames.nextElement().toString();
        set.add(key);
      }

      for (Iterator i = set.iterator(); i.hasNext(); )
      {
        key = (String) i.next();
        LOG.info("  " + Convert.toSapStr(key,40) + ": " + prop.getProperty(key));
      }
    }
  } // end method


  private void logParams()
  {
    if ( LOG.isInfoEnabled() )
    {
      LOG.info("Params");
      LOG.info("   Program ID          : " + SapOutbound.getProgramId()  );
      LOG.info("   Program Action      : " + SapOutbound.getProgramAction() );
      LOG.info("   Program Environment : " + SapOutbound.getProgramEnv() );
      LOG.info("   Properties File     : " + SapOutbound.getPropertiesFile() );
    }
  } // end method

  private void logBuildInfo()
  {
    if ( LOG.isInfoEnabled() )
    {
      LOG.info("Build Info");
      LOG.info("   CVS Tag      : " + BuildInfo.CVS_TAG  );
      LOG.info("   Build Id     : " + BuildInfo.BUILD_ID );
      LOG.info("   Default Provider Url used, when java.naming.provider.url is set to <default>:" );
      LOG.info("   Provider Url : " + ch.ige.util.BuildInfo.PROVIDER_URL );
    }
  } // end method

} // end class


