package ch.ige.ses.client.saprequest;

import java.util.*;
import ch.ige.ses.client.sapinbound.*;
import ch.ige.ses.client.saputil.*;
import ch.ige.ses.exception.*;
import ch.ige.ses.server.sapctrl.*;
import ch.ige.ses.util.*;
import ch.ige.ses.vo.ses.*;
import ch.ige.util.vo.*;


/**
 * Diese Klasse kappselt die Request's des Adress-Clients an SAP.
 *
 * Es können dabei Adressen im SAP System angelegt, mutiert, blockiert
 * und gelöscht werden. Bei der Löschung ist zu beachten, dass die Einträge
 * nicht physisch gelöscht werden, sondern nur zur Löschung markiert werden.
 *
 *    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 *    !                           ACHTUNG                           !
 *    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 *    !                                                             !
 *    !  Diese Klasse verwendet J2EE Restrictions und darf nur      !
 *    !  von Java Client Programmen oder Servlets verwendet werden. !
 *    !                                                             !
 *    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 *
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: SapAdresse.java,v 1.22 2006/12/11 12:43:36 iten Exp $
 */
public class SapAdresse extends SapRequest
{
  //
  // const
  //
  private static final String VERSADR_CHECK_USER_MSG      = "Fehler beim Überprüfen der Abhängigkeiten der Adresse auf dem SAP System";
  public static final int     VERSADR_CHECK_MAX_LIST_SIZE = 1000;
  public static final int     VERSADR_CHECK_TTL_BASE      = 120;    // [sec]
  public static final int     VERSADR_CHECK_TTL_DELTA     = 1;      // [sec]

  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------

  /**
   * Erstellt eine neue SapAdresse, die die Aufrufe des SAP Systems kappselt.
   * Die Aufrufe des SAP Systems via SapAdresse verlaufen im Reply / Request
   * Modus. Das heisst der Aufrufer wartet auf die Antwort des SAP Systems.
   */
  public SapAdresse()
  {
    super();
  }

  //---------------------------------------------------------------------------
  //  public methods
  //---------------------------------------------------------------------------

  /**
   * Überprüft die Abhängigkeiten der Versand Adresse im SAP System. Ist die
   * Versand Adresse im SAP System noch mit einem offenen Auftrag verknüpft,
   * so darf diese nicht gelöscht oder gesperrt werden.
   *
   * @param   vo                  Versand Adressen Check Value Object
   * @throws  SapCallException    Fehler beim Überprüfen der Adresse
   * @return  VersadrCheckVO      VO mit aktualisierten Prüfinformationen
   */
  public VersadrCheckVO check(VersadrCheckVO vo) throws SapCallException
  {
    try
    {
      ArrayList list = new ArrayList();

      list.add(vo);
      list = check(list);
      vo = (VersadrCheckVO)list.get(0);

      return vo;
    }
    catch (SapCallException sce)
    {
      throw sce;
    }
    catch (Exception ex)
    {
      throw new SapCallException
        (VERSADR_CHECK_USER_MSG,"check(VersadrCheckVO) - Exception: " + ex.getMessage());
    }
  }


  /**
   * Überprüft die Abhängigkeiten einer ganze Liste von VersadrCheckVO's im
   * SAP System. Die Versand Adressen welche im SAP System noch mit einem
   * offenen Auftrag verknüpft sind, dürfen nicht gelöscht oder gesperrt werden.
   *
   * Beachte:
   * Die Liste mit den VersadrCheckVO darf maximal VERSADR_CHECK_MAX_LIST_SIZE
   * Einträge enthalten. Ist die Liste grösser, wird eine SapCallException
   * mit einer entsprechenden Fehlermeldung ausgelöst.
   *
   * @param   list                Liste mit zu prüfenden VersadrCheckVO's
   * @throws  SapCallException    Fehler beim Überprüfen der Liste
   * @return  ArrayList           Überprüfte Liste mit VersadrCheckVO's
   */
  public ArrayList check(ArrayList list) throws SapCallException
  {
    if (list.size() > VERSADR_CHECK_MAX_LIST_SIZE)
    {
      throw new SapCallException
       ( VERSADR_CHECK_USER_MSG,
         "check(ArrayList) - list size (" + list.size() + ") > " +
         "max. allowed list size (" + VERSADR_CHECK_MAX_LIST_SIZE +") !");
    }

    return requestAdressCheck(list);
  }


  /**
   * Erstellt eine neue Adresse im SAP System
   *
   * @param   vo                Adressdaten
   * @throws  SapCallException  Fehler beim Erstellen der Adressdaten
   */
  public void create(VersadrVO vo) throws SapCallException
  {
    vo.setIsNew(true);
    requestAdressCreate(vo);
  }


  /**
   * Aktualisiert eine bestehende Adresse im SAP System
   *
   * @param   vo                Adressdaten
   * @throws  SapCallException  Fehler beim Aktualisieren der Adressdaten
   */
  public void update(VersadrVO vo) throws SapCallException
  {
    vo.setIsNew(false);
    requestAdressCreate(vo);
  }


  /**
   * Löschvormerkung für Adresse setzten
   *
   * @param adrNr               Adressnummer
   * @throws SapCallException   Fehler beim Verbindungsaufbau zum SAP System
   */
  public void delete(int adrNr) throws SapCallException
  {
    requestAdressStatus( new SAPAdrStatusVO(adrNr,true,"X") );
  }


  /**
   * Löschvormerkung für Adresse zurücksetzten
   *
   * @param adrNr               Adressnummer
   * @throws SapCallException   Fehler beim Verbindungsaufbau zum SAP System
   */
  public void undelete(int adrNr) throws SapCallException
  {
    requestAdressStatus( new SAPAdrStatusVO(adrNr,true," ") );
  }


  /**
   * Blockierung für Adresse setzten
   *
   * @param adrNr               Adressnummer
   * @throws SapCallException   Fehler beim Verbindungsaufbau zum SAP System
   */
  public void block(int adrNr) throws SapCallException
  {
    requestAdressStatus( new SAPAdrStatusVO(adrNr,false,"X") );
  }


  /**
   * Blockierung für Adresse zurücksetzten
   *
   * @param adrNr               Adressnummer
   * @throws SapCallException   Fehler beim Verbindungsaufbau zum SAP System
   */
  public void unblock(int adrNr) throws SapCallException
  {
    requestAdressStatus( new SAPAdrStatusVO(adrNr,false," ") );
  }

  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  private ArrayList requestAdressCheck (ArrayList list) throws SapCallException
  {
    SapReturn   reply;
    int         timeToLive;

    try
    {
      // set time to live depending on list size
      timeToLive = VERSADR_CHECK_TTL_BASE + list.size() * VERSADR_CHECK_TTL_DELTA;

      // connect and start request
      connect(timeToLive);

      reply = request ( list,
                        Jms.MSG_TYPE_SAP_CUST_CHECK,
                        Jms.MSG_DESC_SAP_CUST_CHECK );

      // check return code
      if (reply.rfcCode != RFC.RET_CODE_SUCCESS)
      {
        throw new SapCallException ( VERSADR_CHECK_USER_MSG,
                                     reply.rfcMessage.toString() );
      }

      // return result
      return ( (ArrayList)reply.sapData );

    }
    finally
    {
      disconnect();
    }
  }


  private void requestAdressCreate (VersadrVO vo) throws SapCallException
  {
    SapReturn   reply;
    ErrorVO     err;

    try
    {
      connect();

      reply = request ( vo,
                        Jms.MSG_TYPE_SAP_CUST_CREATE,
                        Jms.MSG_DESC_SAP_CUST_CREATE );

      if (reply.rfcCode != RFC.RET_CODE_SUCCESS)
      {
        throw new SapCallException ( "Fehler beim Anlegen/Aktualisieren der Adresse auf dem SAP System",
                                      reply.rfcMessage.toString() );
      }

    }
    catch (SapCallException ex)
    {
      if ( timeoutOccured() )
      {
        err = new ErrorVO( "Adress Client",ex.getMessage(),
                           ErrorVO.DATA_TYPE_CD_SAP_ADR_STATUS_VO, vo,
                           null, ErrorVO.EMPL_NOTIFY_KEINE, ErrorVO.EMPL_MSG_TYPE_ADR );
        saveError(err);
      }

      throw new SapCallException(ex.getUserMessage(),ex.getDetailMessage());
    }
    finally
    {
      disconnect();
    }
  }


  private void requestAdressStatus (SAPAdrStatusVO vo) throws SapCallException
  {
    SapReturn   reply;
    ErrorVO     err;

    try
    {
      connect();

      reply = request ( vo,
                        Jms.MSG_TYPE_SAP_CUST_STATUS,
                        Jms.MSG_DESC_SAP_CUST_STATUS );

      if (reply.rfcCode != RFC.RET_CODE_SUCCESS)
      {
        throw new SapCallException ( "Fehler beim Aktualisieren des Adress Status auf dem SAP System",
                                      reply.rfcMessage.toString() );
      }

    }
    catch (SapCallException ex)
    {
      if ( timeoutOccured() )
      {
        err = new ErrorVO( "Adress Client",ex.getMessage(),
                           ErrorVO.DATA_TYPE_CD_SAP_ADR_STATUS_VO, vo,
                           null, ErrorVO.EMPL_NOTIFY_KEINE, ErrorVO.EMPL_MSG_TYPE_ADR );
        saveError(err);
      }

      throw new SapCallException(ex.getUserMessage(),ex.getDetailMessage());
    }
    finally
    {
      disconnect();
    }
  }


  private boolean saveError (ErrorVO err)
  {
    try
    {
      SapDelegateInbound.getInstance().receiveError(err);
      return true;
    }
    catch (Exception ex)
    {
      // save error vo to local file
      new ErrorFile().save(err);

      return false;
    }
  }


} // end class


