package ch.ige.ses.client.saputil;

import java.util.*;
import ch.ige.ses.vo.bagis.*;
import ch.ige.ses.vo.ses.*;

/**
 * Konvertierung der BAGIS Daten ins SAP Format und umgekehrt.
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: Convert.java,v 1.33 2006/12/13 14:54:56 iten Exp $
 */
public class Convert
{
  //
  // const
  //
  private static final char   SAP_NR_LEADING_SIGN    = '0';
  private static final char   SAP_STR_TRAILING_SIGN  = ' ';
  private static final String DEF_SPRACH_CODE        = "D";

  public static final String SAP_MESSAGE_TYPE_ESR         = "ZVL1";    // ESR
  public static final String SAP_MESSAGE_TYPE_ESR_COPY    = "ZVL0";    // ESR Kopie
  public static final String SAP_MESSAGE_TYPE_DEPKTO      = "ZVL2";    // Dep-Kto (Belastungsanzeige)
  public static final String SAP_MESSAGE_TYPE_DEPKTO_COPY = "ZVL8";    // Dep-Kto Kopie
  public static final String SAP_MESSAGE_TYPE_JGBRE       = "ZVL3";    // JGB Rechnung
  public static final String SAP_MESSAGE_TYPE_JGBMA       = "ZVL5";    // JGB Mahnung
  public static final String SAP_MESSAGE_TYPE_AMVE1       = "ZVL6";    // AMV Erinnerung 1
  public static final String SAP_MESSAGE_TYPE_AMVE2       = "ZVL7";    // AMV Erinnerung 2
  //
  // fields
  //

  // BAGIS to SAP
  private static Properties fMapLageinhToSapTargetQu      = new Properties();
  private static Properties fMapSprachCdToSapLangu        = new Properties();
  private static Properties fMapDruckArtToSapDispatchTime = new Properties();
  private static Properties fMapAbsageGrundToSapReasonRej = new Properties();

  private static String[]   fMapZusatzRechEmpfToSapPartnRole =
                              { "R0","R1","R2","R3","R4","R5","R6","R7","R8","R9",
                                "Z0","Z1","Z2","Z3","Z4","Z5","Z6","Z7","Z8","Z9" };

  // SAP to BAGIS
  private static Properties fMapSapStateToStatusDruck     = new Properties();
  private static Properties fMapSapStateToStatusZahlg     = new Properties();


  //
  // static initialization block
  //
  static
  {
    // BAGIS to SAP
    fMapLageinhToSapTargetQu.setProperty("ANZ","ST");      // kein pendant auf sap
    fMapLageinhToSapTargetQu.setProperty("STD","STD");
    fMapLageinhToSapTargetQu.setProperty("STK","ST");

    fMapSprachCdToSapLangu.setProperty("D","D");
    fMapSprachCdToSapLangu.setProperty("F","F");
    fMapSprachCdToSapLangu.setProperty("E","E");
    fMapSprachCdToSapLangu.setProperty("I","I");
    fMapSprachCdToSapLangu.setProperty("USA","E");         // kein pendant auf sap

    fMapDruckArtToSapDispatchTime.setProperty("1","4");    // bagis: sofort drucken
    fMapDruckArtToSapDispatchTime.setProperty("2","3");    // bagis: batch druck
    fMapDruckArtToSapDispatchTime.setProperty("3","3");    // bagis: druck patent jgb rechnungen
    fMapDruckArtToSapDispatchTime.setProperty("4","3");    // bagis: druck patent jgb mahnungen
    fMapDruckArtToSapDispatchTime.setProperty("5","4");    // bagis: löschungsanzeige

    fMapAbsageGrundToSapReasonRej.setProperty("9","Z1");    // bagis: annulation
    fMapAbsageGrundToSapReasonRej.setProperty("A","Z2");    // bagis: fristablauf

    // SAP to BAGIS
    fMapSapStateToStatusDruck.setProperty("1","9");        // gedruckt

    fMapSapStateToStatusZahlg.setProperty("2","4");        // bezahlt
    fMapSapStateToStatusZahlg.setProperty("3","2");        // storniert
    fMapSapStateToStatusZahlg.setProperty("4","9");        // annuliert
    fMapSapStateToStatusZahlg.setProperty("5","A");        // frist abgelaufen (massnahme)
  }


  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------

  /**
   * Constructs a Convert Object<br>
   * Note: In the most cases no spezific instance is used, since mainly all
   * methods to convert data are static !
   */
  public Convert()
  {
    // lazy bone
  }

  //---------------------------------------------------------------------------
  // methods
  //---------------------------------------------------------------------------

  /**
   * convert integer to sap number representation (string with leading zero's)<br>
   * example: value = 5, len = 3 --> sap nr = "005"
   *
   * @param value number
   * @param len sap number length
   * @return sap number as string
   */
  public static String toSapNr (int value, int len)
  {
    StringBuffer buf = new StringBuffer(len);

    buf.append(value);
    while (buf.length() < len)
    {
      buf.insert(0,SAP_NR_LEADING_SIGN);
    }
    return buf.toString();
  }


  /**
   * convert string to sap representation (string with trailing blank's)<br>
   * example: value = hello, len = 10 --> sap string = "hello     "
   *
   * @param value string
   * @param len sap string length
   * @return sap string
   */
  public static String toSapStr (String value, int len)
  {
    StringBuffer buf = new StringBuffer(len);

    buf.append(value);
    while (buf.length() < len)
    {
      buf.append(SAP_STR_TRAILING_SIGN);
    }
    return buf.toString();
  }



  /**
   * convert bagis bfakt_id to sap doc number.<br>
   * note: the number will be in the following range:<br>
   * 0'004'xxx'xxx to 0'019'999'999 (at the moment, the bfakt_id is around 4 mio)
   *
   * @param bfaktId bagis bfakt_id
   * @return sap doc number as string
   */
  public static String toSapDocNumber (int bfaktId)
  {
    return toSapNr(bfaktId,10);
  }

  /**
   * convert bagis kontrollId to sap KONTROLL_ID
   *
   * @param kontrollId    id to control
   * @return              sap KONTROLL_ID as string
   * @author              P. Bolzli
   */
  public static String toSapKontrollId (int kontrollId)
  {
    return toSapNr(kontrollId, 10);
  }


  /**
   * converts sap material to bagis art_nr
   *
   * @param artNr         artikel nummer
   * @return              artNr
   * @author              Patrick Bolzli
   */
  public static String toBagisArtNr(String artNr)
  {
    // auf diese Weise werden die führenden Nullen entfernt! Cast in int und zurück
    return Integer.toString(Integer.parseInt(artNr));
  }

  /**
   * convert sap doc number to bagis bfakt_id.<br>
   *
   * @param docNumber sap doc number
   * @return bagis bfakt_id
   */
  public static int toBagisBfaktId (String docNumber)
  {
    return Integer.parseInt(docNumber);
  }

  /**
   * convert bfakt posnr to sap itm number format.<br>
   *
   * @param bfaktPosnr bagis bfakt posnr
   * @return sap itm number as string
   */
  public static String toSapItmNumber (int bfaktPosnr)
  {
    return toSapNr(bfaktPosnr,6);
  }


  /**
   * convert bagis art_nr to sap material.<br>
   *
   * @param art_nr bagis art_nr
   * @return sap material
   */
  public static String toSapMaterial (String art_nr)
  {
    return toSapNr(Integer.parseInt(art_nr),18);
  }

  /**
   * convert bagis adr_nr to sap customer <br>
   *
   * @param ktoNr bagis adr_nr
   * @return sap customer
   */
  public static String toSapCustomer(int ktoNr)
  {
    return toSapNr(ktoNr,10);
  }


  /**
   * return sap bill date. returns current date if bfakt_dat is null.
   * <br>
   * @param  bfakt_dat bagis bfakt_dat
   * @return sap bill date
   */
  public static Date toSapBillDate (Date bfakt_dat)
  {
    if (bfakt_dat == null)
      return new Date();
    else
      return bfakt_dat;
  }


  /**
   * set sap bill block depending on billing type and bagis regulierer.
   *
   * @param   re  billing data
   * @return      sap bill block
   */
  public static String toSapBillBlock (SAPRechnungVO re)
  {
    String  billBlock = null;

    if (re.isRechnung())
    {
      if (re.regulierer == 0)
      {
        billBlock =  "Z1";              // ESR-Abwicklung
      }
      else
      {
        billBlock = "Z2";               // Deponenten-Abwickung
      }
    }
    else if (re.isJGB())
    {
      billBlock = "Z3";                 // Jahresgebühren
    }
    else if (re.isAMV())
    {
      billBlock = "Z4";                 // Automatische Markenverlängerung
    }

    return billBlock;
  }


  /**
   * map bagis kz_einschr to sap reg mail
   *
   * @param kz_einschr bagis kz_einschr
   * @return sap reg mail
   */
  public static String toSapRegMail (String kz_einschr)
  {
    if  ( (kz_einschr == null) ||
          (kz_einschr.equals(SAPRechnungVO.BOOLEAN_TYPES[0])) )
    {
      return " ";                               // false
    }
    else
    {
      return "X";                               // true
    }
  }


  /**
   * set sap message type depending on billing type and bagis regulierer.
   *
   * @param re    billing data
   * @param   jp  jgb parameters
   * @return      sap bill block
   */
  public static String toSapMessageType (SAPRechnungVO re, SAPJGBParamVO jp)
  {
    String msgType = null;

    if (re.isRechnung())
    {
      if (re.regulierer == 0)
      {
        msgType = SAP_MESSAGE_TYPE_ESR;             // ESR-Abwicklung
      }
      else
      {
        msgType = SAP_MESSAGE_TYPE_DEPKTO;          // Deponenten-Abwickung
      }
    }
    else if (re.isJGB())
    {
      msgType = SAP_MESSAGE_TYPE_JGBRE;             // JGB Rechnung

      if (jp != null)
        if (jp.druck_art != null)
          if ( jp.druck_art.equals("4") )
            msgType = SAP_MESSAGE_TYPE_JGBMA;       // JGB Mahnung
    }
    else if (re.isAMV())
    {
      if (re.amvRechnungCd == AmvRechnungVO.RE_CD_ERINNERUNG_1)
      {
        msgType = SAP_MESSAGE_TYPE_AMVE1;          // AMV Erinnerung 1
      }
      else if (re.amvRechnungCd == AmvRechnungVO.RE_CD_ERINNERUNG_2)
      {
        msgType = SAP_MESSAGE_TYPE_AMVE2;          // AMV Erinnerung 2
      }
    }

    return msgType;
  }

  /**
   * set sap message type for copies (if needed)
   *
   * @param regulierer  bagis regulierer
   * @return            sap bill block (for copies)
   */
  public static String toSapMessageTypeCopy (int regulierer)
  {
    String  msgType;

    if (regulierer == 0)
    {
      msgType = SAP_MESSAGE_TYPE_ESR_COPY;             // ESR-Abwicklung
    }
    else
    {
      msgType = SAP_MESSAGE_TYPE_DEPKTO_COPY;         // Deponenten-Abwickung
    }

    return msgType;
  }


  /**
   * convert bagis lageinh to sap target qu.<br>
   *
   * @param lageinh bagis lageinh
   * @return sap target qu
   */
  public static String toSapTargetQu (String lageinh)
  {
    return fMapLageinhToSapTargetQu.getProperty(lageinh);
  }


  /**
   * convert bagis sprach cd to sap langu.<br>
   *
   * @param sprachCd bagis sprach cd
   * @return sap langu
   */
  public static String toSapLangu(String sprachCd)
  {
    if (sprachCd == null)
      sprachCd = DEF_SPRACH_CODE;

    return fMapSprachCdToSapLangu.getProperty(sprachCd);
  }

  /**
   * convert bagis druck art to sap dispatch time.<br>
   *
   * @param druckArt bagis druck art
   * @return sap dispatch time
   */
  public static String toSapDispatchTime(String druckArt)
  {
    return fMapDruckArtToSapDispatchTime.getProperty(druckArt);
  }


  /**
   * convert bagis listen art to sap list mode.<br>
   * within sap only the folowwing two modes are used:<br>
   *   0  kein Borderau<br>
   *   2  Belastungsauftrag Kontokorrent<br>
   *
   * @param listenArt bagis listen art
   * @return sap list mode
   */
  public static String toSapListMode(String listenArt)
  {
    String listMode = "0";                      // Faktura ohne Borderaux

    if (listenArt != null)
    {
      if (listenArt.equals("2"))                // Faktura mit Borderaux
      {                                         // "Belastungsauftrag Kontokorrent"
        listMode = "2";
      }
      else if (listenArt.equals("6"))           // Ohne Faktura, nur mit Borderaux
      {                                         // "Belastungsauftrag Kontokorrent"
        listMode = "6";
      }
      else if (listenArt.equals("7"))           // Ohne Faktura, ohne Borderaux
      {
        listMode = "7";
      }
    }

    return listMode;
  }


  /**
   * set sap partner role depending on zusatz-rech-empf index.<br>
   *
   * @param idxZusatzRechEmpf index of zusatz-rech-empf
   * @return sap partner role
   */
  public static String toSapPartnerRole(int idxZusatzRechEmpf)
  {
    return fMapZusatzRechEmpfToSapPartnRole[idxZusatzRechEmpf];
  }


  /**
   * convert bagis absageGrund to sap reason-rej.<br>
   *
   * @param absageGrund bagis absageGrund
   * @return sap reason rej
   */
  public static String toSapReasonRej (String absageGrund)
  {
    return fMapAbsageGrundToSapReasonRej.getProperty(absageGrund);
  }


  /**
   * convert sap state to bagis status druck.<br>
   *
   * @param state sap state
   * @return bagis status druck
   */
  public static String toBagisStatusDruck (String state)
  {
    return fMapSapStateToStatusDruck.getProperty(state);
  }


  /**
   * convert sap state to bagis status zahlg.<br>
   *
   * @param state sap state
   * @return bagis status zahlg
   */
  public static String toBagisStatusZahlg (String state)
  {
    return fMapSapStateToStatusZahlg.getProperty(state);
  }

  /**
  * convert bagis Kundengruppe to sap Kundengruppe.<br>
  * within sap only the folowing two values are used:<br>
  *   space  keine Bescheinigung<br>
  *   "001"  mit Bescheinigung<br>
  *
  * @param custGrp1 Bagis KundenGruppe
  * @return sap list mode
  */
  public static String toSapCustGrp1(String custGrp1)
  {
    String  ret = " ";

    if (custGrp1 != null)
    {
      if (custGrp1.equals("001"))
      {
        ret = custGrp1;
      }
    }

    return ret;
  }

} // Convert

