package ch.ige.ses.server.batchctrl;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;

import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.apache.log4j.Logger;

import ch.ige.ses.server.bagisinbound.SESRechnungStatusLookup;
import ch.ige.ses.server.bagisoutbound.SESFacadeLookup;
import ch.ige.ses.server.bagisoutbound.SESRechnungLookup;
import ch.ige.ses.server.dao.bagis.BfakturDAO;
import ch.ige.ses.server.dao.ses.AmvLaufDAO;
import ch.ige.ses.server.dao.ses.AmvRechnungDAO;
import ch.ige.ses.server.dao.ses.JGBLaufDAO;
import ch.ige.ses.server.dao.ses.PropertyDAO;
import ch.ige.ses.server.dao.ses.StakoKontrolleDAO;
import ch.ige.ses.server.dao.ses.TraceDAO;
import ch.ige.ses.server.sapctrl.SapInboundCtrlLocal;
import ch.ige.ses.server.sapctrl.SapInboundCtrlLookup;
import ch.ige.ses.util.MailSender;
import ch.ige.ses.vo.ses.AmvLaufVO;
import ch.ige.ses.vo.ses.AmvRechnungVO;
import ch.ige.ses.vo.ses.JGBLaufVO;
import ch.ige.ses.vo.ses.SAPStakoJobVO;
import ch.ige.ses.vo.ses.StakoKontrolleVO;
import ch.ige.util.LaufNr;


/**
 * Controller Bean für Batch Jobs.
 * <p>
 * Copyright (c) 2003, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: BatchJobBean.java,v 1.61 2007/12/24 10:13:23 iten Exp $
 * 
 * @ejb.bean 
 *  name="BatchJob"  
 *  display-name="BatchJob"
 *  view-type="remote"
 *  jndi-name="BatchJob"
 *  local-jndi-name="BatchJobLocal"
 *  type="Stateless"
 *  transaction-type="Container"
 * @ejb.transaction 
 *  type="Required"
 * @ejb.ejb-ref
 *   ejb-name="SESRechnung"
 *   view-type="local"
 * @ejb.ejb-ref
 *   ejb-name="SESRechnungStatus"
 *   view-type="local"
 * @ejb.ejb-ref
 *   ejb-name="SESFacade"
 *   view-type="local"
 * @ejb.ejb-ref
 *   ejb-name="SapInboundCtrl"
 *   view-type="local"
 */
public class BatchJobBean implements SessionBean
{
  //
  // const
  //
  private static final Logger   LOG = Logger.getLogger(BatchJobBean.class);

  //
  // fields
  //
  SessionContext          sessionContext;

  SESRechnungLookup       rechnung        = new SESRechnungLookup();
  SESRechnungStatusLookup rechnungStatus  = new SESRechnungStatusLookup();
  SESFacadeLookup         facade          = new SESFacadeLookup();
  SapInboundCtrlLookup    sapInboundCtrl  = new SapInboundCtrlLookup();
  
  
  //---------------------------------------------------------------------------
  // bean methods
  //---------------------------------------------------------------------------

  /**
   * create ejb 
   */
  public void ejbCreate() 
  {
    // lazy bone
  }

  public void ejbRemove()
  {
    // lazy bone
  }

  public void ejbActivate()
  {
    // lazy bone
  }

  public void ejbPassivate()
  {
    // lazy bone
  }

  public void setSessionContext(SessionContext sessionContext)
  {
    this.sessionContext = sessionContext;
  }

  //---------------------------------------------------------------------------
  // batch job - common stuff
  //---------------------------------------------------------------------------

  /**
   * send email notification if enabled. the mail settings will be read from
   * the property table.
   *
   * @param jobName       name of bathch job (reserverd for further use)
   * @param subject       mail subject
   * @param body          mail content
   * @throws Exception    unexpected error condition
   * 
   * @ejb.interface-method
   */
  public void batchNotifyEmail (String jobName, String subject, String body) throws Exception
  {
    String      server = null;
    String      enable = null;
    String      to     = null;
    String      cc     = null;
    String      from   = null;


    // init
    LOG.debug ( "batchNotifyEmail - jobName=" + jobName );


    // get mail properties for batch jobs
    server  = getProperty("mail.smtpserver");

    enable  = getProperty("batch." + jobName + ".mail.enable", "batch.default.mail.enable");
    to      = getProperty("batch." + jobName + ".mail.to", "batch.default.mail.to");
    cc      = getProperty("batch." + jobName + ".mail.cc", "batch.default.mail.cc");
    from    = getProperty("batch." + jobName + ".mail.from", "batch.default.mail.from");
    if ( cc.equals("0") )
      cc = null;


    // send email notification
    if (enable.equals("1"))
    {
      LOG.debug ( "batchNotifyEmail - send mail to=" + to );
      MailSender.sendMail(server, to, cc, from, subject, body);
    }
    else
    {
      LOG.debug ( "batchNotifyEmail - mailling disabled " +
                  "(property batch.mail.enable=" + enable + ")" );
    }


    // all done fine
    LOG.debug ( "batchNotifyEmail - all done fine" );
  }


  /**
   * Diese Methode überprüft, ob der Batch Job aktiviert ist.
   * Dabei muss in der Property Tabelle ein Eintrag in der folgenden
   * Form vorhanden sein:
   *    key   = 'batch.jobName.job.enabled'
   *    value = '1' (aktiviert) oder '0' (nicht aktiviert)
   *
   * Beachte: Diese Abfrage muss im Batch Job explizit eingebaut werden.
   *
   * @param jobName     Name des Jobs, wird benutzt um den key zu setzen
   * @return            true falls aktiviert, sonst false
   * @throws Exception  Unerwarteter Fehler
   * 
   * @ejb.interface-method
   */
  public boolean batchJobEnabled (String jobName) throws Exception
  {
    String jobEnabled = null;


    // init
    LOG.debug ( "batchJobEnabled - jobName=" + jobName );

    // check enabled flag
    LOG.debug ( "batchJobEnabled - check enabled flag" );
    jobEnabled  = getProperty("batch." + jobName + ".job.enabled");

    if ( jobEnabled.equals("1") )
    {
      LOG.debug ( "batchJobEnabled - " + jobName + " is enabled");
      return true;
    }
    else
    {
      LOG.debug ( "batchJobEnabled - " + jobName + " is disabled");
      return false;
    }
  }

  /**
   * get property
   * 
   * @param key
   * @return result
   * @throws Exception
   * 
   * @ejb.interface-method
   */
  public String getProperty (String key) throws Exception
  {
    return new PropertyDAO().select(key).value;
  }

  
  /**
   * get property
   * 
   * @param key
   * @param defaultKey
   * @return result
   * @throws Exception
   * 
   * @ejb.interface-method
   */
  public String getProperty (String key, String defaultKey) throws Exception
  {
    try
    {
      return getProperty(key);
    }
    catch (Exception ex)
    {
      return getProperty(defaultKey);
    }
  }


  //---------------------------------------------------------------------------
  // batch job - ses frist ablauf
  //---------------------------------------------------------------------------

  /**
   * frist select bfakt_id list
   * 
   * @return result 
   * @throws Exception
   *  
   * @ejb.interface-method
   */
  public List fristSelectBfaktIdList() throws Exception
  {
    List result = null;

    LOG.debug ( "fristSelectBfaktIdList - started...");
    result = new BfakturDAO().selectBfaktIdForFristAblauf();

    if (result == null)
    {
      LOG.debug ("fristSelectBfaktIdList - finished, result = NULL !");
    }
    else
    {
      LOG.debug ("fristSelectBfaktIdList - finished, " + result.size() + " entries found" );
    }

    return result;
  }

  //---------------------------------------------------------------------------
  // batch job - ses jgb frist ablauf
  //---------------------------------------------------------------------------

  /**
   * jgb frist select bfakt_id list
   * 
   * @return result 
   * @throws Exception 
   * 
   * @ejb.interface-method
   */
  public List jgbfristSelectBfaktIdList() throws Exception
  {
    return new BfakturDAO().selectBfaktIdForFristAblaufJGB();
  }


  //---------------------------------------------------------------------------
  // batch job - ses controlling
  //---------------------------------------------------------------------------

  /**
   * starts new controlling job
   * 
   * @param modus     batch job mode
   * @param bisDate   end date. if null -> start date + one month
   * @author          Patrick Bolzli
   * 
   * @ejb.interface-method
   */
  public void controllingStart(int modus, java.sql.Date bisDate)
  {
    try
    {
      SapInboundCtrlLocal  sic = sapInboundCtrl.getLocalInterface();
      
      SAPStakoJobVO vo    = new SAPStakoJobVO();
      vo.modus            = modus;
      vo.bisDatum         = bisDate;

      sic.sendStakoJob(vo);
    }
    catch (Exception ex)
    {
      LOG.error("BatchJobBean: Exception " +ex.getMessage());
    }
  } // end method


  /**
   * returns the newest stakoKontrolleId
   *
   * @return  kontrollId des Stako-Laufes
   * 
   * @ejb.interface-method
   */
  public int controllingGetNewId()
  {
    StakoKontrolleDAO dao = new StakoKontrolleDAO();
    StakoKontrolleVO kVO = new StakoKontrolleVO();

    try
    {
      kVO = dao.selectMaxKontrollId();
    }
    catch (Exception ex)
    {
      LOG.error("BatchJobBean: Exception " +ex.getMessage());
    }
    return kVO.stakoKontrolleId;
  }


  /**
   * controllingCtrlJobState
   * 
   * @param kontrollID der aktuellen Verarbeitung
   * @return der Status der entsprechenden Verarbeitung
   * @throws Exception
   * 
   * @ejb.interface-method
   */
  public int controllingCtrlJobState (int kontrollID) throws Exception
  {
    StakoKontrolleVO   vo  = null;
    String             msg = null;


    // init
    LOG.debug ( "controllingCtrlJobState - kontrollID=" + kontrollID );


    // get state
    vo = new StakoKontrolleDAO().select(kontrollID);

    if (vo == null)
    {
      msg = "controllingCtrlJobState - Es existiert kein Eintrag in der Tabelle STAKO_KONTROLLE " +
            "für die angegebene kontrollID (" + kontrollID + ")";
      LOG.error(msg);
      throw new Exception(msg);
    }

    // all done fine
    LOG.debug ( "controllingCtrlJobState - all done fine (sap_stat=" + vo.stakoStat + ")" );
    return vo.stakoStat;
  }



  //---------------------------------------------------------------------------
  // batch job - ses jgb start
  //---------------------------------------------------------------------------

  /**
   * start new jgb job and return id from last job. this id will be used to
   * evaluate the new job id for monitioring the state of the batch job.
   *
   * @param modus       batch job mode
   * @param bfaktIdVon  start id
   * @param bfaktIdBis  end id
   * @return            id of last job or 0 if no job was found in table jgb_lauf
   * @throws Exception  unexpected exception
   * 
   * @ejb.interface-method
   */
  public int jgbStart (String modus, int bfaktIdVon, int bfaktIdBis) throws Exception
  {
    JGBLaufVO   vo = null;
    int         id = 0;


    // init
    LOG.debug ( "jgbStart - Modus=" + modus +
                ", bfaktIdVon=" + bfaktIdVon + ", bfaktIdBis=" + bfaktIdBis );

    // get last jgb lauf id
    vo = new JGBLaufDAO().selectMaxLaufID();

    if (vo != null)
      id = vo.jgbLaufId;

    // start jgb
    facade.getLocalInterface().JGBStartSendToJMS(modus, bfaktIdVon, bfaktIdBis);
    
    // all done fine
    LOG.debug ( "jgbStart - all done fine (last jgbLaufId=" + id + ")" );
    return id;
  }

  /**
   * jgb get new lauf id
   * 
   * @param lastLaufId
   * @return result
   * @throws Exception
   * 
   * @ejb.interface-method
   */
  public int jgbGetNewLaufId (int lastLaufId) throws Exception
  {
    JGBLaufVO   vo  = null;
    String      msg = null;

    // init
    LOG.debug ( "jgbGetNewLaufId - lastLaufId=" + lastLaufId );

    // get new jgb lauf id
    vo = new JGBLaufDAO().selectMaxLaufID();

    // compare new id with last one
    if (vo == null)
    {
      msg = "jgbGetNewLaufId - Es existiert kein Eintrag in der Tabelle JGB_LAUF";
      LOG.error(msg);
      throw new Exception(msg);
    }
    else
    {
      if ( vo.jgbLaufId == lastLaufId)
      {
        msg = "jgbGetNewLaufId - Es existiert kein neuer Eintrag in der Tabelle JGB_LAUF";
        LOG.error(msg);
        throw new Exception(msg);
      }
    }

    // all done fine
    LOG.debug ( "jgbGetNewLaufId - all done fine (new jgbLaufId=" + vo.jgbLaufId + ")" );
    return vo.jgbLaufId;
  }

  /**
   * jgb ctrl job state
   * 
   * @param newLaufId
   * @return result
   * @throws Exception
   * 
   * @ejb.interface-method
   */
  public int jgbCtrlJobState (int newLaufId) throws Exception
  {
    JGBLaufVO   vo  = null;
    String      msg = null;


    // init
    LOG.debug ( "jgbCtrlJobState - newLaufId=" + newLaufId );


    // get state
    vo = new JGBLaufDAO().select(newLaufId);

    if (vo == null)
    {
      msg = "jgbCtrlJobState - Es existiert kein Eintrag in der Tabelle JGB_LAUF " +
            "für die angegebene JGB_LAUF_ID (" + newLaufId + ")";
      LOG.error(msg);
      throw new Exception(msg);
    }

    // all done fine
    LOG.debug ( "jgbCtrlJobState - all done fine (sap_stat=" + vo.sapStat + ")" );
    return vo.sapStat;
  }

  //---------------------------------------------------------------------------
  // batch job - amv start
  //---------------------------------------------------------------------------

  //
  // access to bagis db
  //

  /**
   * Liefert eine Liste mit den Rechnungsnummern (bfakt_id's) für die
   * automatische Markenverlängerung für den angegebenen Rechnungscode
   * (Erinnerung 1 oder Erinnerung 2) von Bagis.
   *
   * @param   amvRechnungCd   Rechnungscode (Erinnerung 1 oder 2)
   * @return  ArrayList       Integer Liste mit Rechnungsnummern (bfakt_id)
   * @throws  Exception       Unerwarteter Fehler
   * 
   * @ejb.interface-method
   */
  public ArrayList amvBagisGetRechnungNrList(int amvRechnungCd)
      throws Exception
  {
    return rechnung.getLocalInterface().getRechnungNrListAMV(amvRechnungCd);
  }


  /**
   * Selektiert die Daten für die angegebene Rechnung auf Bagis und sendent
   * diese via JMS an den SapInbound Service zur weiteren Verarbeitung.
   *
   * @param   amvLaufId       AMV Lauf Identifikaton (Id)
   * @param   amvLaufNr       AMV Laufnummer
   * @param   amvRechnungCd   Rechnungscode (Erinnerung 1 oder 2)
   * @param   bfaktId         Rechnungsnummer
   * @throws  Exception       Unerwarteter Fehler
   * 
   * @ejb.interface-method
   */
  public void amvBagisCreateRechnung ( int amvLaufId, String amvLaufNr,
                                       int amvRechnungCd, int bfaktId )
      throws Exception
  {
    rechnung.getLocalInterface().createRechnungAMV
        ( amvLaufId, amvLaufNr, amvRechnungCd, bfaktId );
  }

  /**
   * Aktualisiert das Feld status_druck auf Bagis auf den angegebenen Status
   * für alle Rechnungsnummern in der mitgegebenen Liste.
   *
   * @param   list            Integer Liste mit Rechnungsnummern (bfakt_id)
   * @param   status          Status der gesetzt werden soll
   * @throws  Exception       Unerwarteter Fehler
   * 
   * @ejb.interface-method
   */
  public void amvBagisSetRechnungStatusDruck (ArrayList list, String status) throws Exception
  {
    rechnungStatus.getLocalInterface().setRechnungStatusDruck(list,status);
  }


  //
  // access to ses db
  //

  /**
   * Überprüft ob der letzte ausgeführte Lauf beendet wurde.
   *
   * @return boolean    Lauf wurde beendet (true) oder nicht (false)
   * @throws Exception  Unerwarteter Fehler
   * 
   * @ejb.interface-method
   */
  public boolean amvSesIsLastLaufFinished() throws Exception
  {
    AmvLaufDAO  dao       = new AmvLaufDAO();
    AmvLaufVO   vo        = new AmvLaufVO();
    boolean     finished  = true;

    if (dao.selectCount() > 0)
    {
      vo = dao.select( dao.selectMaxLaufID() );
      if (vo.laufStat != AmvLaufVO.LAUF_STAT_FINISHED)
      {
        finished = false;
      }
    }

    return finished;
  }


  /**
   * Erstellt einen neuen Laufeintrag und liefert die Daten zurück.
   *
   * @return AmvLaufVO  Eintrag zur neuen Verarbeitung
   * @throws Exception  Unerwarteter Fehler
   * 
   * @ejb.interface-method
   */
  public AmvLaufVO amvSesCreateLauf() throws Exception
  {
    AmvLaufDAO  dao     = new AmvLaufDAO();
    AmvLaufVO   vo      = new AmvLaufVO();


    // create new lauf number
    if (dao.selectCount() > 0)
    {
      vo = dao.select( dao.selectMaxLaufID() );
      vo.laufNr = LaufNr.getLaufNr(vo.laufNr);
    }
    else
    {
      vo.laufNr = LaufNr.getLaufNr();
    }

    // insert new lauf data and select created lauf id
    dao.insert(vo);
    vo = dao.select(vo.laufNr);

    // return data
    return vo;
  }


  /**
   * Return Lauf Data for the given Lauf Id
   *
   * @param   amvLaufId   Lauf Id to get data set
   * @return  AmvLaufVO   Lauf Data
   * @throws  Exception   Unexpected Error
   * 
   * @ejb.interface-method
   */
  public AmvLaufVO amvSesGetLauf(int amvLaufId) throws Exception
  {
    AmvLaufVO vo = null;

    vo = new AmvLaufDAO().select(amvLaufId);

    if (vo == null)
    {
      throw new Exception ( "amvSesGetLauf - zur angegebenen amvLaufId " +
                            amvLaufId + " wurde kein Eintrag gefunden !" );
    }

    return vo;
  }


  /**
   * Fügt die Erinnerungen 1 und 2 Rechnungen in die SES Tabelle AMV_RECHNUNG
   * ein und aktualisiert den Status auf der SES Tabelle AMV_LAUF auf
   * "Rechnungsnummern fertig selektiert".
   *
   * @param   amvLaufId   Lauf Id to get data set
   * @param   e1List      Integer Liste mit Erinnerungen 1 (bfakt_id)
   * @param   e2List      Integer Liste mit Erinnerungen 1 (bfakt_id)
   * @throws  Exception   Unexpected Error
   * 
   * @ejb.interface-method
   */
  public void amvSesInsertRechnung ( int amvLaufId,
                                     ArrayList e1List,ArrayList e2List) throws Exception
  {
    int  e1Stat = AmvRechnungVO.RE_STAT_SELECTED;
    int  e2Stat = amvSesIsErinnerung2Enabled() == true ?
                  AmvRechnungVO.RE_STAT_SELECTED : AmvRechnungVO.RE_STAT_SUPPRESS_E2;

    AmvRechnungDAO  dao = new AmvRechnungDAO();
    AmvRechnungVO   vo  = null;
    Integer         id  = null;


    // insert erinnerung 1
    for (Iterator iter = e1List.iterator(); iter.hasNext(); )
    {
      id = (Integer)iter.next();
      vo = new AmvRechnungVO ( amvLaufId,AmvRechnungVO.RE_CD_ERINNERUNG_1,
                               id.intValue(),e1Stat);

      dao.insert(vo);
    }

    // insert erinnerung 2
    for (Iterator iter = e2List.iterator(); iter.hasNext(); )
    {
      id = (Integer)iter.next();
      vo = new AmvRechnungVO ( amvLaufId,AmvRechnungVO.RE_CD_ERINNERUNG_2,
                               id.intValue(),e2Stat);

      dao.insert(vo);
    }

    // update lauf
    new AmvLaufDAO().updateLaufStat(amvLaufId,AmvLaufVO.LAUF_STAT_SELECTED);
  }


  /**
   * Gibt an ob die Erinnerungen 2 nach SAP gesendet werden sollen oder nicht.
   * Die Einstellung erfolgt auf der SES Tabelle Property (via SesMonitor) mit
   * der Eigenschaft "batch.SesAmvStart.e2.enabled".
   *
   * @throws  Exception   Unerwarteter Fehler
   * @return  boolean     Erinnerungen 2 werden übertragen (true)
   *                      oder nicht (false)
   *                      
   * @ejb.interface-method
   */
  public boolean amvSesIsErinnerung2Enabled() throws Exception
  {
    String buf = getProperty("batch.SesAmvStart.e2.enabled");

    if (Integer.parseInt(buf) == 1)
    {
      return true;
    }
    else
    {
      return false;
    }
  }

  /**
   * Gibt an ob vor dem Start eines neuen Laufes überprüft werden soll, ob
   * der alte Lauf abgeschlossen ist oder nicht. Ist die Überprüfung einge-
   * schaltet, so kann nur ein neuer Lauf gestartet werden, wenn der letzte
   * Lauf auf dem Status "finished" ist. Ist die Überprüfung ausgeschaltet,
   * so kann jederzeit ein neuer Lauf gestartet werden.
   *
   * @return  boolean     Überprüfung durchführen (true) oder nicht (false)
   * @throws  Exception   Unexpected Error
   * 
   * @ejb.interface-method
   */
  public boolean amvSesIsCheckLastStateEnabled() throws Exception
  {
    String buf = getProperty("batch.SesAmvStart.checkLastState.enabled");

    if (Integer.parseInt(buf) == 1)
    {
      return true;
    }
    else
    {
      return false;
    }
  }

  /**
   * Aktualisiert einen Eintrag in der SES Tabelle AMV_Rechnung.
   *
   * @param   vo          Amv Rechnungsdaten
   * @throws  Exception   Unerwarteter Fehler
   * 
   * @ejb.interface-method
   */
  public void amvSesSetRechnungStatus(AmvRechnungVO vo) throws Exception
  {
    new AmvRechnungDAO().update(vo);
  }


  /**
   * Aktualisiert das Feld LAUF_STAT der SES Tabelle AMV_LAUF.
   *
   * @param   amvLaufId   Lauf Identifikation
   * @param   laufStat    Status
   * @throws  Exception   Unerwarteter Fehler
   * 
   * @ejb.interface-method
   */
  public void amvSesSetLaufStatus(int amvLaufId, int laufStat) throws Exception
  {
    new AmvLaufDAO().updateLaufStat(amvLaufId,laufStat);
  }


  /**
   * Gibt einen Array mit der Anzahl Rechnungen der Tabelle AMV_RECHNUNG pro
   * Status zurück. Der Array-Index entspricht dabei der Status-Code !
   *
   * @param   amvLaufId   Lauf Id
   * @return  int[]       Array mit der Anzahl Rechnungsempfänger
   * @throws  Exception   Unerwarter Fehler
   * 
   * @ejb.interface-method
   */
  public int[] amvSesGetRechnungCount (int amvLaufId) throws Exception
  {
    int[]           cnt = new int[AmvRechnungVO.RE_STAT_SUPPRESS_E2 + 1];
    AmvRechnungDAO  dao = new AmvRechnungDAO();

    for (int i = 0; i < cnt.length; i++)
    {
      cnt[i] = dao.selectCntByRechnungStat(amvLaufId,i);
    }

    return cnt;
  }


  /**
   * Gibt die einen Array mit der Anzahl Rechnungsempfänger der Tabelle
   * AMV_RECHNUNG pro Rechnungscode (Erinnerung 1 oder 2) und Status zurück.
   * Der Array-Index entspricht dabei der Status-Code !
   *
   * @param   amvLaufId   Lauf Id
   * @param   rechnungCd  Rechnungstyp (Erinnerung 1 oder 2)
   * @return  int[]       Array mit der Anzahl Rechnungsempfänger
   * @throws  Exception   Unerwarter Fehler
   * 
   * @ejb.interface-method
   */
  public int[] amvSesGetRechnungEmpfCount (int amvLaufId, int rechnungCd)
      throws Exception
  {
    int[]           cnt = new int[AmvRechnungVO.RE_STAT_SUPPRESS_E2 + 1];
    AmvRechnungDAO  dao = new AmvRechnungDAO();
    Map             map = null;


    map = dao.selectGroupByStat(amvLaufId,rechnungCd);

    for (int i = 0; i < cnt.length; i++)
    {
      Integer key = new Integer(i);
      Integer val = null;

      if (map.containsKey(key))
      {
        val = (Integer) map.get(key);
        cnt[i] = val.intValue();
        LOG.debug("amvSesGetRechnungCount - key=" + key + ", value=" + val);
      }
    }

    return cnt;
  }


  /**
   * Prüft für einen Lauf, ob die Verarbeitung noch am Laufen ist, oder ob alle
   * Rechnungen an SAP übertragen wurden.
   *
   * @param   amvLaufId   Lauf Identifikation
   * @return  boolean     Lauf ist noch nicht fertig (true) oder ist fertig (false)
   * @throws  Exception   Unerwarter Fehler
   * 
   * @ejb.interface-method
   */
  public boolean amvSesIsRunning (int amvLaufId) throws Exception
  {
    AmvRechnungDAO  dao       = new AmvRechnungDAO();
    int             selected  = 0;
    int             sent      = 0;


    selected = dao.selectCntByRechnungStat(amvLaufId,AmvRechnungVO.RE_STAT_SELECTED);
    sent     = dao.selectCntByRechnungStat(amvLaufId,AmvRechnungVO.RE_STAT_SENT);

    if ((selected == 0) && (sent==0) )
    {
      return false;
    }
    else
    {
      return true;
    }
  }

  /**
   * Prüft ob eine Verarbeitung Fehler hat oder nicht.
   * Beachte:
   * Bevor diese Methode aufgerufen wird, muss sichergestellt werden, dass die
   * Verarbeitung beendet wurde, da nicht verarbeitete Datensätze bei dieser
   * Abfrage nicht berücksichtigt werden.
   *
   * @param   amvLaufId   Lauf Identifikation
   * @return  boolean     Verarbeitung weist Fehler auf (true) oder nicht (false)
   * @throws  Exception   Unerwarter Fehler
   * 
   * @ejb.interface-method
   */
  public boolean amvSesHasErrors (int amvLaufId) throws Exception
  {
    AmvRechnungDAO  dao = new AmvRechnungDAO();
    int             err = 0;


    err = dao.selectCntByRechnungStat(amvLaufId,AmvRechnungVO.RE_STAT_SAP_ERROR);

    if (err == 0)
    {
      return false;
    }
    else
    {
      return true;
    }
  }

  /**
   * Reset Status Feld aller fehlerhaften Datensätze. Die Fehlermeldung wird
   * auf 'reset' gesetzt, damit ersichtlich bleibt, dass der Datensatz bereits
   * zurückgesetzt wurde.
   *
   * @param   amvLaufId     Lauf Identifikation
   * @return  int           Anzahl aktualisierter Datensätze
   * @throws  Exception     Unerwarteter SQL Fehler
   * 
   * @ejb.interface-method
   */
  public int amvSesResetErrors (int amvLaufId) throws Exception
  {
    return new AmvRechnungDAO().resetRechnungStatError(amvLaufId);
  }


  /**
   * Liefert eine Liste mit den Rechnungsnummern (bfakt_id's) für die
   * automatische Markenverlängerung für den angegebenen Rechnungscode
   * (Erinnerung 1 oder Erinnerung 2) von der SES Tabelle AMV_RECHNUNG mit
   * Status "selektiert"
   *
   * @param   amvLaufId       Lauf Identifikation
   * @param   amvRechnungCd   Rechnungscode (Erinnerung 1 oder 2)
   * @return  ArrayList       Integer Liste mit Rechnungsnummern (bfakt_id)
   * @throws  Exception       Unerwarteter Fehler
   * 
   * @ejb.interface-method
   */
  public ArrayList amvSesGetRechnungNrList(int amvLaufId, int amvRechnungCd)
      throws Exception
  {
    return new AmvRechnungDAO().selectIdByCdAndStat
        (amvLaufId, amvRechnungCd, AmvRechnungVO.RE_STAT_SELECTED);
  }


  /**
   * Selektiert eine Array Liste mit Datensätzen der Tabelle AMV_RECHNUNG für
   * die gegebene LaufId und RechnungStat sortiert nach rechnungCd und BfaktId.
   *
   * Werden keine Daten gefunden wird eine leere Liste zurückgegeben.
   *
   * @param   amvLaufId     Lauf Id
   * @param   rechnungStat  Status
   * @throws  Exception     Unerwarteter SQL-Fehler
   * @return  ArrayList     Data Access Object oder null
   * 
   * @ejb.interface-method
   */
  public ArrayList amvSesGetRechnungListByStat(int amvLaufId, int rechnungStat)
    throws Exception
  {
    return new AmvRechnungDAO().selectByRechnungStat(amvLaufId, rechnungStat);
  }


  //---------------------------------------------------------------------------
  // batch job - ses reorganisation
  //---------------------------------------------------------------------------

  /**
   * Löscht alle Einträge der Tabelle, welche älter sind als die angegebene
   * Anzahl Monate.
   *
   * @param   table       Tabelle [jgb_lauf,amv_lauf,stako_kontrolle,trace]
   * @param   months      Anzahl Monate
   * @throws  Exception   unerwartete Fehlermeldung
   * @return  String      Log Meldung mit Angabe zur ausgeführten Aktion
   * 
   * @ejb.interface-method
   */
  public String reorgDatabase (String table, int months) throws Exception
  {
    StringBuffer        result  = new StringBuffer();
    GregorianCalendar   today   = new GregorianCalendar();
    GregorianCalendar   created = new GregorianCalendar();
    List                list    = null;

    //-----------------------------------------------------
    // init
    //-----------------------------------------------------

    LOG.debug("reorgDatabase - start (table=" + table + ") months=" + months +")...");

    created.set(Calendar.MONTH, today.get(Calendar.MONTH) - months);

    result.append ( "\nTable " + table.toUpperCase() +
                    " - entries " + months + " months and older: \n");

    //-----------------------------------------------------
    // clean JGB_LAUF
    // and BFAKTUR_SES, BFAKTJGB_SES, BFAKTPOS_SES, STATISTIK via referential Integrity
    //-----------------------------------------------------
    if (table.equalsIgnoreCase("jgb_lauf"))
    {
      // get entries older than "created date"
      list = new JGBLaufDAO().select(created.getTime());

      // delete entries
      if (list != null && list.size() > 0)
      {
        for (int i = 0; i < list.size(); i++)
        {
          JGBLaufVO vo = (JGBLaufVO)list.get(i);

          if (vo.sapStat == JGBLaufVO.SAP_STAT_FINISHED)
          {
            result.append("  LAUF_NR " + vo.laufNr + " deleted \n");
            new JGBLaufDAO().delete(vo.jgbLaufId);
          }
          else
          {
            result.append ( "  Warning: SAP_STAT isn't FINISHED, " +
                            "LAUF_NR " + vo.laufNr + " NOT deleted! \n" );
          }
        }
      }
      else
      {
        result.append("  no entires found \n");
      }
    }
    //-----------------------------------------------------
    // clean AMV_LAUF
    // and AMV_RECHNUNG via referential Integrity
    //-----------------------------------------------------
    else if (table.equalsIgnoreCase("amv_lauf"))
    {
      // get entries older than "created date"
      list = new AmvLaufDAO().select(created.getTime());

      // delete entries
      if (list != null && list.size() > 0)
      {
        for (int i = 0; i < list.size(); i++)
        {
          AmvLaufVO vo = (AmvLaufVO)list.get(i);

          if (vo.laufStat == AmvLaufVO.LAUF_STAT_FINISHED)
          {
            result.append("  LAUF_NR " + vo.laufNr + " deleted \n");
            new AmvLaufDAO().delete(vo.amvLaufId);
          }
          else
          {
            result.append ( "  Warning: LAUF_STAT isn't FINISHED, " +
                            "LAUF_NR " + vo.laufNr + " NOT deleted! \n" );
          }
        }
      }
      else
      {
        result.append("  no entires found \n");
      }
    }
    //-----------------------------------------------------
    // clean STAKO_KONTROLLE
    //-----------------------------------------------------
    else if (table.equalsIgnoreCase("stako_kontrolle"))
    {
      // get entries older than "created date"
      list = new StakoKontrolleDAO().select(created.getTime());

      // delete entries
      if (list != null && list.size() > 0)
      {
        for (int i = 0; i < list.size(); i++)
        {
          StakoKontrolleVO vo = (StakoKontrolleVO)list.get(i);

          if (vo.stakoStat == StakoKontrolleVO.STATUS_VERARBEITUNG_FINISHED)
          {
            result.append("  STAKO_KONTROLLE_ID " + vo.stakoKontrolleId + " deleted \n");
            new StakoKontrolleDAO().delete(vo.stakoKontrolleId);
          }
          else
          {
            LOG.warn ( "reorgDatabase - STAKO_KONTROLLE_ID " + vo.stakoKontrolleId +
                       " has unfinished state!" );

            result.append ( "  Warning: STAKO_STAT isn't FINISHED, " +
                            "STAKO_KONTROLLE_ID " + vo.stakoKontrolleId + " NOT deleted! \n" );
          }
        }
      }
      else
      {
        result.append("  no entires found \n");
      }
    }
    //-----------------------------------------------------
    // clean TRACE
    //-----------------------------------------------------
    else if (table.equalsIgnoreCase("trace"))
    {
      Properties prop = new TraceDAO().select(created.getTime());

      if ( prop.getProperty("rs").equalsIgnoreCase("true") &&
           ! prop.getProperty("cnt").equals("0") )
      {
        result.append("  TRACE_ID " + prop.getProperty("min") + " to " +
                                      prop.getProperty("max") +
                                      " (" + prop.getProperty("cnt") + " rows) deleted");

        new TraceDAO().delete(created.getTime());
      }
      else
      {
        result.append("  no entires found \n");
      }
    }
    //-----------------------------------------------------
    // table not found/supported
    //-----------------------------------------------------
    else
    {
      throw new Exception( "reorgDatabase - parameter table <" + table + "> " +
                           "is NOT supported");
    }

    //-----------------------------------------------------
    // return result
    //-----------------------------------------------------
    LOG.debug(result.toString());
    return result.toString();
  }



} // end class

