package ch.ige.ses.server.controlling;

import java.sql.SQLException;
import java.util.ArrayList;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.log4j.Logger;

import ch.ige.ses.server.dao.bagis.JoinStakoArtikelDAO;
import ch.ige.ses.server.error.SESErrorWorkLocal;
import ch.ige.ses.server.error.SESErrorWorkLocalHome;
import ch.ige.ses.vo.ses.ErrorVO;

/**
 * SES: Bean zum Einlesen und updaten der Stako relevanten Artikel<br>
 * <p>
 * Copyright (c) 2003, Eidgenössisches Institut für Geistiges Eigentum
 * @author   Stefan Nydegger
 * @version  $Id: StakoBagisReadBean.java,v 1.22 2007/01/10 14:46:37 iten Exp $
 * 
 * @ejb.bean 
 *  name="StakoBagisRead"  
 *  display-name="StakoBagisRead"
 *  view-type="local"
 *  jndi-name="StakoBagisRead"
 *  local-jndi-name="StakoBagisReadLocal"
 *  type="Stateless"
 *  transaction-type="Container"
 * @ejb.transaction 
 *  type="Required"
 * @ejb.ejb-ref
 *   ejb-name="SESErrorWork"
 *   view-type="local"
 */
public class StakoBagisReadBean implements SessionBean
{
  private static final Logger LOG     = Logger.getLogger(StakoBagisReadBean.class);
  
  SESErrorWorkLocalHome sesErrorWorkLocalHome = null;
  SESErrorWorkLocal     sesErrorWorkLocal     = null;
  SessionContext        sessionContext;
  
  //---------------------------------------------------------------------------
  //  bean methods
  //---------------------------------------------------------------------------

  /**
   * create ejb
   */
  public void ejbCreate() 
  {
    try
    {
      //Naming-Kontext ermitteln
      Context ctx = new InitialContext();
      Object ref = ctx.lookup(sesErrorWorkLocalHome.COMP_NAME);
      sesErrorWorkLocalHome = (SESErrorWorkLocalHome)ref;
      sesErrorWorkLocal = sesErrorWorkLocalHome.create();
    }
    catch (CreateException ex)
    {
      LOG.error("StakoBagisReadBean Methode ejbCreate: CreateException: " + ex.getMessage());
    }
    catch (NamingException ex)
    {
      LOG.error("StakoBagisReadBean Methode ejbCreate: NamingException: " + ex.getMessage());
    }
  }
  
  public void ejbRemove()
  {
    // lazy bone
  }
  
  public void ejbActivate()
  {
    // lazy bone
  }

  public void ejbPassivate()
  {
    // lazy bone
  }
  public void setSessionContext(SessionContext sessionContext)
  {
    this.sessionContext = sessionContext;
  }

  //---------------------------------------------------------------------------
  //  methods
  //---------------------------------------------------------------------------
  
  /**
   * Die Methode updateArtikelStatus setzt zuerst den standpreis aller Artikel
   * auf 0, und aktualisiert dann den standpreis bei den übergebenen Artikeln auf 1
   *
   * @param al ArrayList mit adr_nr
   * @return true, wenn Update gelungen<br>
   *         false, wenn unerwarteter Fehler
   *         
   * @ejb.interface-method
   */
  public boolean updateArtikelStatus(ArrayList al)
  {
    JoinStakoArtikelDAO joinStakoArtikelDAO = new JoinStakoArtikelDAO();

    try
    {
      // standpreis aller Artikel auf 0 setzen
      joinStakoArtikelDAO.resetStakoArtikel();

      // standpreis der erhaltenen Artikeln auf 1 setzen
      joinStakoArtikelDAO.updateStakoArtikel(al);
    }
    catch (SQLException sqlex)
    {
     try
     {
       LOG.error("StakoBagisReadBean: Lesen von Bagis fehlgeschlagen");
       //Eintrag in Tabelle ERROR
       ErrorVO errorVO = new ErrorVO();
       errorVO.source = "StakoBagisReadBean Methode: updateArtikelStatus";
       errorVO.employeeNotify = ErrorVO.EMPL_NOTIFY_PENDENT;
       errorVO.employeeMsgType = ErrorVO.EMPL_MSG_TYPE_STAKO;
       errorVO.description = "Updaten standpreis der übergebenen Artikel fehlgeschlagen";
       errorVO.dataTypeCd = ErrorVO.DATA_TYPE_CD_NO_DATA;
       errorVO.errorStat = ErrorVO.ERROR_STAT_PENDENT;
       //Aufruf des Beans SESErrorProcess und Übergabe des ErrorVO
       sesErrorWorkLocal.sendError(errorVO);
     }
     catch (Exception ex)
     {
       LOG.error("StakoBagisReadBean Methode updateArtikelStatus: Eintrag in ERROR nicht möglich");
       throw new EJBException(ex);
     }
   }
   catch (Exception ex)
   {
     throw new EJBException(ex);
   }

    return true;
  }

  /**
   * Die Methode selectEreignisMA() selektiert die Stako relevanten Ereignisse
   * 
   * @param vonDatum Beginn der zu selektierenden Range
   * @param bisDatum   Ende der zu selektierenden Range
   * @param exclMa     Auszuschliessende ma_nr
   * @return ArrayList
   * 
   * @ejb.interface-method
   */
  public ArrayList selectEreignisMA(java.util.Date vonDatum, java.util.Date bisDatum, String exclMa)
  {
    JoinStakoArtikelDAO joinStakoArtikelDAO = new JoinStakoArtikelDAO();

    try
    {
      // holen der Stako relevanten Ereignisse
      return joinStakoArtikelDAO.selectEreignisMA(vonDatum, bisDatum, exclMa);
    }
    catch (SQLException sqlex)
    {
      try
      {
        LOG.error("StakoBagisReadBean: Lesen von Bagis fehlgeschlagen");
        //Eintrag in Tabelle ERROR
        ErrorVO errorVO = new ErrorVO();
        errorVO.source = "StakoBagisReadBean Method: selectEreignisMA";
        errorVO.employeeNotify = ErrorVO.EMPL_NOTIFY_PENDENT;
        errorVO.employeeMsgType = ErrorVO.EMPL_MSG_TYPE_STAKO;
        errorVO.description = "Lesen von Bagis fehlgeschlagen";
        errorVO.dataTypeCd = ErrorVO.DATA_TYPE_CD_NO_DATA;
        errorVO.errorStat = ErrorVO.ERROR_STAT_PENDENT;
        //Aufruf des Beans SESErrorWork und Übergabe des ErrorVO
        sesErrorWorkLocal.sendError(errorVO);
        return null;
      }
      catch (Exception ex)
      {
        LOG.error("StakoBagisReadBean Methode updateArtikelStatus: Eintrag in ERROR nicht möglich");
        throw new EJBException(ex);
      }
    }
  }

  
  /**
   * Die Methode selectBfakturBfaktPos selektiert die Stako relevanten Bfakturen
   * und Positionen
   * 
   * @param vonDatum Beginn der zu selektierenden Range
   * @param bisDatum   Ende der zu selektierenden Range
   * @param exclMa     Auszuschliessende ma_nr
   * @return ArrayList
   * 
   * @ejb.interface-method
   */
  public ArrayList selectBfakturBfaktPos(java.util.Date vonDatum, java.util.Date bisDatum, String exclMa)
  {
    JoinStakoArtikelDAO joinStakoArtikelDAO = new JoinStakoArtikelDAO();
    try
    {
      // holen der Stako relevanten Bfakturen und deren Positionen
      return joinStakoArtikelDAO.selectBfakturBfaktPos(vonDatum, bisDatum, exclMa);
    }
    catch (SQLException sqlex)
    {
      try
      {
        LOG.error("StakoBagisReadBean Method: selectBfakturBfaktPos, Lesen von Bagis fehlgeschlagen");
        //Eintrag in Tabelle ERROR
        ErrorVO errorVO = new ErrorVO();
        errorVO.source = "StakoBagisReadBean Method: selectBfakturBfaktPos";
        errorVO.employeeNotify = ErrorVO.EMPL_NOTIFY_PENDENT;
        errorVO.employeeMsgType = ErrorVO.EMPL_MSG_TYPE_STAKO;
        errorVO.description = "Lesen von Bagis fehlgeschlagen";
        errorVO.dataTypeCd = ErrorVO.DATA_TYPE_CD_NO_DATA;
        errorVO.errorStat = ErrorVO.ERROR_STAT_PENDENT;
        //Aufruf des Beans SESErrorWork und Übergabe des ErrorVO
        sesErrorWorkLocal.sendError(errorVO);
        return null;
      }
      catch (Exception ex)
      {
        LOG.error("StakoBagisReadBean Methode : selectBfakturBfaktPos: Eintrag in ERROR nicht möglich");
        throw new EJBException(ex);
      }
    }
  }
  
} // end of class


