package ch.ige.wdltool.checker;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Locale;

import junit.framework.TestCase;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.time.StopWatch;
import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Logger;

import ch.ige.dbutils.dao.DAOPool;
import ch.ige.dbutils.util.ClientInitializer;
import ch.ige.ipobjects.ServicesFactory;
import ch.ige.wdltool.dao.GeneralTermDAO;
import ch.ige.wdltool.vo.CheckedClassTextGroupVO;
import ch.ige.wdltool.vo.CheckedClassTextPart;
import ch.ige.wdltool.vo.ClassTextPartVO;
import ch.ige.wdltool.vo.WdlCheckResultVO;
import ch.ige.wdlutil.analyze.FormattedWdlTestCases;
import ch.ige.wdlutil.vo.ClassBlockVO;

/**
 * Tests {@link ch.ige.wdltool.checker.WdlChecker}.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author Christian Heinzer
 * @author Eric Lewis
 * @version  $Id: WdlCheckerForWholeWdlTest.java,v 1.3 2008/07/15 08:17:20 lewis_lee Exp $
 */
public class WdlCheckerForWholeWdlTest extends TestCase
{
  static
  {
    BasicConfigurator.configure();
    ClientInitializer.initStandaloneClient();
    ServicesFactory.getStandaloneServices();
  }

  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( WdlCheckerForWholeWdlTest.class );

  /**
   * A border
   */
  private static final String BORDER = "**************************************************************";

  /**
   * The unit under test
   */
  private WdlChecker wdlChecker = new WdlChecker();

  /**
   * Used for performance checks
   */
  private StopWatch stopWatch = new StopWatch();

  /**
   * The real trademarks to check which have a valid syntax
   */
  private static final Integer[] VALID_IP_KEYS = { 1201531251, 101941995, 1201607794, 1201412573, 1201360129, 1200015755, 1200627910, 1200832708, 1201675540,
      1201649793, 1200066186, 1201847506, 1201858529, 1200883245, 1201640203, 1201320971, 1201585829, 1201633900, 149131994 };

  /**
   * The real trademarks to check which have an invalid syntax
   */
  private static final Integer[] INVALID_IP_KEYS = { 141731994, 137901994, 113701995, 100351994, 141551995, 137551995, 137551995, 133641995, 1200038065,
      100031995, 1200729024, 104301994, 133521995, 137271995, 109061994, 113451994, 141781994, 141791994, 109531995, 118151995, 118191995, 122381995,
      126401995, 130541995, 134231995, 134241995, 153881994, 114491995, 122971994, 153951994, 154091995, 122981994, 123111995, 127161994, 142881994, 106341995,
      131411995, 131451995, 1200637252, 1201607794, 1201412573, 1201360129, 1200015755, 1200627910, 1200832708, 1201675540, 191321994, 1201649793, 1201866552,
      1100408333, 1200033866, 1200667233, 1200060372, 1200047193, 1200066186, 1200653105, 1201748960 };

  /**
   * Configure this: If it's <code>true</code>, only trademarks from {@link #VALID_IP_KEYS} are
   * taken, if it's <code>false</code>, also trademarks from {@link #INVALID_IP_KEYS} are added.
   */
  private static final boolean CHECK_PERFORMANCE_ONLY_WITH_VALID_IP_KEYS = false;

  /**
   * The only constructor.
   * @param name The test's name
   */
  public WdlCheckerForWholeWdlTest( String name )
  {
    super( name );
  }

  /**
   * Tests {@link ch.ige.wdltool.checker.WdlChecker#checkWdl(Integer, boolean)}
   * for real, accessing the database, reading all the stuff and generally
   * having a great time  ;-)
   * Since this should not be run as part of the normal unit tests, it doesn't
   * begin with <code>test...</code>
   * @throws Exception if the test fails
   */
  public void checkWdlPerformance() throws Exception
  {
    // Let the terms be cached before testing
    DAOPool.get( GeneralTermDAO.class ).findPositiveTerms( Locale.GERMAN, null );

    List<Integer> ipKeys = new ArrayList<Integer>();
    ipKeys.addAll( Arrays.asList( VALID_IP_KEYS ) );
    if ( !CHECK_PERFORMANCE_ONLY_WITH_VALID_IP_KEYS )
    {
      ipKeys.addAll( Arrays.asList( INVALID_IP_KEYS ) );
    }

    List<WdlCheckPerformance> checkPerformances = new ArrayList<WdlCheckPerformance>();
    for ( Integer ipKey: ipKeys )
    {
      System.out.println( "Checking trademark " + ipKey );
      WdlCheckPerformance checkPerformance = checkTrademark( ipKey, CHECK_PERFORMANCE_ONLY_WITH_VALID_IP_KEYS );
      checkPerformances.add( checkPerformance );
      printResult( checkPerformance, false, false );
    }
    int totalCharactersChecked = 0;
    int totalRuntime = 0;
    for ( WdlCheckPerformance checkPerformance: checkPerformances )
    {
      totalCharactersChecked += checkPerformance.numberOfCharacters;
      totalRuntime += checkPerformance.runtime;
    }
    System.out.println( BORDER );
    System.out.println( BORDER );
    System.out.println( "AVERAGE CHECKED CHARACTERS PER SECOND: " + totalCharactersChecked * 1000 / totalRuntime );
    System.out.println( BORDER );
    System.out.println( BORDER );
  }

  /**
   * Checks whole WDLs.
   * @throws Exception if the test fails
   */
  public void testCheckWdlWithWholeWdls() throws Exception
  {
    // Make sure this doesn't throw an exception
    this.wdlChecker.checkWdl( FormattedWdlTestCases.LONG_CLASS_TEXT_GROUP, Locale.GERMAN, true );

    String wdl = "29 Fleisch, Fisch, Geflügel und Wild; Fleischextrakte;\n" + "   konserviertes, tiefgekühltes, getrocknetes und gekochtes Obst und\n"
        + "   Gemüse; Gallerten (Gelees), Konfitüren, Fruchtmus; Eier, Milch und\n" + "   Milchprodukte; Speiseöle und -fette.";
    WdlCheckResultVO checkResult = this.wdlChecker.checkWdl( wdl, Locale.GERMAN, true );
    // Everything should be green
    ClassBlockVO classBlock29 = checkResult.getWdl().getClassBlocks().get( 0 );
    for ( int i = 0; i < classBlock29.getClassTextGroups().size(); i++ )
    {
      CheckedClassTextGroupVO classTextGroup = (CheckedClassTextGroupVO) classBlock29.getClassTextGroups().get( i );
      for ( ClassTextPartVO classTextPart: classTextGroup.getClassTextParts() )
      {
        String classTextPartString = classTextPart.getClassTextPart();
        LOG.debug( "Checking class text part '" + classTextPartString + "'" );
        if ( classTextPartString.trim().length() > 1 )
        {
          assertEquals( "Checking that class text part '" + classTextPartString + "' is valid", CheckedClassTextPart.CORRECT, classTextPart.getCheckFlag() );
        }
      }
    }

    // Special stopwords in French that end with a <code>'</code>
    wdl = "41 Produits d'éducation.";
    checkResult = this.wdlChecker.checkWdl( wdl, Locale.FRENCH, true );
    ClassBlockVO classBlock41 = checkResult.getWdl().getClassBlocks().get( 0 );
    assertEquals( "Checking class number", 41, classBlock41.getClassNumber().intValue() );
    CheckedClassTextGroupVO classTextGroup41_1 = (CheckedClassTextGroupVO) classBlock41.getClassTextGroups().get( 0 );
    for ( int i = 0; i < classTextGroup41_1.getClassTextParts().size(); i++ )
    {
      ClassTextPartVO classTextPart = classTextGroup41_1.getClassTextParts().get( i );
      switch( i )
      {
        case 0:
          assertEquals( "Checking text part", "Produits", classTextPart.getClassTextPart() );
          assertEquals( "Checking check flag", CheckedClassTextPart.STOPWORD, classTextPart.getCheckFlag() );
          break;
        case 1:
          assertEquals( "Checking text part", " d", classTextPart.getClassTextPart() );
          assertEquals( "Checking check flag", CheckedClassTextPart.STOPWORD, classTextPart.getCheckFlag() );
          break;
        case 2:
          assertEquals( "Checking text part", "'", classTextPart.getClassTextPart() );
          assertEquals( "Checking check flag", CheckedClassTextPart.CORRECT, classTextPart.getCheckFlag() );
          break;
        case 3:
          assertEquals( "Checking text part", "éducation", classTextPart.getClassTextPart() );
          assertEquals( "Checking check flag", CheckedClassTextPart.CORRECT, classTextPart.getCheckFlag() );
          break;
        case 4:
          assertEquals( "Checking text part", ".", classTextPart.getClassTextPart() );
          assertEquals( "Checking check flag", CheckedClassTextPart.CORRECT, classTextPart.getCheckFlag() );
          break;
        default:
          fail( "Faulty number of class text parts found: " + i );
      }
    }
  }

  /**
   * Checks one trademark.
   * @param ipKey The trademark's key
   * @param doStrictSyntaxCheck If this is <code>true</code>, the WDL text must stick to the given syntax exactly, otherwise the parsing is more tolerant
   * @return The performance of the WDL check including the result
   * @throws Exception if the checking fails
   */
  private WdlCheckPerformance checkTrademark( Integer ipKey, boolean doStrictSyntaxCheck ) throws Exception
  {
    WdlCheckPerformance performance = new WdlCheckPerformance();
    this.stopWatch.reset();
    this.stopWatch.start();
    performance.result = this.wdlChecker.checkWdl( ipKey, doStrictSyntaxCheck );
    this.stopWatch.stop();
    performance.ipKey = ipKey;
    performance.runtime = this.stopWatch.getTime();
    performance.numberOfCharacters = performance.result.getWdl().getOriginalWdl().length();
    performance.checkedCharactersPerSecond = performance.numberOfCharacters * 1000 / performance.runtime;
    return performance;
  }

  /**
   * Prints the check result.
   * @param checkPerformance The WDL check performance containing the result
   * @param showOriginalWdl If this is <code>true</code>, the original WDL text is shown
   * @param showCheckedWdl If this is <code>true</code>, the checked WDL result is shown
   */
  private void printResult( WdlCheckPerformance checkPerformance, boolean showOriginalWdl, boolean showCheckedWdl )
  {
    System.out.println( BORDER );
    System.out.println( "Checking trademark '" + checkPerformance.result.getName() + "' (key " + checkPerformance.ipKey + ")" );
    System.out.println( "Runtime: " + checkPerformance.runtime + "ms for " + checkPerformance.numberOfCharacters + " characters" );
    System.out.println( "Corresponds to a speed of " + checkPerformance.checkedCharactersPerSecond + " characters per second" );
    if ( showOriginalWdl )
    {
      System.out.println( "\n\nOriginal WDL is:" );
      System.out.println( checkPerformance.result.getWdl().getOriginalWdl() );
    }
    if ( showCheckedWdl )
    {
      for ( ClassBlockVO classBlock: checkPerformance.result.getWdl().getClassBlocks() )
      {
        System.out.print( "\n\n" );
        if ( classBlock.getClassNumber() == null )
        {
          System.out.println( "(No class number found)" );
        }
        else
        {
          System.out.println( classBlock.getClassNumber() );
        }
        for ( int i = 0; i < classBlock.getClassTextGroups().size(); i++ )
        {
          CheckedClassTextGroupVO classTextGroup = (CheckedClassTextGroupVO) classBlock.getClassTextGroups().get( i );
          for ( ClassTextPartVO classTextPart: classTextGroup.getClassTextParts() )
          {
            System.out.print( "<" + classTextPart.getCheckFlag() + ">" );
            System.out.print( classTextPart.getClassTextPart() );
            System.out.print( "</" + classTextPart.getCheckFlag() + ">" );
          }
          System.out.println( "" );
        }
        if ( StringUtils.isNotEmpty( classBlock.getLimitation() ) )
        {
          System.out.println( "\n" + classBlock.getLimitation() + "\n" );
        }
      }
    }
    System.out.println( BORDER );
  }

  /**
   * Contains the performance data for one check.
   */
  private class WdlCheckPerformance
  {
    /**
     * The IP key
     */
    public Integer ipKey;

    /**
     * The number of characters in the WDL
     */
    public int numberOfCharacters;

    /**
     * The milliseconds that the check used for running
     */
    public long runtime;

    /**
     * The number of characters that were checked per second
     */
    public double checkedCharactersPerSecond;

    /**
     * The result of the check
     */
    public WdlCheckResultVO result;
  }
}
