package ch.ige.wdltool.dao;

import java.sql.Date;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Locale;

import org.apache.commons.lang.ArrayUtils;
import org.apache.log4j.Logger;

import ch.ige.dbutils.dao.DAOPool;
import ch.ige.dbutils.dao.UnitOfWorkException;
import ch.ige.dbutils.runner.IGEQueryRunner;
import ch.ige.dbutils.sql.SQL;
import ch.ige.dbutils.sql.SQLStatement;
import ch.ige.dbutils.sql.SQLStatementCreator;
import ch.ige.dbutils.sql.SQLWhereStatementCreator;
import ch.ige.util.date.DateFormatter;
import ch.ige.util.validation.Check;
import ch.ige.wdltool.enums.DataSource;
import ch.ige.wdltool.enums.TermType;
import ch.ige.wdltool.util.Configuration;
import ch.ige.wdltool.util.TermUtils;
import ch.ige.wdltool.vo.IpiTermCandidateVO;
import ch.ige.wdltool.vo.IpiTermVO;
import ch.ige.wdltool.vo.TermIdCleanupCandidateStatusVO;
import ch.ige.wdltool.vo.TermReplicationVO;

/**
 * Accesses the table <code>TERM</code> for IPI terms.
 * <p>
 * Copyright (c) 2007, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: IpiTermDAO.java,v 1.41 2008/12/04 17:01:49 heinzer Exp $
 */
public class IpiTermDAO extends BaseTermDAO<IpiTermVO, Integer>
{
  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( IpiTermDAO.class );

  /**
   * SQL fragment filtering on expired terms (parameter has to be set on expiry date)
   */
  private static final String EXPIRY_SELECTION_CRITERIA = " AND trunc(renewal_dat) < to_date(?, 'dd.mm.yyyy') ";

  /**
   * Updates a single term to being confirmed if its changed_dat is unchanged.
   */
  private static final SQLStatement<Integer> SET_TERM_TO_CONFIRMED = SQL
      .update( "UPDATE term SET renewal_dat = to_date(?, 'dd.mm.yyyy'), is_active_bool = 1 WHERE term_id = ? AND changed_dat = ?" );

  /**
   * Updates all expired terms to be inactive
   */
  private static final SQLStatement<Integer> INACTIVATE_EXPIRED_TERMS = SQL.update( "UPDATE term SET is_active_bool = 0 " );

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getVOType()
  @Override
  protected Class<IpiTermVO> getVOType()
  {
    return IpiTermVO.class;
  }

  // @see ch.ige.wdltool.dao.BaseTermDAO#getTermType()
  @Override
  protected TermType getTermType()
  {
    return TermType.IPI_TERM;
  }

  // @see ch.ige.wdltool.dao.BaseTermDAO#getTermTypeName()
  @Override
  protected String getTermTypeName()
  {
    return "IPI term";
  }

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getValues(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected Object[] getValues( IpiTermVO term )
  {
    return new Object[] { term.getDataSource(), term.getClassNumber(), term.getTermGroupId(), TermType.IPI_TERM.toDB(), term.getLanguage().getLanguage(),
        term.getTerm(), term.getSearchTerm(), false /* is never generic */, term.isPublic(), term.getPrivateComment(), term.getPublicComment(),
        term.isActive(), term.getRenewal(), term.getUsageCount(), term.getLatestOccurrence(), term.getCreationUser(), term.getChangeUser(),
        term.getChangeDate(), term.isCheckerOnly() };
  }

  /**
   * Finds an IPI term.
   * @param ipiTermId The unique identifier for an IPI term
   * @return The IPI term or <code>null</code> if it doesn't exist
   * @throws SQLException if the query fails
   */
  public IpiTermVO find( final Integer ipiTermId ) throws SQLException
  {
    return (IpiTermVO) super.find( ipiTermId, getVOType() );
  }

  /**
   * Finds all IPI terms.
   * Note: This should not be made public!
   */
  protected List<IpiTermVO> findAll() throws SQLException
  {
    return super.findAll();
  }

  /**
   * Inserts an IPI term and simultaneously removes a corresponding term candidate if it exists.
   * @param term The term to insert
   * @return The new identifier
   * @see ch.ige.wdltool.dao.BaseTermDAO#insert(ch.ige.dbutils.vo.AbstractVO)
   */
  @Override
  public Integer insert( IpiTermVO term )
  {
    try
    {
      return ( insertAndRemoveIpiTermCandidate( term ) ).getNewTermId();
    }
    catch ( SQLException e )
    {
      LOG.error( "Error: could not insert IpiTerm" + e.getMessage(), e );
      return null;
    }
  }

  /**
   * Inserts an IPI term and simultaneously removes a corresponding term candidate if it exists
   * @param term The term to insert
   * @return The new identifier an the flag if a term candidate had been removed
   * @throws SQLException if the query fails
   */
  public TermIdCleanupCandidateStatusVO insertAndRemoveIpiTermCandidate( IpiTermVO term ) throws SQLException
  {
    Check.notNull( term, getTermTypeName() );
    TermUtils.setRenewalTimestamp( term );
    Integer newId = super.insert( term );

    IpiTermCandidateDAO ipiTermCandidateDAO = DAOPool.get( IpiTermCandidateDAO.class );
    List<IpiTermCandidateVO> candidates = ipiTermCandidateDAO.find( term.getLanguage(), term.getClassNumber(), term.getTerm() );
    if ( candidates.size() > 1 )
    {
      LOG.error( "multiple equal candidates have been found and all are going to be deleted: " + term.getTerm() + " in " + term.getLanguage() );
    }
    if ( candidates.size() >= 1 )
    {
      for ( IpiTermCandidateVO candidateVO: candidates )
      {
        ipiTermCandidateDAO.delete( candidateVO.getIdentifier() );
      }
      return new TermIdCleanupCandidateStatusVO( newId, true );
    }
    else
    {
      return new TermIdCleanupCandidateStatusVO( newId, false );
    }
  }

  /**
   * Updates an IPI term and simultaneously removes a corresponding term candidate if it exists.
   * @param term The term to update
   * @see ch.ige.wdltool.dao.BaseTermDAO#update(ch.ige.dbutils.vo.AbstractVO)
   */
  @Override
  public void update( IpiTermVO term )
  {
    try
    {
      updateAndRemoveIpiTermCandidate( term );
    }
    catch ( SQLException e )
    {
      LOG.error( "Error: could not update IpiTerm" + e.getMessage(), e );
      throw new UnitOfWorkException( e );
    }
  }

  /**
   * Updates an IPI term and simultaneously removes a corresponding term candidate if it exists
   * @param term The term to update
   * @return The flag if a term candidate had been removed
   * @throws SQLException if the query fails
   */
  public boolean updateAndRemoveIpiTermCandidate( IpiTermVO term ) throws SQLException
  {
    Check.notNull( term, getTermTypeName() );
    TermUtils.setRenewalTimestamp( term );
    super.update( term );
    IpiTermCandidateDAO ipiTermCandidateDAO = DAOPool.get( IpiTermCandidateDAO.class );
    List<IpiTermCandidateVO> candidates = ipiTermCandidateDAO.find( term.getLanguage(), term.getClassNumber(), term.getTerm() );
    if ( candidates.size() > 1 )
    {
      LOG.error( "multiple equal candidates have been found and all are going to be deleted: " + term.getTerm() + " in " + term.getLanguage() );
    }
    if ( candidates.size() >= 1 )
    {
      for ( IpiTermCandidateVO candidateVO: candidates )
      {
        ipiTermCandidateDAO.delete( candidateVO.getIdentifier() );
      }
      return true;
    }
    else
    {
      return false;
    }
  }

  /**
   * Creates there <code>WHERE</code> statement for {@link #findTermWithinInactiveTerms(Locale, Integer, String)}.
   * @param language The language in which terms are searched for
   * @param classNumber The class number in which the terms are searched for
   * @param term The exact term to search for
   * @return The <code>WHERE</code> statement to execute
   */
  SQLWhereStatementCreator createWhereStatementForFindTermWithinInactiveTerms( Locale language, Integer classNumber, String term )
  {
    checkTermLength( term );

    SQLWhereStatementCreator whereStatement = new SQLWhereStatementCreator();

    // IPI term candidates must never appear
    whereStatement.appendFragmentWithParameters( " term_type = ?", TermType.IPI_TERM.toDB() );
    // Only active terms are taken
    whereStatement.appendFragmentWithParameters( " AND is_active_bool = ?", new Integer( 0 ) );
    if ( language != null )
    {
      whereStatement.appendFragmentWithParameters( " AND language_sn = ?", language.getLanguage() );
    }
    whereStatement.appendFragmentWithParameters( " AND class_id = ?", classNumber );
    whereStatement.appendFragmentWithParameters( " AND term = ?", term );
    return whereStatement;
  }

  /**
   * Finds all inactive terms of a language that correspond to the exact phrase in <code>term</code> within class
   * <code>classNumber</code>. <strong>Note that this is never cached!</strong>
   * @param language The language in which terms are searched for
   * @param classNumber The class in which the terms are searched for
   * @param term The exact term to search for
   * @return The terms that were found or an empty <code>List</code> if none were found
   * @throws SQLException if the query fails
   */
  public List<IpiTermVO> findTermWithinInactiveTerms( Locale language, Integer classNumber, String term ) throws SQLException
  {
    Check.notNull( language, "language" );
    Check.notEmpty( term, "term" );
    Check.notNull( classNumber, "class number" );

    SQLStatementCreator<IpiTermVO> creator = new SQLStatementCreator<IpiTermVO>( getTableName(), getIdColumnName(), getColumnNames(), getIdSequenceName(),
        getAliases() );
    SQLWhereStatementCreator whereStatement = createWhereStatementForFindTermWithinInactiveTerms( language, classNumber, term );
    appendDefaultOrderBy( whereStatement, language );
    SQLStatement<IpiTermVO> completeQuery = creator.selectBy( whereStatement );
    return selectVOList( 1, IpiTermVO.class, completeQuery, whereStatement.getStatementParameters() );
  }

  /**
   * Finds inactive terms.
   * @return The inactive terms
   * @throws SQLException if the query fails
   * @see ch.ige.wdltool.ejb.WdlServices#findInactiveTerms()
   */
  public List<IpiTermVO> findInactiveTerms() throws SQLException
  {
    Integer termValidityPeriod = Configuration.getTermValidityPeriod( DataSource.IPI );
    Integer termConfirmationPeriod = Configuration.getTermConfirmationPeriod( DataSource.IPI );
    return findInactiveTerms( termValidityPeriod, termConfirmationPeriod );
  }

  /**
   * Finds inactive terms.
   * @param termValidityPeriod The number of days that terms are valid
   * @param termConfirmationPeriod The number of days before the expiry date in which the term must be confirmed
   * @return The inactive terms
   * @throws SQLException if the query fails
   * @see ch.ige.wdltool.ejb.WdlServices#findInactiveTerms()
   */
  List<IpiTermVO> findInactiveTerms( int termValidityPeriod, int termConfirmationPeriod ) throws SQLException
  {
    SQLStatementCreator<IpiTermVO> creator = new SQLStatementCreator<IpiTermVO>( getTableName(), getIdColumnName(), getColumnNames(), getIdSequenceName(),
        getAliases() );

    SQLWhereStatementCreator whereStatement = new SQLWhereStatementCreator();
    whereStatement.appendFragmentWithParameters( " term_type = ?", TermType.IPI_TERM.toDB() );
    whereStatement.appendFragment( " AND is_active_bool = 0" );
    final SQLStatement<IpiTermVO> completeQuery = creator.selectBy( whereStatement );

    List<IpiTermVO> inactiveTerms = selectVOList( 5000, IpiTermVO.class, completeQuery, whereStatement.getStatementParameters() );
    setExpirationDateAndSort( inactiveTerms, termValidityPeriod, termConfirmationPeriod );
    return inactiveTerms;
  }

  /**
   * Inactivates expired terms.
   * @return The inactivated terms
   * @see ch.ige.wdltool.ejb.WdlServices#findExpiredTerms()
   */
  public List<IpiTermVO> inactivateExpiredTerms()
  {
    Integer termValidityPeriod = Configuration.getTermValidityPeriod( DataSource.IPI );
    Integer termConfirmationPeriod = Configuration.getTermConfirmationPeriod( DataSource.IPI );
    return inactivateExpiredTerms( new Date( System.currentTimeMillis() ), termValidityPeriod, termConfirmationPeriod );
  }

  /**
   * Inactivates expired terms.
   * @param expiryDate The date for which terms are checked
   * @param termValidityPeriod The number of days that terms are valid
   * @param termConfirmationPeriod The number of days before the expiry date in which the term must be confirmed
   * @return The list of inactivated terms
   */
  List<IpiTermVO> inactivateExpiredTerms( final Date expiryDate, final int termValidityPeriod, final int termConfirmationPeriod )
  {
    return this.execute( new SingleDAOUnitOfWork<List<IpiTermVO>>()
    {
      // @see ch.ige.dbutils.dao.UnitOfWork#execute(ch.ige.dbutils.runner.IGEQueryRunner)
      public void execute( IGEQueryRunner runner ) throws SQLException
      {

        SQLWhereStatementCreator whereStatement = createBaseExpirationStatement();
        whereStatement.appendFragmentWithParameters( EXPIRY_SELECTION_CRITERIA, COMPLEX_SEARCH_DATE_FORMAT.format( getExpirationStart( expiryDate,
            termValidityPeriod, termConfirmationPeriod ) ) );

        SQLStatementCreator<IpiTermVO> creator = new SQLStatementCreator<IpiTermVO>( getTableName(), getIdColumnName(), getColumnNames(), getIdSequenceName(),
            getAliases() );

        final SQLStatement<IpiTermVO> completeQuery = creator.selectBy( whereStatement );
        List<IpiTermVO> termsToUpdate = selectVOList( 5000, IpiTermVO.class, completeQuery, whereStatement.getStatementParameters() );

        if ( LOG.isDebugEnabled() )
        {
          LOG.debug( "Complete SQL Statement inactivation of expired terms: " + INACTIVATE_EXPIRED_TERMS + whereStatement.toString() );
          LOG.debug( "Parameters " + ArrayUtils.toString( whereStatement.getStatementParameters() ) );
        }

        int count = runner.update( INACTIVATE_EXPIRED_TERMS + whereStatement.toString(), whereStatement.getStatementParameters() );

        if ( count != termsToUpdate.size() )
        {
          throw new SQLException( "inactivation of expried terms did not affect the expected number of terms - rolling back..." );
        }
        this.setResult( termsToUpdate );
      }
    } );
  }

  /**
   * Finds expired terms within active terms.
   * @return The expired terms
   * @throws SQLException if the query fails
   * @see ch.ige.wdltool.ejb.WdlServices#findExpiredTerms()
   */
  public List<IpiTermVO> findExpiredTerms() throws SQLException
  {
    Integer termValidityPeriod = Configuration.getTermValidityPeriod( DataSource.IPI );
    Integer termConfirmationPeriod = Configuration.getTermConfirmationPeriod( DataSource.IPI );
    return findExpiredTerms( new Date( System.currentTimeMillis() ), termValidityPeriod, termConfirmationPeriod );
  }

  /**
   * Finds expired terms within active terms.
   * <p>
   * Imagine the following, where <code>T</code> is the term, and the arrow is its <code>termValidityPeriod</code>.
   * The <code>termValidityPeriod</code> is the period between <code>renewal date</code> and
   * <code>conf. start</code> - or in other words:<br>
   * <code>renewal date + termValidityPeriod + termConfirmationPeriod = expiry date</code>:
   * </p>
   * 
   * <pre>
   *         renewal       conf.   expiry
   *          date         start    date
   * |----------|------------|-------|
   *            T----------&gt;
   * 
   * </pre>
   * 
   * <p>
   * As you see, the term is still valid. However, if its renewal date is later back, the picture changes:
   * </p>
   * 
   * <pre>
   *                    renewal       conf.   expiry
   *                     date         start    date
   * |---------------------|------------|-------|
   *                       T---------------------------&gt;
   * </pre>
   * 
   * <p>
   * Here it's obvious that the term has expired.
   * </p>
   * On the expiry date day the terms are still considered expiring (not yet expired)
   * <br>
   * Note that this sets the following additional attributes:
   * <ul>
   * <li>{@link IpiTermVO#getExpirationDate()}</li>
   * </ul>
   * @param expiryDate The date for which terms are checked
   * @param termValidityPeriod The number of days that terms are valid
   * @param termConfirmationPeriod The number of days before the expiry date in which the term must be confirmed
   * @return The expired terms
   * @throws SQLException if the query fails
   * @see ch.ige.wdltool.ejb.WdlServices#findExpiredTerms()
   */
  List<IpiTermVO> findExpiredTerms( Date expiryDate, int termValidityPeriod, int termConfirmationPeriod ) throws SQLException
  {
    SQLStatementCreator<IpiTermVO> creator = new SQLStatementCreator<IpiTermVO>( getTableName(), getIdColumnName(), getColumnNames(), getIdSequenceName(),
        getAliases() );

    SQLWhereStatementCreator whereStatement = createBaseExpirationStatement();
    whereStatement.appendFragmentWithParameters( EXPIRY_SELECTION_CRITERIA, COMPLEX_SEARCH_DATE_FORMAT.format( getExpirationStart( expiryDate,
        termValidityPeriod, termConfirmationPeriod ) ) );

    final SQLStatement<IpiTermVO> completeQuery = creator.selectBy( whereStatement );

    if ( LOG.isDebugEnabled() )
    {
      LOG.debug( "Complete SQL Statement of expired terms: " + completeQuery );
      for ( Object parameter: whereStatement.getStatementParameters() )
      {
        LOG.debug( "Parameter " + parameter.toString() );
      }
    }

    List<IpiTermVO> expiredTerms = selectVOList( 500, IpiTermVO.class, completeQuery, whereStatement.getStatementParameters() );
    setExpirationDateAndSort( expiredTerms, termValidityPeriod, termConfirmationPeriod );
    return expiredTerms;
  }

  /**
   * Finds terms which are going to expire.
   * @return The terms which are going to expire
   * @throws SQLException if the query fails
   * @see ch.ige.wdltool.ejb.WdlServices#findExpiringTerms()
   */
  public List<IpiTermVO> findExpiringTerms() throws SQLException
  {
    Integer termValidityPeriod = Configuration.getTermValidityPeriod( DataSource.IPI );
    Integer termConfirmationPeriod = Configuration.getTermConfirmationPeriod( DataSource.IPI );
    return findExpiringTerms( new Date( System.currentTimeMillis() ), termValidityPeriod, termConfirmationPeriod );
  }

  /**
   * Finds terms which are going to expire.
   * <p>
   * Imagine the following, where <code>T</code> is the term, and the arrow is its <code>termValidityPeriod</code>.
   * The <code>termConfirmationPeriod</code> is the period between <code>start conf.</code> below and
   * <code>expiryDate</code> - or in other words:<br>
   * <code>conf. start + termConfirmationPeriod = expiryDate</code>:
   * </p>
   * 
   * <pre>
   *         renewal       conf.   expiry
   *          date         start    date
   * |----------|------------|-------|
   *            T----------&gt;
   * </pre>
   * 
   * <p>
   * The term is valid, but it will have to be confirmed soon. However, if its renewal date is later back, the picture
   * changes:
   * </p>
   * 
   * <pre>
   *            renewal    conf.   expiry
   *             date      start    date
   * |-------------|---------|-------|
   *               T----------&gt;
   * </pre>
   * 
   * <p>
   * The following term, however is not in the confirmation period anymore, since it has expired.
   * </p>
   * 
   * <pre>
   *                 renewal conf.   expiry
   *                    date start   date
   * |---------------------|-|-------|
   *                       T----------&gt;
   * </pre>
   * 
   * On the expiry date day the terms are still considered expiring (not yet expired)
   * On the conf. start day the terms are not yet considered expiring
   * <br>
   * Note that this sets the following additional attributes:
   * <ul>
   * <li>{@link IpiTermVO#getExpirationDate()}</li>
   * </ul>
   * @param expiryDate The date for which terms are checked
   * @param termValidityPeriod The number of days that terms are valid
   * @param termConfirmationPeriod The number of days before the expiry date in which the term must be confirmed
   * @return The expired terms
   * @throws SQLException if the query fails
   * @see ch.ige.wdltool.ejb.WdlServices#findExpiringTerms()
   */
  List<IpiTermVO> findExpiringTerms( Date expiryDate, int termValidityPeriod, int termConfirmationPeriod ) throws SQLException
  {
    SQLStatementCreator<IpiTermVO> creator = new SQLStatementCreator<IpiTermVO>( getTableName(), getIdColumnName(), getColumnNames(), getIdSequenceName(),
        getAliases() );

    SQLWhereStatementCreator whereStatement = createBaseExpirationStatement();
    whereStatement.appendFragmentWithParameters( " AND trunc(renewal_dat + " + termValidityPeriod + ") >= to_date(?, 'dd.mm.yyyy') ",
        COMPLEX_SEARCH_DATE_FORMAT.format( getConfirmationStart( expiryDate, termConfirmationPeriod ) ) );
    whereStatement.appendFragmentWithParameters( " AND trunc(renewal_dat + " + termValidityPeriod + ") < to_date(?, 'dd.mm.yyyy') ", COMPLEX_SEARCH_DATE_FORMAT
        .format( expiryDate ) );

    final SQLStatement<IpiTermVO> completeQuery = creator.selectBy( whereStatement );
    if ( LOG.isDebugEnabled() )
    {
      LOG.debug( "Complete SQL Statement of expiring terms: " + completeQuery );
      for ( Object parameter: whereStatement.getStatementParameters() )
      {
        LOG.debug( "Parameter " + parameter.toString() );
      }
    }

    List<IpiTermVO> expiredTerms = selectVOList( 500, IpiTermVO.class, completeQuery, whereStatement.getStatementParameters() );
    setExpirationDateAndSort( expiredTerms, termValidityPeriod, termConfirmationPeriod );
    return expiredTerms;
  }

  /**
   * Renews the given terms, that is it confirms them as valid.
   * @param terms The terms to renew
   * @return The ids of terms that could not be renewed because of a collision (updated since the state the term has in
   * the <code>terms</code> list)
   */
  @SuppressWarnings( "unchecked" )
  public List<Integer> renewTerms( final List<IpiTermVO> terms )
  {
    return this.renewTerms( new java.util.Date(), terms );
  }

  /**
   * Updates the given terms. After the update, the terms count as being renewed.
   * @param renewalDate The renewal date to set
   * @param terms The terms to change
   * @return The IDs of terms that could not be renewed because of a collision
   * (updated since the state the term has in the <code>terms</code>)
   */
  @SuppressWarnings( "unchecked" )
  List<Integer> renewTerms( final java.util.Date renewalDate, final List<IpiTermVO> terms )
  {
    final List<Integer> notRenewableTerms = new ArrayList<Integer>();

    Check.notNull( renewalDate, "renewal Date" );
    Check.notNull( terms, "terms" );

    if ( !terms.isEmpty() )
    {

      final String sqlStatement = SET_TERM_TO_CONFIRMED.toString();
      for ( final IpiTermVO termVO: terms )
      {
        this.execute( new SingleDAOUnitOfWork<Integer>()
        {
          // @see ch.ige.dbutils.dao.UnitOfWork#execute(ch.ige.dbutils.runner.IGEQueryRunner)
          public void execute( IGEQueryRunner runner ) throws SQLException
          {
            int count = runner.update( sqlStatement, new Object[] { DateFormatter.shortFormat( renewalDate ), termVO.getIdentifier(), termVO.getChangeDate() } );
            if ( count != 1 )
            {
              LOG.error( "Collision, term with id " + termVO.getIdentifier() + " had been changed since last reading" );
              notRenewableTerms.add( termVO.getIdentifier() );
            }
          }
        } );
      }
    }
    IpiTermDAO.this.clearCache();
    return notRenewableTerms;
  }

  @Override
  public TermReplicationVO getReplicationTerms( String schemaName ) throws SQLException
  {
    SQLStatementCreator<IpiTermVO> statementCreator = new SQLStatementCreator<IpiTermVO>( schemaName + "." + getTableName(), getIdColumnName(),
        getColumnNames(), getIdSequenceName() );
    SQLStatement<Integer> insertStatement = statementCreator.insert( false );

    // Replicates IPI terms that are active and public
    List<IpiTermVO> terms = findAll();

    List<Object[]> termValues = new ArrayList<Object[]>( terms.size() );
    for ( IpiTermVO ipiTerm: terms )
    {
      if ( ipiTerm.isActive() && ipiTerm.isPublic() )
      {
        Object[] insertValues = getValues( ipiTerm );
        termValues.add( ArrayUtils.addAll( new Object[] { ipiTerm.getIdentifier() }, insertValues ) );
      }
    }

    return new TermReplicationVO( insertStatement, termValues );
  }

  /**
   * Creates the base expiration statement for terms.
   * @return The base expiration statement
   */
  private SQLWhereStatementCreator createBaseExpirationStatement()
  {
    SQLWhereStatementCreator whereStatement = new SQLWhereStatementCreator();
    whereStatement.appendFragmentWithParameters( " term_type = ?", TermType.IPI_TERM.toDB() );
    whereStatement.appendFragment( " AND renewal_dat IS NOT NULL AND is_active_bool = 1" );
    return whereStatement;
  }

  /**
   * Sets the expiration date in all expired terms and sorts by it.
   * @param terms The terms which are expired or going to expire
   * @param termValidityPeriod The number of days that terms are valid
   * @param termConfirmationPeriod The number of days before the expiry date in which the term must be confirmed
   */
  private void setExpirationDateAndSort( List<IpiTermVO> terms, int termValidityPeriod, int termConfirmationPeriod )
  {
    for ( IpiTermVO expiredTerm: terms )
    {
      Calendar termExpirationDate = Calendar.getInstance();
      termExpirationDate.setTime( expiredTerm.getRenewal() );
      termExpirationDate.add( Calendar.DAY_OF_MONTH, termValidityPeriod + termConfirmationPeriod );
      expiredTerm.setExpirationDate( new Date( termExpirationDate.getTime().getTime() ) );
    }
    Collections.sort( terms, new ExpirationDateComparator() );
  }

  /**
   * Gets the start of the confirmation period.
   * @param expiryDate The date for which terms are checked
   * @param termConfirmationPeriod The number of days before the expiry date in which the term must be confirmed
   * @return The start of the confirmation period
   */
  private java.util.Date getConfirmationStart( Date expiryDate, int termConfirmationPeriod )
  {
    Calendar confirmationStart = Calendar.getInstance();
    confirmationStart.setTime( expiryDate );
    confirmationStart.add( Calendar.DAY_OF_MONTH, -termConfirmationPeriod );
    return confirmationStart.getTime();
  }

  /**
   * Gets the start of the expiration period.
   * @param expiryDate The date for which terms are checked
   * @param termConfirmationPeriod The number of days before the expiry date in which the term must be confirmed
   * @param termValidityPeriod The number of days that terms are valid
   * @return The start of the confirmation period
   */
  private java.util.Date getExpirationStart( Date expiryDate, int termValidityPeriod, int termConfirmationPeriod )
  {
    java.util.Date confirmationDate = getConfirmationStart( expiryDate, termConfirmationPeriod );
    Calendar expirationStart = Calendar.getInstance();
    expirationStart.setTime( confirmationDate );
    expirationStart.add( Calendar.DAY_OF_MONTH, -termValidityPeriod );
    return expirationStart.getTime();
  }

  /**
   * Compares two expired terms by expiration date (ascending).
   */
  private static class ExpirationDateComparator implements Comparator<IpiTermVO>
  {
    // @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
    public int compare( IpiTermVO term1, IpiTermVO term2 )
    {
      java.util.Date expirationDate1 = term1.getExpirationDate();
      java.util.Date expirationDate2 = term2.getExpirationDate();
      return expirationDate1.compareTo( expirationDate2 );
    }
  }
}
