package ch.ige.wdltool.dao;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang.ArrayUtils;

import ch.ige.dbutils.sql.SQLStatement;
import ch.ige.dbutils.sql.SQLStatementCreator;
import ch.ige.wdltool.enums.TermType;
import ch.ige.wdltool.vo.NegativeIpiTermVO;
import ch.ige.wdltool.vo.TermReplicationVO;

/**
 * Accesses the table <code>TERM</code> for negative IPI terms.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: NegativeIpiTermDAO.java,v 1.11 2008/12/04 17:01:49 heinzer Exp $
 */
public class NegativeIpiTermDAO extends BaseTermDAO<NegativeIpiTermVO, Integer>
{
  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getVOType()
  @Override
  protected Class<NegativeIpiTermVO> getVOType()
  {
    return NegativeIpiTermVO.class;
  }

  // @see ch.ige.wdltool.dao.BaseTermDAO#getTermType()
  @Override
  protected TermType getTermType()
  {
    return TermType.NEGATIVE_IPI_TERM;
  }

  // @see ch.ige.wdltool.dao.BaseTermDAO#getTermTypeName()
  @Override
  protected String getTermTypeName()
  {
    return "Negative IPI term";
  }

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getValues(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected Object[] getValues( NegativeIpiTermVO term )
  {
    return new Object[] { term.getDataSource(), term.getClassNumber(), null /* belongs to no term group */, TermType.NEGATIVE_IPI_TERM.toDB(),
        term.getLanguage().getLanguage(), term.getTerm(), term.getSearchTerm(), false /* is never generic */, term.isPublic(), term.getPrivateComment(),
        term.getPublicComment(), true /* is always active */, null /* doesn't expire */, null /* is not counted for usage */,
        null /* is not checked for usage */, term.getCreationUser(), term.getChangeUser(), term.getChangeDate(), term.isCheckerOnly() };
  }

  /**
   * Finds a negative IPI term.
   * @param negativeIpiTermId The unique identifier for a negative IPI term
   * @return The negative IPI term or <code>null</code> if it doesn't exist
   * @throws SQLException if the query fails
   */
  public NegativeIpiTermVO find( final Integer negativeIpiTermId ) throws SQLException
  {
    return (NegativeIpiTermVO) super.find( negativeIpiTermId, getVOType() );
  }

  /**
   * Finds all negative IPI terms.
   * Note: This should not be made public!
   */
  protected List<NegativeIpiTermVO> findAll() throws SQLException
  {
    return super.findAll();
  }

  @Override
  public TermReplicationVO getReplicationTerms( String schemaName ) throws SQLException
  {
    SQLStatementCreator<NegativeIpiTermVO> statementCreator = new SQLStatementCreator<NegativeIpiTermVO>( schemaName + "." + getTableName(), getIdColumnName(),
        getColumnNames(), getIdSequenceName() );
    SQLStatement<Integer> insertStatement = statementCreator.insert( false );

    // Replicates public Negative IPI terms
    List<NegativeIpiTermVO> terms = findAll();

    List<Object[]> termValues = new ArrayList<Object[]>( terms.size() );
    for ( NegativeIpiTermVO negativeIpiTerm: terms )
    {
      if ( negativeIpiTerm.isPublic() )
      {
        Object[] insertValues = getValues( negativeIpiTerm );
        termValues.add( ArrayUtils.addAll( new Object[] { negativeIpiTerm.getIdentifier() }, insertValues ) );
      }
    }

    return new TermReplicationVO( insertStatement, termValues );
  }
}
