package ch.ige.wdltool.ejb.impl;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import ch.ige.dbutils.dao.DAOPool;
import ch.ige.dbutils.vo.AbstractVO;
import ch.ige.wdltool.dao.GeneralTermDAO;
import ch.ige.wdltool.dao.IpiTermCandidateDAO;
import ch.ige.wdltool.dao.StopwordDAO;
import ch.ige.wdltool.exception.ExceptionThrower;
import ch.ige.wdltool.exception.WdlObjectNotFoundException;
import ch.ige.wdltool.exception.WdlUpdateException;
import ch.ige.wdltool.exception.WdlValidationException;
import ch.ige.wdltool.exception.WrappedRuntimeException;
import ch.ige.wdltool.vo.AbstractTermVO;
import ch.ige.wdltool.vo.ClassVO;
import ch.ige.wdltool.vo.IpiTermCandidateVO;
import ch.ige.wdltool.vo.NegativeIpiTermVO;
import ch.ige.wdltool.vo.StopwordVO;

/**
 * Implements CRUD services for IPI term candidates. <strong>Note that IPI term candidates are never cached!</strong>
 * <p>
 * Copyright (c) 2007, Swiss Federal Institute of Intellectual Property
 * </p>
 * 
 * @author Eric Lewis
 * @version $Id: IpiTermCandidateImpl.java,v 1.17 2008/05/08 10:44:32 lewis_lee Exp $
 */
public class IpiTermCandidateImpl extends BaseTermServicesImpl<IpiTermCandidateDAO, IpiTermCandidateVO, Integer>
{
  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( IpiTermCandidateImpl.class );

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#getObjectName()
  @Override
  protected String getObjectName()
  {
    return "IPI term candidate";
  }

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#getDAOClass()
  @Override
  protected Class<IpiTermCandidateDAO> getDAOClass()
  {
    return IpiTermCandidateDAO.class;
  }

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#find(ch.ige.dbutils.dao.DAO, java.io.Serializable)
  @Override
  protected IpiTermCandidateVO find( IpiTermCandidateDAO dao, Integer id ) throws SQLException
  {
    return dao.find( id );
  }

  /**
   * Finds an IPI term candidate.
   * @param language The language in which terms are searched for
   * @param classNumber The class in which terms are searched for
   * @param searchTerm The term to look for
   * @return The IPI term candidate if it exists, <code>null</code> otherwise.
   * @throws WdlObjectNotFoundException if the IPI term candidate can't be found
   * @throws WrappedRuntimeException if any argument is wrong
   * @see ch.ige.wdltool.ejb.WdlServices#findIpiTermCandidate(java.util.Locale, java.lang.Integer, java.lang.String)
   */
  public IpiTermCandidateVO findIpiTermCandidate( Locale language, Integer classNumber, String searchTerm ) throws WdlObjectNotFoundException,
      WrappedRuntimeException
  {
    List<IpiTermCandidateVO> terms = new ArrayList<IpiTermCandidateVO>();
    try
    {
      terms = DAOPool.get( IpiTermCandidateDAO.class ).find( language, classNumber, searchTerm );
    }
    catch ( SQLException e )
    {
      ExceptionThrower.throwObjectNotFoundException( "Couldn't search for term candidate in: " + language + " and class: " + classNumber + " with term: "
          + searchTerm, e, null );
    }
    catch ( RuntimeException e )
    {
      ExceptionThrower.throwRuntimeException( e.getMessage(), e );
    }
    LOG.debug( "searched term: " + searchTerm + ", found " + terms.size() );
    if ( terms.size() == 0 )
    {
      return null;
    }
    else
    {
      if ( terms.size() != 1 )
      { // this should not happen, could lead to strange effects concerning usage count
        throw new IllegalStateException( "Found several equal term candidates for lang " + language + ", term " + searchTerm );
      }
      return terms.get( 0 );
    }
  }

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#insert(ch.ige.dbutils.dao.DAO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected IpiTermCandidateVO insert( IpiTermCandidateDAO dao, IpiTermCandidateVO vo )
  {
    dao.insert( vo );
    return vo;
  }

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#update(ch.ige.dbutils.dao.DAO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void update( IpiTermCandidateDAO dao, IpiTermCandidateVO vo )
  {
    dao.update( vo );
  }

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#delete(ch.ige.dbutils.dao.DAO, java.io.Serializable)
  @Override
  protected void delete( IpiTermCandidateDAO dao, Integer id )
  {
    dao.delete( id );
  }

  /**
   * Gets the terms candidates (uncached) for a given language ordered by usage_count.
   * 
   * @param dao The DAO to use
   * @param language The language, if <code>null</code> all languages are searched.
   * @return The term candidates that were found or an empty <code>List</code> if none were found
   * @throws WrappedRuntimeException
   * @throws SQLException
   */
  protected List<IpiTermCandidateVO> findByLanguage( IpiTermCandidateDAO dao, Locale language ) throws WrappedRuntimeException, SQLException
  {
    try
    {
      return dao.findAll( language );
    }
    catch ( RuntimeException e )
    {
      ExceptionThrower.throwRuntimeException( "Can't find term candidates", e );
    }
    return null;
  }

  /**
   * Activates a term candidate.
   * 
   * @param dao The DAO to use
   * @param vo The term candidate set active
   * @throws WrappedRuntimeException if the term's data is faulty
   * @throws WdlUpdateException if the term can't be activated
   * @throws WdlValidationException if the term's data is invalid
   */
  protected void activate( IpiTermCandidateDAO dao, IpiTermCandidateVO vo ) throws WrappedRuntimeException, WdlUpdateException, WdlValidationException
  {
    String errorMessage = "Couldn't activate candidate " + getObjectName() + " with ID " + vo.getIdentifier();
    try
    {
      validate( vo, true );
      dao.activate( vo );
    }
    catch ( SQLException e )
    {
      ExceptionThrower.throwUpdateException( errorMessage, e, vo );
    }
    catch ( RuntimeException e )
    {
      ExceptionThrower.throwRuntimeException( errorMessage, e );
    }
  }

  /**
   * Gets the variants that can be extracted from the IPI term candidate's term. This means splitting it up into its
   * class text sub-groups, ignoring all of them that correspond to an existing stopword or an existing active term.
   * 
   * @param ipiTermCandidate The IPI term candidate containing the term
   * @return The term variants, which can be empty
   * @throws WrappedRuntimeException if the term candidate's data is faulty
   * @throws WdlObjectNotFoundException if a query for terms or stopword fails
   */
  public List<String> extractTermVariants( IpiTermCandidateVO ipiTermCandidate ) throws WrappedRuntimeException, WdlObjectNotFoundException
  {
    String errorMessage = "Found no terms or stopwords for IPI term candidate";
    List<String> termVariants = new ArrayList<String>();
    if ( StringUtils.isNotEmpty( ipiTermCandidate.getTerm() ) )
    {
      try
      {
        String[] candidates = ipiTermCandidate.getTerm().split( ",\\s*" );

        Map<String, AbstractVO> prohibited = getStopwordsAndActiveTerms( ipiTermCandidate );
        for ( String candidate: candidates )
        {
          boolean addVariant = false;
          LOG.debug( "Checking term variant '" + candidate + "'" );
          String lowerCaseCandidate = candidate.toLowerCase().trim();
          if ( prohibited.containsKey( lowerCaseCandidate ) )
          {
            LOG.debug( "Term variant '" + candidate + "' found in " + prohibited.get( lowerCaseCandidate ) );
            AbstractVO abstractVO = prohibited.get( lowerCaseCandidate );
            // Stopwords with the same text are always prohibited
            if ( abstractVO instanceof AbstractTermVO )
            {
              AbstractTermVO term = (AbstractTermVO) abstractVO;
              // If the terms don't have the same class number, it's ok
              if ( term.getClassNumber() == null )
              {
                LOG.debug( "No class number found" );
                addVariant = true;
              }
              else if ( !term.getClassNumber().equals( ipiTermCandidate.getClassNumber() ) )
              {
                LOG.debug( "Class numbers are different" );
                addVariant = true;
              }
            }
          }
          else
          {
            addVariant = true;
          }

          if ( addVariant )
          {
            LOG.debug( "Variant '" + candidate + "' is added" );
            termVariants.add( candidate );
          }
          else
          {
            LOG.debug( "Variant '" + candidate + "' is ignored" );
          }
        }
      }
      catch ( SQLException e )
      {
        ExceptionThrower.throwObjectNotFoundException( errorMessage, e, ipiTermCandidate.getIdentifier() );
      }
      catch ( RuntimeException e )
      {
        ExceptionThrower.throwRuntimeException( errorMessage, e );
      }
    }
    return termVariants;
  }

  /**
   * Gets all stopwords and active terms.
   * 
   * @param ipiTermCandidate The IPI term candidate to check
   * @return The stopwords and active terms, where the key is the lowercase word/term and the value is the value object
   * @throws SQLException if the query fails
   */
  private Map<String, AbstractVO> getStopwordsAndActiveTerms( IpiTermCandidateVO ipiTermCandidate ) throws SQLException
  {
    List<StopwordVO> stopwords = DAOPool.get( StopwordDAO.class ).findAll( ipiTermCandidate.getLanguage() );
    List<AbstractTermVO> terms = DAOPool.get( GeneralTermDAO.class ).findAllTerms( ipiTermCandidate.getLanguage() );

    Map<String, AbstractVO> prohibited = new HashMap<String, AbstractVO>( stopwords.size() + terms.size() / ClassVO.HIGHEST_CLASS_NUMBER );
    for ( StopwordVO stopword: stopwords )
    {
      prohibited.put( stopword.getWord().toLowerCase(), stopword );
    }
    LOG.debug( "For language '" + ipiTermCandidate.getLanguage() + "', added " + stopwords.size() + " stopwords" );

    int addedTerms = 0;
    for ( AbstractTermVO term: terms )
    {
      if ( !( term instanceof IpiTermCandidateVO ) && term.getLanguage().equals( ipiTermCandidate.getLanguage() ) )
      {
        // Negative terms normally have no class number, but only matching class numbers are taken.
        if ( term instanceof NegativeIpiTermVO || term.getClassNumber().equals( ipiTermCandidate.getClassNumber() ) )
        {
          prohibited.put( term.getTerm().toLowerCase(), term );
          addedTerms++;
        }
      }
    }
    LOG.debug( "For language '" + ipiTermCandidate.getLanguage() + "', added " + addedTerms + " terms" );

    return prohibited;
  }
}
