package ch.ige.wdltool.ejb.impl;

import java.sql.SQLException;
import java.util.List;

import ch.ige.dbutils.dao.UnitOfWorkException;
import ch.ige.wdltool.dao.IpiTermGroupDAO;
import ch.ige.wdltool.exception.ExceptionThrower;
import ch.ige.wdltool.exception.WdlCreateException;
import ch.ige.wdltool.exception.WdlObjectNotFoundException;
import ch.ige.wdltool.exception.WrappedRuntimeException;
import ch.ige.wdltool.vo.IpiTermGroupVO;

/**
 * Implements services for IPI term groups.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: IpiTermGroupImpl.java,v 1.7 2008/01/18 12:08:09 lewis_lee Exp $
 */
public class IpiTermGroupImpl extends BaseServicesImpl<IpiTermGroupDAO, IpiTermGroupVO, Integer>
{
  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#getObjectName()
  @Override
  protected String getObjectName()
  {
    return "IPI term group";
  }

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#getDAOClass()
  @Override
  protected Class<IpiTermGroupDAO> getDAOClass()
  {
    return IpiTermGroupDAO.class;
  }

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#findAll(ch.ige.dbutils.dao.DAO)
  @Override
  protected List<IpiTermGroupVO> findAll( IpiTermGroupDAO dao ) throws SQLException
  {
    return dao.findAll();
  }

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#find(ch.ige.dbutils.dao.DAO, java.io.Serializable)
  @Override
  protected IpiTermGroupVO find( IpiTermGroupDAO dao, Integer id ) throws SQLException
  {
    return dao.find( id );
  }

  /**
   * Finds an IPI term group.
   * @param ipiTermNumber The term group's number (not the ID!)
   * @return The IPI term group containing the IPI term number
   * @throws WdlObjectNotFoundException 
   * @throws WrappedRuntimeException if the parameter is faulty
   * @see ch.ige.wdltool.ejb.WdlServices#findByIpiTermNumber(Integer)
   */
  protected IpiTermGroupVO findByIpiTermNumber( Integer ipiTermNumber ) throws WdlObjectNotFoundException, WrappedRuntimeException
  {
    final String errorMessage = "Didn't find " + getObjectName() + " with term number " + ipiTermNumber;
    IpiTermGroupVO termGroup = null;
    try
    {
      termGroup = getDAOInstance().findByNumber( ipiTermNumber );
      if ( termGroup == null )
      {
        ExceptionThrower.throwObjectNotFoundException( errorMessage, ipiTermNumber );
      }
    }
    catch ( RuntimeException e )
    {
      ExceptionThrower.throwRuntimeException( errorMessage, e );
    }
    catch ( SQLException e )
    {
      ExceptionThrower.throwObjectNotFoundException( errorMessage, e, ipiTermNumber );
    }
    return termGroup;
  }

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#insert(ch.ige.dbutils.dao.AbstractDAO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected IpiTermGroupVO insert( IpiTermGroupDAO dao, IpiTermGroupVO vo )
  {
    dao.insert( vo );
    return vo;
  }

  /**
   * Gets the next free IPI term group number and inserts a
   * IPI term group with it.
   * @return The inserted IPI term group with the next free IPI term group number
   * @throws SQLException if the IPI term can't be inserted
   * @throws WdlCreateException if the IPI term can't be inserted 
   * @see ch.ige.wdltool.ejb.WdlServices#createNextFreeIpiTermGroup()
   */
  protected IpiTermGroupVO createNextFreeIpiTermGroup() throws SQLException, WdlCreateException
  {
    try
    {
      return getDAOInstance().createNextFreeIpiTermGroup();
    }
    catch ( UnitOfWorkException e )
    {
      ExceptionThrower.throwCreateException( "Couldn't insert IPI term group", e, null );
    }
    return null;
  }
}
