package ch.ige.wdltool.ejb.impl;

import java.sql.SQLException;
import java.util.List;

import ch.ige.dbutils.dao.DAOPool;
import ch.ige.dbutils.dao.UnitOfWorkException;
import ch.ige.wdltool.dao.NiceTermDAO;
import ch.ige.wdltool.exception.ExceptionThrower;
import ch.ige.wdltool.exception.WdlValidationException;
import ch.ige.wdltool.exception.WrappedRuntimeException;
import ch.ige.wdltool.vo.NiceTermVO;
import ch.ige.wdltool.vo.TermCleanupCandidateStatusVO;
import ch.ige.wdltool.vo.TermIdCleanupCandidateStatusVO;

/**
 * Implements CRUD services for Nice terms.
 * <p>
 * Copyright (c) 2007, Swiss Federal Institute of Intellectual Property
 * </p>
 * 
 * @author Eric Lewis
 * @version $Id: NiceTermImpl.java,v 1.12 2008/12/09 16:08:56 bandschapp Exp $
 */
public class NiceTermImpl extends BaseTermServicesImpl<NiceTermDAO, NiceTermVO, Integer>
{
  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#getObjectName()
  @Override
  protected String getObjectName()
  {
    return "Nice term";
  }

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#getDAOClass()
  @Override
  protected Class<NiceTermDAO> getDAOClass()
  {
    return NiceTermDAO.class;
  }

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#find(ch.ige.dbutils.dao.AbstractDAO, java.io.Serializable)
  @Override
  protected NiceTermVO find( NiceTermDAO dao, Integer id ) throws SQLException
  {
    return dao.find( id );
  }

  /**
   * Finds the Generic Nice Terms.
   * @return The Generic Nice Terms.
   * @throws WrappedRuntimeException if the query or request fails.
   */
  protected List<NiceTermVO> findGenericTerms() throws WrappedRuntimeException
  {
    try
    {
      return DAOPool.get( NiceTermDAO.class ).findGenericTerms();
    }
    catch ( RuntimeException e )
    {
      ExceptionThrower.throwRuntimeException( "Can't find generic Nice terms", e );
    }
    return null;
  }

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#insert(ch.ige.dbutils.dao.DAO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected NiceTermVO insert( NiceTermDAO dao, NiceTermVO vo )
  {
    dao.insert( vo );
    return vo;
  }

  /**
   * Inserts a Nice term when no corresponding stopword or active term already exists. During the insert operation a
   * corresponding term candidate is removed if it exists.
   * 
   * @param vo The VO to insert
   * @return The new VO an the flag if a term candidate had been removed
   * @throws SQLException
   * @throws WdlValidationException
   * @throws WrappedRuntimeException 
   */
  public TermCleanupCandidateStatusVO<NiceTermVO> insertAndRemoveIpiTermCandidate( NiceTermVO vo ) throws SQLException, WdlValidationException,
      WrappedRuntimeException
  {
    try
    {
      checkValueObject( vo, false );
      TermIdCleanupCandidateStatusVO termIdCleanupCandidateStatusVO = DAOPool.get( getDAOClass() ).insertAndRemoveIpiTermCandidate( vo );
      return new TermCleanupCandidateStatusVO<NiceTermVO>( vo, termIdCleanupCandidateStatusVO.isCandidateRemoved() );
    }
    catch ( RuntimeException e )
    {
      ExceptionThrower.throwRuntimeException( e.getMessage(), e );
    }
    return null;
  }

  /**
   * Updates a Nice term when no corresponding stopword or active term already exists. During the update operation a
   * corresponding term candidate is removed if it exists.
   * 
   * @param dao The DAO to use
   * @param vo The VO to update
   * @throws IllegalArgumentException if the VO is faulty
   * @throws UnitOfWorkException if the update fails
   * @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#update(ch.ige.dbutils.dao.DAO, ch.ige.dbutils.vo.AbstractVO)
   */
  @Override
  protected void update( NiceTermDAO dao, NiceTermVO vo )
  {
    dao.update( vo );
  }

  /**
   * Updates a Nice term and simultaneously removes a corresponding term candidate if it exists.
   * 
   * @param vo The VO to insert
   * @return the flag if a term candidate had been removed
   * @throws SQLException
   * @throws WdlValidationException
   * @throws WrappedRuntimeException 
   */
  public boolean updateAndRemoveIpiTermCandidate( NiceTermVO vo ) throws SQLException, WdlValidationException, WrappedRuntimeException
  {
    try
    {
      checkValueObject( vo, true );
      return DAOPool.get( getDAOClass() ).updateAndRemoveIpiTermCandidate( vo );
    }
    catch ( RuntimeException e )
    {
      ExceptionThrower.throwRuntimeException( e.getMessage(), e );
    }
    return false;
  }

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#delete(ch.ige.dbutils.dao.AbstractDAO, java.io.Serializable)
  @Override
  protected void delete( NiceTermDAO dao, Integer id ) throws IllegalArgumentException, UnitOfWorkException
  {
    dao.delete( id );
  }
}
