package ch.ige.wdltool.ejb.impl;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Locale;

import org.apache.log4j.Logger;

import ch.ige.dbutils.dao.DAOPool;
import ch.ige.wdltool.dao.StopwordDAO;
import ch.ige.wdltool.exception.ExceptionThrower;
import ch.ige.wdltool.exception.WdlObjectNotFoundException;
import ch.ige.wdltool.exception.WrappedRuntimeException;
import ch.ige.wdltool.vo.StopwordVO;

/**
 * Implements CRUD services for stopwords.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: StopwordImpl.java,v 1.5 2008/01/28 15:24:50 lewis_lee Exp $
 */
public class StopwordImpl extends BaseServicesImpl<StopwordDAO, StopwordVO, Integer>
{
  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( StopwordImpl.class );

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#getObjectName()
  @Override
  protected String getObjectName()
  {
    return "stopword";
  }

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#getDAOClass()
  @Override
  protected Class<StopwordDAO> getDAOClass()
  {
    return StopwordDAO.class;
  }

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#find(ch.ige.dbutils.dao.AbstractDAO, java.io.Serializable)
  @Override
  protected StopwordVO find( StopwordDAO dao, Integer id ) throws SQLException
  {
    return dao.find( id );
  }

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#findAll(ch.ige.dbutils.dao.DAO)
  @Override
  protected List<StopwordVO> findAll( StopwordDAO dao ) throws SQLException
  {
    return dao.findAll();
  }

  /**
   * Finds all stopwords of a language that correspond to any word within <code>searchStopword</code>.
   * This splits up <code>searchStopword</code> into separate words and tries to find
   * any of those words in the search stopwords.
   * @param language The language in which stopwords are searched for; if this is <code>null</code>, all languages are taken
   * @param searchStopword The stopword to search for, which can contain any number of blank-separated words
   * @return The stopwords that were found or an empty <code>List</code> if none were found
   * @throws WdlObjectNotFoundException if the stopword can't be found
   * @throws WrappedRuntimeException if any argument is wrong
   * @see ch.ige.wdltool.ejb.WdlServices#findAnyWordWithinStopwords(java.util.Locale, java.lang.String)
   */
  public List<StopwordVO> findAnyWordWithinStopwords( Locale language, String searchStopword ) throws WdlObjectNotFoundException, WrappedRuntimeException
  {
    List<StopwordVO> stopwords = new ArrayList<StopwordVO>();
    try
    {
      if ( language == null )
      {
        stopwords = DAOPool.get( getDAOClass() ).findAnyWord( searchStopword );
      }
      else
      {
        stopwords = DAOPool.get( getDAOClass() ).findAnyWord( language, searchStopword );
      }
    }
    catch ( SQLException e )
    {
      ExceptionThrower.throwObjectNotFoundException( "Couldn't search for any word in: " + language + " with search stopword: " + searchStopword, e, null );
    }
    catch ( RuntimeException e )
    {
      ExceptionThrower.throwRuntimeException( e.getMessage(), e );
    }
    LOG.debug( "Searched word: " + searchStopword + ", found " + stopwords.size() );
    return stopwords;
  }

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#insert(ch.ige.dbutils.dao.AbstractDAO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected StopwordVO insert( StopwordDAO dao, StopwordVO vo )
  {
    dao.insert( vo );
    return vo;
  }

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#update(ch.ige.dbutils.dao.AbstractDAO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void update( StopwordDAO dao, StopwordVO vo )
  {
    dao.update( vo );
  }

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#delete(ch.ige.dbutils.dao.AbstractDAO, java.io.Serializable)
  @Override
  protected void delete( StopwordDAO dao, Integer id )
  {
    dao.delete( id );
  }

  // @see ch.ige.wdltool.ejb.impl.BaseServicesImpl#delete(ch.ige.dbutils.dao.DAO, java.util.Collection)
  @Override
  protected void delete( StopwordDAO dao, Collection<Integer> ids )
  {
    dao.delete( ids );
  }
}
