package ch.ige.wdltool.ejb.impl;

import ch.ige.wdltool.exception.WdlObjectNotFoundException;
import ch.ige.wdltool.exception.WdlUpdateException;
import ch.ige.wdltool.exception.WdlValidationException;
import ch.ige.wdltool.exception.WrappedRuntimeException;
import ch.ige.wdltool.vo.IpiTermCandidateVO;
import ch.ige.wdltool.vo.IpiTermVO;
import ch.ige.wdltool.vo.NegativeIpiTermVO;

/**
 * Implements services for conversion of one term type into another.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Christian Heinzer
 * @version  $Id: TermConversionImpl.java,v 1.8 2008/04/28 13:20:38 heinzer Exp $
 */
public class TermConversionImpl
{
  /**
   * The implementation of wdl services to use
   */
  private WdlServicesImpl services;

  /**
   * Public constructor.
   * @param services The Implementation of the services to use
   */
  public TermConversionImpl( WdlServicesImpl services )
  {
    this.services = services;
  }

  /**
   * Converts an IPI term into a negative IPI term and updates it.
   * @param negativeIpiTerm The negative IPI term to update to. The term with this <code>ID</code> is still of type IPI term in the DB. 
   * @throws WdlUpdateException 
   * @throws WdlObjectNotFoundException 
   * @throws WdlValidationException 
   * @throws WrappedRuntimeException 
   */
  public void convertIpiTermToNegativeIpiTerm( NegativeIpiTermVO negativeIpiTerm ) throws WdlUpdateException, WdlObjectNotFoundException,
      WdlValidationException, WrappedRuntimeException
  {
    //cleanup the old ipi term prior to update - several fields make no sense for the new negative ipi term
    IpiTermVO oldTerm = this.services.getIpiTermImpl().find( negativeIpiTerm.getIdentifier() );
    oldTerm.setTermGroupId( null );
    oldTerm.setRenewal( null );
    this.services.getIpiTermImpl().update( oldTerm );
    this.services.getNegativeIpiTermImpl().update( negativeIpiTerm );
  }

  /**
   * Converts a negative IPI term into an IPI term, updates it and removes a corresponding IPI term candidate - if it exists.
   * Note that for security reasons, this checks whether the term already exists, either as term
   * or as stopword.
   * @param ipiTerm The IPI term to update. The term with this <code>ID</code> is still of type negative IPI term in the DB. 
   * @return a status flag determining whether a candidate has been deleted because of the operation
   * @throws WrappedRuntimeException 
   * @throws WdlUpdateException 
   * @throws WdlValidationException 
   */
  public boolean convertNegativeIpiTermToIpiTermAndRemoveIpiTermCandidate( IpiTermVO ipiTerm ) throws WrappedRuntimeException, WdlUpdateException,
      WdlValidationException
  {
    //cleanup of the ipi term as precondition for conversion
    //setting fields to default values is not necessary as an IPI term can use all fields from the negative IPI term.
    return this.services.updateIpiTermAndRemoveIpiTermCandidate( ipiTerm );
  }

  /**
   * Converts an inactive IPI term into an IPI term candidate and updates it.
   * @param ipiTermCandidate The IPI term candidate to update. The term with this <code>ID</code> is still of type IPI term in the DB.
   * @throws WrappedRuntimeException 
   * @throws WdlUpdateException 
   * @throws WdlValidationException 
   */
  public void convertInactiveTermToIpiTermCandidate( IpiTermCandidateVO ipiTermCandidate ) throws WrappedRuntimeException, WdlUpdateException,
      WdlValidationException
  {
    this.services.updateIpiTermCandidate( ipiTermCandidate );
  }
}
