package ch.ige.wdltool.util;

import java.io.File;
import java.sql.SQLException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.commons.configuration.CompositeConfiguration;
import org.apache.commons.configuration.SystemConfiguration;
import org.apache.log4j.Logger;

import ch.ige.dbutils.dao.DAOPool;
import ch.ige.dbutils.vo.ConfigurationTable;
import ch.ige.util.validation.Check;
import ch.ige.wdltool.dao.ConfigurationDAO;
import ch.ige.wdltool.dao.DataSourceDAO;
import ch.ige.wdltool.ejb.WdlServices;
import ch.ige.wdltool.enums.DataSource;
import ch.ige.wdltool.vo.DataSourceVO;

/**
 * Accesses the configuration, which is saved in the database and can be overwritten by system properties.
 * For general properties the keys are public static final Strings of this class.
 * For special (e.g. data source) properties the keys are assembled and can be retrieved by <code>getKey()</code> methods. 
 * 
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   heinzer
 * @version  $Id: Configuration.java,v 1.20 2008/03/31 09:58:23 lewis_lee Exp $
 */
public class Configuration
{
  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( Configuration.class );

  /**
   * Key for boolean property specifying whether links are created for correct words (shown in green). 
   */
  public static final String CREATE_LINKS_FOR_CORRECT_WORDS = "LINKS_FOR_CORRECT_WORDS";

  /**
   * The key suffix for the cleaner activity
   */
  public static final String DATA_SOURCE_CLEANER_ACTIVE = "_CLEANER_ACTIVE";

  /**
   * The key suffix for the term validity period
   */
  public static final String DATA_SOURCE_TERM_VALIDITY_PERIOD = "_TERM_VALIDITY_PERIOD";

  /**
   * The key suffix for the term confirmation period
   */
  public static final String DATA_SOURCE_TERM_CONFIRMATION_PERIOD = "_TERM_CONFIRMATION_PERIOD";

  /**
   * The key for the directory where analyzer log files are saved
   */
  public static final String ANALYZER_LOG_DIRECTORY = "ANALYZER_LOG_DIR";

  /**
   * The key for the directory where checker log files are saved
   */
  public static final String CHECKER_LOG_DIRECTORY = "CHECKER_LOG_DIR";

  /**
   * The key for the directory where cleaner log files are saved
   */
  public static final String CLEANER_LOG_DIRECTORY = "CLEANER_LOG_DIR";

  /**
   * The key for the directory where replication log files are saved
   */
  public static final String REPLICATION_LOG_DIRECTORY = "REPLICATION_LOG_DIR";

  /**
   * Does the actual reading and overriding work.
   */
  private static org.apache.commons.configuration.Configuration configuration = null;

  static
  {
    readConfig();
  }

  /**
   * Rebuilds the whole configuration.
   * To be used if properties in the database have changed.
   */
  public static synchronized void readConfig()
  {
    ConfigurationDAO configurationDAO = DAOPool.get( ConfigurationDAO.class );

    //read general configuration entries
    org.apache.commons.configuration.Configuration dbConfiguration = null;
    ConfigurationTable configurationTable = configurationDAO.getConfigurationTable( "CONFIGURATION", null, "KEY", "VALUE", null );
    try
    {
      dbConfiguration = configurationTable.getFullConfiguration( true );

      //read specific configuration enties - not directly supported by commons.config: add manually
      DataSourceDAO dataSourceDAO = DAOPool.get( DataSourceDAO.class );
      for ( DataSourceVO dataSourceVO: dataSourceDAO.findAll() )
      {
        DataSource dataSource = dataSourceVO.getIdentifier();
        dbConfiguration.addProperty( getDataSourceKey( dataSource, DATA_SOURCE_TERM_VALIDITY_PERIOD ), dataSourceVO.getTermValidityPeriod() );
        dbConfiguration.addProperty( getDataSourceKey( dataSource, DATA_SOURCE_TERM_CONFIRMATION_PERIOD ), dataSourceVO.getTermConfirmationPeriod() );
        dbConfiguration.addProperty( getDataSourceKey( dataSource, DATA_SOURCE_CLEANER_ACTIVE ), dataSourceVO.isCleanerActive() );
      }
    }
    catch ( SQLException e )
    {
      LOG.error( "Could not read configuration from WDL database: " + e.getMessage(), e );
    }

    // Merge DB and system configuration
    CompositeConfiguration compositeConfiguration = new CompositeConfiguration();
    compositeConfiguration.addConfiguration( new SystemConfiguration() ); // System configuration has precedence
    compositeConfiguration.addConfiguration( dbConfiguration );
    configuration = compositeConfiguration;
  }

  //---------------------------------------------------------------------------------
  // Naming
  //---------------------------------------------------------------------------------

  /**
   * Gets the resource key specific for the given data source.
   * @param dataSource The data source
   * @param keySuffix The suffix to add for the real configuration key, for instance {@link #DATA_SOURCE_TERM_CONFIRMATION_PERIOD}
   * @return The full key used for the configuration
   */
  public static String getDataSourceKey( DataSource dataSource, String keySuffix )
  {
    return dataSource.name().toUpperCase() + keySuffix;
  }

  //---------------------------------------------------------------------------------
  // General properties methods
  //---------------------------------------------------------------------------------

  /**
   * Returns a general property where the key is known.
   * @param key The key to retrieve
   * @return value for specified key
   */
  private static synchronized String getProperty( String key )
  {
    Check.notEmpty( key, "key" );
    return configuration.getString( key );
  }

  /**
   * Sets a general property with a known key.
   * @param key The key to set, e.g. {@link #ANALYZER_LOG_DIRECTORY}
   * @param value The value to set
   */
  private static synchronized void setProperty( String key, String value )
  {
    Check.notEmpty( key, "key" );
    Check.notEmpty( value, key );

    try
    {
      DAOPool.get( ConfigurationDAO.class ).save( key, value );
      readConfig();
    }
    catch ( SQLException e )
    {
      throw new RuntimeException( "Couldn't save property '" + key + "'", e );
    }
  }

  //---------------------------------------------------------------------------------
  // Data source properties methods
  //---------------------------------------------------------------------------------

  /**
   * Gets a period configuration for a data source.
   * @param dataSource The data source
   * @param keySuffix The suffix to add for the real configuration key
   * @return The period configuration or <code>null</code> if there's none
   */
  private static synchronized Integer getDataSourcePeriod( final DataSource dataSource, final String keySuffix )
  {
    checkDataSource( dataSource );
    return configuration.getInteger( getDataSourceKey( dataSource, keySuffix ), null );
  }

  /**
   * Sets a data source's attribute and saves it.
   * @param dataSource The data source to change
   * @param dataSourceChanger Sets the value in the data source
   */
  private static synchronized void setDataSourceAttribute( final DataSource dataSource, final DataSourceChanger dataSourceChanger )
  {
    checkDataSource( dataSource );
    try
    {
      DataSourceDAO dataSourceDAO = DAOPool.get( DataSourceDAO.class );
      DataSourceVO existingDataSource = dataSourceDAO.find( dataSource );
      dataSourceChanger.setAttribute( existingDataSource );
      dataSourceDAO.update( existingDataSource );
    }
    catch ( SQLException e )
    {
      throw new IllegalArgumentException( "Unknown data source " + dataSource, e );
    }
  }

  /**
   * Normally, only {@link DataSource#IPI} can be configured.
   * @param dataSource The data source to check
   * @throws IllegalArgumentException if the data source can't be configured
   * 
   */
  private static void checkDataSource( final DataSource dataSource ) throws IllegalArgumentException
  {
    Check.notNull( dataSource, "data source" );
    if ( !dataSource.equals( DataSource.IPI ) )
    {
      throw new IllegalArgumentException( "Only IPI data source can be configured" );
    }
  }

  //---------------------------------------------------------------------------------
  // Specific properties
  //---------------------------------------------------------------------------------

  /**
   * Gets a configured directory.
   * @param key The key in the configuration
   * @return The directory or <code>null</code> if the key doesn't exist
   */
  private static synchronized File getDirectory( String key )
  {
    String filename = getProperty( key );
    if ( filename == null )
    {
      return null;
    }
    else
    {
      return new File( filename );
    }
  }

  /**
   * Sets a configured directory.
   * @param key The key to set, e.g. {@link #ANALYZER_LOG_DIRECTORY}
   * @param directory The directory to set (value)
   */
  private static synchronized void setDirectory( String key, File directory )
  {
    Check.notNull( directory, key );
    setProperty( key, directory.toString() );
  }

  /**
   * Gets a new log file.
   * @param directoryKey The key that finds the log directory
   * @param dateFormat The date format to use for the timestamp
   * @param fileNameStub The file name stub, like <code>cleaner</code>
   * @param fileExtension The file extension, like <code>.txt</code>
   * @return The complete log file or <code>null</code> if the directory can't be fetched
   */
  private static File getNewLogFile( String directoryKey, String dateFormat, String fileNameStub, String fileExtension )
  {
    DateFormat simpleDateFormat = new SimpleDateFormat( dateFormat );
    String now = simpleDateFormat.format( new Date( System.currentTimeMillis() ) );
    String directory = getProperty( directoryKey );
    if ( directory == null )
    {
      return null;
    }
    else
    {
      return new File( directory + "\\" + fileNameStub + "_" + now + fileExtension );
    }
  }

  //---------------------------------------------------------------------------------
  // Normal properties
  //---------------------------------------------------------------------------------

  /**
   * See {@link WdlServices#getAnalyzerLogDirectory()}.
   * @return The analyzer log directory
   */
  public static synchronized File getAnalyzerLogDirectory()
  {
    return getDirectory( ANALYZER_LOG_DIRECTORY );
  }

  /**
   * See {@link WdlServices#getNewAnalyzerLogFileName()}.
   * @return The new analyzer log file
   */
  public static synchronized File getNewAnalyzerLogFileName()
  {
    return getNewLogFile( ANALYZER_LOG_DIRECTORY, "yyyy-MM-dd", "wdl_analyse", ".txt" );
  }

  /**
   * Gets a new file for the replication log
   * @return The new replication log file
   */
  public static synchronized File getNewReplicationLogFileName()
  {
    return getNewLogFile( REPLICATION_LOG_DIRECTORY, "yyyy-MM-dd", "ea_replication", ".txt" );
  }

  /**
   * See {@link WdlServices#getReplicationLogDirectory()}.
   * @return The new replication log directory
   */
  public static synchronized File getReplicationLogDirectory()
  {
    return getDirectory( REPLICATION_LOG_DIRECTORY );
  }

  /**
   * See {@link WdlServices#setAnalyzerLogDirectory(File)}.
   * @param logDirectory The analyzer log directory
   */
  public static synchronized void setAnalyzerLogDirectory( File logDirectory )
  {
    setDirectory( ANALYZER_LOG_DIRECTORY, logDirectory );
  }

  /**
   * See {@link WdlServices#getCheckerLogDirectory()}.
   * @return The checker log directory
   */
  public static synchronized File getCheckerLogDirectory()
  {
    return getDirectory( CHECKER_LOG_DIRECTORY );
  }

  /**
   * See {@link WdlServices#setCheckerLogDirectory(File)}.
   * @param logDirectory The checker log directory
   */
  public static synchronized void setCheckerLogDirectory( File logDirectory )
  {
    setDirectory( CHECKER_LOG_DIRECTORY, logDirectory );
  }

  /**
   * See {@link WdlServices#getNewCheckerLogFileName()}.
   * @return The new checker log file
   */
  public static synchronized File getNewCheckerLogFileName()
  {
    return getNewLogFile( CHECKER_LOG_DIRECTORY, "yyyy-MM-dd'T'HHmmss", "wdl_checker", ".txt" );
  }

  /**
   * See {@link WdlServices#getNewCheckerSummaryFileName()}.
   * @return The new checker log file
   */
  public static synchronized File getNewCheckerSummaryFileName()
  {
    return getNewLogFile( CHECKER_LOG_DIRECTORY, "yyyy-MM-dd'T'HHmmss", "wdl_checker_summary", ".csv" );
  }

  /**
   * See {@link WdlServices#getCleanerLogDirectory()}.
   * @return The cleaner log directory
   */
  public static synchronized File getCleanerLogDirectory()
  {
    return getDirectory( CLEANER_LOG_DIRECTORY );
  }

  /**
   * See {@link WdlServices#setCleanerLogDirectory(File)}.
   * @param logDirectory The cleaner log directory
   */
  public static synchronized void setCleanerLogDirectory( File logDirectory )
  {
    setDirectory( CLEANER_LOG_DIRECTORY, logDirectory );
  }

  /**
   * See {@link WdlServices#getNewCleanerLogFileName()}.
   * @return The new cleaner log file
   */
  public static synchronized File getNewCleanerLogFileName()
  {
    return getNewLogFile( CLEANER_LOG_DIRECTORY, "yyyy-MM-dd'T'HHmmss", "wdl_cleaner", ".txt" );
  }

  /**
   * See {@link WdlServices#createLinksForCorrectWords()}.
   * @return <code>true</code> if links are created, otherwise <code>false</code>
   */
  public static synchronized Boolean createLinksForCorrectWords()
  {
    return configuration.getBoolean( CREATE_LINKS_FOR_CORRECT_WORDS, false );
  }

  /**
   * See {@link WdlServices#setCreateLinksForCorrectWords(boolean)}.
   * @param createLinksForCorrectWords <code>true</code> if links are created, otherwise <code>false</code>
   */
  public static synchronized void setCreateLinksForCorrectWords( Boolean createLinksForCorrectWords )
  {
    setProperty( CREATE_LINKS_FOR_CORRECT_WORDS, createLinksForCorrectWords.toString() );
  }

  //---------------------------------------------------------------------------------
  // Data source properties
  //---------------------------------------------------------------------------------

  /**
   * See {@link WdlServices#getTermConfirmationPeriod(DataSource)}.
   * @param dataSource The data source 
   * @return The data source's configuration
   */
  public static synchronized Integer getTermConfirmationPeriod( final DataSource dataSource )
  {
    return getDataSourcePeriod( dataSource, DATA_SOURCE_TERM_CONFIRMATION_PERIOD );
  }

  /**
   * See {@link WdlServices#setTermConfirmationPeriod(DataSource, Integer)}.
   * @param dataSource The data source
   * @param termConfirmationPeriod The term confirmation period
   */
  public static synchronized void setTermConfirmationPeriod( final DataSource dataSource, final Integer termConfirmationPeriod )
  {
    Check.notBelow( termConfirmationPeriod, "term confirmation period", 1 );
    setDataSourceAttribute( dataSource, new DataSourceChanger()
    {
      public void setAttribute( DataSourceVO dataSourceVO )
      {
        dataSourceVO.setTermConfirmationPeriod( termConfirmationPeriod );
      }
    } );
  }

  /**
   * See {@link WdlServices#getTermValidityPeriod(DataSource)}.
   * @param dataSource The data source 
   * @return The data source's configuration
   */
  public static synchronized Integer getTermValidityPeriod( final DataSource dataSource )
  {
    return getDataSourcePeriod( dataSource, DATA_SOURCE_TERM_VALIDITY_PERIOD );
  }

  /**
   * See {@link WdlServices#setTermValidityPeriod(DataSource, Integer)}.
   * @param dataSource The data source
   * @param termValidityPeriod The term validity period
   */
  public static synchronized void setTermValidityPeriod( final DataSource dataSource, final Integer termValidityPeriod )
  {
    Check.notBelow( termValidityPeriod, "term validity period", 1 );
    setDataSourceAttribute( dataSource, new DataSourceChanger()
    {
      public void setAttribute( DataSourceVO dataSourceVO )
      {
        dataSourceVO.setTermValidityPeriod( termValidityPeriod );
      }
    } );
  }

  //----------------------------------------------------------------

  /**
   * Used for changing a data source.
   */
  private interface DataSourceChanger
  {
    /**
     * Sets an attribute.
     * @param dataSource The data source to change
     */
    public void setAttribute( DataSourceVO dataSource );
  }
}
